@@ -6,29 +6,28 @@
 #include "CanvasLayerD3D10.h"
 
 #include "../d3d9/Nv3DVUtils.h"
-#include "gfxImageSurface.h"
 #include "gfxWindowsSurface.h"
 #include "gfxWindowsPlatform.h"
-#include "SurfaceStream.h"
 #include "SharedSurfaceANGLE.h"
+#include "SharedSurfaceGL.h"
 #include "gfxContext.h"
 #include "GLContext.h"
-
-using namespace mozilla::gl;
-using namespace mozilla::gfx;
+#include "gfxPrefs.h"
 
 namespace mozilla {
 namespace layers {
 
+using namespace mozilla::gl;
+using namespace mozilla::gfx;
+
 CanvasLayerD3D10::CanvasLayerD3D10(LayerManagerD3D10 *aManager)
   : CanvasLayer(aManager, nullptr)
   , LayerD3D10(aManager)
-  , mDataIsPremultiplied(false)
-  , mNeedsYFlip(false)
+  , mDataIsPremultiplied(true)
+  , mOriginPos(gl::OriginPos::TopLeft)
   , mHasAlpha(true)
 {
     mImplData = static_cast<LayerD3D10*>(this);
-    mForceReadback = Preferences::GetBool("webgl.force-layers-readback", false);
 }
 
 CanvasLayerD3D10::~CanvasLayerD3D10()
@@ -40,71 +39,48 @@ CanvasLayerD3D10::Initialize(const Data& aData)
 {
   NS_ASSERTION(mSurface == nullptr, "BasicCanvasLayer::Initialize called twice!");
 
-  if (aData.mSurface) {
-    mSurface = aData.mSurface;
-    NS_ASSERTION(!aData.mGLContext && !aData.mDrawTarget,
-                 "CanvasLayer can't have both surface and WebGLContext/DrawTarget");
-    mNeedsYFlip = false;
-    mDataIsPremultiplied = true;
-  } else if (aData.mGLContext) {
+  if (aData.mGLContext) {
     mGLContext = aData.mGLContext;
     NS_ASSERTION(mGLContext->IsOffscreen(), "Canvas GLContext must be offscreen.");
     mDataIsPremultiplied = aData.mIsGLAlphaPremult;
-    mNeedsYFlip = true;
+    mOriginPos = gl::OriginPos::TopLeft;
 
     GLScreenBuffer* screen = mGLContext->Screen();
-    SurfaceStreamType streamType =
-        SurfaceStream::ChooseGLStreamType(SurfaceStream::MainThread,
-                                          screen->PreserveBuffer());
 
-    SurfaceFactory_GL* factory = nullptr;
-    if (!mForceReadback) {
-      factory = SurfaceFactory_ANGLEShareHandle::Create(mGLContext,
-                                                        device(),
-                                                        screen->Caps());
+    UniquePtr<SurfaceFactory> factory = nullptr;
+    if (!gfxPrefs::WebGLForceLayersReadback()) {
+      if (mGLContext->IsANGLE()) {
+        factory = SurfaceFactory_ANGLEShareHandle::Create(mGLContext,
+                                                          screen->mCaps);
+      }
     }
 
     if (factory) {
-      screen->Morph(factory, streamType);
+      screen->Morph(Move(factory));
     }
   } else if (aData.mDrawTarget) {
     mDrawTarget = aData.mDrawTarget;
-    mNeedsYFlip = false;
-    mDataIsPremultiplied = true;
-    void *texture = mDrawTarget->GetNativeSurface(NATIVE_SURFACE_D3D10_TEXTURE);
+    void *texture = mDrawTarget->GetNativeSurface(NativeSurfaceType::D3D10_TEXTURE);
 
     if (texture) {
       mTexture = static_cast<ID3D10Texture2D*>(texture);
 
-      NS_ASSERTION(!aData.mGLContext && !aData.mSurface,
-                   "CanvasLayer can't have both surface and WebGLContext/Surface");
+      NS_ASSERTION(!aData.mGLContext,
+                   "CanvasLayer can't have both DrawTarget and WebGLContext/Surface");
 
       mBounds.SetRect(0, 0, aData.mSize.width, aData.mSize.height);
       device()->CreateShaderResourceView(mTexture, nullptr, getter_AddRefs(mSRView));
       return;
-    } 
-    
+    }
+
     // XXX we should store mDrawTarget and use it directly in UpdateSurface,
     // bypassing Thebes
-    mSurface = gfxPlatform::GetPlatform()->GetThebesSurfaceForDrawTarget(mDrawTarget);
+    mSurface = mDrawTarget->Snapshot();
   } else {
-    NS_ERROR("CanvasLayer created without mSurface, mDrawTarget or mGLContext?");
+    MOZ_CRASH("CanvasLayer created without mSurface, mDrawTarget or mGLContext?");
   }
 
   mBounds.SetRect(0, 0, aData.mSize.width, aData.mSize.height);
-
-  if (mSurface && mSurface->GetType() == gfxASurface::SurfaceTypeD2D) {
-    void *data = mSurface->GetData(&gKeyD3D10Texture);
-    if (data) {
-      mTexture = static_cast<ID3D10Texture2D*>(data);
-      mIsD2DTexture = true;
-      device()->CreateShaderResourceView(mTexture, nullptr, getter_AddRefs(mSRView));
-      mHasAlpha =
-        mSurface->GetContentType() == gfxASurface::CONTENT_COLOR_ALPHA;
-      return;
-    }
-  }
-
   mIsD2DTexture = false;
 
   // Create a texture in case we need to readback.
@@ -131,88 +107,72 @@ CanvasLayerD3D10::UpdateSurface()
   if (mDrawTarget) {
     mDrawTarget->Flush();
   } else if (mIsD2DTexture) {
-    mSurface->Flush();
     return;
   }
 
+  if (!mTexture) {
+    return;
+  }
+
+  SharedSurface* surf = nullptr;
   if (mGLContext) {
-    SharedSurface* surf = mGLContext->RequestFrame();
+    auto screen = mGLContext->Screen();
+    MOZ_ASSERT(screen);
+
+    surf = screen->Front()->Surf();
     if (!surf)
+      return;
+    surf->WaitSync();
+
+    if (surf->mType == SharedSurfaceType::EGLSurfaceANGLE) {
+      SharedSurface_ANGLEShareHandle* shareSurf = SharedSurface_ANGLEShareHandle::Cast(surf);
+      HANDLE shareHandle = shareSurf->GetShareHandle();
+
+      HRESULT hr = device()->OpenSharedResource(shareHandle,
+                                                __uuidof(ID3D10Texture2D),
+                                                getter_AddRefs(mTexture));
+      if (FAILED(hr))
         return;
 
-    switch (surf->Type()) {
-      case SharedSurfaceType::EGLSurfaceANGLE: {
-        SharedSurface_ANGLEShareHandle* shareSurf = SharedSurface_ANGLEShareHandle::Cast(surf);
-
-        mSRView = shareSurf->GetSRV();
+      hr = device()->CreateShaderResourceView(mTexture,
+                                              nullptr,
+                                              getter_AddRefs(mSRView));
+      if (FAILED(hr))
         return;
-      }
-      case SharedSurfaceType::Basic: {
-        SharedSurface_Basic* shareSurf = SharedSurface_Basic::Cast(surf);
-        // WebGL reads entire surface.
-        D3D10_MAPPED_TEXTURE2D map;
 
-        HRESULT hr = mTexture->Map(0, D3D10_MAP_WRITE_DISCARD, 0, &map);
-
-        if (FAILED(hr)) {
-          NS_WARNING("Failed to map CanvasLayer texture.");
-          return;
-        }
-
-        gfxImageSurface* frameData = shareSurf->GetData();
-        // Scope for gfxContext, so it's destroyed before Unmap.
-        {
-          nsRefPtr<gfxImageSurface> mapSurf = 
-              new gfxImageSurface((uint8_t*)map.pData,
-                                  shareSurf->Size(),
-                                  map.RowPitch,
-                                  gfxASurface::ImageFormatARGB32);
-
-          nsRefPtr<gfxContext> ctx = new gfxContext(mapSurf);
-          ctx->SetOperator(gfxContext::OPERATOR_SOURCE);
-          ctx->SetSource(frameData);
-          ctx->Paint();
-
-          mapSurf->Flush();
-        }
-
-        mTexture->Unmap(0);
-        mSRView = mUploadSRView;
-        break;
-      }
-
-      default:
-        MOZ_CRASH("Unhandled SharedSurfaceType.");
-    }
-  } else if (mSurface) {
-    RECT r;
-    r.left = 0;
-    r.top = 0;
-    r.right = mBounds.width;
-    r.bottom = mBounds.height;
-
-    D3D10_MAPPED_TEXTURE2D map;
-    HRESULT hr = mTexture->Map(0, D3D10_MAP_WRITE_DISCARD, 0, &map);
-
-    if (FAILED(hr)) {
-      NS_WARNING("Failed to lock CanvasLayer texture.");
       return;
     }
-
-    nsRefPtr<gfxImageSurface> dstSurface;
-
-    dstSurface = new gfxImageSurface((unsigned char*)map.pData,
-                                     gfxIntSize(mBounds.width, mBounds.height),
-                                     map.RowPitch,
-                                     gfxASurface::ImageFormatARGB32);
-    nsRefPtr<gfxContext> ctx = new gfxContext(dstSurface);
-    ctx->SetOperator(gfxContext::OPERATOR_SOURCE);
-    ctx->SetSource(mSurface);
-    ctx->Paint();
-    
-    mTexture->Unmap(0);
-    mSRView = mUploadSRView;
   }
+
+  D3D10_MAPPED_TEXTURE2D map;
+  HRESULT hr = mTexture->Map(0, D3D10_MAP_WRITE_DISCARD, 0, &map);
+
+  if (FAILED(hr)) {
+    gfxWarning() << "Failed to lock CanvasLayer texture.";
+    return;
+  }
+
+  RefPtr<DrawTarget> destTarget =
+    Factory::CreateDrawTargetForD3D10Texture(mTexture,
+                                             SurfaceFormat::R8G8B8A8);
+
+  if (!destTarget) {
+    gfxWarning() << "Invalid D3D10 texture target R8G8B8A8";
+    return;
+  }
+
+  if (surf) {
+    if (!ReadbackSharedSurface(surf, destTarget)) {
+      gfxWarning() << "Failed to readback into texture.";
+    }
+  } else if (mSurface) {
+    Rect r(Point(0, 0), ToRect(mBounds).Size());
+    destTarget->DrawSurface(mSurface, r, r, DrawSurfaceOptions(),
+                            DrawOptions(1.0F, CompositionOp::OP_SOURCE));
+  }
+
+  mTexture->Unmap(0);
+  mSRView = mUploadSRView;
 }
 
 Layer*
@@ -240,7 +200,7 @@ CanvasLayerD3D10::RenderLayer()
   shaderFlags |= mDataIsPremultiplied
                 ? SHADER_PREMUL : SHADER_NON_PREMUL | SHADER_RGBA;
   shaderFlags |= mHasAlpha ? SHADER_RGBA : SHADER_RGB;
-  shaderFlags |= mFilter == gfxPattern::FILTER_NEAREST
+  shaderFlags |= mFilter == GraphicsFilter::FILTER_NEAREST
                 ? SHADER_POINT : SHADER_LINEAR;
   ID3D10EffectTechnique* technique = SelectShader(shaderFlags);
 
@@ -256,7 +216,9 @@ CanvasLayerD3D10::RenderLayer()
       (float)mBounds.height)
     );
 
-  if (mNeedsYFlip) {
+  const bool needsYFlip = (mOriginPos == gl::OriginPos::BottomLeft);
+
+  if (needsYFlip) {
     effect()->GetVariableByName("vTextureCoords")->AsVector()->SetFloatVector(
       ShaderConstantRectD3D10(
         0,
@@ -269,7 +231,7 @@ CanvasLayerD3D10::RenderLayer()
   technique->GetPassByIndex(0)->Apply(0);
   device()->Draw(4, 0);
 
-  if (mNeedsYFlip) {
+  if (needsYFlip) {
     effect()->GetVariableByName("vTextureCoords")->AsVector()->
       SetFloatVector(ShaderConstantRectD3D10(0, 0, 1.0f, 1.0f));
   }
