@@ -10,132 +10,227 @@
 #include <stdint.h>                     // for uint32_t, uint64_t
 #include <sys/types.h>                  // for int32_t
 #include "gfxCore.h"                    // for NS_GFX
+#include "mozilla/ToString.h"           // for mozilla::ToString
 #include "nsCoord.h"                    // for nscoord
 #include "nsError.h"                    // for nsresult
 #include "nsPoint.h"                    // for nsIntPoint, nsPoint
 #include "nsRect.h"                     // for nsIntRect, nsRect
 #include "nsMargin.h"                   // for nsIntMargin
-#include "nsString.h"               // for nsCString
+#include "nsStringGlue.h"               // for nsCString
 #include "xpcom-config.h"               // for CPP_THROW_NEW
+#include "mozilla/TypedEnum.h"          // for the VisitEdges typed enum
+#include "mozilla/Move.h"               // for mozilla::Move
 
 class nsIntRegion;
+class gfx3DMatrix;
 
-/**
- * Implementation of regions.
- * A region is represented as circular double-linked list of nsRegion::RgnRect structures.
- * Rectangles in this list do not overlap and are sorted by (y, x) coordinates.
+#include "pixman.h"
+
+/* For information on the internal representation look at pixman-region.c
  *
- * nsRegions use nscoord coordinates and nsRects.
+ * This replaces an older homebrew implementation of nsRegion. The
+ * representation used here may use more rectangles than nsRegion however, the
+ * representation is canonical.  This means that there's no need for an
+ * Optimize() method because for a paticular region there is only one
+ * representation. This means that nsIntRegion will have more predictable
+ * performance characteristics than the old nsRegion and should not become
+ * degenerate.
+ *
+ * The pixman region code originates from X11 which has spread to a variety of
+ * projects including Qt, Gtk, Wine. It should perform reasonably well.
  */
-class NS_GFX nsRegion
+
+MOZ_BEGIN_ENUM_CLASS(VisitSide)
+	TOP,
+	BOTTOM,
+	LEFT,
+	RIGHT
+MOZ_END_ENUM_CLASS(VisitSide)
+
+class nsRegionRectIterator;
+
+class nsRegion
 {
+
   friend class nsRegionRectIterator;
-  friend class RgnRectMemoryAllocator;
-
-
-// Special version of nsRect structure for speed optimizations in nsRegion code.
-// Most important functions could be made inline and be sure that passed rectangles
-// will always be non-empty.
-// 
-// Do not add any new member variables to this structure! 
-// Otherwise it will break casts from nsRect to nsRectFast, which expect data parts to be identical.
-  struct nsRectFast : public nsRect
-  {
-    nsRectFast () {}      // No need to call parent constructor to set default values
-    nsRectFast (int32_t aX, int32_t aY, int32_t aWidth, int32_t aHeight) : nsRect (aX, aY, aWidth, aHeight) {}
-    nsRectFast (const nsRect& aRect) : nsRect (aRect) {}
-
-    // Override nsRect methods to make them inline. Do not check for emptiness.
-    inline bool Contains (const nsRect& aRect) const;
-    inline bool Intersects (const nsRect& aRect) const;
-    inline bool IntersectRect (const nsRect& aRect1, const nsRect& aRect2);
-    inline void UnionRect (const nsRect& aRect1, const nsRect& aRect2);
-  };
-
-
-  struct RgnRect : public nsRectFast
-  {
-    RgnRect* prev;
-    RgnRect* next;
-
-    RgnRect () {}                           // No need to call parent constructor to set default values
-    RgnRect (int32_t aX, int32_t aY, int32_t aWidth, int32_t aHeight) : nsRectFast (aX, aY, aWidth, aHeight) {}
-    RgnRect (const nsRectFast& aRect) : nsRectFast (aRect) {}
-
-    void* operator new (size_t) CPP_THROW_NEW;
-    void  operator delete (void* aRect, size_t);
-
-    RgnRect& operator = (const RgnRect& aRect)      // Do not overwrite prev/next pointers
-    {
-      x = aRect.x;
-      y = aRect.y;
-      width = aRect.width;
-      height = aRect.height;
-      return *this;
-    }
-  };
-
 
 public:
-  nsRegion () { Init (); }
-  nsRegion (const nsRect& aRect) { Init (); Copy (aRect); }
-  nsRegion (const nsRegion& aRegion) { Init (); Copy (aRegion); }
- ~nsRegion () { SetToElements (0); }
+  nsRegion () { pixman_region32_init(&mImpl); }
+  MOZ_IMPLICIT nsRegion (const nsRect& aRect) { pixman_region32_init_rect(&mImpl,
+                                                                          aRect.x,
+                                                                          aRect.y,
+                                                                          aRect.width,
+                                                                          aRect.height); }
+  nsRegion (const nsRegion& aRegion) { pixman_region32_init(&mImpl); pixman_region32_copy(&mImpl,aRegion.Impl()); }
+  nsRegion (nsRegion&& aRegion) { mImpl = aRegion.mImpl; pixman_region32_init(&aRegion.mImpl); }
+  nsRegion& operator = (nsRegion&& aRegion) {
+      pixman_region32_fini(&mImpl);
+      mImpl = aRegion.mImpl;
+      pixman_region32_init(&aRegion.mImpl);
+      return *this;
+  }
+ ~nsRegion () { pixman_region32_fini(&mImpl); }
   nsRegion& operator = (const nsRect& aRect) { Copy (aRect); return *this; }
   nsRegion& operator = (const nsRegion& aRegion) { Copy (aRegion); return *this; }
-
-
-  nsRegion& And  (const nsRegion& aRgn1,   const nsRegion& aRgn2);
-  nsRegion& And  (const nsRegion& aRegion, const nsRect& aRect);
-  nsRegion& And  (const nsRect& aRect, const nsRegion& aRegion)
+  bool operator==(const nsRegion& aRgn) const
   {
-    return  And  (aRegion, aRect);
+    return IsEqual(aRgn);
   }
-  nsRegion& And  (const nsRect& aRect1, const nsRect& aRect2)
+  bool operator!=(const nsRegion& aRgn) const
+  {
+    return !(*this == aRgn);
+  }
+
+  friend std::ostream& operator<<(std::ostream& stream, const nsRegion& m);
+
+  void Swap(nsRegion* aOther)
+  {
+    pixman_region32_t tmp = mImpl;
+    mImpl = aOther->mImpl;
+    aOther->mImpl = tmp;
+  }
+
+  static
+  nsresult InitStatic()
+  {
+    return NS_OK;
+  }
+
+  static
+  void ShutdownStatic() {}
+
+  void AndWith(const nsRegion& aOther)
+  {
+    And(*this, aOther);
+  }
+  void AndWith(const nsRect& aOther)
+  {
+    And(*this, aOther);
+  }
+  nsRegion& And(const nsRegion& aRgn1,   const nsRegion& aRgn2)
+  {
+    pixman_region32_intersect(&mImpl, aRgn1.Impl(), aRgn2.Impl());
+    return *this;
+  }
+  nsRegion& And(const nsRect& aRect, const nsRegion& aRegion)
+  {
+    return And(aRegion, aRect);
+  }
+  nsRegion& And(const nsRegion& aRegion, const nsRect& aRect)
+  {
+    pixman_region32_intersect_rect(&mImpl, aRegion.Impl(), aRect.x, aRect.y, aRect.width, aRect.height);
+    return *this;
+  }
+  nsRegion& And(const nsRect& aRect1, const nsRect& aRect2)
   {
     nsRect TmpRect;
 
-    TmpRect.IntersectRect (aRect1, aRect2);
-    return Copy (TmpRect);
+    TmpRect.IntersectRect(aRect1, aRect2);
+    return Copy(TmpRect);
   }
 
-  nsRegion& Or   (const nsRegion& aRgn1,   const nsRegion& aRgn2);
-  nsRegion& Or   (const nsRegion& aRegion, const nsRect& aRect);
-  nsRegion& Or   (const nsRect& aRect, const nsRegion& aRegion)
+  void OrWith(const nsRegion& aOther)
   {
-    return  Or   (aRegion, aRect);
+    Or(*this, aOther);
   }
-  nsRegion& Or   (const nsRect& aRect1, const nsRect& aRect2)
+  void OrWith(const nsRect& aOther)
+  {
+    Or(*this, aOther);
+  }
+  nsRegion& Or(const nsRegion& aRgn1, const nsRegion& aRgn2)
+  {
+    pixman_region32_union(&mImpl, aRgn1.Impl(), aRgn2.Impl());
+    return *this;
+  }
+  nsRegion& Or(const nsRegion& aRegion, const nsRect& aRect)
+  {
+    pixman_region32_union_rect(&mImpl, aRegion.Impl(), aRect.x, aRect.y, aRect.width, aRect.height);
+    return *this;
+  }
+  nsRegion& Or(const nsRect& aRect, const nsRegion& aRegion)
+  {
+    return  Or(aRegion, aRect);
+  }
+  nsRegion& Or(const nsRect& aRect1, const nsRect& aRect2)
   {
     Copy (aRect1);
     return Or (*this, aRect2);
   }
 
-  nsRegion& Xor  (const nsRegion& aRgn1,   const nsRegion& aRgn2);
-  nsRegion& Xor  (const nsRegion& aRegion, const nsRect& aRect);
-  nsRegion& Xor  (const nsRect& aRect, const nsRegion& aRegion)
+  void XorWith(const nsRegion& aOther)
   {
-    return  Xor  (aRegion, aRect);
+    Xor(*this, aOther);
   }
-  nsRegion& Xor  (const nsRect& aRect1, const nsRect& aRect2)
+  void XorWith(const nsRect& aOther)
   {
-    Copy (aRect1);
-    return Xor (*this, aRect2);
+    Xor(*this, aOther);
+  }
+  nsRegion& Xor(const nsRegion& aRgn1,   const nsRegion& aRgn2)
+  {
+    // this could be implemented better if pixman had direct
+    // support for xoring regions.
+    nsRegion p;
+    p.Sub(aRgn1, aRgn2);
+    nsRegion q;
+    q.Sub(aRgn2, aRgn1);
+    return Or(p, q);
+  }
+  nsRegion& Xor(const nsRegion& aRegion, const nsRect& aRect)
+  {
+    return Xor(aRegion, nsRegion(aRect));
+  }
+  nsRegion& Xor(const nsRect& aRect, const nsRegion& aRegion)
+  {
+    return Xor(nsRegion(aRect), aRegion);
+  }
+  nsRegion& Xor(const nsRect& aRect1, const nsRect& aRect2)
+  {
+    return Xor(nsRegion(aRect1), nsRegion(aRect2));
   }
 
-  nsRegion& Sub  (const nsRegion& aRgn1,   const nsRegion& aRgn2);
-  nsRegion& Sub  (const nsRegion& aRegion, const nsRect& aRect);
-  nsRegion& Sub  (const nsRect& aRect, const nsRegion& aRegion)
+  nsRegion ToAppUnits (nscoord aAppUnitsPerPixel) const;
+
+  void SubOut(const nsRegion& aOther)
   {
-    return Sub (nsRegion (aRect), aRegion);
+    Sub(*this, aOther);
   }
-  nsRegion& Sub  (const nsRect& aRect1, const nsRect& aRect2)
+  void SubOut(const nsRect& aOther)
   {
-    Copy (aRect1);
-    return Sub (*this, aRect2);
+    Sub(*this, aOther);
+  }
+  nsRegion& Sub(const nsRegion& aRgn1, const nsRegion& aRgn2)
+  {
+    pixman_region32_subtract(&mImpl, aRgn1.Impl(), aRgn2.Impl());
+    return *this;
+  }
+  nsRegion& Sub(const nsRegion& aRegion, const nsRect& aRect)
+  {
+    return Sub(aRegion, nsRegion(aRect));
+  }
+  nsRegion& Sub(const nsRect& aRect, const nsRegion& aRegion)
+  {
+    return Sub(nsRegion(aRect), aRegion);
+  }
+  nsRegion& Sub(const nsRect& aRect1, const nsRect& aRect2)
+  {
+    Copy(aRect1);
+    return Sub(*this, aRect2);
   }
 
-  bool Contains (const nsRect& aRect) const;
+  /**
+   * Returns true iff the given point is inside the region. A region
+   * created from a rect (x=0, y=0, w=100, h=100) will NOT contain
+   * the point x=100, y=100.
+   */
+  bool Contains (int aX, int aY) const
+  {
+    return pixman_region32_contains_point(Impl(), aX, aY, nullptr);
+  }
+  bool Contains (const nsRect& aRect) const
+  {
+    pixman_box32_t box = RectToBox(aRect);
+    return pixman_region32_contains_rectangle(Impl(), &box) == PIXMAN_REGION_IN;
+  }
   bool Contains (const nsRegion& aRgn) const;
   bool Intersects (const nsRect& aRect) const;
 
@@ -143,11 +238,10 @@ public:
   {
     MoveBy (nsPoint (aXOffset, aYOffset));
   }
-  void MoveBy (nsPoint aPt);
+  void MoveBy (nsPoint aPt) { pixman_region32_translate(&mImpl, aPt.x, aPt.y); }
   void SetEmpty ()
   {
-    SetToElements (0);
-    mBoundRect.SetRect (0, 0, 0, 0);
+    pixman_region32_clear(&mImpl);
   }
 
   nsRegion MovedBy(int32_t aXOffset, int32_t aYOffset) const
@@ -177,11 +271,20 @@ public:
     return copy;
   }
 
-  bool IsEmpty () const { return mRectCount == 0; }
-  bool IsComplex () const { return mRectCount > 1; }
-  bool IsEqual (const nsRegion& aRegion) const;
-  uint32_t GetNumRects () const { return mRectCount; }
-  const nsRect& GetBounds () const { return mBoundRect; }
+  bool IsEmpty () const { return !pixman_region32_not_empty(Impl()); }
+  bool IsComplex () const { return GetNumRects() > 1; }
+  bool IsEqual (const nsRegion& aRegion) const
+  {
+    return pixman_region32_equal(Impl(), aRegion.Impl());
+  }
+  uint32_t GetNumRects () const
+  {
+    // Work around pixman bug. Sometimes pixman creates regions with 1 rect
+    // that's empty.
+    uint32_t result = pixman_region32_n_rects(Impl());
+    return (result == 1 && GetBounds().IsEmpty()) ? 0 : result;
+  }
+  const nsRect GetBounds () const { return BoxToRect(mImpl.extents); }
   uint64_t Area () const;
   // Converts this region from aFromAPP, an appunits per pixel ratio, to
   // aToAPP. This applies nsRect::ConvertAppUnitsRoundOut/In to each rect of
@@ -190,6 +293,7 @@ public:
   nsRegion ConvertAppUnitsRoundIn (int32_t aFromAPP, int32_t aToAPP) const;
   nsRegion& ScaleRoundOut(float aXScale, float aYScale);
   nsRegion& ScaleInverseRoundOut(float aXScale, float aYScale);
+  nsRegion& Transform (const gfx3DMatrix &aTransform);
   nsIntRegion ScaleToOutsidePixels (float aXScale, float aYScale, nscoord aAppUnitsPerPixel) const;
   nsIntRegion ScaleToInsidePixels (float aXScale, float aYScale, nscoord aAppUnitsPerPixel) const;
   nsIntRegion ScaleToNearestPixels (float aXScale, float aYScale, nscoord aAppUnitsPerPixel) const;
@@ -211,101 +315,145 @@ public:
    * the same as for the current region.
    */
   void SimplifyOutward (uint32_t aMaxRects);
+  /**
+   * Simplify the region by adding at most aThreshold area between spans of
+   * rects.  The simplified region will be a superset of the original region.
+   * The simplified region's bounding box will be the same as for the current
+   * region.
+   */
+  void SimplifyOutwardByArea(uint32_t aThreshold);
   /**
    * Make sure the region has at most aMaxRects by removing area from
    * it if necessary. The simplified region will be a subset of the
    * original region.
    */
   void SimplifyInward (uint32_t aMaxRects);
+
   /**
-   * Efficiently try to remove a rectangle from this region. The actual
-   * area removed could be some sub-area contained by the rectangle
-   * (even possibly nothing at all).
-   * 
-   * We remove all rectangles that are contained by aRect.
+   * VisitEdges is a weird kind of function that we use for padding
+   * out surfaces to prevent texture filtering artifacts.
+   * It calls the visitFn callback for each of the exterior edges of
+   * the regions. The top and bottom edges will be expanded 1 pixel
+   * to the left and right if there's an outside corner. The order
+   * the edges are visited is not guaranteed.
+   *
+   * visitFn has a side parameter that can be TOP,BOTTOM,LEFT,RIGHT
+   * and specifies which kind of edge is being visited. x1, y1, x2, y2
+   * are the coordinates of the line. (x1 == x2) || (y1 == y2)
    */
-  void SimpleSubtract (const nsRect& aRect);
-  /**
-   * Efficiently try to remove a region from this region. The actual
-   * area removed could be some sub-area contained by aRegion
-   * (even possibly nothing at all).
-   * 
-   * We remove all rectangles of this region that are contained by
-   * a rectangle of aRegion.
-   */
-  void SimpleSubtract (const nsRegion& aRegion);
+  typedef void (*visitFn)(void *closure, VisitSide side, int x1, int y1, int x2, int y2);
+  void VisitEdges(visitFn, void *closure);
 
   nsCString ToString() const;
-
-  /**
-   * Initialize any static data associated with nsRegion.
-   */
-  static nsresult InitStatic();
-
-  /**
-   * Deinitialize static data.
-   */
-  static void ShutdownStatic();
-
 private:
-  uint32_t    mRectCount;
-  RgnRect*    mCurRect;
-  RgnRect     mRectListHead;
-  nsRectFast  mBoundRect;
+  pixman_region32_t mImpl;
+
+#ifndef MOZ_TREE_PIXMAN
+  // For compatibility with pixman versions older than 0.25.2.
+  static inline void
+  pixman_region32_clear(pixman_region32_t *region)
+  {
+    pixman_region32_fini(region);
+    pixman_region32_init(region);
+  }
+#endif
 
-  void Init ();
-  nsRegion& Copy (const nsRegion& aRegion);
-  nsRegion& Copy (const nsRect& aRect);
-  void InsertBefore (RgnRect* aNewRect, RgnRect* aRelativeRect);
-  void InsertAfter (RgnRect* aNewRect, RgnRect* aRelativeRect);
-  void SetToElements (uint32_t aCount);
-  RgnRect* Remove (RgnRect* aRect);
-  void InsertInPlace (RgnRect* aRect, bool aOptimizeOnFly = false);
-  inline void SaveLinkChain ();
-  inline void RestoreLinkChain ();
-  void Optimize ();
-  void SubRegion (const nsRegion& aRegion, nsRegion& aResult) const;
-  void SubRect (const nsRectFast& aRect, nsRegion& aResult, nsRegion& aCompleted) const;
-  void SubRect (const nsRectFast& aRect, nsRegion& aResult) const
-  {    SubRect (aRect, aResult, aResult);  }
-  void Merge (const nsRegion& aRgn1, const nsRegion& aRgn2);
-  void MoveInto (nsRegion& aDestRegion, const RgnRect* aStartRect);
-  void MoveInto (nsRegion& aDestRegion)
-  {    MoveInto (aDestRegion, mRectListHead.next);  }
   nsIntRegion ToPixels(nscoord aAppUnitsPerPixel, bool aOutsidePixels) const;
+
+  nsRegion& Copy (const nsRegion& aRegion)
+  {
+    pixman_region32_copy(&mImpl, aRegion.Impl());
+    return *this;
+  }
+
+  nsRegion& Copy (const nsRect& aRect)
+  {
+    // pixman needs to distinguish between an empty region and a region
+    // with one rect so that it can return a different number of rectangles.
+    // Empty rect: data = empty_box
+    //     1 rect: data = null
+    //    >1 rect: data = rects
+    if (aRect.IsEmpty()) {
+      pixman_region32_clear(&mImpl);
+    } else {
+      pixman_box32_t box = RectToBox(aRect);
+      pixman_region32_reset(&mImpl, &box);
+    }
+    return *this;
+  }
+
+  static inline pixman_box32_t RectToBox(const nsRect &aRect)
+  {
+    pixman_box32_t box = { aRect.x, aRect.y, aRect.XMost(), aRect.YMost() };
+    return box;
+  }
+
+  static inline pixman_box32_t RectToBox(const nsIntRect &aRect)
+  {
+    pixman_box32_t box = { aRect.x, aRect.y, aRect.XMost(), aRect.YMost() };
+    return box;
+  }
+
+
+  static inline nsRect BoxToRect(const pixman_box32_t &aBox)
+  {
+    return nsRect(aBox.x1, aBox.y1,
+                  aBox.x2 - aBox.x1,
+                  aBox.y2 - aBox.y1);
+  }
+
+  pixman_region32_t* Impl() const
+  {
+    return const_cast<pixman_region32_t*>(&mImpl);
+  }
+
 };
 
 
-
-// Allow read-only access to region rectangles by iterating the list
-
 class NS_GFX nsRegionRectIterator
 {
   const nsRegion*  mRegion;
-  const nsRegion::RgnRect* mCurPtr;
+  int i;
+  int n;
+  nsRect rect;
+  pixman_box32_t *boxes;
 
 public:
-  nsRegionRectIterator (const nsRegion& aRegion)
+  explicit nsRegionRectIterator (const nsRegion& aRegion)
   {
     mRegion = &aRegion;
-    mCurPtr = &aRegion.mRectListHead;
+    i = 0;
+    boxes = pixman_region32_rectangles(aRegion.Impl(), &n);
+    // Work around pixman bug. Sometimes pixman creates regions with 1 rect
+    // that's empty.
+    if (n == 1 && nsRegion::BoxToRect(boxes[0]).IsEmpty()) {
+      n = 0;
+    }
   }
 
   const nsRect* Next ()
   {
-    mCurPtr = mCurPtr->next;
-    return (mCurPtr != &mRegion->mRectListHead) ? mCurPtr : nullptr;
+    if (i == n)
+      return nullptr;
+    rect = nsRegion::BoxToRect(boxes[i]);
+    NS_ASSERTION(!rect.IsEmpty(), "Shouldn't return empty rect");
+    i++;
+    return &rect;
   }
 
   const nsRect* Prev ()
   {
-    mCurPtr = mCurPtr->prev;
-    return (mCurPtr != &mRegion->mRectListHead) ? mCurPtr : nullptr;
+    if (i == -1)
+      return nullptr;
+    rect = nsRegion::BoxToRect(boxes[i]);
+    NS_ASSERTION(!rect.IsEmpty(), "Shouldn't return empty rect");
+    i--;
+    return &rect;
   }
 
   void Reset ()
   {
-    mCurPtr = &mRegion->mRectListHead;
+    i = 0;
   }
 };
 
@@ -315,19 +463,43 @@ public:
 class NS_GFX nsIntRegion
 {
   friend class nsIntRegionRectIterator;
+  friend class nsRegion;
 
 public:
   nsIntRegion () {}
-  nsIntRegion (const nsIntRect& aRect) : mImpl (ToRect(aRect)) {}
+  MOZ_IMPLICIT nsIntRegion (const nsIntRect& aRect) : mImpl (ToRect(aRect)) {}
   nsIntRegion (const nsIntRegion& aRegion) : mImpl (aRegion.mImpl) {}
+  nsIntRegion (nsIntRegion&& aRegion) : mImpl (mozilla::Move(aRegion.mImpl)) {}
   nsIntRegion& operator = (const nsIntRect& aRect) { mImpl = ToRect (aRect); return *this; }
   nsIntRegion& operator = (const nsIntRegion& aRegion) { mImpl = aRegion.mImpl; return *this; }
+  nsIntRegion& operator = (nsIntRegion&& aRegion) { mImpl = mozilla::Move(aRegion.mImpl); return *this; }
 
   bool operator==(const nsIntRegion& aRgn) const
   {
     return IsEqual(aRgn);
   }
+  bool operator!=(const nsIntRegion& aRgn) const
+  {
+    return !(*this == aRgn);
+  }
 
+  friend std::ostream& operator<<(std::ostream& stream, const nsIntRegion& m) {
+    return stream << m.mImpl;
+  }
+
+  void Swap(nsIntRegion* aOther)
+  {
+    mImpl.Swap(&aOther->mImpl);
+  }
+
+  void AndWith(const nsIntRegion& aOther)
+  {
+    And(*this, aOther);
+  }
+  void AndWith(const nsIntRect& aOther)
+  {
+    And(*this, aOther);
+  }
   nsIntRegion& And  (const nsIntRegion& aRgn1,   const nsIntRegion& aRgn2)
   {
     mImpl.And (aRgn1.mImpl, aRgn2.mImpl);
@@ -351,6 +523,14 @@ public:
     return *this;
   }
 
+  void OrWith(const nsIntRegion& aOther)
+  {
+    Or(*this, aOther);
+  }
+  void OrWith(const nsIntRect& aOther)
+  {
+    Or(*this, aOther);
+  }
   nsIntRegion& Or   (const nsIntRegion& aRgn1,   const nsIntRegion& aRgn2)
   {
     mImpl.Or (aRgn1.mImpl, aRgn2.mImpl);
@@ -371,6 +551,14 @@ public:
     return Or (*this, aRect2);
   }
 
+  void XorWith(const nsIntRegion& aOther)
+  {
+    Xor(*this, aOther);
+  }
+  void XorWith(const nsIntRect& aOther)
+  {
+    Xor(*this, aOther);
+  }
   nsIntRegion& Xor  (const nsIntRegion& aRgn1,   const nsIntRegion& aRgn2)
   {
     mImpl.Xor (aRgn1.mImpl, aRgn2.mImpl);
@@ -391,6 +579,14 @@ public:
     return Xor (*this, aRect2);
   }
 
+  void SubOut(const nsIntRegion& aOther)
+  {
+    Sub(*this, aOther);
+  }
+  void SubOut(const nsIntRect& aOther)
+  {
+    Sub(*this, aOther);
+  }
   nsIntRegion& Sub  (const nsIntRegion& aRgn1,   const nsIntRegion& aRgn2)
   {
     mImpl.Sub (aRgn1.mImpl, aRgn2.mImpl);
@@ -411,6 +607,15 @@ public:
     return Sub (*this, aRect2);
   }
 
+  /**
+   * Returns true iff the given point is inside the region. A region
+   * created from a rect (x=0, y=0, w=100, h=100) will NOT contain
+   * the point x=100, y=100.
+   */
+  bool Contains (int aX, int aY) const
+  {
+    return mImpl.Contains(aX, aY);
+  }
   bool Contains (const nsIntRect& aRect) const
   {
     return mImpl.Contains (ToRect (aRect));
@@ -487,6 +692,12 @@ public:
     return *this;
   }
 
+  nsIntRegion& Transform (const gfx3DMatrix &aTransform)
+  {
+    mImpl.Transform(aTransform);
+    return *this;
+  }
+
   /**
    * Make sure the region has at most aMaxRects by adding area to it
    * if necessary. The simplified region will be a superset of the
@@ -497,6 +708,10 @@ public:
   {
     mImpl.SimplifyOutward (aMaxRects);
   }
+  void SimplifyOutwardByArea (uint32_t aThreshold)
+  {
+    mImpl.SimplifyOutwardByArea (aThreshold);
+  }
   /**
    * Make sure the region has at most aMaxRects by removing area from
    * it if necessary. The simplified region will be a subset of the
@@ -506,28 +721,11 @@ public:
   {
     mImpl.SimplifyInward (aMaxRects);
   }
-  /**
-   * Efficiently try to remove a rectangle from this region. The actual
-   * area removed could be some sub-area contained by the rectangle
-   * (even possibly nothing at all).
-   * 
-   * We remove all rectangles that are contained by aRect.
-   */
-  void SimpleSubtract (const nsIntRect& aRect)
+
+  typedef void (*visitFn)(void *closure, VisitSide side, int x1, int y1, int x2, int y2);
+  void VisitEdges (visitFn visit, void *closure)
   {
-    mImpl.SimpleSubtract (ToRect (aRect));
-  }
-  /**
-   * Efficiently try to remove a region from this region. The actual
-   * area removed could be some sub-area contained by aRegion
-   * (even possibly nothing at all).
-   * 
-   * We remove all rectangles of this region that are contained by
-   * a rectangle of aRegion.
-   */
-  void SimpleSubtract (const nsIntRegion& aRegion)
-  {
-    mImpl.SimpleSubtract (aRegion.mImpl);
+    mImpl.VisitEdges (visit, closure);
   }
 
   nsCString ToString() const { return mImpl.ToString(); }
@@ -551,7 +749,7 @@ class NS_GFX nsIntRegionRectIterator
   nsIntRect mTmp;
 
 public:
-  nsIntRegionRectIterator (const nsIntRegion& aRegion) : mImpl (aRegion.mImpl) {}
+  explicit nsIntRegionRectIterator (const nsIntRegion& aRegion) : mImpl (aRegion.mImpl) {}
 
   const nsIntRect* Next ()
   {
@@ -576,5 +774,4 @@ public:
     mImpl.Reset ();
   }
 };
-
 #endif
