@@ -15,7 +15,7 @@ class gfxImageSurface;
  */
 class gfxReusableImageSurfaceWrapper : public gfxReusableSurfaceWrapper {
 public:
-  gfxReusableImageSurfaceWrapper(gfxImageSurface* aSurface);
+  explicit gfxReusableImageSurfaceWrapper(gfxImageSurface* aSurface);
 protected:
   ~gfxReusableImageSurfaceWrapper();
 
@@ -26,7 +26,7 @@ public:
   void ReadLock() MOZ_OVERRIDE;
   void ReadUnlock() MOZ_OVERRIDE;
 
-  Type GetType()
+  Type GetType() MOZ_OVERRIDE
   {
     return TYPE_IMAGE;
   }
