@@ -35,21 +35,23 @@ namespace mozilla {
 namespace image {
 
 #if defined(PR_LOGGING)
-static PRLogModuleInfo *
+static PRLogModuleInfo*
 GetJPEGLog()
 {
-  static PRLogModuleInfo *sJPEGLog;
-  if (!sJPEGLog)
+  static PRLogModuleInfo* sJPEGLog;
+  if (!sJPEGLog) {
     sJPEGLog = PR_NewLogModule("JPEGDecoder");
+  }
   return sJPEGLog;
 }
 
-static PRLogModuleInfo *
+static PRLogModuleInfo*
 GetJPEGDecoderAccountingLog()
 {
-  static PRLogModuleInfo *sJPEGDecoderAccountingLog;
-  if (!sJPEGDecoderAccountingLog)
+  static PRLogModuleInfo* sJPEGDecoderAccountingLog;
+  if (!sJPEGDecoderAccountingLog) {
     sJPEGDecoderAccountingLog = PR_NewLogModule("JPEGDecoderAccounting");
+  }
   return sJPEGDecoderAccountingLog;
 }
 #else
@@ -58,7 +60,7 @@ GetJPEGDecoderAccountingLog()
 #endif
 
 static qcms_profile*
-GetICCProfile(struct jpeg_decompress_struct &info)
+GetICCProfile(struct jpeg_decompress_struct& info)
 {
   JOCTET* profilebuf;
   uint32_t profileLength;
@@ -78,11 +80,12 @@ METHODDEF(void) skip_input_data (j_decompress_ptr jd, long num_bytes);
 METHODDEF(void) term_source (j_decompress_ptr jd);
 METHODDEF(void) my_error_exit (j_common_ptr cinfo);
 
-/* Normal JFIF markers can't have more bytes than this. */
+// Normal JFIF markers can't have more bytes than this.
 #define MAX_JPEG_MARKER_LENGTH  (((uint32_t)1 << 16) - 1)
 
 
-nsJPEGDecoder::nsJPEGDecoder(RasterImage& aImage, Decoder::DecodeStyle aDecodeStyle)
+nsJPEGDecoder::nsJPEGDecoder(RasterImage* aImage,
+                             Decoder::DecodeStyle aDecodeStyle)
  : Decoder(aImage)
  , mDecodeStyle(aDecodeStyle)
 {
@@ -118,10 +121,12 @@ nsJPEGDecoder::~nsJPEGDecoder()
   jpeg_destroy_decompress(&mInfo);
 
   PR_FREEIF(mBackBuffer);
-  if (mTransform)
+  if (mTransform) {
     qcms_transform_release(mTransform);
-  if (mInProfile)
+  }
+  if (mInProfile) {
     qcms_profile_release(mInProfile);
+  }
 
   PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
          ("nsJPEGDecoder::~nsJPEGDecoder: Destroying JPEG decoder %p",
@@ -138,37 +143,37 @@ void
 nsJPEGDecoder::InitInternal()
 {
   mCMSMode = gfxPlatform::GetCMSMode();
-  if ((mDecodeFlags & DECODER_NO_COLORSPACE_CONVERSION) != 0)
+  if (GetDecodeFlags() & imgIContainer::FLAG_DECODE_NO_COLORSPACE_CONVERSION) {
     mCMSMode = eCMSMode_Off;
+  }
 
-  /* We set up the normal JPEG error routines, then override error_exit. */
+  // We set up the normal JPEG error routines, then override error_exit.
   mInfo.err = jpeg_std_error(&mErr.pub);
-  /*   mInfo.err = jpeg_std_error(&mErr.pub); */
+  //   mInfo.err = jpeg_std_error(&mErr.pub);
   mErr.pub.error_exit = my_error_exit;
-  /* Establish the setjmp return context for my_error_exit to use. */
+  // Establish the setjmp return context for my_error_exit to use.
   if (setjmp(mErr.setjmp_buffer)) {
-    /* If we get here, the JPEG code has signaled an error.
-     * We need to clean up the JPEG object, close the input file, and return.
-     */
+    // If we get here, the JPEG code has signaled an error.
+    // We need to clean up the JPEG object, close the input file, and return.
     PostDecoderError(NS_ERROR_FAILURE);
     return;
   }
 
-  /* Step 1: allocate and initialize JPEG decompression object */
+  // Step 1: allocate and initialize JPEG decompression object
   jpeg_create_decompress(&mInfo);
-  /* Set the source manager */
+  // Set the source manager
   mInfo.src = &mSourceMgr;
 
-  /* Step 2: specify data source (eg, a file) */
+  // Step 2: specify data source (eg, a file)
 
-  /* Setup callback functions. */
+  // Setup callback functions.
   mSourceMgr.init_source = init_source;
   mSourceMgr.fill_input_buffer = fill_input_buffer;
   mSourceMgr.skip_input_data = skip_input_data;
   mSourceMgr.resync_to_restart = jpeg_resync_to_restart;
   mSourceMgr.term_source = term_source;
 
-  /* Record app markers for ICC data */
+  // Record app markers for ICC data
   for (uint32_t m = 0; m < 16; m++)
     jpeg_save_markers(&mInfo, JPEG_APP0 + m, 0xFFFF);
 }
@@ -176,48 +181,39 @@ nsJPEGDecoder::InitInternal()
 void
 nsJPEGDecoder::FinishInternal()
 {
-  /* If we're not in any sort of error case, flush the decoder.
-   *
-   * XXXbholley - It seems wrong that this should be necessary, but at the
-   * moment I'm just folding the contents of Flush() into Close() so that
-   * we can get rid of it.
-   *
-   * XXX(seth): It'd be great to get rid of this. For now, we treat this as a
-   * write to a synchronous decoder, which means that this must be called only
-   * on the main thread. (That's asserted in Decoder::Finish and
-   * Decoder::FinishSharedDecoder.)
-   */
+  // If we're not in any sort of error case, force our state to JPEG_DONE.
   if ((mState != JPEG_DONE && mState != JPEG_SINK_NON_JPEG_TRAILER) &&
       (mState != JPEG_ERROR) &&
-      !IsSizeDecode())
-    this->Write(nullptr, 0, DECODE_SYNC);
+      !IsSizeDecode()) {
+    mState = JPEG_DONE;
+  }
 }
 
 void
-nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount, DecodeStrategy)
+nsJPEGDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
 {
-  mSegment = (const JOCTET *)aBuffer;
+  mSegment = (const JOCTET*)aBuffer;
   mSegmentLen = aCount;
 
   NS_ABORT_IF_FALSE(!HasError(), "Shouldn't call WriteInternal after error!");
 
-  /* Return here if there is a fatal error within libjpeg. */
+  // Return here if there is a fatal error within libjpeg.
   nsresult error_code;
   // This cast to nsresult makes sense because setjmp() returns whatever we
   // passed to longjmp(), which was actually an nsresult.
   if ((error_code = (nsresult)setjmp(mErr.setjmp_buffer)) != NS_OK) {
     if (error_code == NS_ERROR_FAILURE) {
       PostDataError();
-      /* Error due to corrupt stream - return NS_OK and consume silently
-         so that libpr0n doesn't throw away a partial image load */
+      // Error due to corrupt stream - return NS_OK and consume silently
+      // so that ImageLib doesn't throw away a partial image load
       mState = JPEG_SINK_NON_JPEG_TRAILER;
       PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
              ("} (setjmp returned NS_ERROR_FAILURE)"));
       return;
     } else {
-      /* Error due to reasons external to the stream (probably out of
-         memory) - let libpr0n attempt to clean up, even though
-         mozilla is seconds away from falling flat on its face. */
+      // Error due to reasons external to the stream (probably out of
+      // memory) - let ImageLib attempt to clean up, even though
+      // mozilla is seconds away from falling flat on its face.
       PostDecoderError(error_code);
       mState = JPEG_ERROR;
       PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
@@ -230,103 +226,119 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount, DecodeStrateg
          ("[this=%p] nsJPEGDecoder::Write -- processing JPEG data\n", this));
 
   switch (mState) {
-  case JPEG_HEADER:
-  {
-    LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::Write -- entering JPEG_HEADER case");
+    case JPEG_HEADER: {
+      LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::Write -- entering JPEG_HEADER"
+                " case");
 
-    /* Step 3: read file parameters with jpeg_read_header() */
-    if (jpeg_read_header(&mInfo, TRUE) == JPEG_SUSPENDED) {
-      PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
-             ("} (JPEG_SUSPENDED)"));
-      return; /* I/O suspension */
-    }
+      // Step 3: read file parameters with jpeg_read_header()
+      if (jpeg_read_header(&mInfo, TRUE) == JPEG_SUSPENDED) {
+        PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+               ("} (JPEG_SUSPENDED)"));
+        return; // I/O suspension
+      }
 
-    // Post our size to the superclass
-    PostSize(mInfo.image_width, mInfo.image_height, ReadOrientationFromEXIF());
-    if (HasError()) {
-      // Setting the size led to an error.
-      mState = JPEG_ERROR;
-      return;
-    }
+      int sampleSize = mImage->GetRequestedSampleSize();
+      if (sampleSize > 0) {
+        mInfo.scale_num = 1;
+        mInfo.scale_denom = sampleSize;
+      }
 
-    /* If we're doing a size decode, we're done. */
-    if (IsSizeDecode())
-      return;
+      // Used to set up image size so arrays can be allocated
+      jpeg_calc_output_dimensions(&mInfo);
 
-    /* We're doing a full decode. */
-    if (mCMSMode != eCMSMode_Off &&
-        (mInProfile = GetICCProfile(mInfo)) != nullptr) {
-      uint32_t profileSpace = qcms_profile_get_color_space(mInProfile);
-      bool mismatch = false;
+      // Post our size to the superclass
+      PostSize(mInfo.output_width, mInfo.output_height,
+               ReadOrientationFromEXIF());
+      if (HasError()) {
+        // Setting the size led to an error.
+        mState = JPEG_ERROR;
+        return;
+      }
+
+      // If we're doing a size decode, we're done.
+      if (IsSizeDecode()) {
+        return;
+      }
+
+      // We're doing a full decode.
+      if (mCMSMode != eCMSMode_Off &&
+          (mInProfile = GetICCProfile(mInfo)) != nullptr) {
+        uint32_t profileSpace = qcms_profile_get_color_space(mInProfile);
+        bool mismatch = false;
 
 #ifdef DEBUG_tor
       fprintf(stderr, "JPEG profileSpace: 0x%08X\n", profileSpace);
 #endif
       switch (mInfo.jpeg_color_space) {
-      case JCS_GRAYSCALE:
-        if (profileSpace == icSigRgbData)
-          mInfo.out_color_space = JCS_RGB;
-        else if (profileSpace != icSigGrayData)
-          mismatch = true;
-        break;
-      case JCS_RGB:
-        if (profileSpace != icSigRgbData)
-          mismatch =  true;
-        break;
-      case JCS_YCbCr:
-        if (profileSpace == icSigRgbData)
-          mInfo.out_color_space = JCS_RGB;
-        else
-	  // qcms doesn't support ycbcr
-          mismatch = true;
-        break;
-      case JCS_CMYK:
-      case JCS_YCCK:
-	  // qcms doesn't support cmyk
-          mismatch = true;
-        break;
-      default:
-        mState = JPEG_ERROR;
-        PostDataError();
-        PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
-               ("} (unknown colorpsace (1))"));
-        return;
-      }
-
-      if (!mismatch) {
-        qcms_data_type type;
-        switch (mInfo.out_color_space) {
         case JCS_GRAYSCALE:
-          type = QCMS_DATA_GRAY_8;
+          if (profileSpace == icSigRgbData) {
+            mInfo.out_color_space = JCS_RGB;
+          } else if (profileSpace != icSigGrayData) {
+            mismatch = true;
+          }
           break;
         case JCS_RGB:
-          type = QCMS_DATA_RGB_8;
+          if (profileSpace != icSigRgbData) {
+            mismatch =  true;
+          }
+          break;
+        case JCS_YCbCr:
+          if (profileSpace == icSigRgbData) {
+            mInfo.out_color_space = JCS_RGB;
+          } else {
+            // qcms doesn't support ycbcr
+            mismatch = true;
+          }
+          break;
+        case JCS_CMYK:
+        case JCS_YCCK:
+            // qcms doesn't support cmyk
+            mismatch = true;
           break;
         default:
           mState = JPEG_ERROR;
           PostDataError();
           PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
-                 ("} (unknown colorpsace (2))"));
+                 ("} (unknown colorpsace (1))"));
           return;
+      }
+
+      if (!mismatch) {
+        qcms_data_type type;
+        switch (mInfo.out_color_space) {
+          case JCS_GRAYSCALE:
+            type = QCMS_DATA_GRAY_8;
+            break;
+          case JCS_RGB:
+            type = QCMS_DATA_RGB_8;
+            break;
+          default:
+            mState = JPEG_ERROR;
+            PostDataError();
+            PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+                   ("} (unknown colorpsace (2))"));
+            return;
         }
 #if 0
-        /* We don't currently support CMYK profiles. The following
-         * code dealt with lcms types. Add something like this
-         * back when we gain support for CMYK.
-         */
-        /* Adobe Photoshop writes YCCK/CMYK files with inverted data */
-        if (mInfo.out_color_space == JCS_CMYK)
+        // We don't currently support CMYK profiles. The following
+        // code dealt with lcms types. Add something like this
+        // back when we gain support for CMYK.
+
+        // Adobe Photoshop writes YCCK/CMYK files with inverted data
+        if (mInfo.out_color_space == JCS_CMYK) {
           type |= FLAVOR_SH(mInfo.saw_Adobe_marker ? 1 : 0);
+        }
 #endif
 
         if (gfxPlatform::GetCMSOutputProfile()) {
 
-          /* Calculate rendering intent. */
+          // Calculate rendering intent.
           int intent = gfxPlatform::GetRenderingIntent();
-          if (intent == -1)
-              intent = qcms_profile_get_rendering_intent(mInProfile);
+          if (intent == -1) {
+            intent = qcms_profile_get_rendering_intent(mInProfile);
+          }
 
-          /* Create the color management transform. */
+          // Create the color management transform.
           mTransform = qcms_transform_create(mInProfile,
                                           type,
                                           gfxPlatform::GetCMSOutputProfile(),
@@ -342,41 +354,37 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount, DecodeStrateg
 
     if (!mTransform) {
       switch (mInfo.jpeg_color_space) {
-      case JCS_GRAYSCALE:
-      case JCS_RGB:
-      case JCS_YCbCr:
-        // if we're not color managing we can decode directly to
-        // MOZ_JCS_EXT_NATIVE_ENDIAN_XRGB
-        if (mCMSMode != eCMSMode_All) {
-            mInfo.out_color_space = MOZ_JCS_EXT_NATIVE_ENDIAN_XRGB;
-            mInfo.out_color_components = 4;
-        } else {
-            mInfo.out_color_space = JCS_RGB;
-        }
-        break;
-      case JCS_CMYK:
-      case JCS_YCCK:
-        /* libjpeg can convert from YCCK to CMYK, but not to RGB */
-        mInfo.out_color_space = JCS_CMYK;
-        break;
-      default:
-        mState = JPEG_ERROR;
-        PostDataError();
-        PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
-               ("} (unknown colorpsace (3))"));
-        return;
-        break;
+        case JCS_GRAYSCALE:
+        case JCS_RGB:
+        case JCS_YCbCr:
+          // if we're not color managing we can decode directly to
+          // MOZ_JCS_EXT_NATIVE_ENDIAN_XRGB
+          if (mCMSMode != eCMSMode_All) {
+              mInfo.out_color_space = MOZ_JCS_EXT_NATIVE_ENDIAN_XRGB;
+              mInfo.out_color_components = 4;
+          } else {
+              mInfo.out_color_space = JCS_RGB;
+          }
+          break;
+        case JCS_CMYK:
+        case JCS_YCCK:
+          // libjpeg can convert from YCCK to CMYK, but not to RGB
+          mInfo.out_color_space = JCS_CMYK;
+          break;
+        default:
+          mState = JPEG_ERROR;
+          PostDataError();
+          PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
+                 ("} (unknown colorpsace (3))"));
+          return;
+          break;
       }
     }
 
-    /*
-     * Don't allocate a giant and superfluous memory buffer
-     * when not doing a progressive decode.
-     */
-    mInfo.buffered_image = mDecodeStyle == PROGRESSIVE && jpeg_has_multiple_scans(&mInfo);
-
-    /* Used to set up image size so arrays can be allocated */
-    jpeg_calc_output_dimensions(&mInfo);
+    // Don't allocate a giant and superfluous memory buffer
+    // when not doing a progressive decode.
+    mInfo.buffered_image = mDecodeStyle == PROGRESSIVE &&
+                           jpeg_has_multiple_scans(&mInfo);
 
     if (!mImageData) {
       mState = JPEG_ERROR;
@@ -387,64 +395,65 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount, DecodeStrateg
     }
 
     PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
-           ("        JPEGDecoderAccounting: nsJPEGDecoder::Write -- created image frame with %ux%u pixels",
-            mInfo.image_width, mInfo.image_height));
+           ("        JPEGDecoderAccounting: nsJPEGDecoder::"
+            "Write -- created image frame with %ux%u pixels",
+            mInfo.output_width, mInfo.output_height));
 
     mState = JPEG_START_DECOMPRESS;
   }
 
-  case JPEG_START_DECOMPRESS:
-  {
-    LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::Write -- entering JPEG_START_DECOMPRESS case");
-    /* Step 4: set parameters for decompression */
+  case JPEG_START_DECOMPRESS: {
+    LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::Write -- entering"
+                            " JPEG_START_DECOMPRESS case");
+    // Step 4: set parameters for decompression
+
+    // FIXME -- Should reset dct_method and dither mode
+    // for final pass of progressive JPEG
 
-    /* FIXME -- Should reset dct_method and dither mode
-     * for final pass of progressive JPEG
-     */
     mInfo.dct_method =  JDCT_ISLOW;
     mInfo.dither_mode = JDITHER_FS;
     mInfo.do_fancy_upsampling = TRUE;
     mInfo.enable_2pass_quant = FALSE;
     mInfo.do_block_smoothing = TRUE;
 
-    /* Step 5: Start decompressor */
+    // Step 5: Start decompressor
     if (jpeg_start_decompress(&mInfo) == FALSE) {
       PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
              ("} (I/O suspension after jpeg_start_decompress())"));
-      return; /* I/O suspension */
+      return; // I/O suspension
     }
 
 
-    /* If this is a progressive JPEG ... */
-    mState = mInfo.buffered_image ? JPEG_DECOMPRESS_PROGRESSIVE : JPEG_DECOMPRESS_SEQUENTIAL;
+    // If this is a progressive JPEG ...
+    mState = mInfo.buffered_image ?
+             JPEG_DECOMPRESS_PROGRESSIVE : JPEG_DECOMPRESS_SEQUENTIAL;
   }
 
-  case JPEG_DECOMPRESS_SEQUENTIAL:
-  {
-    if (mState == JPEG_DECOMPRESS_SEQUENTIAL)
-    {
-      LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::Write -- JPEG_DECOMPRESS_SEQUENTIAL case");
-      
+  case JPEG_DECOMPRESS_SEQUENTIAL: {
+    if (mState == JPEG_DECOMPRESS_SEQUENTIAL) {
+      LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::Write -- "
+                              "JPEG_DECOMPRESS_SEQUENTIAL case");
+
       bool suspend;
       OutputScanlines(&suspend);
-      
+
       if (suspend) {
         PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
                ("} (I/O suspension after OutputScanlines() - SEQUENTIAL)"));
-        return; /* I/O suspension */
+        return; // I/O suspension
       }
-      
-      /* If we've completed image output ... */
-      NS_ASSERTION(mInfo.output_scanline == mInfo.output_height, "We didn't process all of the data!");
+
+      // If we've completed image output ...
+      NS_ASSERTION(mInfo.output_scanline == mInfo.output_height,
+                   "We didn't process all of the data!");
       mState = JPEG_DONE;
     }
   }
 
-  case JPEG_DECOMPRESS_PROGRESSIVE:
-  {
-    if (mState == JPEG_DECOMPRESS_PROGRESSIVE)
-    {
-      LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::Write -- JPEG_DECOMPRESS_PROGRESSIVE case");
+  case JPEG_DECOMPRESS_PROGRESSIVE: {
+    if (mState == JPEG_DECOMPRESS_PROGRESSIVE) {
+      LOG_SCOPE(GetJPEGLog(),
+                "nsJPEGDecoder::Write -- JPEG_DECOMPRESS_PROGRESSIVE case");
 
       int status;
       do {
@@ -456,9 +465,9 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount, DecodeStrateg
         if (mInfo.output_scanline == 0) {
           int scan = mInfo.input_scan_number;
 
-          /* if we haven't displayed anything yet (output_scan_number==0)
-             and we have enough data for a complete scan, force output
-             of the last full scan */
+          // if we haven't displayed anything yet (output_scan_number==0)
+          // and we have enough data for a complete scan, force output
+          // of the last full scan
           if ((mInfo.output_scan_number == 0) &&
               (scan > 1) &&
               (status != JPEG_REACHED_EOI))
@@ -466,34 +475,36 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount, DecodeStrateg
 
           if (!jpeg_start_output(&mInfo, scan)) {
             PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
-                   ("} (I/O suspension after jpeg_start_output() - PROGRESSIVE)"));
-            return; /* I/O suspension */
+                   ("} (I/O suspension after jpeg_start_output() -"
+                    " PROGRESSIVE)"));
+            return; // I/O suspension
           }
         }
 
-        if (mInfo.output_scanline == 0xffffff)
+        if (mInfo.output_scanline == 0xffffff) {
           mInfo.output_scanline = 0;
+        }
 
         bool suspend;
         OutputScanlines(&suspend);
 
         if (suspend) {
           if (mInfo.output_scanline == 0) {
-            /* didn't manage to read any lines - flag so we don't call
-               jpeg_start_output() multiple times for the same scan */
+            // didn't manage to read any lines - flag so we don't call
+            // jpeg_start_output() multiple times for the same scan
             mInfo.output_scanline = 0xffffff;
           }
           PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
                  ("} (I/O suspension after OutputScanlines() - PROGRESSIVE)"));
-          return; /* I/O suspension */
+          return; // I/O suspension
         }
 
-        if (mInfo.output_scanline == mInfo.output_height)
-        {
+        if (mInfo.output_scanline == mInfo.output_height) {
           if (!jpeg_finish_output(&mInfo)) {
             PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
-                   ("} (I/O suspension after jpeg_finish_output() - PROGRESSIVE)"));
-            return; /* I/O suspension */
+                   ("} (I/O suspension after jpeg_finish_output() -"
+                    " PROGRESSIVE)"));
+            return; // I/O suspension
           }
 
           if (jpeg_input_complete(&mInfo) &&
@@ -508,31 +519,33 @@ nsJPEGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount, DecodeStrateg
     }
   }
 
-  case JPEG_DONE:
-  {
-    LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::ProcessData -- entering JPEG_DONE case");
+  case JPEG_DONE: {
+    LOG_SCOPE(GetJPEGLog(), "nsJPEGDecoder::ProcessData -- entering"
+                            " JPEG_DONE case");
 
-    /* Step 7: Finish decompression */
+    // Step 7: Finish decompression
 
     if (jpeg_finish_decompress(&mInfo) == FALSE) {
       PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
              ("} (I/O suspension after jpeg_finish_decompress() - DONE)"));
-      return; /* I/O suspension */
+      return; // I/O suspension
     }
 
     mState = JPEG_SINK_NON_JPEG_TRAILER;
 
-    /* we're done dude */
+    // we're done dude
     break;
   }
   case JPEG_SINK_NON_JPEG_TRAILER:
     PR_LOG(GetJPEGLog(), PR_LOG_DEBUG,
-           ("[this=%p] nsJPEGDecoder::ProcessData -- entering JPEG_SINK_NON_JPEG_TRAILER case\n", this));
+           ("[this=%p] nsJPEGDecoder::ProcessData -- entering"
+            " JPEG_SINK_NON_JPEG_TRAILER case\n", this));
 
     break;
 
   case JPEG_ERROR:
-    NS_ABORT_IF_FALSE(0, "Should always return immediately after error and not re-enter decoder");
+    NS_ABORT_IF_FALSE(0, "Should always return immediately after error and"
+                         " not re-enter decoder");
   }
 
   PR_LOG(GetJPEGDecoderAccountingLog(), PR_LOG_DEBUG,
@@ -547,13 +560,15 @@ nsJPEGDecoder::ReadOrientationFromEXIF()
 
   // Locate the APP1 marker, where EXIF data is stored, in the marker list.
   for (marker = mInfo.marker_list ; marker != nullptr ; marker = marker->next) {
-    if (marker->marker == JPEG_APP0 + 1) 
+    if (marker->marker == JPEG_APP0 + 1) {
       break;
+    }
   }
 
   // If we're at the end of the list, there's no EXIF data.
-  if (!marker)
+  if (!marker) {
     return Orientation();
+  }
 
   // Extract the orientation information.
   EXIFData exif = EXIFParser::Parse(marker->data,
@@ -564,7 +579,7 @@ nsJPEGDecoder::ReadOrientationFromEXIF()
 void
 nsJPEGDecoder::NotifyDone()
 {
-  PostFrameStop(FrameBlender::kFrameOpaque);
+  PostFrameStop(Opacity::OPAQUE);
   PostDecodeDone();
 }
 
@@ -576,40 +591,40 @@ nsJPEGDecoder::OutputScanlines(bool* suspend)
   const uint32_t top = mInfo.output_scanline;
 
   while ((mInfo.output_scanline < mInfo.output_height)) {
-      /* Use the Cairo image buffer as scanline buffer */
-      uint32_t *imageRow = ((uint32_t*)mImageData) +
+      // Use the Cairo image buffer as scanline buffer
+      uint32_t* imageRow = ((uint32_t*)mImageData) +
                            (mInfo.output_scanline * mInfo.output_width);
 
       if (mInfo.out_color_space == MOZ_JCS_EXT_NATIVE_ENDIAN_XRGB) {
-        /* Special case: scanline will be directly converted into packed ARGB */
+        // Special case: scanline will be directly converted into packed ARGB
         if (jpeg_read_scanlines(&mInfo, (JSAMPARRAY)&imageRow, 1) != 1) {
-          *suspend = true; /* suspend */
+          *suspend = true; // suspend
           break;
         }
-        continue; /* all done for this row! */
+        continue; // all done for this row!
       }
 
       JSAMPROW sampleRow = (JSAMPROW)imageRow;
       if (mInfo.output_components == 3) {
-        /* Put the pixels at end of row to enable in-place expansion */
+        // Put the pixels at end of row to enable in-place expansion
         sampleRow += mInfo.output_width;
       }
 
-      /* Request one scanline.  Returns 0 or 1 scanlines. */    
+      // Request one scanline.  Returns 0 or 1 scanlines.
       if (jpeg_read_scanlines(&mInfo, &sampleRow, 1) != 1) {
-        *suspend = true; /* suspend */
+        *suspend = true; // suspend
         break;
       }
 
       if (mTransform) {
         JSAMPROW source = sampleRow;
         if (mInfo.out_color_space == JCS_GRAYSCALE) {
-          /* Convert from the 1byte grey pixels at begin of row 
-             to the 3byte RGB byte pixels at 'end' of row */
+          // Convert from the 1byte grey pixels at begin of row
+          // to the 3byte RGB byte pixels at 'end' of row
           sampleRow += mInfo.output_width;
         }
         qcms_transform_data(mTransform, source, sampleRow, mInfo.output_width);
-        /* Move 3byte RGB data to end of row */
+        // Move 3byte RGB data to end of row
         if (mInfo.out_color_space == JCS_CMYK) {
           memmove(sampleRow + mInfo.output_width,
                   sampleRow,
@@ -618,17 +633,20 @@ nsJPEGDecoder::OutputScanlines(bool* suspend)
         }
       } else {
         if (mInfo.out_color_space == JCS_CMYK) {
-          /* Convert from CMYK to RGB */
-          /* We cannot convert directly to Cairo, as the CMSRGBTransform may wants to do a RGB transform... */
-          /* Would be better to have platform CMSenabled transformation from CMYK to (A)RGB... */
+          // Convert from CMYK to RGB
+          // We cannot convert directly to Cairo, as the CMSRGBTransform
+          // may wants to do a RGB transform...
+          // Would be better to have platform CMSenabled transformation
+          // from CMYK to (A)RGB...
           cmyk_convert_rgb((JSAMPROW)imageRow, mInfo.output_width);
           sampleRow += mInfo.output_width;
         }
         if (mCMSMode == eCMSMode_All) {
-          /* No embedded ICC profile - treat as sRGB */
-          qcms_transform *transform = gfxPlatform::GetCMSRGBTransform();
+          // No embedded ICC profile - treat as sRGB
+          qcms_transform* transform = gfxPlatform::GetCMSRGBTransform();
           if (transform) {
-            qcms_transform_data(transform, sampleRow, sampleRow, mInfo.output_width);
+            qcms_transform_data(transform, sampleRow, sampleRow,
+                                mInfo.output_width);
           }
         }
       }
@@ -638,7 +656,8 @@ nsJPEGDecoder::OutputScanlines(bool* suspend)
 
       // copy as bytes until source pointer is 32-bit-aligned
       for (; (NS_PTR_TO_UINT32(sampleRow) & 0x3) && idx; --idx) {
-        *imageRow++ = gfxPackedPixel(0xFF, sampleRow[0], sampleRow[1], sampleRow[2]);
+        *imageRow++ = gfxPackedPixel(0xFF, sampleRow[0], sampleRow[1],
+                                     sampleRow[2]);
         sampleRow += 3;
       }
 
@@ -653,7 +672,8 @@ nsJPEGDecoder::OutputScanlines(bool* suspend)
       // copy remaining pixel(s)
       while (idx--) {
         // 32-bit read of final pixel will exceed buffer, so read bytes
-        *imageRow++ = gfxPackedPixel(0xFF, sampleRow[0], sampleRow[1], sampleRow[2]);
+        *imageRow++ = gfxPackedPixel(0xFF, sampleRow[0], sampleRow[1],
+                                     sampleRow[2]);
         sampleRow += 3;
       }
   }
@@ -666,13 +686,13 @@ nsJPEGDecoder::OutputScanlines(bool* suspend)
 }
 
 
-/* Override the standard error method in the IJG JPEG decoder code. */
+// Override the standard error method in the IJG JPEG decoder code.
 METHODDEF(void)
 my_error_exit (j_common_ptr cinfo)
 {
-  decoder_error_mgr *err = (decoder_error_mgr *) cinfo->err;
+  decoder_error_mgr* err = (decoder_error_mgr*) cinfo->err;
 
-  /* Convert error to a browser error code */
+  // Convert error to a browser error code
   nsresult error_code = err->pub.msg_code == JERR_OUT_OF_MEMORY
                       ? NS_ERROR_OUT_OF_MEMORY
                       : NS_ERROR_FAILURE;
@@ -680,19 +700,18 @@ my_error_exit (j_common_ptr cinfo)
 #ifdef DEBUG
   char buffer[JMSG_LENGTH_MAX];
 
-  /* Create the message */
+  // Create the message
   (*err->pub.format_message) (cinfo, buffer);
 
   fprintf(stderr, "JPEG decoding error:\n%s\n", buffer);
 #endif
 
-  /* Return control to the setjmp point.  We pass an nsresult masquerading as
-   * an int, which works because the setjmp() caller casts it back. */
+  // Return control to the setjmp point.  We pass an nsresult masquerading as
+  // an int, which works because the setjmp() caller casts it back.
   longjmp(err->setjmp_buffer, static_cast<int>(error_code));
 }
 
-/******************************************************************************/
-/*-----------------------------------------------------------------------------
+/*******************************************************************************
  * This is the callback routine from the IJG JPEG library used to supply new
  * data to the decompressor when its input buffer is exhausted.  It juggles
  * multiple buffers in an attempt to avoid unnecessary copying of input data.
@@ -719,15 +738,14 @@ my_error_exit (j_common_ptr cinfo)
  * Returns:
  *  TRUE if additional data is available, FALSE if no data present and
  *   the JPEG library should therefore suspend processing of input stream
- *---------------------------------------------------------------------------*/
+ ******************************************************************************/
 
 /******************************************************************************/
 /* data source manager method                                                 */
 /******************************************************************************/
 
-
 /******************************************************************************/
-/* data source manager method 
+/* data source manager method
         Initialize source.  This is called by jpeg_read_header() before any
         data is actually read.  May leave
         bytes_in_buffer set to 0 (in which case a fill_input_buffer() call
@@ -752,21 +770,19 @@ init_source (j_decompress_ptr jd)
 METHODDEF(void)
 skip_input_data (j_decompress_ptr jd, long num_bytes)
 {
-  struct jpeg_source_mgr *src = jd->src;
-  nsJPEGDecoder *decoder = (nsJPEGDecoder *)(jd->client_data);
+  struct jpeg_source_mgr* src = jd->src;
+  nsJPEGDecoder* decoder = (nsJPEGDecoder*)(jd->client_data);
 
   if (num_bytes > (long)src->bytes_in_buffer) {
-    /*
-     * Can't skip it all right now until we get more data from
-     * network stream. Set things up so that fill_input_buffer
-     * will skip remaining amount.
-     */
+    // Can't skip it all right now until we get more data from
+    // network stream. Set things up so that fill_input_buffer
+    // will skip remaining amount.
     decoder->mBytesToSkip = (size_t)num_bytes - src->bytes_in_buffer;
     src->next_input_byte += src->bytes_in_buffer;
     src->bytes_in_buffer = 0;
 
   } else {
-    /* Simple case. Just advance buffer pointer */
+    // Simple case. Just advance buffer pointer
 
     src->bytes_in_buffer -= (size_t)num_bytes;
     src->next_input_byte += num_bytes;
@@ -789,32 +805,33 @@ skip_input_data (j_decompress_ptr jd, long num_bytes)
 METHODDEF(boolean)
 fill_input_buffer (j_decompress_ptr jd)
 {
-  struct jpeg_source_mgr *src = jd->src;
-  nsJPEGDecoder *decoder = (nsJPEGDecoder *)(jd->client_data);
+  struct jpeg_source_mgr* src = jd->src;
+  nsJPEGDecoder* decoder = (nsJPEGDecoder*)(jd->client_data);
 
   if (decoder->mReading) {
-    const JOCTET *new_buffer = decoder->mSegment;
+    const JOCTET* new_buffer = decoder->mSegment;
     uint32_t new_buflen = decoder->mSegmentLen;
-  
-    if (!new_buffer || new_buflen == 0)
-      return false; /* suspend */
+
+    if (!new_buffer || new_buflen == 0) {
+      return false; // suspend
+    }
 
     decoder->mSegmentLen = 0;
 
     if (decoder->mBytesToSkip) {
       if (decoder->mBytesToSkip < new_buflen) {
-        /* All done skipping bytes; Return what's left. */
+        // All done skipping bytes; Return what's left.
         new_buffer += decoder->mBytesToSkip;
         new_buflen -= decoder->mBytesToSkip;
         decoder->mBytesToSkip = 0;
       } else {
-        /* Still need to skip some more data in the future */
+        // Still need to skip some more data in the future
         decoder->mBytesToSkip -= (size_t)new_buflen;
-        return false; /* suspend */
+        return false; // suspend
       }
     }
 
-      decoder->mBackBufferUnreadLen = src->bytes_in_buffer;
+    decoder->mBackBufferUnreadLen = src->bytes_in_buffer;
 
     src->next_input_byte = new_buffer;
     src->bytes_in_buffer = (size_t)new_buflen;
@@ -824,25 +841,27 @@ fill_input_buffer (j_decompress_ptr jd)
   }
 
   if (src->next_input_byte != decoder->mSegment) {
-    /* Backtrack data has been permanently consumed. */
+    // Backtrack data has been permanently consumed.
     decoder->mBackBufferUnreadLen = 0;
     decoder->mBackBufferLen = 0;
   }
 
-  /* Save remainder of netlib buffer in backtrack buffer */
-  const uint32_t new_backtrack_buflen = src->bytes_in_buffer + decoder->mBackBufferLen;
- 
-  /* Make sure backtrack buffer is big enough to hold new data. */
+  // Save remainder of netlib buffer in backtrack buffer
+  const uint32_t new_backtrack_buflen = src->bytes_in_buffer +
+                                        decoder->mBackBufferLen;
+
+  // Make sure backtrack buffer is big enough to hold new data.
   if (decoder->mBackBufferSize < new_backtrack_buflen) {
-    /* Check for malformed MARKER segment lengths, before allocating space for it */
+    // Check for malformed MARKER segment lengths, before allocating space
+    // for it
     if (new_backtrack_buflen > MAX_JPEG_MARKER_LENGTH) {
       my_error_exit((j_common_ptr)(&decoder->mInfo));
     }
 
-    /* Round up to multiple of 256 bytes. */
+    // Round up to multiple of 256 bytes.
     const size_t roundup_buflen = ((new_backtrack_buflen + 255) >> 8) << 8;
-    JOCTET *buf = (JOCTET *)PR_REALLOC(decoder->mBackBuffer, roundup_buflen);
-    /* Check for OOM */
+    JOCTET* buf = (JOCTET*)PR_REALLOC(decoder->mBackBuffer, roundup_buflen);
+    // Check for OOM
     if (!buf) {
       decoder->mInfo.err->msg_code = JERR_OUT_OF_MEMORY;
       my_error_exit((j_common_ptr)(&decoder->mInfo));
@@ -851,13 +870,14 @@ fill_input_buffer (j_decompress_ptr jd)
     decoder->mBackBufferSize = roundup_buflen;
   }
 
-  /* Copy remainder of netlib segment into backtrack buffer. */
+  // Copy remainder of netlib segment into backtrack buffer.
   memmove(decoder->mBackBuffer + decoder->mBackBufferLen,
           src->next_input_byte,
           src->bytes_in_buffer);
 
-  /* Point to start of data to be rescanned. */
-  src->next_input_byte = decoder->mBackBuffer + decoder->mBackBufferLen - decoder->mBackBufferUnreadLen;
+  // Point to start of data to be rescanned.
+  src->next_input_byte = decoder->mBackBuffer + decoder->mBackBufferLen -
+                         decoder->mBackBufferUnreadLen;
   src->bytes_in_buffer += decoder->mBackBufferUnreadLen;
   decoder->mBackBufferLen = (size_t)new_backtrack_buflen;
   decoder->mReading = true;
@@ -869,13 +889,13 @@ fill_input_buffer (j_decompress_ptr jd)
 /* data source manager method */
 /*
  * Terminate source --- called by jpeg_finish_decompress() after all
- * data has been read to clean up JPEG source manager. NOT called by 
+ * data has been read to clean up JPEG source manager. NOT called by
  * jpeg_abort() or jpeg_destroy().
  */
 METHODDEF(void)
 term_source (j_decompress_ptr jd)
 {
-  nsJPEGDecoder *decoder = (nsJPEGDecoder *)(jd->client_data);
+  nsJPEGDecoder* decoder = (nsJPEGDecoder*)(jd->client_data);
 
   // This function shouldn't be called if we ran into an error we didn't
   // recover from.
@@ -890,16 +910,15 @@ term_source (j_decompress_ptr jd)
 } // namespace mozilla
 
 
-/**************** Inverted CMYK -> RGB conversion **************/
-/*
- * Input is (Inverted) CMYK stored as 4 bytes per pixel.
- * Output is RGB stored as 3 bytes per pixel.
- * @param row Points to row buffer containing the CMYK bytes for each pixel in the row.
- * @param width Number of pixels in the row.
- */
+///*************** Inverted CMYK -> RGB conversion *************************
+/// Input is (Inverted) CMYK stored as 4 bytes per pixel.
+/// Output is RGB stored as 3 bytes per pixel.
+/// @param row Points to row buffer containing the CMYK bytes for each pixel
+/// in the row.
+/// @param width Number of pixels in the row.
 static void cmyk_convert_rgb(JSAMPROW row, JDIMENSION width)
 {
-  /* Work from end to front to shrink from 4 bytes per pixel to 3 */
+  // Work from end to front to shrink from 4 bytes per pixel to 3
   JSAMPROW in = row + width*4;
   JSAMPROW out = in;
 
@@ -924,7 +943,7 @@ static void cmyk_convert_rgb(JSAMPROW row, JDIMENSION width)
     // R = 1 - C => 1 - (1 - iC*iK) => iC*iK
     // G = 1 - M => 1 - (1 - iM*iK) => iM*iK
     // B = 1 - Y => 1 - (1 - iY*iK) => iY*iK
-  
+
     // Convert from Inverted CMYK (0..255) to RGB (0..255)
     const uint32_t iC = in[0];
     const uint32_t iM = in[1];
