@@ -12,6 +12,9 @@
 JSObject *
 js_InitRegExpClass(JSContext *cx, js::HandleObject obj);
 
+bool
+regexp_flags(JSContext *cx, unsigned argc, JS::Value *vp);
+
 /*
  * The following builtin natives are extern'd for pointer comparison in
  * other parts of the engine.
@@ -25,7 +28,7 @@ enum RegExpStaticsUpdate { UpdateRegExpStatics, DontUpdateRegExpStatics };
 
 RegExpRunStatus
 ExecuteRegExp(JSContext *cx, HandleObject regexp, HandleString string,
-              MatchConduit &matches, RegExpStaticsUpdate staticsUpdate);
+              MatchPairs *matches, RegExpStaticsUpdate staticsUpdate);
 
 /*
  * Legacy behavior of ExecuteRegExp(), which is baked into the JSAPI.
@@ -36,17 +39,17 @@ ExecuteRegExp(JSContext *cx, HandleObject regexp, HandleString string,
  */
 bool
 ExecuteRegExpLegacy(JSContext *cx, RegExpStatics *res, RegExpObject &reobj,
-                    Handle<JSLinearString*> input, const jschar *chars, size_t length,
-                    size_t *lastIndex, bool test, MutableHandleValue rval);
+                    HandleLinearString input, size_t *lastIndex, bool test,
+                    MutableHandleValue rval);
 
 /* Translation from MatchPairs to a JS array in regexp_exec()'s output format. */
 bool
-CreateRegExpMatchResult(JSContext *cx, HandleString string, MatchPairs &matches,
+CreateRegExpMatchResult(JSContext *cx, HandleString input, const MatchPairs &matches,
                         MutableHandleValue rval);
 
-bool
-CreateRegExpMatchResult(JSContext *cx, HandleString input, const jschar *chars, size_t length,
-                        MatchPairs &matches, MutableHandleValue rval);
+extern bool
+regexp_exec_raw(JSContext *cx, HandleObject regexp, HandleString input, MatchPairs *maybeMatches,
+                MutableHandleValue output);
 
 extern bool
 regexp_exec(JSContext *cx, unsigned argc, Value *vp);
