@@ -34,8 +34,8 @@ typedef Rooted<JSFlatString*>    RootedFlatString;
 static const JSClass sLibraryClass = {
   "Library",
   JSCLASS_HAS_RESERVED_SLOTS(LIBRARY_SLOTS),
-  JS_PropertyStub, JS_DeletePropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
-  JS_EnumerateStub,JS_ResolveStub, JS_ConvertStub, Library::Finalize
+  nullptr, nullptr, nullptr, nullptr,
+  nullptr, nullptr, nullptr, Library::Finalize
 };
 
 #define CTYPESFN_FLAGS \
@@ -50,19 +50,19 @@ static const JSFunctionSpec sLibraryFunctions[] = {
 bool
 Library::Name(JSContext* cx, unsigned argc, jsval *vp)
 {
-  if (argc != 1) {
+  CallArgs args = CallArgsFromVp(argc, vp);
+  if (args.length() != 1) {
     JS_ReportError(cx, "libraryName takes one argument");
     return false;
   }
 
-  jsval arg = JS_ARGV(cx, vp)[0];
+  Value arg = args[0];
   JSString* str = nullptr;
-  if (JSVAL_IS_STRING(arg)) {
-    str = JSVAL_TO_STRING(arg);
-  }
-  else {
+  if (arg.isString()) {
+    str = arg.toString();
+  } else {
     JS_ReportError(cx, "name argument must be a string");
-      return false;
+    return false;
   }
 
   AutoString resultString;
@@ -75,16 +75,16 @@ Library::Name(JSContext* cx, unsigned argc, jsval *vp)
   if (!result)
     return false;
 
-  JS_SET_RVAL(cx, vp, STRING_TO_JSVAL(result));
+  args.rval().setString(result);
   return true;
 }
 
 JSObject*
-Library::Create(JSContext* cx, jsval path_, JSCTypesCallbacks* callbacks)
+Library::Create(JSContext* cx, jsval path_, const JSCTypesCallbacks* callbacks)
 {
   RootedValue path(cx, path_);
   RootedObject libraryObj(cx,
-                          JS_NewObject(cx, &sLibraryClass, nullptr, nullptr));
+                          JS_NewObject(cx, &sLibraryClass, NullPtr(), NullPtr()));
   if (!libraryObj)
     return nullptr;
 
@@ -95,22 +95,22 @@ Library::Create(JSContext* cx, jsval path_, JSCTypesCallbacks* callbacks)
   if (!JS_DefineFunctions(cx, libraryObj, sLibraryFunctions))
     return nullptr;
 
-  if (!JSVAL_IS_STRING(path)) {
+  if (!path.isString()) {
     JS_ReportError(cx, "open takes a string argument");
     return nullptr;
   }
 
   PRLibSpec libSpec;
-  RootedFlatString pathStr(cx, JS_FlattenString(cx, JSVAL_TO_STRING(path)));
+  RootedFlatString pathStr(cx, JS_FlattenString(cx, path.toString()));
   if (!pathStr)
     return nullptr;
+  AutoStableStringChars pathStrChars(cx);
+  if (!pathStrChars.initTwoByte(cx, pathStr))
+    return nullptr;
 #ifdef XP_WIN
   // On Windows, converting to native charset may corrupt path string.
   // So, we have to use Unicode path directly.
-  const PRUnichar* pathChars = JS_GetFlatStringChars(pathStr);
-  if (!pathChars)
-    return nullptr;
-
+  char16ptr_t pathChars = pathStrChars.twoByteChars();
   libSpec.value.pathname_u = pathChars;
   libSpec.type = PR_LibSpec_PathnameU;
 #else
@@ -118,8 +118,8 @@ Library::Create(JSContext* cx, jsval path_, JSCTypesCallbacks* callbacks)
   // provided.
   char* pathBytes;
   if (callbacks && callbacks->unicodeToNative) {
-    pathBytes = 
-      callbacks->unicodeToNative(cx, pathStr->chars(), pathStr->length());
+    pathBytes =
+      callbacks->unicodeToNative(cx, pathStrChars.twoByteChars(), pathStr->length());
     if (!pathBytes)
       return nullptr;
 
@@ -127,7 +127,7 @@ Library::Create(JSContext* cx, jsval path_, JSCTypesCallbacks* callbacks)
     // Fallback: assume the platform native charset is UTF-8. This is true
     // for Mac OS X, Android, and probably Linux.
     size_t nbytes =
-      GetDeflatedUTF8StringLength(cx, pathStr->chars(), pathStr->length());
+      GetDeflatedUTF8StringLength(cx, pathStrChars.twoByteChars(), pathStr->length());
     if (nbytes == (size_t) -1)
       return nullptr;
 
@@ -135,7 +135,7 @@ Library::Create(JSContext* cx, jsval path_, JSCTypesCallbacks* callbacks)
     if (!pathBytes)
       return nullptr;
 
-    ASSERT_OK(DeflateStringToUTF8Buffer(cx, pathStr->chars(),
+    ASSERT_OK(DeflateStringToUTF8Buffer(cx, pathStrChars.twoByteChars(),
                 pathStr->length(), pathBytes, &nbytes));
     pathBytes[nbytes] = 0;
   }
@@ -175,10 +175,10 @@ Library::IsLibrary(JSObject* obj)
 PRLibrary*
 Library::GetLibrary(JSObject* obj)
 {
-  JS_ASSERT(IsLibrary(obj));
+  MOZ_ASSERT(IsLibrary(obj));
 
   jsval slot = JS_GetReservedSlot(obj, SLOT_LIBRARY);
-  return static_cast<PRLibrary*>(JSVAL_TO_PRIVATE(slot));
+  return static_cast<PRLibrary*>(slot.toPrivate());
 }
 
 static void
@@ -198,6 +198,7 @@ Library::Finalize(JSFreeOp *fop, JSObject* obj)
 bool
 Library::Open(JSContext* cx, unsigned argc, jsval *vp)
 {
+  CallArgs args = CallArgsFromVp(argc, vp);
   JSObject* ctypesObj = JS_THIS_OBJECT(cx, vp);
   if (!ctypesObj)
     return false;
@@ -206,22 +207,23 @@ Library::Open(JSContext* cx, unsigned argc, jsval *vp)
     return false;
   }
 
-  if (argc != 1 || JSVAL_IS_VOID(JS_ARGV(cx, vp)[0])) {
+  if (args.length() != 1 || args[0].isUndefined()) {
     JS_ReportError(cx, "open requires a single argument");
     return false;
   }
 
-  JSObject* library = Create(cx, JS_ARGV(cx, vp)[0], GetCallbacks(ctypesObj));
+  JSObject* library = Create(cx, args[0], GetCallbacks(ctypesObj));
   if (!library)
     return false;
 
-  JS_SET_RVAL(cx, vp, OBJECT_TO_JSVAL(library));
+  args.rval().setObject(*library);
   return true;
 }
 
 bool
 Library::Close(JSContext* cx, unsigned argc, jsval* vp)
 {
+  CallArgs args = CallArgsFromVp(argc, vp);
   JSObject* obj = JS_THIS_OBJECT(cx, vp);
   if (!obj)
     return false;
@@ -230,7 +232,7 @@ Library::Close(JSContext* cx, unsigned argc, jsval* vp)
     return false;
   }
 
-  if (argc != 0) {
+  if (args.length() != 0) {
     JS_ReportError(cx, "close doesn't take any arguments");
     return false;
   }
@@ -239,13 +241,14 @@ Library::Close(JSContext* cx, unsigned argc, jsval* vp)
   UnloadLibrary(obj);
   JS_SetReservedSlot(obj, SLOT_LIBRARY, PRIVATE_TO_JSVAL(nullptr));
 
-  JS_SET_RVAL(cx, vp, JSVAL_VOID);
+  args.rval().setUndefined();
   return true;
 }
 
 bool
 Library::Declare(JSContext* cx, unsigned argc, jsval* vp)
 {
+  CallArgs args = CallArgsFromVp(argc, vp);
   RootedObject obj(cx, JS_THIS_OBJECT(cx, vp));
   if (!obj)
     return false;
@@ -269,26 +272,25 @@ Library::Declare(JSContext* cx, unsigned argc, jsval* vp)
   //    back will be of type 'type', and will point into the symbol data.
   //    This data will be both readable and writable via the usual CData
   //    accessors. If 'type' is a PointerType to a FunctionType, the result will
-  //    be a function pointer, as with 1). 
-  if (argc < 2) {
+  //    be a function pointer, as with 1).
+  if (args.length() < 2) {
     JS_ReportError(cx, "declare requires at least two arguments");
     return false;
   }
 
-  jsval* argv = JS_ARGV(cx, vp);
-  if (!JSVAL_IS_STRING(argv[0])) {
+  if (!args[0].isString()) {
     JS_ReportError(cx, "first argument must be a string");
     return false;
   }
 
   RootedObject fnObj(cx, nullptr);
   RootedObject typeObj(cx);
-  bool isFunction = argc > 2;
+  bool isFunction = args.length() > 2;
   if (isFunction) {
     // Case 1).
     // Create a FunctionType representing the function.
     fnObj = FunctionType::CreateInternal(cx,
-              argv[1], argv[2], &argv[3], argc - 3);
+              args[1], args[2], &args.array()[3], args.length() - 3);
     if (!fnObj)
       return false;
 
@@ -298,14 +300,14 @@ Library::Declare(JSContext* cx, unsigned argc, jsval* vp)
       return false;
   } else {
     // Case 2).
-    if (JSVAL_IS_PRIMITIVE(argv[1]) ||
-        !CType::IsCType(JSVAL_TO_OBJECT(argv[1])) ||
-        !CType::IsSizeDefined(JSVAL_TO_OBJECT(argv[1]))) {
+    if (args[1].isPrimitive() ||
+        !CType::IsCType(args[1].toObjectOrNull()) ||
+        !CType::IsSizeDefined(args[1].toObjectOrNull())) {
       JS_ReportError(cx, "second argument must be a type of defined size");
       return false;
     }
 
-    typeObj = JSVAL_TO_OBJECT(argv[1]);
+    typeObj = args[1].toObjectOrNull();
     if (CType::GetTypeCode(typeObj) == TYPE_pointer) {
       fnObj = PointerType::GetBaseType(typeObj);
       isFunction = fnObj && CType::GetTypeCode(fnObj) == TYPE_function;
@@ -314,7 +316,7 @@ Library::Declare(JSContext* cx, unsigned argc, jsval* vp)
 
   void* data;
   PRFuncPtr fnptr;
-  JSString* nameStr = JSVAL_TO_STRING(argv[0]);
+  JSString* nameStr = args[0].toString();
   AutoCString symbol;
   if (isFunction) {
     // Build the symbol, with mangling if necessary.
@@ -345,7 +347,7 @@ Library::Declare(JSContext* cx, unsigned argc, jsval* vp)
   if (!result)
     return false;
 
-  JS_SET_RVAL(cx, vp, OBJECT_TO_JSVAL(result));
+  args.rval().setObject(*result);
 
   // Seal the CData object, to prevent modification of the function pointer.
   // This permanently associates this object with the library, and avoids
