@@ -65,66 +65,75 @@ var gcTypes = {}; // map from parent struct => Set of GC typed children
 var gcPointers = {}; // map from parent struct => Set of GC typed children
 var gcFields = {};
 
-function addGCType(name, child, why)
+// "typeName is a (pointer to a)*'depth' GC type because it contains a field
+// named 'child' of type 'why' (or pointer to 'why' if ptrdness == 1), which
+// itself a GCThing or GCPointer."
+function markGCType(typeName, child, why, depth, ptrdness)
 {
-    if (!why)
-        why = '<annotation>';
-    if (!child)
-        child = 'annotation';
+    // Some types, like UniquePtr, do not mark/trace/relocate their contained
+    // pointers and so should not hold them live across a GC. UniquePtr in
+    // particular should be the only thing pointing to a structure containing a
+    // GCPointer, so nothing else can be tracing it and it'll die when the
+    // UniquePtr goes out of scope. So we say that a UniquePtr's memory is just
+    // as unsafe as the stack for storing GC pointers.
+    if (!ptrdness && isUnsafeStorage(typeName)) {
+        printErr("Unsafe! " + typeName);
+        // The UniquePtr itself is on the stack but when you dereference the
+        // contained pointer, you get to the unsafe memory that we are treating
+        // as if it were the stack (aka depth 0). Note that
+        // UniquePtr<UniquePtr<JSObject*>> is fine, so we don't want to just
+        // hardcode the depth.
+        ptrdness = -1;
+    }
 
-    if (isRootedTypeName(name))
+    depth += ptrdness;
+    if (depth > 2)
         return;
 
-    if (!(name in gcTypes))
-        gcTypes[name] = Set();
-    gcTypes[name].add(why);
+    if (depth == 0 && isRootedTypeName(typeName))
+        return;
+    if (depth == 1 && isRootedPointerTypeName(typeName))
+        return;
 
-    if (!(name in gcFields))
-        gcFields[name] = Map();
-    gcFields[name].set(why, child);
+    if (depth == 0) {
+        if (!(typeName in gcTypes))
+            gcTypes[typeName] = Set();
+        gcTypes[typeName].add(why);
+    } else if (depth == 1) {
+        if (!(typeName in gcPointers))
+            gcPointers[typeName] = Set();
+        gcPointers[typeName].add(why);
+    }
 
-    if (name in structureParents) {
-        for (var field of structureParents[name]) {
-            var [ holder, fieldName ] = field;
-            addGCType(holder, name, fieldName);
+    if (!(typeName in gcFields))
+        gcFields[typeName] = Map();
+    gcFields[typeName].set(why, [ child, ptrdness ]);
+
+    if (typeName in structureParents) {
+        for (var field of structureParents[typeName]) {
+            var [ holderType, fieldName ] = field;
+            markGCType(holderType, typeName, fieldName, depth, 0);
         }
     }
-    if (name in pointerParents) {
-        for (var field of pointerParents[name]) {
-            var [ holder, fieldName ] = field;
-            addGCPointer(holder, name, fieldName);
+    if (typeName in pointerParents) {
+        for (var field of pointerParents[typeName]) {
+            var [ holderType, fieldName ] = field;
+            markGCType(holderType, typeName, fieldName, depth, 1);
         }
     }
 }
 
-function addGCPointer(name, child, why)
+function addGCType(typeName, child, why, depth, ptrdness)
 {
-    if (!why)
-        why = '<annotation>';
-    if (!child)
-        child = 'annotation';
-
-    // Ignore types that are properly rooted.
-    if (isRootedPointerTypeName(name))
-        return;
-
-    if (!(name in gcPointers))
-        gcPointers[name] = Set();
-    gcPointers[name].add(why);
-
-    if (!(name in gcFields))
-        gcFields[name] = Map();
-    gcFields[name].set(why, child);
-
-    if (name in structureParents) {
-        for (var field of structureParents[name]) {
-            var [ holder, fieldName ] = field;
-            addGCPointer(holder, name, fieldName);
-        }
-    }
+    markGCType(typeName, 'annotation', '<annotation>', 0, 0);
 }
 
-addGCType('js::ObjectImpl');
+function addGCPointer(typeName)
+{
+    markGCType(typeName, 'annotation', '<pointer-annotation>', 1, 0);
+}
+
+addGCType('JSObject');
 addGCType('JSString');
 addGCType('js::Shape');
 addGCType('js::BaseShape');
@@ -133,6 +142,8 @@ addGCType('js::LazyScript');
 addGCType('js::ion::IonCode');
 addGCPointer('JS::Value');
 addGCPointer('jsid');
+
+// AutoCheckCannotGC should also not be held live across a GC function.
 addGCPointer('JS::AutoCheckCannotGC');
 
 function explain(csu, indent, seen) {
@@ -142,14 +153,26 @@ function explain(csu, indent, seen) {
     if (!(csu in gcFields))
         return;
     if (gcFields[csu].has('<annotation>')) {
-        print(indent + "because I said so");
+        print(indent + "which is a GCThing because I said so");
         return;
     }
-    for (var [ field, child ] of gcFields[csu]) {
+    if (gcFields[csu].has('<pointer-annotation>')) {
+        print(indent + "which is a GCPointer because I said so");
+        return;
+    }
+    for (var [ field, [ child, ptrdness ] ] of gcFields[csu]) {
         var inherit = "";
         if (field == "field:0")
             inherit = " (probably via inheritance)";
-        print(indent + "contains field '" + field + "' of type " + child + inherit);
+        var msg = indent + "contains field '" + field + "' ";
+        if (ptrdness == -1)
+            msg += "(with a pointer to unsafe storage) holding a ";
+        else if (ptrdness == 0)
+            msg += "of type ";
+        else
+            msg += "pointing to type ";
+        msg += child + inherit;
+        print(msg);
         if (!seen.has(child))
             explain(child, indent + "  ", seen);
     }
