@@ -29,7 +29,7 @@ class FullParseHandler
     bool foldConstants;
 
     ParseNode *allocParseNode(size_t size) {
-        JS_ASSERT(size == sizeof(ParseNode));
+        MOZ_ASSERT(size == sizeof(ParseNode));
         return static_cast<ParseNode *>(allocator.allocNode());
     }
 
@@ -87,7 +87,12 @@ class FullParseHandler
     const Token &currentToken() { return tokenStream.currentToken(); }
 
     ParseNode *newName(PropertyName *name, uint32_t blockid, const TokenPos &pos) {
-        return new_<NameNode>(PNK_NAME, JSOP_NAME, name, blockid, pos);
+        return new_<NameNode>(PNK_NAME, JSOP_GETNAME, name, blockid, pos);
+    }
+
+    ParseNode *newComputedName(ParseNode *expr, uint32_t begin, uint32_t end) {
+        TokenPos pos(begin, end);
+        return new_<UnaryNode>(PNK_COMPUTED_NAME, JSOP_NOP, pos, expr);
     }
 
     Definition *newPlaceholder(JSAtom *atom, uint32_t blockid, const TokenPos &pos) {
@@ -117,7 +122,42 @@ class FullParseHandler
     }
 
     ParseNode *newStringLiteral(JSAtom *atom, const TokenPos &pos) {
-        return new_<NullaryNode>(PNK_STRING, JSOP_STRING, pos, atom);
+        return new_<NullaryNode>(PNK_STRING, JSOP_NOP, pos, atom);
+    }
+
+    ParseNode *newTemplateStringLiteral(JSAtom *atom, const TokenPos &pos) {
+        return new_<NullaryNode>(PNK_TEMPLATE_STRING, JSOP_NOP, pos, atom);
+    }
+
+    ParseNode *newCallSiteObject(uint32_t begin, unsigned blockidGen) {
+        ParseNode *callSite = new_<CallSiteNode>(begin);
+        if (!callSite)
+            return null();
+
+        Node propExpr = newArrayLiteral(getPosition(callSite).begin, blockidGen);
+        if (!propExpr)
+            return null();
+
+        if (!addArrayElement(callSite, propExpr))
+            return null();
+
+        return callSite;
+    }
+
+    bool addToCallSiteObject(ParseNode *callSiteObj, ParseNode *rawNode, ParseNode *cookedNode) {
+        MOZ_ASSERT(callSiteObj->isKind(PNK_CALLSITEOBJ));
+
+        if (!addArrayElement(callSiteObj, cookedNode))
+            return false;
+        if (!addArrayElement(callSiteObj->pn_head, rawNode))
+            return false;
+
+        /*
+         * We don't know when the last noSubstTemplate will come in, and we
+         * don't want to deal with this outside this method
+         */
+        setEndPosition(callSiteObj, callSiteObj->pn_head);
+        return true;
     }
 
     ParseNode *newThisLiteral(const TokenPos &pos) {
@@ -132,7 +172,7 @@ class FullParseHandler
     // Specifically, a Boxer has a .newObjectBox(T) method that accepts a
     // Rooted<RegExpObject*> argument and returns an ObjectBox*.
     template <class Boxer>
-    ParseNode *newRegExp(HandleObject reobj, const TokenPos &pos, Boxer &boxer) {
+    ParseNode *newRegExp(RegExpObject *reobj, const TokenPos &pos, Boxer &boxer) {
         ObjectBox *objbox = boxer.newObjectBox(reobj);
         if (!objbox)
             return null();
@@ -191,6 +231,17 @@ class FullParseHandler
 
     // Expressions
 
+    ParseNode *newArrayComprehension(ParseNode *body, unsigned blockid, const TokenPos &pos) {
+        MOZ_ASSERT(pos.begin <= body->pn_pos.begin);
+        MOZ_ASSERT(body->pn_pos.end <= pos.end);
+        ParseNode *pn = new_<ListNode>(PNK_ARRAYCOMP, pos);
+        if (!pn)
+            return nullptr;
+        pn->pn_blockid = blockid;
+        pn->append(body);
+        return pn;
+    }
+
     ParseNode *newArrayLiteral(uint32_t begin, unsigned blockid) {
         ParseNode *literal = new_<ListNode>(PNK_ARRAY, TokenPos(begin, begin + 1));
         // Later in this stack: remove dependency on this opcode.
@@ -235,28 +286,34 @@ class FullParseHandler
         return literal;
     }
 
-    bool addPropertyDefinition(ParseNode *literal, ParseNode *name, ParseNode *expr) {
-        ParseNode *propdef = newBinary(PNK_COLON, name, expr, JSOP_INITPROP);
+    bool addPrototypeMutation(ParseNode *literal, uint32_t begin, ParseNode *expr) {
+        // Object literals with mutated [[Prototype]] are non-constant so that
+        // singleton objects will have Object.prototype as their [[Prototype]].
+        setListFlag(literal, PNX_NONCONST);
+
+        ParseNode *mutation = newUnary(PNK_MUTATEPROTO, JSOP_NOP, begin, expr);
+        if (!mutation)
+            return false;
+        literal->append(mutation);
+        return true;
+    }
+
+    bool addPropertyDefinition(ParseNode *literal, ParseNode *name, ParseNode *expr,
+                               bool isShorthand = false) {
+        MOZ_ASSERT(literal->isArity(PN_LIST));
+        ParseNode *propdef = newBinary(isShorthand ? PNK_SHORTHAND : PNK_COLON, name, expr,
+                                       JSOP_INITPROP);
+        if (isShorthand)
+            literal->pn_xflags |= PNX_NONCONST;
         if (!propdef)
             return false;
         literal->append(propdef);
         return true;
     }
 
-    bool addShorthandPropertyDefinition(ParseNode *literal, ParseNode *name) {
-        JS_ASSERT(literal->isArity(PN_LIST));
-        literal->pn_xflags |= PNX_DESTRUCT | PNX_NONCONST;  // XXX why PNX_DESTRUCT?
-
-        ParseNode *propdef = newBinary(PNK_COLON, name, name, JSOP_INITPROP);
-        if (!propdef)
-            return false;
-        literal->append(propdef);
-        return true;
-    }
-
-    bool addAccessorPropertyDefinition(ParseNode *literal, ParseNode *name, ParseNode *fn, JSOp op)
+    bool addMethodDefinition(ParseNode *literal, ParseNode *name, ParseNode *fn, JSOp op)
     {
-        JS_ASSERT(literal->isArity(PN_LIST));
+        MOZ_ASSERT(literal->isArity(PN_LIST));
         literal->pn_xflags |= PNX_NONCONST;
 
         ParseNode *propdef = newBinary(PNK_COLON, name, fn, op);
@@ -266,6 +323,17 @@ class FullParseHandler
         return true;
     }
 
+    ParseNode *newYieldExpression(uint32_t begin, ParseNode *value, ParseNode *gen,
+                                  JSOp op = JSOP_YIELD) {
+        TokenPos pos(begin, value ? value->pn_pos.end : begin + 1);
+        return new_<BinaryNode>(PNK_YIELD, op, pos, value, gen);
+    }
+
+    ParseNode *newYieldStarExpression(uint32_t begin, ParseNode *value, ParseNode *gen) {
+        TokenPos pos(begin, value->pn_pos.end);
+        return new_<BinaryNode>(PNK_YIELD_STAR, JSOP_NOP, pos, value, gen);
+    }
+
     // Statements
 
     ParseNode *newStatementList(unsigned blockid, const TokenPos &pos) {
@@ -277,7 +345,7 @@ class FullParseHandler
 
     template <typename PC>
     void addStatementToList(ParseNode *list, ParseNode *stmt, PC *pc) {
-        JS_ASSERT(list->isKind(PNK_STATEMENTLIST));
+        MOZ_ASSERT(list->isKind(PNK_STATEMENTLIST));
 
         if (stmt->isKind(PNK_FUNCTION)) {
             if (pc->atBodyLevel()) {
@@ -287,14 +355,39 @@ class FullParseHandler
                 list->pn_xflags |= PNX_FUNCDEFS;
             } else {
                 // General deoptimization was done in Parser::functionDef.
-                JS_ASSERT_IF(pc->sc->isFunctionBox(),
-                             pc->sc->asFunctionBox()->hasExtensibleScope());
+                MOZ_ASSERT_IF(pc->sc->isFunctionBox(),
+                              pc->sc->asFunctionBox()->hasExtensibleScope());
             }
         }
 
         list->append(stmt);
     }
 
+    bool prependInitialYield(ParseNode *stmtList, ParseNode *genName) {
+        MOZ_ASSERT(stmtList->isKind(PNK_STATEMENTLIST));
+
+        TokenPos yieldPos(stmtList->pn_pos.begin, stmtList->pn_pos.begin + 1);
+        ParseNode *makeGen = new_<NullaryNode>(PNK_GENERATOR, yieldPos);
+        if (!makeGen)
+            return false;
+
+        MOZ_ASSERT(genName->getOp() == JSOP_GETNAME);
+        genName->setOp(JSOP_SETNAME);
+        genName->markAsAssigned();
+        ParseNode *genInit = newBinary(PNK_ASSIGN, genName, makeGen);
+
+        ParseNode *initialYield = newYieldExpression(yieldPos.begin, nullptr, genInit,
+                                                     JSOP_INITIALYIELD);
+        if (!initialYield)
+            return false;
+
+        initialYield->pn_next = stmtList->pn_head;
+        stmtList->pn_head = initialYield;
+        stmtList->pn_count++;
+
+        return true;
+    }
+
     ParseNode *newEmptyStatement(const TokenPos &pos) {
         return new_<UnaryNode>(PNK_SEMI, JSOP_NOP, pos, (ParseNode *) nullptr);
     }
@@ -324,7 +417,7 @@ class FullParseHandler
     }
 
     ParseNode *newExprStatement(ParseNode *expr, uint32_t end) {
-        JS_ASSERT(expr->pn_pos.end <= end);
+        MOZ_ASSERT(expr->pn_pos.end <= end);
         return new_<UnaryNode>(PNK_SEMI, JSOP_NOP, TokenPos(expr->pn_pos.begin, end), expr);
     }
 
@@ -363,7 +456,7 @@ class FullParseHandler
     ParseNode *newForHead(ParseNodeKind kind, ParseNode *pn1, ParseNode *pn2, ParseNode *pn3,
                           const TokenPos &pos)
     {
-        JS_ASSERT(kind == PNK_FORIN || kind == PNK_FOROF || kind == PNK_FORHEAD);
+        MOZ_ASSERT(kind == PNK_FORIN || kind == PNK_FOROF || kind == PNK_FORHEAD);
         return new_<TernaryNode>(kind, JSOP_NOP, pn1, pn2, pn3, pos);
     }
 
@@ -385,13 +478,15 @@ class FullParseHandler
         return new_<BreakStatement>(label, pos);
     }
 
-    ParseNode *newReturnStatement(ParseNode *expr, const TokenPos &pos) {
-        JS_ASSERT_IF(expr, pos.encloses(expr->pn_pos));
-        return new_<UnaryNode>(PNK_RETURN, JSOP_RETURN, pos, expr);
+    ParseNode *newReturnStatement(ParseNode *expr, ParseNode *genrval, const TokenPos &pos) {
+        MOZ_ASSERT_IF(expr, pos.encloses(expr->pn_pos));
+        return new_<BinaryNode>(PNK_RETURN, JSOP_RETURN, pos, expr, genrval);
     }
 
-    ParseNode *newWithStatement(uint32_t begin, ParseNode *expr, ParseNode *body) {
-        return new_<BinaryNode>(PNK_WITH, JSOP_NOP, TokenPos(begin, body->pn_pos.end), expr, body);
+    ParseNode *newWithStatement(uint32_t begin, ParseNode *expr, ParseNode *body,
+                                ObjectBox *staticWith) {
+        return new_<BinaryObjNode>(PNK_WITH, JSOP_NOP, TokenPos(begin, body->pn_pos.end),
+                                   expr, body, staticWith);
     }
 
     ParseNode *newLabeledStatement(PropertyName *label, ParseNode *stmt, uint32_t begin) {
@@ -399,7 +494,7 @@ class FullParseHandler
     }
 
     ParseNode *newThrowStatement(ParseNode *expr, const TokenPos &pos) {
-        JS_ASSERT(pos.encloses(expr->pn_pos));
+        MOZ_ASSERT(pos.encloses(expr->pn_pos));
         return new_<UnaryNode>(PNK_THROW, JSOP_THROW, pos, expr);
     }
 
@@ -424,21 +519,22 @@ class FullParseHandler
     inline bool addCatchBlock(ParseNode *catchList, ParseNode *letBlock,
                               ParseNode *catchName, ParseNode *catchGuard, ParseNode *catchBody);
 
-    inline void setLeaveBlockResult(ParseNode *block, ParseNode *kid, bool leaveBlockExpr);
-
     inline void setLastFunctionArgumentDefault(ParseNode *funcpn, ParseNode *pn);
     inline ParseNode *newFunctionDefinition();
     void setFunctionBody(ParseNode *pn, ParseNode *kid) {
         pn->pn_body = kid;
     }
     void setFunctionBox(ParseNode *pn, FunctionBox *funbox) {
-        JS_ASSERT(pn->isKind(PNK_FUNCTION));
+        MOZ_ASSERT(pn->isKind(PNK_FUNCTION));
         pn->pn_funbox = funbox;
     }
     void addFunctionArgument(ParseNode *pn, ParseNode *argpn) {
         pn->pn_body->append(argpn);
     }
+
     inline ParseNode *newLexicalScope(ObjectBox *blockbox);
+    inline void setLexicalScopeBody(ParseNode *block, ParseNode *body);
+
     bool isOperationWithoutParens(ParseNode *pn, ParseNodeKind kind) {
         return pn->isKind(kind) && !pn->isInParens();
     }
@@ -450,7 +546,7 @@ class FullParseHandler
     }
     void setBeginPosition(ParseNode *pn, uint32_t begin) {
         pn->pn_pos.begin = begin;
-        JS_ASSERT(pn->pn_pos.begin <= pn->pn_pos.end);
+        MOZ_ASSERT(pn->pn_pos.begin <= pn->pn_pos.end);
     }
 
     void setEndPosition(ParseNode *pn, ParseNode *oth) {
@@ -458,7 +554,7 @@ class FullParseHandler
     }
     void setEndPosition(ParseNode *pn, uint32_t end) {
         pn->pn_pos.end = end;
-        JS_ASSERT(pn->pn_pos.begin <= pn->pn_pos.end);
+        MOZ_ASSERT(pn->pn_pos.begin <= pn->pn_pos.end);
     }
 
     void setPosition(ParseNode *pn, const TokenPos &pos) {
@@ -498,7 +594,7 @@ class FullParseHandler
         pn->pn_dflags |= flag;
     }
     void setListFlag(ParseNode *pn, unsigned flag) {
-        JS_ASSERT(pn->isArity(PN_LIST));
+        MOZ_ASSERT(pn->isArity(PN_LIST));
         pn->pn_xflags |= flag;
     }
     ParseNode *setInParens(ParseNode *pn) {
@@ -537,12 +633,15 @@ class FullParseHandler
     static Definition::Kind getDefinitionKind(Definition *dn) {
         return dn->kind();
     }
+    static bool isPlaceholderDefinition(Definition *dn) {
+        return dn->isPlaceholder();
+    }
     void linkUseToDef(ParseNode *pn, Definition *dn)
     {
-        JS_ASSERT(!pn->isUsed());
-        JS_ASSERT(!pn->isDefn());
-        JS_ASSERT(pn != dn->dn_uses);
-        JS_ASSERT(dn->isDefn());
+        MOZ_ASSERT(!pn->isUsed());
+        MOZ_ASSERT(!pn->isDefn());
+        MOZ_ASSERT(pn != dn->dn_uses);
+        MOZ_ASSERT(dn->isDefn());
         pn->pn_link = dn->dn_uses;
         dn->dn_uses = pn;
         dn->pn_dflags |= pn->pn_dflags & PND_USE2DEF_FLAGS;
@@ -555,8 +654,8 @@ class FullParseHandler
     void deoptimizeUsesWithin(Definition *dn, const TokenPos &pos)
     {
         for (ParseNode *pnu = dn->dn_uses; pnu; pnu = pnu->pn_link) {
-            JS_ASSERT(pnu->isUsed());
-            JS_ASSERT(!pnu->isDefn());
+            MOZ_ASSERT(pnu->isUsed());
+            MOZ_ASSERT(!pnu->isDefn());
             if (pnu->pn_pos.begin >= pos.begin && pnu->pn_pos.end <= pos.end)
                 pnu->pn_dflags |= PND_DEOPTIMIZED;
         }
@@ -564,6 +663,13 @@ class FullParseHandler
     bool dependencyCovered(ParseNode *pn, unsigned blockid, bool functionScope) {
         return pn->pn_blockid >= blockid;
     }
+    void markMaybeUninitializedLexicalUseInSwitch(ParseNode *pn, Definition *dn,
+                                                  uint16_t firstDominatingLexicalSlot)
+    {
+        MOZ_ASSERT(pn->isUsed());
+        if (dn->isLexical() && dn->pn_cookie.slot() < firstDominatingLexicalSlot)
+            pn->pn_dflags |= PND_LEXICAL;
+    }
 
     static uintptr_t definitionToBits(Definition *dn) {
         return uintptr_t(dn);
@@ -582,7 +688,7 @@ class FullParseHandler
         return lazyOuterFunction_;
     }
     JSFunction *nextLazyInnerFunction() {
-        JS_ASSERT(lazyInnerFunctionIndex < lazyOuterFunction()->numInnerFunctions());
+        MOZ_ASSERT(lazyInnerFunctionIndex < lazyOuterFunction()->numInnerFunctions());
         return lazyOuterFunction()->innerFunctions()[lazyInnerFunctionIndex++];
     }
 };
@@ -600,15 +706,6 @@ FullParseHandler::addCatchBlock(ParseNode *catchList, ParseNode *letBlock,
     return true;
 }
 
-inline void
-FullParseHandler::setLeaveBlockResult(ParseNode *block, ParseNode *kid, bool leaveBlockExpr)
-{
-    JS_ASSERT(block->isOp(JSOP_LEAVEBLOCK));
-    if (leaveBlockExpr)
-        block->setOp(JSOP_LEAVEBLOCKEXPR);
-    block->pn_expr = kid;
-}
-
 inline void
 FullParseHandler::setLastFunctionArgumentDefault(ParseNode *funcpn, ParseNode *defaultValue)
 {
@@ -637,13 +734,18 @@ FullParseHandler::newLexicalScope(ObjectBox *blockbox)
     if (!pn)
         return nullptr;
 
-    pn->setOp(JSOP_LEAVEBLOCK);
     pn->pn_objbox = blockbox;
     pn->pn_cookie.makeFree();
     pn->pn_dflags = 0;
     return pn;
 }
 
+inline void
+FullParseHandler::setLexicalScopeBody(ParseNode *block, ParseNode *kid)
+{
+    block->pn_expr = kid;
+}
+
 inline bool
 FullParseHandler::finishInitializerAssignment(ParseNode *pn, ParseNode *init, JSOp op)
 {
@@ -655,11 +757,14 @@ FullParseHandler::finishInitializerAssignment(ParseNode *pn, ParseNode *init, JS
         pn->pn_expr = init;
     }
 
-    pn->setOp((pn->pn_dflags & PND_BOUND)
-              ? JSOP_SETLOCAL
-              : (op == JSOP_DEFCONST)
-              ? JSOP_SETCONST
-              : JSOP_SETNAME);
+    if (op == JSOP_INITLEXICAL)
+        pn->setOp(op);
+    else if (pn->pn_dflags & PND_BOUND)
+        pn->setOp(JSOP_SETLOCAL);
+    else if (op == JSOP_DEFCONST)
+        pn->setOp(JSOP_SETCONST);
+    else
+        pn->setOp(JSOP_SETNAME);
 
     pn->markAsAssigned();
 
