@@ -15,20 +15,11 @@ using namespace js::frontend;
 
 using mozilla::IsFinite;
 
-/*
- * Asserts to verify assumptions behind pn_ macros.
- */
-#define pn_offsetof(m)  offsetof(ParseNode, m)
-
-JS_STATIC_ASSERT(pn_offsetof(pn_link) == pn_offsetof(dn_uses));
-
-#undef pn_offsetof
-
 #ifdef DEBUG
 void
 ParseNode::checkListConsistency()
 {
-    JS_ASSERT(isArity(PN_LIST));
+    MOZ_ASSERT(isArity(PN_LIST));
     ParseNode **tail;
     uint32_t count = 0;
     if (pn_head) {
@@ -39,8 +30,8 @@ ParseNode::checkListConsistency()
     } else {
         tail = &pn_head;
     }
-    JS_ASSERT(pn_tail == tail);
-    JS_ASSERT(pn_count == count);
+    MOZ_ASSERT(pn_tail == tail);
+    MOZ_ASSERT(pn_count == count);
 }
 #endif
 
@@ -49,7 +40,7 @@ void
 ParseNodeAllocator::freeNode(ParseNode *pn)
 {
     /* Catch back-to-back dup recycles. */
-    JS_ASSERT(pn != freelist);
+    MOZ_ASSERT(pn != freelist);
 
     /*
      * It's too hard to clear these nodes from the AtomDefnMaps, etc. that
@@ -57,8 +48,8 @@ ParseNodeAllocator::freeNode(ParseNode *pn)
      * recognize and process these, since their children do need to be dealt
      * with.
      */
-    JS_ASSERT(!pn->isUsed());
-    JS_ASSERT(!pn->isDefn());
+    MOZ_ASSERT(!pn->isUsed());
+    MOZ_ASSERT(!pn->isDefn());
 
 #ifdef DEBUG
     /* Poison the node, to catch attempts to use it without initializing it. */
@@ -96,7 +87,7 @@ class NodeStack {
         top = pn->pn_head;
     }
     ParseNode *pop() {
-        JS_ASSERT(!empty());
+        MOZ_ASSERT(!empty());
         ParseNode *hold = top; /* my kingdom for a prog1 */
         top = top->pn_next;
         return hold;
@@ -170,6 +161,7 @@ PushNodeChildren(ParseNode *pn, NodeStack *stack)
         stack->pushUnlessNull(pn->pn_kid3);
         break;
       case PN_BINARY:
+      case PN_BINARY_OBJ:
         if (pn->pn_left != pn->pn_right)
             stack->pushUnlessNull(pn->pn_left);
         stack->pushUnlessNull(pn->pn_right);
@@ -268,7 +260,7 @@ ParseNode::append(ParseNodeKind kind, JSOp op, ParseNode *left, ParseNode *right
     if (!left || !right)
         return nullptr;
 
-    JS_ASSERT(left->isKind(kind) && left->isOp(op) && (js_CodeSpec[op].format & JOF_LEFTASSOC));
+    MOZ_ASSERT(left->isKind(kind) && left->isOp(op) && (js_CodeSpec[op].format & JOF_LEFTASSOC));
 
     ListNode *list;
     if (left->pn_arity == PN_LIST) {
@@ -279,26 +271,10 @@ ParseNode::append(ParseNodeKind kind, JSOp op, ParseNode *left, ParseNode *right
         if (!list)
             return nullptr;
         list->append(pn2);
-        if (kind == PNK_ADD) {
-            if (pn1->isKind(PNK_STRING))
-                list->pn_xflags |= PNX_STRCAT;
-            else if (!pn1->isKind(PNK_NUMBER))
-                list->pn_xflags |= PNX_CANTFOLD;
-            if (pn2->isKind(PNK_STRING))
-                list->pn_xflags |= PNX_STRCAT;
-            else if (!pn2->isKind(PNK_NUMBER))
-                list->pn_xflags |= PNX_CANTFOLD;
-        }
     }
 
     list->append(right);
     list->pn_pos.end = right->pn_pos.end;
-    if (kind == PNK_ADD) {
-        if (right->isKind(PNK_STRING))
-            list->pn_xflags |= PNX_STRCAT;
-        else if (!right->isKind(PNK_NUMBER))
-            list->pn_xflags |= PNX_CANTFOLD;
-    }
 
     return list;
 }
@@ -325,24 +301,6 @@ ParseNode::newBinaryOrAppend(ParseNodeKind kind, JSOp op, ParseNode *left, Parse
     if (left->isKind(kind) && left->isOp(op) && (js_CodeSpec[op].format & JOF_LEFTASSOC))
         return append(kind, op, left, right, handler);
 
-    /*
-     * Fold constant addition immediately, to conserve node space and, what's
-     * more, so js::FoldConstants never sees mixed addition and concatenation
-     * operations with more than one leading non-string operand in a PN_LIST
-     * generated for expressions such as 1 + 2 + "pt" (which should evaluate
-     * to "3pt", not "12pt").
-     */
-    if (kind == PNK_ADD &&
-        left->isKind(PNK_NUMBER) &&
-        right->isKind(PNK_NUMBER) &&
-        foldConstants)
-    {
-        left->pn_dval += right->pn_dval;
-        left->pn_pos.end = right->pn_pos.end;
-        handler->freeTree(right);
-        return left;
-    }
-
     return handler->new_<BinaryNode>(kind, op, left, right);
 }
 
@@ -350,18 +308,16 @@ const char *
 Definition::kindString(Kind kind)
 {
     static const char * const table[] = {
-        "", js_var_str, js_const_str, js_let_str, js_function_str, "argument", "unknown"
+        "", js_var_str, js_const_str, js_const_str, js_let_str, "argument", js_function_str, "unknown"
     };
 
-    JS_ASSERT(unsigned(kind) <= unsigned(ARG));
+    MOZ_ASSERT(unsigned(kind) <= unsigned(ARG));
     return table[kind];
 }
 
 namespace js {
 namespace frontend {
 
-#if JS_HAS_DESTRUCTURING
-
 /*
  * This function assumes the cloned tree is for use in the same statement and
  * binding context as the original tree.
@@ -372,6 +328,11 @@ Parser<FullParseHandler>::cloneParseTree(ParseNode *opn)
 {
     JS_CHECK_RECURSION(context, return nullptr);
 
+    if (opn->isKind(PNK_COMPUTED_NAME)) {
+        report(ParseError, false, opn, JSMSG_COMPUTED_NAME_IN_PATTERN);
+        return null();
+    }
+
     ParseNode *pn = handler.new_<ParseNode>(opn->getKind(), opn->getOp(), opn->getArity(),
                                             opn->pn_pos);
     if (!pn)
@@ -418,6 +379,15 @@ Parser<FullParseHandler>::cloneParseTree(ParseNode *opn)
         pn->pn_iflags = opn->pn_iflags;
         break;
 
+      case PN_BINARY_OBJ:
+        NULLCHECK(pn->pn_left = cloneParseTree(opn->pn_left));
+        if (opn->pn_right != opn->pn_left)
+            NULLCHECK(pn->pn_right = cloneParseTree(opn->pn_right));
+        else
+            pn->pn_right = pn->pn_left;
+        pn->pn_binary_obj = opn->pn_binary_obj;
+        break;
+
       case PN_UNARY:
         NULLCHECK(pn->pn_kid = cloneParseTree(opn->pn_kid));
         break;
@@ -433,6 +403,7 @@ Parser<FullParseHandler>::cloneParseTree(ParseNode *opn)
             Definition *dn = pn->pn_lexdef;
 
             pn->pn_link = dn->dn_uses;
+            pn->pn_dflags = opn->pn_dflags;
             dn->dn_uses = pn;
         } else if (opn->pn_expr) {
             NULLCHECK(pn->pn_expr = cloneParseTree(opn->pn_expr));
@@ -457,8 +428,6 @@ Parser<FullParseHandler>::cloneParseTree(ParseNode *opn)
     return pn;
 }
 
-#endif /* JS_HAS_DESTRUCTURING */
-
 /*
  * Used by Parser::forStatement and comprehensionTail to clone the TARGET in
  *   for (var/const/let TARGET in EXPR)
@@ -481,27 +450,38 @@ Parser<FullParseHandler>::cloneLeftHandSide(ParseNode *opn)
     pn->setDefn(opn->isDefn());
     pn->setUsed(opn->isUsed());
 
-#if JS_HAS_DESTRUCTURING
     if (opn->isArity(PN_LIST)) {
-        JS_ASSERT(opn->isKind(PNK_ARRAY) || opn->isKind(PNK_OBJECT));
+        MOZ_ASSERT(opn->isKind(PNK_ARRAY) || opn->isKind(PNK_OBJECT));
         pn->makeEmpty();
         for (ParseNode *opn2 = opn->pn_head; opn2; opn2 = opn2->pn_next) {
             ParseNode *pn2;
             if (opn->isKind(PNK_OBJECT)) {
-                JS_ASSERT(opn2->isArity(PN_BINARY));
-                JS_ASSERT(opn2->isKind(PNK_COLON));
+                if (opn2->isKind(PNK_MUTATEPROTO)) {
+                    ParseNode *target = cloneLeftHandSide(opn2->pn_kid);
+                    if (!target)
+                        return nullptr;
+                    pn2 = handler.new_<UnaryNode>(PNK_MUTATEPROTO, JSOP_NOP, opn2->pn_pos, target);
+                } else {
+                    MOZ_ASSERT(opn2->isArity(PN_BINARY));
+                    MOZ_ASSERT(opn2->isKind(PNK_COLON) || opn2->isKind(PNK_SHORTHAND));
 
-                ParseNode *tag = cloneParseTree(opn2->pn_left);
-                if (!tag)
-                    return nullptr;
-                ParseNode *target = cloneLeftHandSide(opn2->pn_right);
+                    ParseNode *tag = cloneParseTree(opn2->pn_left);
+                    if (!tag)
+                        return nullptr;
+                    ParseNode *target = cloneLeftHandSide(opn2->pn_right);
+                    if (!target)
+                        return nullptr;
+
+                    pn2 = handler.new_<BinaryNode>(opn2->getKind(), JSOP_INITPROP, opn2->pn_pos, tag, target);
+                }
+            } else if (opn2->isArity(PN_NULLARY)) {
+                MOZ_ASSERT(opn2->isKind(PNK_ELISION));
+                pn2 = cloneParseTree(opn2);
+            } else if (opn2->isKind(PNK_SPREAD)) {
+                ParseNode *target = cloneLeftHandSide(opn2->pn_kid);
                 if (!target)
                     return nullptr;
-
-                pn2 = handler.new_<BinaryNode>(PNK_COLON, JSOP_INITPROP, opn2->pn_pos, tag, target);
-            } else if (opn2->isArity(PN_NULLARY)) {
-                JS_ASSERT(opn2->isKind(PNK_ELISION));
-                pn2 = cloneParseTree(opn2);
+                pn2 = handler.new_<UnaryNode>(PNK_SPREAD, JSOP_NOP, opn2->pn_pos, target);
             } else {
                 pn2 = cloneLeftHandSide(opn2);
             }
@@ -513,10 +493,9 @@ Parser<FullParseHandler>::cloneLeftHandSide(ParseNode *opn)
         pn->pn_xflags = opn->pn_xflags;
         return pn;
     }
-#endif
 
-    JS_ASSERT(opn->isArity(PN_NAME));
-    JS_ASSERT(opn->isKind(PNK_NAME));
+    MOZ_ASSERT(opn->isArity(PN_NAME));
+    MOZ_ASSERT(opn->isKind(PNK_NAME));
 
     /* If opn is a definition or use, make pn a use. */
     pn->pn_u.name = opn->pn_u.name;
@@ -531,7 +510,7 @@ Parser<FullParseHandler>::cloneLeftHandSide(ParseNode *opn)
         if (opn->isDefn()) {
             /* We copied some definition-specific state into pn. Clear it out. */
             pn->pn_cookie.makeFree();
-            pn->pn_dflags &= ~PND_BOUND;
+            pn->pn_dflags &= ~(PND_LEXICAL | PND_BOUND);
             pn->setDefn(false);
 
             handler.linkUseToDef(pn, (Definition *) opn);
@@ -588,6 +567,9 @@ ParseNode::dump(int indent)
       case PN_BINARY:
         ((BinaryNode *) this)->dump(indent);
         break;
+      case PN_BINARY_OBJ:
+        ((BinaryObjNode *) this)->dump(indent);
+        break;
       case PN_TERNARY:
         ((TernaryNode *) this)->dump(indent);
         break;
@@ -628,7 +610,7 @@ NullaryNode::dump()
       }
 
       case PNK_STRING:
-        JSString::dumpChars(pn_atom->chars(), pn_atom->length());
+        pn_atom->dumpCharsNoNewline();
         break;
 
       default:
@@ -658,6 +640,18 @@ BinaryNode::dump(int indent)
     fprintf(stderr, ")");
 }
 
+void
+BinaryObjNode::dump(int indent)
+{
+    const char *name = parseNodeNames[getKind()];
+    fprintf(stderr, "(%s ", name);
+    indent += strlen(name) + 2;
+    DumpParseTree(pn_left, indent);
+    IndentNewLine(indent);
+    DumpParseTree(pn_right, indent);
+    fprintf(stderr, ")");
+}
+
 void
 TernaryNode::dump(int indent)
 {
@@ -700,6 +694,24 @@ ListNode::dump(int indent)
     fprintf(stderr, "])");
 }
 
+template <typename CharT>
+static void
+DumpName(const CharT *s, size_t len)
+{
+    if (len == 0)
+        fprintf(stderr, "#<zero-length name>");
+
+    for (size_t i = 0; i < len; i++) {
+        char16_t c = s[i];
+        if (c > 32 && c < 127)
+            fputc(c, stderr);
+        else if (c <= 255)
+            fprintf(stderr, "\\x%02x", unsigned(c));
+        else
+            fprintf(stderr, "\\u%04x", unsigned(c));
+    }
+}
+
 void
 NameNode::dump(int indent)
 {
@@ -710,18 +722,11 @@ NameNode::dump(int indent)
         if (!pn_atom) {
             fprintf(stderr, "#<null name>");
         } else {
-            const jschar *s = pn_atom->chars();
-            size_t len = pn_atom->length();
-            if (len == 0)
-                fprintf(stderr, "#<zero-length name>");
-            for (size_t i = 0; i < len; i++) {
-                if (s[i] > 32 && s[i] < 127)
-                    fputc(s[i], stderr);
-                else if (s[i] <= 255)
-                    fprintf(stderr, "\\x%02x", (unsigned int) s[i]);
-                else
-                    fprintf(stderr, "\\u%04x", (unsigned int) s[i]);
-            }
+            JS::AutoCheckCannotGC nogc;
+            if (pn_atom->hasLatin1Chars())
+                DumpName(pn_atom->latin1Chars(nogc), pn_atom->length());
+            else
+                DumpName(pn_atom->twoByteChars(nogc), pn_atom->length());
         }
 
         if (isKind(PNK_DOT)) {
@@ -732,7 +737,7 @@ NameNode::dump(int indent)
         return;
     }
 
-    JS_ASSERT(!isUsed());
+    MOZ_ASSERT(!isUsed());
     const char *name = parseNodeNames[getKind()];
     if (isUsed())
         fprintf(stderr, "(%s)", name);
@@ -745,12 +750,12 @@ NameNode::dump(int indent)
 }
 #endif
 
-ObjectBox::ObjectBox(JSObject *object, ObjectBox* traceLink)
+ObjectBox::ObjectBox(NativeObject *object, ObjectBox* traceLink)
   : object(object),
     traceLink(traceLink),
     emitLink(nullptr)
 {
-    JS_ASSERT(!object->is<JSFunction>());
+    MOZ_ASSERT(!object->is<JSFunction>());
 }
 
 ObjectBox::ObjectBox(JSFunction *function, ObjectBox* traceLink)
@@ -758,14 +763,14 @@ ObjectBox::ObjectBox(JSFunction *function, ObjectBox* traceLink)
     traceLink(traceLink),
     emitLink(nullptr)
 {
-    JS_ASSERT(object->is<JSFunction>());
-    JS_ASSERT(asFunctionBox()->function() == function);
+    MOZ_ASSERT(object->is<JSFunction>());
+    MOZ_ASSERT(asFunctionBox()->function() == function);
 }
 
 FunctionBox *
 ObjectBox::asFunctionBox()
 {
-    JS_ASSERT(isFunctionBox());
+    MOZ_ASSERT(isFunctionBox());
     return static_cast<FunctionBox *>(this);
 }
 
