@@ -206,6 +206,10 @@ class SharedContext
     bool needStrictChecks() {
         return strict || extraWarnings;
     }
+
+    bool isDotVariable(JSAtom *atom) const {
+        return atom == context->names().dotGenerator || atom == context->names().dotGenRVal;
+    }
 };
 
 class GlobalSharedContext : public SharedContext
@@ -227,7 +231,7 @@ class GlobalSharedContext : public SharedContext
 inline GlobalSharedContext *
 SharedContext::asGlobalSharedContext()
 {
-    JS_ASSERT(isGlobalSharedContext());
+    MOZ_ASSERT(isGlobalSharedContext());
     return static_cast<GlobalSharedContext*>(this);
 }
 
@@ -245,12 +249,13 @@ class FunctionBox : public ObjectBox, public SharedContext
     bool            inWith:1;               /* some enclosing scope is a with-statement */
     bool            inGenexpLambda:1;       /* lambda from generator expression */
     bool            hasDestructuringArgs:1; /* arguments list contains destructuring expression */
-    bool            useAsm:1;               /* function contains "use asm" directive */
-    bool            insideUseAsm:1;         /* nested function of function of "use asm" directive */
+    bool            useAsm:1;               /* see useAsmOrInsideUseAsm */
+    bool            insideUseAsm:1;         /* see useAsmOrInsideUseAsm */
 
     // Fields for use in heuristics.
     bool            usesArguments:1;  /* contains a free use of 'arguments' */
     bool            usesApply:1;      /* contains an f.apply() call */
+    bool            usesThis:1;       /* contains 'this' */
 
     FunctionContextFlags funCxFlags;
 
@@ -271,7 +276,7 @@ class FunctionBox : public ObjectBox, public SharedContext
         // A generator kind can be set at initialization, or when "yield" is
         // first seen.  In both cases the transition can only happen from
         // NotGenerator.
-        JS_ASSERT(!isGenerator());
+        MOZ_ASSERT(!isGenerator());
         generatorKindBits_ = GeneratorKindAsBits(kind);
     }
 
@@ -285,15 +290,18 @@ class FunctionBox : public ObjectBox, public SharedContext
     void setHasExtensibleScope()           { funCxFlags.hasExtensibleScope       = true; }
     void setNeedsDeclEnvObject()           { funCxFlags.needsDeclEnvObject       = true; }
     void setArgumentsHasLocalBinding()     { funCxFlags.argumentsHasLocalBinding = true; }
-    void setDefinitelyNeedsArgsObj()       { JS_ASSERT(funCxFlags.argumentsHasLocalBinding);
+    void setDefinitelyNeedsArgsObj()       { MOZ_ASSERT(funCxFlags.argumentsHasLocalBinding);
                                              funCxFlags.definitelyNeedsArgsObj   = true; }
 
     bool hasDefaults() const {
         return length != function()->nargs() - function()->hasRest();
     }
 
-    // Return whether this function has either specified "use asm" or is
-    // (transitively) nested inside a function that has.
+    // Return whether this or an enclosing function is being parsed and
+    // validated as asm.js. Note: if asm.js validation fails, this will be false
+    // while the function is being reparsed. This flag can be used to disable
+    // certain parsing features that are necessary in general, but unnecessary
+    // for validated asm.js.
     bool useAsmOrInsideUseAsm() const {
         return useAsm || insideUseAsm;
     }
@@ -317,7 +325,7 @@ class FunctionBox : public ObjectBox, public SharedContext
 inline FunctionBox *
 SharedContext::asFunctionBox()
 {
-    JS_ASSERT(isFunctionBox());
+    MOZ_ASSERT(isFunctionBox());
     return static_cast<FunctionBox*>(this);
 }
 
@@ -424,9 +432,12 @@ struct StmtInfoBase {
         return isNestedScope;
     }
 
+    void setStaticScope() {
+    }
+
     StaticBlockObject& staticBlock() const {
-        JS_ASSERT(isNestedScope);
-        JS_ASSERT(isBlockScope);
+        MOZ_ASSERT(isNestedScope);
+        MOZ_ASSERT(isBlockScope);
         return staticScope->as<StaticBlockObject>();
     }
 
@@ -483,7 +494,7 @@ FinishPopStatement(ContextT *ct)
     if (stmt->linksScope()) {
         ct->topScopeStmt = stmt->downScope;
         if (stmt->isNestedScope) {
-            JS_ASSERT(stmt->staticScope);
+            MOZ_ASSERT(stmt->staticScope);
             ct->staticScope = stmt->staticScope->enclosingNestedScope();
         }
     }
