@@ -18,7 +18,7 @@ namespace js {
 void
 ValueReadBarrier(const Value &value)
 {
-    JS_ASSERT(!CurrentThreadIsIonCompiling());
+    MOZ_ASSERT(!CurrentThreadIsIonCompiling());
     if (value.isObject())
         JSObject::readBarrier(&value.toObject());
     else if (value.isString())
@@ -26,12 +26,12 @@ ValueReadBarrier(const Value &value)
     else if (value.isSymbol())
         JS::Symbol::readBarrier(value.toSymbol());
     else
-        JS_ASSERT(!value.isMarkable());
+        MOZ_ASSERT(!value.isMarkable());
 }
 
 #ifdef DEBUG
 bool
-HeapSlot::preconditionForSet(JSObject *owner, Kind kind, uint32_t slot)
+HeapSlot::preconditionForSet(NativeObject *owner, Kind kind, uint32_t slot)
 {
     return kind == Slot
          ? &owner->getSlotRef(slot) == this
@@ -39,7 +39,7 @@ HeapSlot::preconditionForSet(JSObject *owner, Kind kind, uint32_t slot)
 }
 
 bool
-HeapSlot::preconditionForSet(Zone *zone, JSObject *owner, Kind kind, uint32_t slot)
+HeapSlot::preconditionForSet(Zone *zone, NativeObject *owner, Kind kind, uint32_t slot)
 {
     bool ok = kind == Slot
             ? &owner->getSlotRef(slot) == this
@@ -48,7 +48,7 @@ HeapSlot::preconditionForSet(Zone *zone, JSObject *owner, Kind kind, uint32_t sl
 }
 
 bool
-HeapSlot::preconditionForWriteBarrierPost(JSObject *obj, Kind kind, uint32_t slot, Value target) const
+HeapSlot::preconditionForWriteBarrierPost(NativeObject *obj, Kind kind, uint32_t slot, Value target) const
 {
     return kind == Slot
          ? obj->getSlotAddressUnchecked(slot)->get() == target
