@@ -18,6 +18,7 @@
 
 #include "gc/GCInternals.h"
 #include "gc/Marking.h"
+#include "gc/Zone.h"
 
 #include "vm/Symbol.h"
 
@@ -28,67 +29,67 @@ using namespace js::gc;
 using mozilla::DebugOnly;
 
 JS_PUBLIC_API(void)
-JS_CallValueTracer(JSTracer *trc, Value *valuep, const char *name)
+JS_CallUnbarrieredValueTracer(JSTracer *trc, Value *valuep, const char *name)
 {
     MarkValueUnbarriered(trc, valuep, name);
 }
 
 JS_PUBLIC_API(void)
-JS_CallIdTracer(JSTracer *trc, jsid *idp, const char *name)
+JS_CallUnbarrieredIdTracer(JSTracer *trc, jsid *idp, const char *name)
 {
     MarkIdUnbarriered(trc, idp, name);
 }
 
 JS_PUBLIC_API(void)
-JS_CallObjectTracer(JSTracer *trc, JSObject **objp, const char *name)
+JS_CallUnbarrieredObjectTracer(JSTracer *trc, JSObject **objp, const char *name)
 {
     MarkObjectUnbarriered(trc, objp, name);
 }
 
 JS_PUBLIC_API(void)
-JS_CallStringTracer(JSTracer *trc, JSString **strp, const char *name)
+JS_CallUnbarrieredStringTracer(JSTracer *trc, JSString **strp, const char *name)
 {
     MarkStringUnbarriered(trc, strp, name);
 }
 
 JS_PUBLIC_API(void)
-JS_CallScriptTracer(JSTracer *trc, JSScript **scriptp, const char *name)
+JS_CallUnbarrieredScriptTracer(JSTracer *trc, JSScript **scriptp, const char *name)
 {
     MarkScriptUnbarriered(trc, scriptp, name);
 }
 
 JS_PUBLIC_API(void)
-JS_CallHeapValueTracer(JSTracer *trc, JS::Heap<JS::Value> *valuep, const char *name)
+JS_CallValueTracer(JSTracer *trc, JS::Heap<JS::Value> *valuep, const char *name)
 {
     MarkValueUnbarriered(trc, valuep->unsafeGet(), name);
 }
 
 JS_PUBLIC_API(void)
-JS_CallHeapIdTracer(JSTracer *trc, JS::Heap<jsid> *idp, const char *name)
+JS_CallIdTracer(JSTracer *trc, JS::Heap<jsid> *idp, const char *name)
 {
     MarkIdUnbarriered(trc, idp->unsafeGet(), name);
 }
 
 JS_PUBLIC_API(void)
-JS_CallHeapObjectTracer(JSTracer *trc, JS::Heap<JSObject *> *objp, const char *name)
+JS_CallObjectTracer(JSTracer *trc, JS::Heap<JSObject *> *objp, const char *name)
 {
     MarkObjectUnbarriered(trc, objp->unsafeGet(), name);
 }
 
 JS_PUBLIC_API(void)
-JS_CallHeapStringTracer(JSTracer *trc, JS::Heap<JSString *> *strp, const char *name)
+JS_CallStringTracer(JSTracer *trc, JS::Heap<JSString *> *strp, const char *name)
 {
     MarkStringUnbarriered(trc, strp->unsafeGet(), name);
 }
 
 JS_PUBLIC_API(void)
-JS_CallHeapScriptTracer(JSTracer *trc, JS::Heap<JSScript *> *scriptp, const char *name)
+JS_CallScriptTracer(JSTracer *trc, JS::Heap<JSScript *> *scriptp, const char *name)
 {
     MarkScriptUnbarriered(trc, scriptp->unsafeGet(), name);
 }
 
 JS_PUBLIC_API(void)
-JS_CallHeapFunctionTracer(JSTracer *trc, JS::Heap<JSFunction *> *funp, const char *name)
+JS_CallFunctionTracer(JSTracer *trc, JS::Heap<JSFunction *> *funp, const char *name)
 {
     MarkObjectUnbarriered(trc, funp->unsafeGet(), name);
 }
@@ -119,6 +120,60 @@ JS_TraceRuntime(JSTracer *trc)
     TraceRuntime(trc);
 }
 
+JS_PUBLIC_API(void)
+JS_TraceIncomingCCWs(JSTracer *trc, const JS::ZoneSet &zones)
+{
+    for (js::ZonesIter z(trc->runtime(), SkipAtoms); !z.done(); z.next()) {
+        Zone *zone = z.get();
+        if (!zone || zones.has(zone))
+            continue;
+
+        for (js::CompartmentsInZoneIter c(zone); !c.done(); c.next()) {
+            JSCompartment *comp = c.get();
+            if (!comp)
+                continue;
+
+            for (JSCompartment::WrapperEnum e(comp); !e.empty(); e.popFront()) {
+                const CrossCompartmentKey &key = e.front().key();
+                JSObject *obj;
+                JSScript *script;
+
+                switch (key.kind) {
+                  case CrossCompartmentKey::StringWrapper:
+                    // StringWrappers are just used to avoid copying strings
+                    // across zones multiple times, and don't hold a strong
+                    // reference.
+                    continue;
+
+                  case CrossCompartmentKey::ObjectWrapper:
+                  case CrossCompartmentKey::DebuggerObject:
+                  case CrossCompartmentKey::DebuggerSource:
+                  case CrossCompartmentKey::DebuggerEnvironment:
+                    obj = static_cast<JSObject *>(key.wrapped);
+                    // Ignore CCWs whose wrapped value doesn't live in our given
+                    // set of zones.
+                    if (!zones.has(obj->zone()))
+                        continue;
+
+                    MarkObjectUnbarriered(trc, &obj, "cross-compartment wrapper");
+                    MOZ_ASSERT(obj == key.wrapped);
+                    break;
+
+                  case CrossCompartmentKey::DebuggerScript:
+                    script = static_cast<JSScript *>(key.wrapped);
+                    // Ignore CCWs whose wrapped value doesn't live in our given
+                    // set of zones.
+                    if (!zones.has(script->zone()))
+                        continue;
+                    MarkScriptUnbarriered(trc, &script, "cross-compartment wrapper");
+                    MOZ_ASSERT(script == key.wrapped);
+                    break;
+                }
+            }
+        }
+    }
+}
+
 static size_t
 CountDecimalDigits(size_t num)
 {
@@ -148,6 +203,10 @@ JS_GetTraceThingInfo(char *buf, size_t bufsize, JSTracer *trc, void *thing,
         break;
       }
 
+      case JSTRACE_SCRIPT:
+        name = "script";
+        break;
+
       case JSTRACE_STRING:
         name = ((JSString *)thing)->isDependent()
                ? "substring"
@@ -158,29 +217,29 @@ JS_GetTraceThingInfo(char *buf, size_t bufsize, JSTracer *trc, void *thing,
         name = "symbol";
         break;
 
-      case JSTRACE_SCRIPT:
-        name = "script";
-        break;
-
-      case JSTRACE_LAZY_SCRIPT:
-        name = "lazyscript";
+      case JSTRACE_BASE_SHAPE:
+        name = "base_shape";
         break;
 
       case JSTRACE_JITCODE:
         name = "jitcode";
         break;
 
+      case JSTRACE_LAZY_SCRIPT:
+        name = "lazyscript";
+        break;
+
       case JSTRACE_SHAPE:
         name = "shape";
         break;
 
-      case JSTRACE_BASE_SHAPE:
-        name = "base_shape";
-        break;
-
       case JSTRACE_TYPE_OBJECT:
         name = "type_object";
         break;
+
+      default:
+        name = "INVALID";
+        break;
     }
 
     n = strlen(name);
@@ -204,13 +263,20 @@ JS_GetTraceThingInfo(char *buf, size_t bufsize, JSTracer *trc, void *thing,
                     PutEscapedString(buf, bufsize, fun->displayAtom(), 0);
                 }
             } else if (obj->getClass()->flags & JSCLASS_HAS_PRIVATE) {
-                JS_snprintf(buf, bufsize, " %p", obj->getPrivate());
+                JS_snprintf(buf, bufsize, " %p", obj->as<NativeObject>().getPrivate());
             } else {
                 JS_snprintf(buf, bufsize, " <no private>");
             }
             break;
           }
 
+          case JSTRACE_SCRIPT:
+          {
+            JSScript *script = static_cast<JSScript *>(thing);
+            JS_snprintf(buf, bufsize, " %s:%u", script->filename(), unsigned(script->lineno()));
+            break;
+          }
+
           case JSTRACE_STRING:
           {
             *buf++ = ' ';
@@ -251,18 +317,7 @@ JS_GetTraceThingInfo(char *buf, size_t bufsize, JSTracer *trc, void *thing,
             break;
           }
 
-          case JSTRACE_SCRIPT:
-          {
-            JSScript *script = static_cast<JSScript *>(thing);
-            JS_snprintf(buf, bufsize, " %s:%u", script->filename(), unsigned(script->lineno()));
-            break;
-          }
-
-          case JSTRACE_LAZY_SCRIPT:
-          case JSTRACE_JITCODE:
-          case JSTRACE_SHAPE:
-          case JSTRACE_BASE_SHAPE:
-          case JSTRACE_TYPE_OBJECT:
+          default:
             break;
         }
     }
@@ -292,7 +347,7 @@ JSTracer::hasTracingDetails() const
 const char *
 JSTracer::tracingName(const char *fallback) const
 {
-    JS_ASSERT(hasTracingDetails());
+    MOZ_ASSERT(hasTracingDetails());
     return debugPrinter_ ? fallback : (const char *)debugPrintArg_;
 }
 
@@ -362,7 +417,7 @@ MarkStack::init(JSGCMode gcMode)
 {
     setBaseCapacity(gcMode);
 
-    JS_ASSERT(!stack_);
+    MOZ_ASSERT(!stack_);
     uintptr_t *newStack = js_pod_malloc<uintptr_t>(baseCapacity_);
     if (!newStack)
         return false;
@@ -393,7 +448,7 @@ MarkStack::setBaseCapacity(JSGCMode mode)
 void
 MarkStack::setMaxCapacity(size_t maxCapacity)
 {
-    JS_ASSERT(isEmpty());
+    MOZ_ASSERT(isEmpty());
     maxCapacity_ = maxCapacity;
     if (baseCapacity_ > maxCapacity_)
         baseCapacity_ = maxCapacity_;
@@ -476,25 +531,25 @@ GCMarker::init(JSGCMode gcMode)
 void
 GCMarker::start()
 {
-    JS_ASSERT(!started);
+    MOZ_ASSERT(!started);
     started = true;
     color = BLACK;
 
-    JS_ASSERT(!unmarkedArenaStackTop);
-    JS_ASSERT(markLaterArenas == 0);
+    MOZ_ASSERT(!unmarkedArenaStackTop);
+    MOZ_ASSERT(markLaterArenas == 0);
 
 }
 
 void
 GCMarker::stop()
 {
-    JS_ASSERT(isDrained());
+    MOZ_ASSERT(isDrained());
 
-    JS_ASSERT(started);
+    MOZ_ASSERT(started);
     started = false;
 
-    JS_ASSERT(!unmarkedArenaStackTop);
-    JS_ASSERT(markLaterArenas == 0);
+    MOZ_ASSERT(!unmarkedArenaStackTop);
+    MOZ_ASSERT(markLaterArenas == 0);
 
     /* Free non-ballast stack memory. */
     stack.reset();
@@ -509,20 +564,20 @@ GCMarker::reset()
     color = BLACK;
 
     stack.reset();
-    JS_ASSERT(isMarkStackEmpty());
+    MOZ_ASSERT(isMarkStackEmpty());
 
     while (unmarkedArenaStackTop) {
         ArenaHeader *aheader = unmarkedArenaStackTop;
-        JS_ASSERT(aheader->hasDelayedMarking);
-        JS_ASSERT(markLaterArenas);
+        MOZ_ASSERT(aheader->hasDelayedMarking);
+        MOZ_ASSERT(markLaterArenas);
         unmarkedArenaStackTop = aheader->getNextDelayedMarking();
         aheader->unsetDelayedMarking();
         aheader->markOverflow = 0;
         aheader->allocatedDuringIncremental = 0;
         markLaterArenas--;
     }
-    JS_ASSERT(isDrained());
-    JS_ASSERT(!markLaterArenas);
+    MOZ_ASSERT(isDrained());
+    MOZ_ASSERT(!markLaterArenas);
 }
 
 void
@@ -533,14 +588,14 @@ GCMarker::markDelayedChildren(ArenaHeader *aheader)
         aheader->markOverflow = 0;
 
         for (ArenaCellIterUnderGC i(aheader); !i.done(); i.next()) {
-            Cell *t = i.getCell();
+            TenuredCell *t = i.getCell();
             if (always || t->isMarked()) {
                 t->markIfUnmarked();
                 JS_TraceChildren(this, t, MapAllocToTraceKind(aheader->getAllocKind()));
             }
         }
     } else {
-        JS_ASSERT(aheader->allocatedDuringIncremental);
+        MOZ_ASSERT(aheader->allocatedDuringIncremental);
         PushArena(this, aheader);
     }
     aheader->allocatedDuringIncremental = 0;
@@ -554,11 +609,10 @@ GCMarker::markDelayedChildren(ArenaHeader *aheader)
 bool
 GCMarker::markDelayedChildren(SliceBudget &budget)
 {
-    gcstats::MaybeAutoPhase ap;
-    if (runtime()->gc.state() == MARK)
-        ap.construct(runtime()->gc.stats, gcstats::PHASE_MARK_DELAYED);
+    GCRuntime &gc = runtime()->gc;
+    gcstats::AutoPhase ap(gc.stats, gc.state() == MARK, gcstats::PHASE_MARK_DELAYED);
 
-    JS_ASSERT(unmarkedArenaStackTop);
+    MOZ_ASSERT(unmarkedArenaStackTop);
     do {
         /*
          * If marking gets delayed at the same arena again, we must repeat
@@ -566,8 +620,8 @@ GCMarker::markDelayedChildren(SliceBudget &budget)
          * clear its hasDelayedMarking flag before we begin the marking.
          */
         ArenaHeader *aheader = unmarkedArenaStackTop;
-        JS_ASSERT(aheader->hasDelayedMarking);
-        JS_ASSERT(markLaterArenas);
+        MOZ_ASSERT(aheader->hasDelayedMarking);
+        MOZ_ASSERT(markLaterArenas);
         unmarkedArenaStackTop = aheader->getNextDelayedMarking();
         aheader->unsetDelayedMarking();
         markLaterArenas--;
@@ -577,7 +631,7 @@ GCMarker::markDelayedChildren(SliceBudget &budget)
         if (budget.isOverBudget())
             return false;
     } while (unmarkedArenaStackTop);
-    JS_ASSERT(!markLaterArenas);
+    MOZ_ASSERT(!markLaterArenas);
 
     return true;
 }
@@ -586,9 +640,9 @@ GCMarker::markDelayedChildren(SliceBudget &budget)
 void
 GCMarker::checkZone(void *p)
 {
-    JS_ASSERT(started);
+    MOZ_ASSERT(started);
     DebugOnly<Cell *> cell = static_cast<Cell *>(p);
-    JS_ASSERT_IF(cell->isTenured(), cell->tenuredZone()->isCollecting());
+    MOZ_ASSERT_IF(cell->isTenured(), cell->asTenured().zone()->isCollecting());
 }
 #endif
 
@@ -601,24 +655,24 @@ GCMarker::hasBufferedGrayRoots() const
 void
 GCMarker::startBufferingGrayRoots()
 {
-    JS_ASSERT(grayBufferState == GRAY_BUFFER_UNUSED);
+    MOZ_ASSERT(grayBufferState == GRAY_BUFFER_UNUSED);
     grayBufferState = GRAY_BUFFER_OK;
     for (GCZonesIter zone(runtime()); !zone.done(); zone.next())
-        JS_ASSERT(zone->gcGrayRoots.empty());
+        MOZ_ASSERT(zone->gcGrayRoots.empty());
 
-    JS_ASSERT(!callback);
+    MOZ_ASSERT(!callback);
     callback = GrayCallback;
-    JS_ASSERT(IS_GC_MARKING_TRACER(this));
+    MOZ_ASSERT(IS_GC_MARKING_TRACER(this));
 }
 
 void
 GCMarker::endBufferingGrayRoots()
 {
-    JS_ASSERT(callback == GrayCallback);
+    MOZ_ASSERT(callback == GrayCallback);
     callback = nullptr;
-    JS_ASSERT(IS_GC_MARKING_TRACER(this));
-    JS_ASSERT(grayBufferState == GRAY_BUFFER_OK ||
-              grayBufferState == GRAY_BUFFER_FAILED);
+    MOZ_ASSERT(IS_GC_MARKING_TRACER(this));
+    MOZ_ASSERT(grayBufferState == GRAY_BUFFER_OK ||
+               grayBufferState == GRAY_BUFFER_FAILED);
 }
 
 void
@@ -631,24 +685,21 @@ GCMarker::resetBufferedGrayRoots()
 void
 GCMarker::markBufferedGrayRoots(JS::Zone *zone)
 {
-    JS_ASSERT(grayBufferState == GRAY_BUFFER_OK);
-    JS_ASSERT(zone->isGCMarkingGray());
+    MOZ_ASSERT(grayBufferState == GRAY_BUFFER_OK);
+    MOZ_ASSERT(zone->isGCMarkingGray() || zone->isGCCompacting());
 
     for (GrayRoot *elem = zone->gcGrayRoots.begin(); elem != zone->gcGrayRoots.end(); elem++) {
 #ifdef DEBUG
         setTracingDetails(elem->debugPrinter, elem->debugPrintArg, elem->debugPrintIndex);
 #endif
-        void *tmp = elem->thing;
-        setTracingLocation((void *)&elem->thing);
-        MarkKind(this, &tmp, elem->kind);
-        JS_ASSERT(tmp == elem->thing);
+        MarkKind(this, &elem->thing, elem->kind);
     }
 }
 
 void
 GCMarker::appendGrayRoot(void *thing, JSGCTraceKind kind)
 {
-    JS_ASSERT(started);
+    MOZ_ASSERT(started);
 
     if (grayBufferState == GRAY_BUFFER_FAILED)
         return;
@@ -660,9 +711,22 @@ GCMarker::appendGrayRoot(void *thing, JSGCTraceKind kind)
     root.debugPrintIndex = debugPrintIndex();
 #endif
 
-    Zone *zone = static_cast<Cell *>(thing)->tenuredZone();
+    Zone *zone = TenuredCell::fromPointer(thing)->zone();
     if (zone->isCollecting()) {
-        zone->maybeAlive = true;
+        // See the comment on SetMaybeAliveFlag to see why we only do this for
+        // objects and scripts. We rely on gray root buffering for this to work,
+        // but we only need to worry about uncollected dead compartments during
+        // incremental GCs (when we do gray root buffering).
+        switch (kind) {
+          case JSTRACE_OBJECT:
+            static_cast<JSObject *>(thing)->compartment()->maybeAlive = true;
+            break;
+          case JSTRACE_SCRIPT:
+            static_cast<JSScript *>(thing)->compartment()->maybeAlive = true;
+            break;
+          default:
+            break;
+        }
         if (!zone->gcGrayRoots.append(root)) {
             resetBufferedGrayRoots();
             grayBufferState = GRAY_BUFFER_FAILED;
@@ -673,8 +737,8 @@ GCMarker::appendGrayRoot(void *thing, JSGCTraceKind kind)
 void
 GCMarker::GrayCallback(JSTracer *trc, void **thingp, JSGCTraceKind kind)
 {
-    JS_ASSERT(thingp);
-    JS_ASSERT(*thingp);
+    MOZ_ASSERT(thingp);
+    MOZ_ASSERT(*thingp);
     GCMarker *gcmarker = static_cast<GCMarker *>(trc);
     gcmarker->appendGrayRoot(*thingp, kind);
 }
