@@ -9,8 +9,8 @@
 
 #include "jstypes.h"
 
-#include "jit/IonFrameIterator.h"
-#include "jit/IonFrames.h"
+#include "jit/JitFrameIterator.h"
+#include "jit/JitFrames.h"
 #include "vm/Stack.h"
 
 namespace js {
@@ -86,14 +86,11 @@ namespace jit {
 // pointers. To account for this we segregate frames into a limited set of
 // "frame sizes", and create a table for each frame size. We also have the
 // option of not using bailout tables, for platforms or situations where the
-// 10 byte cost is more optimal than a bailout table. See IonFrames.h for more
+// 10 byte cost is more optimal than a bailout table. See JitFrames.h for more
 // detail.
 
 static const BailoutId INVALID_BAILOUT_ID = BailoutId(-1);
 
-static const uint32_t BAILOUT_KIND_BITS = 3;
-static const uint32_t BAILOUT_RESUME_BITS = 1;
-
 // Keep this arbitrarily small for now, for testing.
 static const uint32_t BAILOUT_TABLE_SIZE = 16;
 
@@ -103,6 +100,10 @@ static const uint32_t BAILOUT_RETURN_OK = 0;
 static const uint32_t BAILOUT_RETURN_FATAL_ERROR = 1;
 static const uint32_t BAILOUT_RETURN_OVERRECURSED = 2;
 
+// This address is a magic number made to cause crashes while indicating that we
+// are making an attempt to mark the stack during a bailout.
+static uint8_t * const FAKE_JIT_TOP_FOR_BAILOUT = reinterpret_cast<uint8_t *>(0xba1);
+
 class JitCompartment;
 
 // BailoutStack is an architecture specific pointer to the stack, given by the
@@ -112,39 +113,44 @@ class InvalidationBailoutStack;
 
 // Must be implemented by each architecture.
 
-// This iterator is constructed at a time where there is no exit frame at the
-// moment. They must be initialized to the first JS frame instead of the exit
-// frame as usually done with IonFrameIterator.
-class IonBailoutIterator : public IonFrameIterator
+// This structure is constructed before recovering the baseline frames for a
+// bailout. It records all information extracted from the stack, and which are
+// needed for the JitFrameIterator.
+class BailoutFrameInfo
 {
     MachineState machine_;
-    uint32_t snapshotOffset_;
+    uint8_t *framePointer_;
     size_t topFrameSize_;
     IonScript *topIonScript_;
+    uint32_t snapshotOffset_;
+    JitActivation *activation_;
+
+    void attachOnJitActivation(const JitActivationIterator &activations);
 
   public:
-    IonBailoutIterator(const JitActivationIterator &activations, BailoutStack *sp);
-    IonBailoutIterator(const JitActivationIterator &activations, InvalidationBailoutStack *sp);
-    IonBailoutIterator(const JitActivationIterator &activations, const IonFrameIterator &frame);
+    BailoutFrameInfo(const JitActivationIterator &activations, BailoutStack *sp);
+    BailoutFrameInfo(const JitActivationIterator &activations, InvalidationBailoutStack *sp);
+    BailoutFrameInfo(const JitActivationIterator &activations, const JitFrameIterator &frame);
+    ~BailoutFrameInfo();
 
+    uint8_t *fp() const {
+        return framePointer_;
+    }
     SnapshotOffset snapshotOffset() const {
-        JS_ASSERT(topIonScript_);
         return snapshotOffset_;
     }
-    const MachineState &machineState() const {
+    const MachineState machineState() const {
         return machine_;
     }
     size_t topFrameSize() const {
-        JS_ASSERT(topIonScript_);
         return topFrameSize_;
     }
     IonScript *ionScript() const {
-        if (topIonScript_)
-            return topIonScript_;
-        return IonFrameIterator::ionScript();
+        return topIonScript_;
+    }
+    JitActivation *activation() const {
+        return activation_;
     }
-
-    void dump() const;
 };
 
 bool EnsureHasScopeObjects(JSContext *cx, AbstractFramePtr fp);
@@ -158,18 +164,52 @@ uint32_t Bailout(BailoutStack *sp, BaselineBailoutInfo **info);
 uint32_t InvalidationBailout(InvalidationBailoutStack *sp, size_t *frameSizeOut,
                              BaselineBailoutInfo **info);
 
-struct ExceptionBailoutInfo
+class ExceptionBailoutInfo
 {
-    size_t frameNo;
-    jsbytecode *resumePC;
-    size_t numExprSlots;
+    size_t frameNo_;
+    jsbytecode *resumePC_;
+    size_t numExprSlots_;
+
+  public:
+    ExceptionBailoutInfo(size_t frameNo, jsbytecode *resumePC, size_t numExprSlots)
+      : frameNo_(frameNo),
+        resumePC_(resumePC),
+        numExprSlots_(numExprSlots)
+    { }
+
+    ExceptionBailoutInfo()
+      : frameNo_(0),
+        resumePC_(nullptr),
+        numExprSlots_(0)
+    { }
+
+    bool catchingException() const {
+        return !!resumePC_;
+    }
+    bool propagatingIonExceptionForDebugMode() const {
+        return !resumePC_;
+    }
+
+    size_t frameNo() const {
+        MOZ_ASSERT(catchingException());
+        return frameNo_;
+    }
+    jsbytecode *resumePC() const {
+        MOZ_ASSERT(catchingException());
+        return resumePC_;
+    }
+    size_t numExprSlots() const {
+        MOZ_ASSERT(catchingException());
+        return numExprSlots_;
+    }
 };
 
 // Called from the exception handler to enter a catch or finally block.
 // Returns a BAILOUT_* error code.
 uint32_t ExceptionHandlerBailout(JSContext *cx, const InlineFrameIterator &frame,
+                                 ResumeFromException *rfe,
                                  const ExceptionBailoutInfo &excInfo,
-                                 BaselineBailoutInfo **bailoutInfo);
+                                 bool *overrecursed, bool *poppedLastSPSFrameOut);
 
 uint32_t FinishBailoutToBaseline(BaselineBailoutInfo *bailoutInfo);
 
