@@ -7,8 +7,6 @@
 #ifndef jit_BaselineFrame_inl_h
 #define jit_BaselineFrame_inl_h
 
-#ifdef JS_ION
-
 #include "jit/BaselineFrame.h"
 
 #include "jscntxt.h"
@@ -16,14 +14,16 @@
 
 #include "vm/ScopeObject.h"
 
+#include "jsscriptinlines.h"
+
 namespace js {
 namespace jit {
 
 inline void
 BaselineFrame::pushOnScopeChain(ScopeObject &scope)
 {
-    JS_ASSERT(*scopeChain() == scope.enclosingScope() ||
-              *scopeChain() == scope.as<CallObject>().enclosingScope().as<DeclEnvObject>().enclosingScope());
+    MOZ_ASSERT(*scopeChain() == scope.enclosingScope() ||
+               *scopeChain() == scope.as<CallObject>().enclosingScope().as<DeclEnvObject>().enclosingScope());
     scopeChain_ = &scope;
 }
 
@@ -33,44 +33,42 @@ BaselineFrame::popOffScopeChain()
     scopeChain_ = &scopeChain_->as<ScopeObject>().enclosingScope();
 }
 
+inline void
+BaselineFrame::popWith(JSContext *cx)
+{
+    if (MOZ_UNLIKELY(isDebuggee()))
+        DebugScopes::onPopWith(this);
+
+    MOZ_ASSERT(scopeChain()->is<DynamicWithObject>());
+    popOffScopeChain();
+}
+
 inline bool
 BaselineFrame::pushBlock(JSContext *cx, Handle<StaticBlockObject *> block)
 {
-    JS_ASSERT_IF(hasBlockChain(), blockChain() == *block->enclosingBlock());
+    MOZ_ASSERT(block->needsClone());
 
-    if (block->needsClone()) {
-        ClonedBlockObject *clone = ClonedBlockObject::create(cx, block, this);
-        if (!clone)
-            return false;
+    ClonedBlockObject *clone = ClonedBlockObject::create(cx, block, this);
+    if (!clone)
+        return false;
+    pushOnScopeChain(*clone);
 
-        pushOnScopeChain(*clone);
-    }
-
-    setBlockChain(*block);
     return true;
 }
 
 inline void
 BaselineFrame::popBlock(JSContext *cx)
 {
-    JS_ASSERT(hasBlockChain());
+    MOZ_ASSERT(scopeChain_->is<ClonedBlockObject>());
 
-    if (cx->compartment()->debugMode())
-        DebugScopes::onPopBlock(cx, this);
-
-    if (blockChain_->needsClone()) {
-        JS_ASSERT(scopeChain_->as<ClonedBlockObject>().staticBlock() == *blockChain_);
-        popOffScopeChain();
-    }
-
-    setBlockChain(*blockChain_->enclosingBlock());
+    popOffScopeChain();
 }
 
 inline CallObject &
 BaselineFrame::callObj() const
 {
-    JS_ASSERT(hasCallObj());
-    JS_ASSERT(fun()->isHeavyweight());
+    MOZ_ASSERT(hasCallObj());
+    MOZ_ASSERT(fun()->isHeavyweight());
 
     JSObject *obj = scopeChain();
     while (!obj->is<CallObject>())
@@ -78,9 +76,14 @@ BaselineFrame::callObj() const
     return obj->as<CallObject>();
 }
 
+inline void
+BaselineFrame::unsetIsDebuggee()
+{
+    MOZ_ASSERT(!script()->isDebuggee());
+    flags_ &= ~DEBUGGEE;
+}
+
 } // namespace jit
 } // namespace js
 
-#endif // JS_ION
-
 #endif /* jit_BaselineFrame_inl_h */
