@@ -9,96 +9,100 @@
 
 #include "mozilla/MathAlgorithms.h"
 
-#include "jit/IonAllocPolicy.h"
+#include "jit/JitAllocPolicy.h"
 
 namespace js {
 namespace jit {
 
 // Provides constant time set insertion and removal, and fast linear
 // set operations such as intersection, difference, and union.
-// N.B. All set operations must be performed on sets with the same maximum.
-class BitSet : private TempObject
+// N.B. All set operations must be performed on sets with the same number
+// of bits.
+class BitSet
 {
   public:
+    static const size_t BitsPerWord = 8 * sizeof(uint32_t);
+
     static size_t RawLengthForBits(size_t bits) {
-        return 1 + bits / (8 * sizeof(uint32_t));
+        return (bits + BitsPerWord - 1) / BitsPerWord;
     }
 
   private:
-    BitSet(unsigned int max) :
-        max_(max),
-        bits_(nullptr) {}
-
-    unsigned int max_;
     uint32_t *bits_;
+    const unsigned int numBits_;
 
     static inline uint32_t bitForValue(unsigned int value) {
-        return 1l << (uint32_t)(value % (8 * sizeof(uint32_t)));
+        return 1l << uint32_t(value % BitsPerWord);
     }
 
     static inline unsigned int wordForValue(unsigned int value) {
-        return value / (8 * sizeof(uint32_t));
+        return value / BitsPerWord;
     }
 
     inline unsigned int numWords() const {
-        return RawLengthForBits(max_);
+        return RawLengthForBits(numBits_);
     }
 
-    bool init(TempAllocator &alloc);
+    BitSet(const BitSet &) = delete;
+    void operator=(const BitSet &) = delete;
 
   public:
     class Iterator;
 
-    static BitSet *New(TempAllocator &alloc, unsigned int max);
+    explicit BitSet(unsigned int numBits) :
+        bits_(nullptr),
+        numBits_(numBits) {}
 
-    unsigned int getMax() const {
-        return max_;
+    bool init(TempAllocator &alloc);
+
+    unsigned int getNumBits() const {
+        return numBits_;
     }
 
     // O(1): Check if this set contains the given value.
     bool contains(unsigned int value) const {
-        JS_ASSERT(bits_);
-        JS_ASSERT(value <= max_);
+        MOZ_ASSERT(bits_);
+        MOZ_ASSERT(value < numBits_);
 
         return !!(bits_[wordForValue(value)] & bitForValue(value));
     }
 
-    // O(max): Check if this set contains any value.
+    // O(numBits): Check if this set contains any value.
     bool empty() const;
 
     // O(1): Insert the given value into this set.
     void insert(unsigned int value) {
-        JS_ASSERT(bits_);
-        JS_ASSERT(value <= max_);
+        MOZ_ASSERT(bits_);
+        MOZ_ASSERT(value < numBits_);
 
         bits_[wordForValue(value)] |= bitForValue(value);
     }
 
-    // O(max): Insert every element of the given set into this set.
-    void insertAll(const BitSet *other);
+    // O(numBits): Insert every element of the given set into this set.
+    void insertAll(const BitSet &other);
 
     // O(1): Remove the given value from this set.
     void remove(unsigned int value) {
-        JS_ASSERT(bits_);
-        JS_ASSERT(value <= max_);
+        MOZ_ASSERT(bits_);
+        MOZ_ASSERT(value < numBits_);
 
         bits_[wordForValue(value)] &= ~bitForValue(value);
     }
 
-    // O(max): Remove the every element of the given set from this set.
-    void removeAll(const BitSet *other);
+    // O(numBits): Remove the every element of the given set from this set.
+    void removeAll(const BitSet &other);
 
-    // O(max): Intersect this set with the given set.
-    void intersect(const BitSet *other);
+    // O(numBits): Intersect this set with the given set.
+    void intersect(const BitSet &other);
 
-    // O(max): Intersect this set with the given set; return whether the
+    // O(numBits): Intersect this set with the given set; return whether the
     // intersection caused the set to change.
-    bool fixedPointIntersect(const BitSet *other);
+    bool fixedPointIntersect(const BitSet &other);
 
-    // O(max): Does inplace complement of the set.
+    // O(numBits): Does inplace complement of the set.
     void complement();
 
-    // O(max): Clear this set.
+    // O(numBits): Clear this set.
     void clear();
 
     uint32_t *raw() const {
@@ -117,15 +121,36 @@ class BitSet::Iterator
     unsigned word_;
     uint32_t value_;
 
+    void skipEmpty() {
+        // Skip words containing only zeros.
+        unsigned numWords = set_.numWords();
+        const uint32_t *bits = set_.bits_;
+        while (value_ == 0) {
+            word_++;
+            if (word_ == numWords)
+                return;
+
+            index_ = word_ * BitSet::BitsPerWord;
+            value_ = bits[word_];
+        }
+
+        // Be careful: the result of CountTrailingZeroes32 is undefined if the
+        // input is 0.
+        int numZeros = mozilla::CountTrailingZeroes32(value_);
+        index_ += numZeros;
+        value_ >>= numZeros;
+
+        MOZ_ASSERT_IF(index_ < set_.numBits_, set_.contains(index_));
+    }
+
   public:
-    Iterator(BitSet &set) :
+    explicit Iterator(BitSet &set) :
       set_(set),
       index_(0),
       word_(0),
       value_(set.bits_[0])
     {
-        if (!set_.contains(index_))
-            (*this)++;
+        skipEmpty();
     }
 
     inline bool more() const {
@@ -135,35 +160,18 @@ class BitSet::Iterator
         return more();
     }
 
-    inline Iterator& operator++(int dummy) {
-        JS_ASSERT(more());
-        JS_ASSERT(index_ <= set_.max_);
+    inline void operator++() {
+        MOZ_ASSERT(more());
+        MOZ_ASSERT(index_ < set_.numBits_);
 
         index_++;
         value_ >>= 1;
 
-        // Skip words containing only zeros.
-        while (value_ == 0) {
-            word_++;
-            if (!more())
-                return *this;
-
-            index_ = word_ * sizeof(value_) * 8;
-            value_ = set_.bits_[word_];
-        }
-
-        // Be careful: the result of CountTrailingZeroes32 is undefined if the
-        // input is 0.
-        int numZeros = mozilla::CountTrailingZeroes32(value_);
-        index_ += numZeros;
-        value_ >>= numZeros;
-
-        JS_ASSERT_IF(index_ <= set_.max_, set_.contains(index_));
-        return *this;
+        skipEmpty();
     }
 
     unsigned int operator *() {
-        JS_ASSERT(index_ <= set_.max_);
+        MOZ_ASSERT(index_ < set_.numBits_);
         return index_;
     }
 };
