@@ -7,8 +7,6 @@
 #ifndef jit_InlineList_h
 #define jit_InlineList_h
 
-#include "mozilla/DebugOnly.h"
-
 #include "jsutil.h"
 
 namespace js {
@@ -22,9 +20,11 @@ class InlineForwardListNode
   public:
     InlineForwardListNode() : next(nullptr)
     { }
-    InlineForwardListNode(InlineForwardListNode<T> *n) : next(n)
+    explicit InlineForwardListNode(InlineForwardListNode<T> *n) : next(n)
     { }
 
+    InlineForwardListNode(const InlineForwardListNode<T> &) = delete;
+
   protected:
     friend class InlineForwardList<T>;
     friend class InlineForwardListIterator<T>;
@@ -40,7 +40,9 @@ class InlineForwardList : protected InlineForwardListNode<T>
     typedef InlineForwardListNode<T> Node;
 
     Node *tail_;
-    mozilla::DebugOnly<int> modifyCount_;
+#ifdef DEBUG
+    int modifyCount_;
+#endif
 
     InlineForwardList<T> *thisFromConstructor() {
         return this;
@@ -50,7 +52,9 @@ class InlineForwardList : protected InlineForwardListNode<T>
     InlineForwardList()
       : tail_(thisFromConstructor())
     {
+#ifdef DEBUG
         modifyCount_ = 0;
+#endif
     }
 
   public:
@@ -63,61 +67,64 @@ class InlineForwardList : protected InlineForwardListNode<T>
     iterator end() const {
         return iterator(nullptr);
     }
-    iterator removeAt(iterator &where) {
+    void removeAt(iterator where) {
         iterator iter(where);
         iter++;
-        iter.prev = where.prev;
+#ifdef DEBUG
         iter.modifyCount_++;
-
-        // Once the element 'where' points at has been removed, it is no longer
-        // safe to do any operations that would touch 'iter', as the element
-        // may be added to another list, etc. This nullptr ensures that any
-        // improper uses of this function will fail quickly and loudly.
+#endif
         removeAfter(where.prev, where.iter);
-        where.prev = where.iter = nullptr;
-
-        return iter;
     }
     void pushFront(Node *t) {
         insertAfter(this, t);
     }
     void pushBack(Node *t) {
+        MOZ_ASSERT(t->next == nullptr);
+#ifdef DEBUG
         modifyCount_++;
+#endif
         tail_->next = t;
-        t->next = nullptr;
         tail_ = t;
     }
     T *popFront() {
-        JS_ASSERT(!empty());
+        MOZ_ASSERT(!empty());
         T* result = static_cast<T *>(this->next);
         removeAfter(this, result);
         return result;
     }
     T *back() {
-        JS_ASSERT(!empty());
+        MOZ_ASSERT(!empty());
         return static_cast<T *>(tail_);
     }
     void insertAfter(Node *at, Node *item) {
+        MOZ_ASSERT(item->next == nullptr);
+#ifdef DEBUG
         modifyCount_++;
+#endif
         if (at == tail_)
             tail_ = item;
         item->next = at->next;
         at->next = item;
     }
     void removeAfter(Node *at, Node *item) {
+#ifdef DEBUG
         modifyCount_++;
+#endif
         if (item == tail_)
             tail_ = at;
-        JS_ASSERT(at->next == item);
+        MOZ_ASSERT(at->next == item);
         at->next = item->next;
+        item->next = nullptr;
     }
     void splitAfter(Node *at, InlineForwardList<T> *to) {
-        JS_ASSERT(to->empty());
+        MOZ_ASSERT(to->empty());
         if (!at)
             at = this;
         if (at == tail_)
             return;
+#ifdef DEBUG
         modifyCount_++;
+#endif
         to->next = at->next;
         to->tail_ = tail_;
         tail_ = at;
@@ -129,7 +136,9 @@ class InlineForwardList : protected InlineForwardListNode<T>
     void clear() {
         this->next = nullptr;
         tail_ = this;
+#ifdef DEBUG
         modifyCount_ = 0;
+#endif
     }
 };
 
@@ -141,35 +150,33 @@ private:
 
     typedef InlineForwardListNode<T> Node;
 
-    InlineForwardListIterator<T>(const InlineForwardList<T> *owner)
+    explicit InlineForwardListIterator<T>(const InlineForwardList<T> *owner)
       : prev(const_cast<Node *>(static_cast<const Node *>(owner))),
         iter(owner ? owner->next : nullptr)
 #ifdef DEBUG
       , owner_(owner),
-        modifyCount_(owner ? owner->modifyCount_.value : 0)
+        modifyCount_(owner ? owner->modifyCount_ : 0)
 #endif
     { }
 
 public:
     InlineForwardListIterator<T> & operator ++() {
-        JS_ASSERT(modifyCount_ == owner_->modifyCount_);
+        MOZ_ASSERT(modifyCount_ == owner_->modifyCount_);
         prev = iter;
         iter = iter->next;
         return *this;
     }
     InlineForwardListIterator<T> operator ++(int) {
-        JS_ASSERT(modifyCount_ == owner_->modifyCount_);
         InlineForwardListIterator<T> old(*this);
-        prev = iter;
-        iter = iter->next;
+        operator++();
         return old;
     }
     T * operator *() const {
-        JS_ASSERT(modifyCount_ == owner_->modifyCount_);
+        MOZ_ASSERT(modifyCount_ == owner_->modifyCount_);
         return static_cast<T *>(iter);
     }
     T * operator ->() const {
-        JS_ASSERT(modifyCount_ == owner_->modifyCount_);
+        MOZ_ASSERT(modifyCount_ == owner_->modifyCount_);
         return static_cast<T *>(iter);
     }
     bool operator !=(const InlineForwardListIterator<T> &where) const {
@@ -185,8 +192,8 @@ private:
 
 #ifdef DEBUG
     const InlineForwardList<T> *owner_;
+    int modifyCount_;
 #endif
-    mozilla::DebugOnly<int> modifyCount_;
 };
 
 template <typename T> class InlineList;
@@ -204,6 +211,27 @@ class InlineListNode : public InlineForwardListNode<T>
         prev(p)
     { }
 
+    // Move constructor. Nodes may be moved without being removed from their
+    // containing lists. For example, this allows list nodes to be safely
+    // stored in a resizable Vector -- when the Vector resizes, the new storage
+    // is initialized by this move constructor. |other| is a reference to the
+    // old node which the |this| node here is replacing.
+    InlineListNode(InlineListNode<T> &&other)
+      : InlineForwardListNode<T>(other.next)
+    {
+        InlineListNode<T> *newNext = static_cast<InlineListNode<T> *>(other.next);
+        InlineListNode<T> *newPrev = other.prev;
+        prev = newPrev;
+
+        // Update the pointers in the adjacent nodes to point to this node's new
+        // location.
+        newNext->prev = this;
+        newPrev->next = this;
+    }
+
+    InlineListNode(const InlineListNode<T> &) = delete;
+    void operator=(const InlineListNode<T> &) = delete;
+
   protected:
     friend class InlineList<T>;
     friend class InlineListIterator<T>;
@@ -217,13 +245,8 @@ class InlineList : protected InlineListNode<T>
 {
     typedef InlineListNode<T> Node;
 
-    // Silence MSVC warning C4355
-    InlineList<T> *thisFromConstructor() {
-        return this;
-    }
-
   public:
-    InlineList() : InlineListNode<T>(thisFromConstructor(), thisFromConstructor())
+    InlineList() : InlineListNode<T>(this, this)
     { }
 
   public:
@@ -249,34 +272,26 @@ class InlineList : protected InlineListNode<T>
     reverse_iterator rend() const {
         return reverse_iterator(this);
     }
-    template <typename itertype>
-    itertype removeAt(itertype &where) {
-        itertype iter(where);
-        iter++;
-
-        // Once the element 'where' points at has been removed, it is no longer
-        // safe to do any operations that would touch 'iter', as the element
-        // may be added to another list, etc. This nullptr ensures that any
-        // improper uses of this function will fail quickly and loudly.
-        remove(where.iter);
-        where.iter = nullptr;
-
-        return iter;
-    }
     void pushFront(Node *t) {
         insertAfter(this, t);
     }
+    void pushFrontUnchecked(Node *t) {
+        insertAfterUnchecked(this, t);
+    }
     void pushBack(Node *t) {
         insertBefore(this, t);
     }
+    void pushBackUnchecked(Node *t) {
+        insertBeforeUnchecked(this, t);
+    }
     T *popFront() {
-        JS_ASSERT(!empty());
+        MOZ_ASSERT(!empty());
         T *t = static_cast<T *>(this->next);
         remove(t);
         return t;
     }
     T *popBack() {
-        JS_ASSERT(!empty());
+        MOZ_ASSERT(!empty());
         T *t = static_cast<T *>(this->prev);
         remove(t);
         return t;
@@ -287,21 +302,48 @@ class InlineList : protected InlineListNode<T>
         return *iter;
     }
     void insertBefore(Node *at, Node *item) {
+        MOZ_ASSERT(item->prev == nullptr);
+        MOZ_ASSERT(item->next == nullptr);
+        insertBeforeUnchecked(at, item);
+    }
+    void insertBeforeUnchecked(Node *at, Node *item) {
+        Node *atPrev = at->prev;
         item->next = at;
-        item->prev = at->prev;
-        at->prev->next = item;
+        item->prev = atPrev;
+        atPrev->next = item;
         at->prev = item;
     }
     void insertAfter(Node *at, Node *item) {
-        item->next = at->next;
+        MOZ_ASSERT(item->prev == nullptr);
+        MOZ_ASSERT(item->next == nullptr);
+        insertAfterUnchecked(at, item);
+    }
+    void insertAfterUnchecked(Node *at, Node *item) {
+        Node *atNext = static_cast<Node *>(at->next);
+        item->next = atNext;
         item->prev = at;
-        static_cast<Node *>(at->next)->prev = item;
+        atNext->prev = item;
         at->next = item;
     }
     void remove(Node *t) {
-        t->prev->next = t->next;
-        static_cast<Node *>(t->next)->prev = t->prev;
-        t->next = t->prev = nullptr;
+        Node *tNext = static_cast<Node *>(t->next);
+        Node *tPrev = t->prev;
+        tPrev->next = tNext;
+        tNext->prev = tPrev;
+        t->next = nullptr;
+        t->prev = nullptr;
+    }
+    // Remove |old| from the list and insert |now| in its place.
+    void replace(Node *old, Node *now) {
+        MOZ_ASSERT(now->next == nullptr && now->prev == nullptr);
+        Node *listNext = static_cast<Node *>(old->next);
+        Node *listPrev = old->prev;
+        listPrev->next = now;
+        listNext->prev = now;
+        now->next = listNext;
+        now->prev = listPrev;
+        old->next = nullptr;
+        old->prev = nullptr;
     }
     void clear() {
         this->next = this->prev = this;
@@ -309,6 +351,15 @@ class InlineList : protected InlineListNode<T>
     bool empty() const {
         return begin() == end();
     }
+    void takeElements(InlineList &l) {
+        MOZ_ASSERT(&l != this, "cannot takeElements from this");
+        Node *lprev = l.prev;
+        static_cast<Node *>(l.next)->prev = this;
+        lprev->next = this->next;
+        static_cast<Node *>(this->next)->prev = l.prev;
+        this->next = l.next;
+        l.clear();
+    }
 };
 
 template <typename T>
@@ -319,7 +370,7 @@ class InlineListIterator
 
     typedef InlineListNode<T> Node;
 
-    InlineListIterator(const Node *iter)
+    explicit InlineListIterator(const Node *iter)
       : iter(const_cast<Node *>(iter))
     { }
 
@@ -330,12 +381,16 @@ class InlineListIterator
     }
     InlineListIterator<T> operator ++(int) {
         InlineListIterator<T> old(*this);
-        iter = static_cast<Node *>(iter->next);
+        operator++();
         return old;
     }
+    InlineListIterator<T> & operator --() {
+        iter = iter->prev;
+        return *this;
+    }
     InlineListIterator<T> operator --(int) {
         InlineListIterator<T> old(*this);
-        iter = iter->prev;
+        operator--();
         return old;
     }
     T * operator *() const {
@@ -363,7 +418,7 @@ class InlineListReverseIterator
 
     typedef InlineListNode<T> Node;
 
-    InlineListReverseIterator(const Node *iter)
+    explicit InlineListReverseIterator(const Node *iter)
       : iter(const_cast<Node *>(iter))
     { }
 
@@ -374,7 +429,16 @@ class InlineListReverseIterator
     }
     InlineListReverseIterator<T> operator ++(int) {
         InlineListReverseIterator<T> old(*this);
-        iter = iter->prev;
+        operator++();
+        return old;
+    }
+    InlineListReverseIterator<T> & operator --() {
+        iter = static_cast<Node *>(iter->next);
+        return *this;
+    }
+    InlineListReverseIterator<T> operator --(int) {
+        InlineListReverseIterator<T> old(*this);
+        operator--();
         return old;
     }
     T * operator *() {
@@ -425,10 +489,10 @@ class InlineConcatList
 
     void append(InlineConcatList<T> *adding)
     {
-        JS_ASSERT(tail);
-        JS_ASSERT(!tail->next);
-        JS_ASSERT(adding->tail);
-        JS_ASSERT(!adding->tail->next);
+        MOZ_ASSERT(tail);
+        MOZ_ASSERT(!tail->next);
+        MOZ_ASSERT(adding->tail);
+        MOZ_ASSERT(!adding->tail->next);
 
         tail->next = adding;
         tail = adding->tail;
@@ -449,18 +513,18 @@ class InlineConcatListIterator
 
     typedef InlineConcatList<T> Node;
 
-    InlineConcatListIterator(const Node *iter)
+    explicit InlineConcatListIterator(const Node *iter)
       : iter(const_cast<Node *>(iter))
     { }
 
   public:
     InlineConcatListIterator<T> & operator ++() {
-        iter = iter->next;
-        return *iter;
+        iter = static_cast<Node *>(iter->next);
+        return *this;
     }
     InlineConcatListIterator<T> operator ++(int) {
         InlineConcatListIterator<T> old(*this);
-        iter = static_cast<Node *>(iter->next);
+        operator++();
         return old;
     }
     T * operator *() const {
@@ -480,6 +544,106 @@ class InlineConcatListIterator
     Node *iter;
 };
 
+template <typename T> class InlineSpaghettiStack;
+template <typename T> class InlineSpaghettiStackNode;
+template <typename T> class InlineSpaghettiStackIterator;
+
+template <typename T>
+class InlineSpaghettiStackNode : public InlineForwardListNode<T>
+{
+    typedef InlineForwardListNode<T> Parent;
+
+  public:
+    InlineSpaghettiStackNode() : Parent()
+    { }
+
+    explicit InlineSpaghettiStackNode(InlineSpaghettiStackNode<T> *n)
+      : Parent(n)
+    { }
+
+    InlineSpaghettiStackNode(const InlineSpaghettiStackNode<T> &) = delete;
+
+  protected:
+    friend class InlineSpaghettiStack<T>;
+    friend class InlineSpaghettiStackIterator<T>;
+};
+
+template <typename T>
+class InlineSpaghettiStack : protected InlineSpaghettiStackNode<T>
+{
+    friend class InlineSpaghettiStackIterator<T>;
+
+    typedef InlineSpaghettiStackNode<T> Node;
+
+  public:
+    InlineSpaghettiStack()
+    { }
+
+  public:
+    typedef InlineSpaghettiStackIterator<T> iterator;
+
+  public:
+    iterator begin() const {
+        return iterator(this);
+    }
+    iterator end() const {
+        return iterator(nullptr);
+    }
+
+    void push(Node *t) {
+        MOZ_ASSERT(t->next == nullptr);
+        t->next = this->next;
+        this->next = t;
+    }
+
+    void copy(const InlineSpaghettiStack<T> &stack) {
+        this->next = stack.next;
+    }
+
+    bool empty() const {
+        return this->next == nullptr;
+    }
+};
+
+template <typename T>
+class InlineSpaghettiStackIterator
+{
+  private:
+    friend class InlineSpaghettiStack<T>;
+
+    typedef InlineSpaghettiStackNode<T> Node;
+
+    explicit InlineSpaghettiStackIterator<T>(const InlineSpaghettiStack<T> *owner)
+      : iter(owner ? static_cast<Node *>(owner->next) : nullptr)
+    { }
+
+  public:
+    InlineSpaghettiStackIterator<T> & operator ++() {
+        iter = static_cast<Node *>(iter->next);
+        return *this;
+    }
+    InlineSpaghettiStackIterator<T> operator ++(int) {
+        InlineSpaghettiStackIterator<T> old(*this);
+        operator++();
+        return old;
+    }
+    T *operator *() const {
+        return static_cast<T *>(iter);
+    }
+    T *operator ->() const {
+        return static_cast<T *>(iter);
+    }
+    bool operator !=(const InlineSpaghettiStackIterator<T> &where) const {
+        return iter != where.iter;
+    }
+    bool operator ==(const InlineSpaghettiStackIterator<T> &where) const {
+        return iter == where.iter;
+    }
+
+  private:
+    Node *iter;
+};
+
 } // namespace js
 
 #endif /* jit_InlineList_h */
