@@ -14,14 +14,12 @@
 #include "jsinfer.h"
 #include "jstypes.h"
 
-#include "assembler/jit/ExecutableAllocator.h"
 #include "gc/Heap.h"
+#include "jit/ExecutableAllocator.h"
 #include "jit/IonOptimizationLevels.h"
 #include "jit/IonTypes.h"
-
-namespace JSC {
-    class ExecutablePool;
-}
+#include "js/UbiNode.h"
+#include "vm/TraceLogging.h"
 
 namespace js {
 
@@ -32,12 +30,13 @@ namespace jit {
 class MacroAssembler;
 class CodeOffsetLabel;
 class PatchableBackedge;
+class IonBuilder;
 
-class JitCode : public gc::BarrieredCell<JitCode>
+class JitCode : public gc::TenuredCell
 {
   protected:
     uint8_t *code_;
-    JSC::ExecutablePool *pool_;
+    ExecutablePool *pool_;
     uint32_t bufferSize_;             // Total buffer size. Does not include headerSize_.
     uint32_t insnSize_;               // Instruction stream size.
     uint32_t dataSize_;               // Size of the read-only data area.
@@ -45,9 +44,11 @@ class JitCode : public gc::BarrieredCell<JitCode>
     uint32_t dataRelocTableBytes_;    // Size of the data relocation table.
     uint32_t preBarrierTableBytes_;   // Size of the prebarrier table.
     uint8_t headerSize_ : 5;          // Number of bytes allocated before codeStart.
-    uint8_t kind_ : 3;                // JSC::CodeKind, for the memory reporters.
+    uint8_t kind_ : 3;                // jit::CodeKind, for the memory reporters.
     bool invalidated_ : 1;            // Whether the code object has been invalidated.
                                       // This is necessary to prevent GC tracing.
+    bool hasBytecodeMap_ : 1;         // Whether the code object has been registered with
+                                      // native=>bytecode mapping tables.
 
 #if JS_BITS_PER_WORD == 32
     // Ensure JitCode is gc::Cell aligned.
@@ -58,8 +59,8 @@ class JitCode : public gc::BarrieredCell<JitCode>
       : code_(nullptr),
         pool_(nullptr)
     { }
-    JitCode(uint8_t *code, uint32_t bufferSize, uint32_t headerSize, JSC::ExecutablePool *pool,
-            JSC::CodeKind kind)
+    JitCode(uint8_t *code, uint32_t bufferSize, uint32_t headerSize, ExecutablePool *pool,
+            CodeKind kind)
       : code_(code),
         pool_(pool),
         bufferSize_(bufferSize),
@@ -70,9 +71,10 @@ class JitCode : public gc::BarrieredCell<JitCode>
         preBarrierTableBytes_(0),
         headerSize_(headerSize),
         kind_(kind),
-        invalidated_(false)
+        invalidated_(false),
+        hasBytecodeMap_(false)
     {
-        MOZ_ASSERT(JSC::CodeKind(kind_) == kind);
+        MOZ_ASSERT(CodeKind(kind_) == kind);
         MOZ_ASSERT(headerSize_ == headerSize);
     }
 
@@ -93,15 +95,23 @@ class JitCode : public gc::BarrieredCell<JitCode>
     uint8_t *raw() const {
         return code_;
     }
+    uint8_t *rawEnd() const {
+        return code_ + insnSize_;
+    }
     size_t instructionsSize() const {
         return insnSize_;
     }
     void trace(JSTracer *trc);
     void finalize(FreeOp *fop);
+    void fixupAfterMovingGC() {}
     void setInvalidated() {
         invalidated_ = true;
     }
 
+    void setHasBytecodeMap() {
+        hasBytecodeMap_ = true;
+    }
+
     void togglePreBarriers(bool enabled);
 
     // If this JitCode object has been, effectively, corrupted due to
@@ -119,7 +129,7 @@ class JitCode : public gc::BarrieredCell<JitCode>
 
     static JitCode *FromExecutable(uint8_t *buffer) {
         JitCode *code = *(JitCode **)(buffer - sizeof(JitCode *));
-        JS_ASSERT(code->raw() == buffer);
+        MOZ_ASSERT(code->raw() == buffer);
         return code;
     }
 
@@ -136,7 +146,7 @@ class JitCode : public gc::BarrieredCell<JitCode>
     // automatically released, so the code may be freed.
     template <AllowGC allowGC>
     static JitCode *New(JSContext *cx, uint8_t *code, uint32_t bufferSize, uint32_t headerSize,
-                        JSC::ExecutablePool *pool, JSC::CodeKind kind);
+                        ExecutablePool *pool, CodeKind kind);
 
   public:
     static inline ThingRootKind rootKind() { return THING_ROOT_JIT_CODE; }
@@ -151,19 +161,6 @@ class IonCache;
 struct PatchableBackedgeInfo;
 struct CacheLocation;
 
-// Describes a single AsmJSModule which jumps (via an FFI exit with the given
-// index) directly into an IonScript.
-struct DependentAsmJSModuleExit
-{
-    const AsmJSModule *module;
-    size_t exitIndex;
-
-    DependentAsmJSModuleExit(const AsmJSModule *module, size_t exitIndex)
-      : module(module),
-        exitIndex(exitIndex)
-    { }
-};
-
 // An IonScript attaches Ion-generated information to a JSScript.
 struct IonScript
 {
@@ -235,7 +232,7 @@ struct IonScript
     uint32_t frameSlots_;
 
     // Frame size is the value that can be added to the StackPointer along
-    // with the frame prefix to get a valid IonJSFrameLayout.
+    // with the frame prefix to get a valid JitFrameLayout.
     uint32_t frameSize_;
 
     // Table mapping bailout IDs to snapshot offsets.
@@ -259,18 +256,12 @@ struct IonScript
     uint32_t constantTable_;
     uint32_t constantEntries_;
 
-    // List of scripts that we call.
-    //
-    // Currently this is only non-nullptr for parallel IonScripts.
-    uint32_t callTargetList_;
-    uint32_t callTargetEntries_;
-
     // List of patchable backedges which are threaded into the runtime's list.
     uint32_t backedgeList_;
     uint32_t backedgeEntries_;
 
     // Number of references from invalidation records.
-    uint32_t refcount_;
+    uint32_t invalidationCount_;
 
     // If this is a parallel script, the number of major GC collections it has
     // been idle, otherwise 0.
@@ -289,9 +280,10 @@ struct IonScript
     // a LOOPENTRY pc other than osrPc_.
     uint32_t osrPcMismatchCounter_;
 
-    // If non-null, the list of AsmJSModules
-    // that contain an optimized call directly into this IonScript.
-    Vector<DependentAsmJSModuleExit> *dependentAsmJSModules;
+    // The tracelogger event used to log the start/stop of this IonScript.
+    TraceLoggerEvent traceLoggerScriptEvent_;
+
+    IonBuilder *pendingBuilder_;
 
   private:
     inline uint8_t *bottomBuffer() {
@@ -302,6 +294,15 @@ struct IonScript
     }
 
   public:
+
+    // SHOULD ONLY BE CALLED FROM JSScript
+    void setPendingBuilderPrivate(IonBuilder *builder) {
+        pendingBuilder_ = builder;
+    }
+    IonBuilder *pendingBuilder() const {
+        return pendingBuilder_;
+    }
+
     SnapshotOffset *bailoutTable() {
         return (SnapshotOffset *) &bottomBuffer()[bailoutTable_];
     }
@@ -326,25 +327,9 @@ struct IonScript
     uint8_t *runtimeData() {
         return  &bottomBuffer()[runtimeData_];
     }
-    JSScript **callTargetList() {
-        return (JSScript **) &bottomBuffer()[callTargetList_];
-    }
     PatchableBackedge *backedgeList() {
         return (PatchableBackedge *) &bottomBuffer()[backedgeList_];
     }
-    bool addDependentAsmJSModule(JSContext *cx, DependentAsmJSModuleExit exit);
-    void removeDependentAsmJSModule(DependentAsmJSModuleExit exit) {
-        if (!dependentAsmJSModules)
-            return;
-        for (size_t i = 0; i < dependentAsmJSModules->length(); i++) {
-            if (dependentAsmJSModules->begin()[i].module == exit.module &&
-                dependentAsmJSModules->begin()[i].exitIndex == exit.exitIndex)
-            {
-                dependentAsmJSModules->erase(dependentAsmJSModules->begin() + i);
-                break;
-            }
-        }
-    }
 
   private:
     void trace(JSTracer *trc);
@@ -360,8 +345,7 @@ struct IonScript
                           size_t constants, size_t safepointIndexEntries,
                           size_t osiIndexEntries, size_t cacheEntries,
                           size_t runtimeSize, size_t safepointsSize,
-                          size_t callTargetEntries, size_t backedgeEntries,
-                          OptimizationLevel optimizationLevel);
+                          size_t backedgeEntries, OptimizationLevel optimizationLevel);
     static void Trace(JSTracer *trc, IonScript *script);
     static void Destroy(FreeOp *fop, IonScript *script);
 
@@ -374,8 +358,8 @@ struct IonScript
     static inline size_t offsetOfSkipArgCheckEntryOffset() {
         return offsetof(IonScript, skipArgCheckEntryOffset_);
     }
-    static inline size_t offsetOfRefcount() {
-        return offsetof(IonScript, refcount_);
+    static inline size_t offsetOfInvalidationCount() {
+        return offsetof(IonScript, invalidationCount_);
     }
     static inline size_t offsetOfRecompiling() {
         return offsetof(IonScript, recompiling_);
@@ -386,7 +370,7 @@ struct IonScript
         return method_;
     }
     void setMethod(JitCode *code) {
-        JS_ASSERT(!invalidated());
+        MOZ_ASSERT(!invalidated());
         method_ = code;
     }
     void setDeoptTable(JitCode *code) {
@@ -399,14 +383,14 @@ struct IonScript
         return osrPc_;
     }
     void setOsrEntryOffset(uint32_t offset) {
-        JS_ASSERT(!osrEntryOffset_);
+        MOZ_ASSERT(!osrEntryOffset_);
         osrEntryOffset_ = offset;
     }
     uint32_t osrEntryOffset() const {
         return osrEntryOffset_;
     }
     void setSkipArgCheckEntryOffset(uint32_t offset) {
-        JS_ASSERT(!skipArgCheckEntryOffset_);
+        MOZ_ASSERT(!skipArgCheckEntryOffset_);
         skipArgCheckEntryOffset_ = offset;
     }
     uint32_t getSkipArgCheckEntryOffset() const {
@@ -421,19 +405,19 @@ struct IonScript
         return method()->raw() <= addr && addr <= method()->raw() + method()->instructionsSize();
     }
     void setInvalidationEpilogueOffset(uint32_t offset) {
-        JS_ASSERT(!invalidateEpilogueOffset_);
+        MOZ_ASSERT(!invalidateEpilogueOffset_);
         invalidateEpilogueOffset_ = offset;
     }
     uint32_t invalidateEpilogueOffset() const {
-        JS_ASSERT(invalidateEpilogueOffset_);
+        MOZ_ASSERT(invalidateEpilogueOffset_);
         return invalidateEpilogueOffset_;
     }
     void setInvalidationEpilogueDataOffset(uint32_t offset) {
-        JS_ASSERT(!invalidateEpilogueDataOffset_);
+        MOZ_ASSERT(!invalidateEpilogueDataOffset_);
         invalidateEpilogueDataOffset_ = offset;
     }
     uint32_t invalidateEpilogueDataOffset() const {
-        JS_ASSERT(invalidateEpilogueDataOffset_);
+        MOZ_ASSERT(invalidateEpilogueDataOffset_);
         return invalidateEpilogueDataOffset_;
     }
     void incNumBailouts() {
@@ -472,6 +456,9 @@ struct IonScript
     bool hasSPSInstrumentation() const {
         return hasSPSInstrumentation_;
     }
+    void setTraceLoggerEvent(TraceLoggerEvent &event) {
+        traceLoggerScriptEvent_ = event;
+    }
     const uint8_t *snapshots() const {
         return reinterpret_cast<const uint8_t *>(this) + snapshots_;
     }
@@ -493,14 +480,11 @@ struct IonScript
     size_t safepointsSize() const {
         return safepointsSize_;
     }
-    size_t callTargetEntries() const {
-        return callTargetEntries_;
-    }
     size_t sizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf) const {
         return mallocSizeOf(this);
     }
     PreBarrieredValue &getConstant(size_t index) {
-        JS_ASSERT(index < numConstants());
+        MOZ_ASSERT(index < numConstants());
         return constants()[index];
     }
     size_t numConstants() const {
@@ -513,23 +497,23 @@ struct IonScript
         return frameSize_;
     }
     SnapshotOffset bailoutToSnapshot(uint32_t bailoutId) {
-        JS_ASSERT(bailoutId < bailoutEntries_);
+        MOZ_ASSERT(bailoutId < bailoutEntries_);
         return bailoutTable()[bailoutId];
     }
     const SafepointIndex *getSafepointIndex(uint32_t disp) const;
     const SafepointIndex *getSafepointIndex(uint8_t *retAddr) const {
-        JS_ASSERT(containsCodeAddress(retAddr));
+        MOZ_ASSERT(containsCodeAddress(retAddr));
         return getSafepointIndex(retAddr - method()->raw());
     }
     const OsiIndex *getOsiIndex(uint32_t disp) const;
     const OsiIndex *getOsiIndex(uint8_t *retAddr) const;
     inline IonCache &getCacheFromIndex(uint32_t index) {
-        JS_ASSERT(index < cacheEntries_);
+        MOZ_ASSERT(index < cacheEntries_);
         uint32_t offset = cacheIndex()[index];
         return getCache(offset);
     }
     inline IonCache &getCache(uint32_t offset) {
-        JS_ASSERT(offset < runtimeSize_);
+        MOZ_ASSERT(offset < runtimeSize_);
         return *(IonCache *) &runtimeData()[offset];
     }
     size_t numCaches() const {
@@ -539,7 +523,7 @@ struct IonScript
         return runtimeSize_;
     }
     CacheLocation *getCacheLocs(uint32_t locIndex) {
-        JS_ASSERT(locIndex < runtimeSize_);
+        MOZ_ASSERT(locIndex < runtimeSize_);
         return (CacheLocation *) &runtimeData()[locIndex];
     }
     void toggleBarriers(bool enabled);
@@ -555,24 +539,27 @@ struct IonScript
     void copyRuntimeData(const uint8_t *data);
     void copyCacheEntries(const uint32_t *caches, MacroAssembler &masm);
     void copySafepoints(const SafepointWriter *writer);
-    void copyCallTargetEntries(JSScript **callTargets);
     void copyPatchableBackedges(JSContext *cx, JitCode *code,
                                 PatchableBackedgeInfo *backedges,
                                 MacroAssembler &masm);
 
     bool invalidated() const {
-        return refcount_ != 0;
+        return invalidationCount_ != 0;
     }
-    size_t refcount() const {
-        return refcount_;
+
+    // Invalidate the current compilation.
+    bool invalidate(JSContext *cx, bool resetUses, const char *reason);
+
+    size_t invalidationCount() const {
+        return invalidationCount_;
     }
-    void incref() {
-        refcount_++;
+    void incrementInvalidationCount() {
+        invalidationCount_++;
     }
-    void decref(FreeOp *fop) {
-        JS_ASSERT(refcount_);
-        refcount_--;
-        if (!refcount_)
+    void decrementInvalidationCount(FreeOp *fop) {
+        MOZ_ASSERT(invalidationCount_);
+        invalidationCount_--;
+        if (!invalidationCount_)
             Destroy(fop, this);
     }
     const types::RecompileInfo& recompileInfo() const {
@@ -636,6 +623,9 @@ struct IonBlockCounts
     // was generated from.
     uint32_t offset_;
 
+    // File and line of the inner script this block was generated from.
+    char *description_;
+
     // ids for successors of this block.
     uint32_t numSuccessors_;
     uint32_t *successors_;
@@ -648,9 +638,10 @@ struct IonBlockCounts
 
   public:
 
-    bool init(uint32_t id, uint32_t offset, uint32_t numSuccessors) {
+    bool init(uint32_t id, uint32_t offset, char *description, uint32_t numSuccessors) {
         id_ = id;
         offset_ = offset;
+        description_ = description;
         numSuccessors_ = numSuccessors;
         if (numSuccessors) {
             successors_ = js_pod_calloc<uint32_t>(numSuccessors);
@@ -661,6 +652,7 @@ struct IonBlockCounts
     }
 
     void destroy() {
+        js_free(description_);
         js_free(successors_);
         js_free(code_);
     }
@@ -673,17 +665,21 @@ struct IonBlockCounts
         return offset_;
     }
 
+    const char *description() const {
+        return description_;
+    }
+
     size_t numSuccessors() const {
         return numSuccessors_;
     }
 
     void setSuccessor(size_t i, uint32_t id) {
-        JS_ASSERT(i < numSuccessors_);
+        MOZ_ASSERT(i < numSuccessors_);
         successors_[i] = id;
     }
 
     uint32_t successor(size_t i) const {
-        JS_ASSERT(i < numSuccessors_);
+        MOZ_ASSERT(i < numSuccessors_);
         return successors_[i];
     }
 
@@ -744,7 +740,7 @@ struct IonScriptCounts
     }
 
     IonBlockCounts &block(size_t i) {
-        JS_ASSERT(i < numBlocks_);
+        MOZ_ASSERT(i < numBlocks_);
         return blocks_[i];
     }
 
@@ -797,4 +793,12 @@ IsMarked(const jit::VMFunction *)
 
 } // namespace js
 
+// JS::ubi::Nodes can point to js::jit::JitCode instances; they're js::gc::Cell
+// instances with no associated compartment.
+namespace JS {
+namespace ubi {
+template<> struct Concrete<js::jit::JitCode> : TracerConcrete<js::jit::JitCode> { };
+}
+}
+
 #endif /* jit_IonCode_h */
