@@ -13,6 +13,10 @@
 #include "jit/IonAnalysis.h"
 #include "jit/MIR.h"
 
+// windows.h defines those, which messes with the definitions below.
+#undef min
+#undef max
+
 namespace js {
 namespace jit {
 
@@ -28,21 +32,30 @@ struct LoopIterationBound : public TempObject
     // Loop for which this bound applies.
     MBasicBlock *header;
 
-    // Test from which this bound was derived. Code in the loop body which this
+    // Test from which this bound was derived; after executing exactly 'bound'
+    // times this test will exit the loop. Code in the loop body which this
     // test dominates (will include the backedge) will execute at most 'bound'
     // times. Other code in the loop will execute at most '1 + Max(bound, 0)'
     // times.
     MTest *test;
 
-    // Symbolic bound computed for the number of backedge executions.
-    LinearSum sum;
+    // Symbolic bound computed for the number of backedge executions. The terms
+    // in this bound are all loop invariant.
+    LinearSum boundSum;
 
-    LoopIterationBound(MBasicBlock *header, MTest *test, LinearSum sum)
-      : header(header), test(test), sum(sum)
+    // Linear sum for the number of iterations already executed, at the start
+    // of the loop header. This will use loop invariant terms and header phis.
+    LinearSum currentSum;
+
+    LoopIterationBound(MBasicBlock *header, MTest *test, LinearSum boundSum, LinearSum currentSum)
+      : header(header), test(test),
+        boundSum(boundSum), currentSum(currentSum)
     {
     }
 };
 
+typedef Vector<LoopIterationBound *, 0, SystemAllocPolicy> LoopIterationBoundVector;
+
 // A symbolic upper or lower bound computed for a term.
 struct SymbolicBound : public TempObject
 {
@@ -86,21 +99,24 @@ class RangeAnalysis
     TempAllocator &alloc() const;
 
   public:
-    MOZ_CONSTEXPR RangeAnalysis(MIRGenerator *mir, MIRGraph &graph) :
+    RangeAnalysis(MIRGenerator *mir, MIRGraph &graph) :
         mir(mir), graph_(graph) {}
     bool addBetaNodes();
     bool analyze();
     bool addRangeAssertions();
     bool removeBetaNodes();
+    bool prepareForUCE(bool *shouldRemoveDeadCode);
     bool truncate();
 
+    // Any iteration bounds discovered for loops in the graph.
+    LoopIterationBoundVector loopIterationBounds;
+
   private:
     bool analyzeLoop(MBasicBlock *header);
     LoopIterationBound *analyzeLoopIterationCount(MBasicBlock *header,
                                                   MTest *test, BranchDirection direction);
     void analyzeLoopPhi(MBasicBlock *header, LoopIterationBound *loopBound, MPhi *phi);
     bool tryHoistBoundsCheck(MBasicBlock *header, MBoundsCheck *ins);
-    bool markBlocksInLoopBody(MBasicBlock *header, MBasicBlock *current);
 };
 
 class Range : public TempObject {
@@ -116,10 +132,10 @@ class Range : public TempObject {
     // Maximal exponenent under which we have no precission loss on double
     // operations. Double has 52 bits of mantissa, so 2^52+1 cannot be
     // represented without loss.
-    static const uint16_t MaxTruncatableExponent = mozilla::DoubleExponentShift;
+    static const uint16_t MaxTruncatableExponent = mozilla::FloatingPoint<double>::kExponentShift;
 
     // Maximum exponent for finite values.
-    static const uint16_t MaxFiniteExponent = mozilla::DoubleExponentBias;
+    static const uint16_t MaxFiniteExponent = mozilla::FloatingPoint<double>::kExponentBias;
 
     // An special exponent value representing all non-NaN values. This
     // includes finite values and the infinities.
@@ -138,6 +154,15 @@ class Range : public TempObject {
     static const int64_t NoInt32UpperBound = int64_t(JSVAL_INT_MAX) + 1;
     static const int64_t NoInt32LowerBound = int64_t(JSVAL_INT_MIN) - 1;
 
+    enum FractionalPartFlag {
+        ExcludesFractionalParts = false,
+        IncludesFractionalParts = true
+    };
+    enum NegativeZeroFlag {
+        ExcludesNegativeZero = false,
+        IncludesNegativeZero = true
+    };
+
   private:
     // Absolute ranges.
     //
@@ -170,56 +195,58 @@ class Range : public TempObject {
     // the Int32 this over approximation is rectified.
 
     int32_t lower_;
-    bool hasInt32LowerBound_;
-
     int32_t upper_;
+
+    bool hasInt32LowerBound_;
     bool hasInt32UpperBound_;
 
-    bool canHaveFractionalPart_;
+    FractionalPartFlag canHaveFractionalPart_ : 1;
+    NegativeZeroFlag canBeNegativeZero_ : 1;
     uint16_t max_exponent_;
 
     // Any symbolic lower or upper bound computed for this term.
     const SymbolicBound *symbolicLower_;
     const SymbolicBound *symbolicUpper_;
 
-    // This function simply makes several JS_ASSERTs to verify the internal
+    // This function simply makes several MOZ_ASSERTs to verify the internal
     // consistency of this range.
     void assertInvariants() const {
         // Basic sanity :).
-        JS_ASSERT(lower_ <= upper_);
+        MOZ_ASSERT(lower_ <= upper_);
 
         // When hasInt32LowerBound_ or hasInt32UpperBound_ are false, we set
         // lower_ and upper_ to these specific values as it simplifies the
         // implementation in some places.
-        JS_ASSERT_IF(!hasInt32LowerBound_, lower_ == JSVAL_INT_MIN);
-        JS_ASSERT_IF(!hasInt32UpperBound_, upper_ == JSVAL_INT_MAX);
+        MOZ_ASSERT_IF(!hasInt32LowerBound_, lower_ == JSVAL_INT_MIN);
+        MOZ_ASSERT_IF(!hasInt32UpperBound_, upper_ == JSVAL_INT_MAX);
 
         // max_exponent_ must be one of three possible things.
-        JS_ASSERT(max_exponent_ <= MaxFiniteExponent ||
-                  max_exponent_ == IncludesInfinity ||
-                  max_exponent_ == IncludesInfinityAndNaN);
+        MOZ_ASSERT(max_exponent_ <= MaxFiniteExponent ||
+                   max_exponent_ == IncludesInfinity ||
+                   max_exponent_ == IncludesInfinityAndNaN);
 
         // Forbid the max_exponent_ field from implying better bounds for
         // lower_/upper_ fields. We have to add 1 to the max_exponent_ when
-        // canHaveFractionalPart_ is true in order to accomodate fractional
-        // offsets. For example, 2147483647.9 is greater than INT32_MAX, so a
-        // range containing that value will have hasInt32UpperBound_ set to
-        // false, however that value also has exponent 30, which is strictly
-        // less than MaxInt32Exponent. For another example, 1.9 has an exponent
-        // of 0 but requires upper_ to be at least 2, which has exponent 1.
-        JS_ASSERT_IF(!hasInt32LowerBound_ || !hasInt32UpperBound_,
-                     max_exponent_ + canHaveFractionalPart_ >= MaxInt32Exponent);
-        JS_ASSERT(max_exponent_ + canHaveFractionalPart_ >=
-                  mozilla::FloorLog2(mozilla::Abs(upper_)));
-        JS_ASSERT(max_exponent_ + canHaveFractionalPart_ >=
-                  mozilla::FloorLog2(mozilla::Abs(lower_)));
+        // canHaveFractionalPart_ is true in order to accomodate
+        // fractional offsets. For example, 2147483647.9 is greater than
+        // INT32_MAX, so a range containing that value will have
+        // hasInt32UpperBound_ set to false, however that value also has
+        // exponent 30, which is strictly less than MaxInt32Exponent. For
+        // another example, 1.9 has an exponent of 0 but requires upper_ to be
+        // at least 2, which has exponent 1.
+        mozilla::DebugOnly<uint32_t> adjustedExponent = max_exponent_ +
+            (canHaveFractionalPart_ ? 1 : 0);
+        MOZ_ASSERT_IF(!hasInt32LowerBound_ || !hasInt32UpperBound_,
+                      adjustedExponent >= MaxInt32Exponent);
+        MOZ_ASSERT(adjustedExponent >= mozilla::FloorLog2(mozilla::Abs(upper_)));
+        MOZ_ASSERT(adjustedExponent >= mozilla::FloorLog2(mozilla::Abs(lower_)));
 
         // The following are essentially static assertions, but FloorLog2 isn't
         // trivially suitable for constexpr :(.
-        JS_ASSERT(mozilla::FloorLog2(JSVAL_INT_MIN) == MaxInt32Exponent);
-        JS_ASSERT(mozilla::FloorLog2(JSVAL_INT_MAX) == 30);
-        JS_ASSERT(mozilla::FloorLog2(UINT32_MAX) == MaxUInt32Exponent);
-        JS_ASSERT(mozilla::FloorLog2(0) == 0);
+        MOZ_ASSERT(mozilla::FloorLog2(JSVAL_INT_MIN) == MaxInt32Exponent);
+        MOZ_ASSERT(mozilla::FloorLog2(JSVAL_INT_MAX) == 30);
+        MOZ_ASSERT(mozilla::FloorLog2(UINT32_MAX) == MaxUInt32Exponent);
+        MOZ_ASSERT(mozilla::FloorLog2(0) == 0);
     }
 
     // Set the lower_ and hasInt32LowerBound_ values.
@@ -259,7 +286,7 @@ class Range : public TempObject {
          // The number of bits needed to encode |max| is the power of 2 plus one.
          uint32_t max = Max(mozilla::Abs(lower()), mozilla::Abs(upper()));
          uint16_t result = mozilla::FloorLog2(max);
-         JS_ASSERT(result == (max == 0 ? 0 : mozilla::ExponentComponent(double(max))));
+         MOZ_ASSERT(result == (max == 0 ? 0 : mozilla::ExponentComponent(double(max))));
          return result;
     }
 
@@ -272,12 +299,17 @@ class Range : public TempObject {
     // Given an exponent value and pointers to the lower and upper bound values,
     // this function refines the lower and upper bound values to the tighest
     // bound for integer values implied by the exponent.
-    static void refineInt32BoundsByExponent(uint16_t e, int32_t *l, int32_t *h) {
+    static void refineInt32BoundsByExponent(uint16_t e,
+                                            int32_t *l, bool *lb,
+                                            int32_t *h, bool *hb)
+    {
        if (e < MaxInt32Exponent) {
            // pow(2, max_exponent_+1)-1 to compute a maximum absolute value.
            int32_t limit = (uint32_t(1) << (e + 1)) - 1;
            *h = Min(*h, limit);
            *l = Max(*l, -limit);
+           *hb = true;
+           *lb = true;
        }
     }
 
@@ -300,29 +332,43 @@ class Range : public TempObject {
             // If we have a completely precise range, the value is an integer,
             // since we can only represent integers.
             if (canHaveFractionalPart_ && lower_ == upper_) {
-                canHaveFractionalPart_ = false;
+                canHaveFractionalPart_ = ExcludesFractionalParts;
                 assertInvariants();
             }
         }
+
+        // If the range doesn't include zero, it doesn't include negative zero.
+        if (canBeNegativeZero_ && !canBeZero()) {
+            canBeNegativeZero_ = ExcludesNegativeZero;
+            assertInvariants();
+        }
     }
 
     // Set the range fields to the given raw values.
-    void rawInitialize(int32_t l, bool lb, int32_t h, bool hb, bool f, uint16_t e) {
+    void rawInitialize(int32_t l, bool lb, int32_t h, bool hb,
+                       FractionalPartFlag canHaveFractionalPart,
+                       NegativeZeroFlag canBeNegativeZero,
+                       uint16_t e)
+    {
         lower_ = l;
-        hasInt32LowerBound_ = lb;
         upper_ = h;
+        hasInt32LowerBound_ = lb;
         hasInt32UpperBound_ = hb;
-        canHaveFractionalPart_ = f;
+        canHaveFractionalPart_ = canHaveFractionalPart;
+        canBeNegativeZero_ = canBeNegativeZero;
         max_exponent_ = e;
         optimize();
     }
 
     // Construct a range from the given raw values.
-    Range(int32_t l, bool lb, int32_t h, bool hb, bool f, uint16_t e)
+    Range(int32_t l, bool lb, int32_t h, bool hb,
+          FractionalPartFlag canHaveFractionalPart,
+          NegativeZeroFlag canBeNegativeZero,
+          uint16_t e)
       : symbolicLower_(nullptr),
         symbolicUpper_(nullptr)
      {
-        rawInitialize(l, lb, h, hb, f, e);
+        rawInitialize(l, lb, h, hb, canHaveFractionalPart, canBeNegativeZero, e);
      }
 
   public:
@@ -333,19 +379,23 @@ class Range : public TempObject {
         setUnknown();
     }
 
-    Range(int64_t l, int64_t h, bool f = false, uint16_t e = MaxInt32Exponent)
+    Range(int64_t l, int64_t h,
+          FractionalPartFlag canHaveFractionalPart,
+          NegativeZeroFlag canBeNegativeZero,
+          uint16_t e)
       : symbolicLower_(nullptr),
         symbolicUpper_(nullptr)
     {
-        set(l, h, f, e);
+        set(l, h, canHaveFractionalPart, canBeNegativeZero, e);
     }
 
     Range(const Range &other)
       : lower_(other.lower_),
-        hasInt32LowerBound_(other.hasInt32LowerBound_),
         upper_(other.upper_),
+        hasInt32LowerBound_(other.hasInt32LowerBound_),
         hasInt32UpperBound_(other.hasInt32UpperBound_),
         canHaveFractionalPart_(other.canHaveFractionalPart_),
+        canBeNegativeZero_(other.canBeNegativeZero_),
         max_exponent_(other.max_exponent_),
         symbolicLower_(nullptr),
         symbolicUpper_(nullptr)
@@ -356,18 +406,27 @@ class Range : public TempObject {
     // Construct a range from the given MDefinition. This differs from the
     // MDefinition's range() method in that it describes the range of values
     // *after* any bailout checks.
-    Range(const MDefinition *def);
+    explicit Range(const MDefinition *def);
 
     static Range *NewInt32Range(TempAllocator &alloc, int32_t l, int32_t h) {
-        return new(alloc) Range(l, h, false, MaxInt32Exponent);
+        return new(alloc) Range(l, h, ExcludesFractionalParts, ExcludesNegativeZero, MaxInt32Exponent);
+    }
+
+    // Construct an int32 range containing just i. This is just a convenience
+    // wrapper around NewInt32Range.
+    static Range *NewInt32SingletonRange(TempAllocator &alloc, int32_t i) {
+        return NewInt32Range(alloc, i, i);
     }
 
     static Range *NewUInt32Range(TempAllocator &alloc, uint32_t l, uint32_t h) {
         // For now, just pass them to the constructor as int64_t values.
         // They'll become unbounded if they're not in the int32_t range.
-        return new(alloc) Range(l, h, false, MaxUInt32Exponent);
+        return new(alloc) Range(l, h, ExcludesFractionalParts, ExcludesNegativeZero, MaxUInt32Exponent);
     }
 
+    // Construct a range containing values >= l and <= h. Note that this
+    // function treats negative zero as equal to zero, as >= and <= do. If the
+    // range includes zero, it is assumed to include negative zero too.
     static Range *NewDoubleRange(TempAllocator &alloc, double l, double h) {
         if (mozilla::IsNaN(l) && mozilla::IsNaN(h))
             return nullptr;
@@ -377,6 +436,19 @@ class Range : public TempObject {
         return r;
     }
 
+    // Construct the strictest possible range containing d, or null if d is NaN.
+    // This function treats negative zero as distinct from zero, since this
+    // makes the strictest possible range containin zero a range which
+    // contains one value rather than two.
+    static Range *NewDoubleSingletonRange(TempAllocator &alloc, double d) {
+        if (mozilla::IsNaN(d))
+            return nullptr;
+
+        Range *r = new(alloc) Range();
+        r->setDoubleSingleton(d);
+        return r;
+    }
+
     void print(Sprinter &sp) const;
     void dump(FILE *fp) const;
     void dump() const;
@@ -405,6 +477,9 @@ class Range : public TempObject {
     static Range *abs(TempAllocator &alloc, const Range *op);
     static Range *min(TempAllocator &alloc, const Range *lhs, const Range *rhs);
     static Range *max(TempAllocator &alloc, const Range *lhs, const Range *rhs);
+    static Range *floor(TempAllocator &alloc, const Range *op);
+    static Range *ceil(TempAllocator &alloc, const Range *op);
+    static Range *sign(TempAllocator &alloc, const Range *op);
 
     static bool negativeZeroMul(const Range *lhs, const Range *rhs);
 
@@ -416,6 +491,7 @@ class Range : public TempObject {
         return !hasInt32LowerBound_ &&
                !hasInt32UpperBound_ &&
                canHaveFractionalPart_ &&
+               canBeNegativeZero_ &&
                max_exponent_ == IncludesInfinityAndNaN;
     }
 
@@ -434,21 +510,32 @@ class Range : public TempObject {
 
     // Test whether the value is known to be representable as an int32.
     bool isInt32() const {
-        return hasInt32Bounds() && !canHaveFractionalPart();
+        return hasInt32Bounds() &&
+               !canHaveFractionalPart_ &&
+               !canBeNegativeZero_;
     }
 
     // Test whether the given value is known to be either 0 or 1.
     bool isBoolean() const {
-        return lower() >= 0 && upper() <= 1 && !canHaveFractionalPart();
+        return lower() >= 0 && upper() <= 1 &&
+               !canHaveFractionalPart_ &&
+               !canBeNegativeZero_;
     }
 
     bool canHaveRoundingErrors() const {
-        return canHaveFractionalPart() || max_exponent_ >= MaxTruncatableExponent;
+        return canHaveFractionalPart_ ||
+               canBeNegativeZero_ ||
+               max_exponent_ >= MaxTruncatableExponent;
     }
 
-    // Test whether the range contains zero.
+    // Test if an integer x belongs to the range.
+    bool contains(int32_t x) const {
+        return x >= lower_ && x <= upper_;
+    }
+
+    // Test whether the range contains zero (of either sign).
     bool canBeZero() const {
-        return lower_ <= 0 && upper_ >= 0;
+        return contains(0);
     }
 
     // Test whether the range contains NaN values.
@@ -461,12 +548,16 @@ class Range : public TempObject {
         return max_exponent_ >= IncludesInfinity;
     }
 
-    bool canHaveFractionalPart() const {
+    FractionalPartFlag canHaveFractionalPart() const {
         return canHaveFractionalPart_;
     }
 
+    NegativeZeroFlag canBeNegativeZero() const {
+        return canBeNegativeZero_;
+    }
+
     uint16_t exponent() const {
-        JS_ASSERT(!canBeInfiniteOrNaN());
+        MOZ_ASSERT(!canBeInfiniteOrNaN());
         return max_exponent_;
     }
 
@@ -476,13 +567,13 @@ class Range : public TempObject {
 
     // Return the lower bound. Asserts that the value has an int32 bound.
     int32_t lower() const {
-        JS_ASSERT(hasInt32LowerBound());
+        MOZ_ASSERT(hasInt32LowerBound());
         return lower_;
     }
 
     // Return the upper bound. Asserts that the value has an int32 bound.
     int32_t upper() const {
-        JS_ASSERT(hasInt32UpperBound());
+        MOZ_ASSERT(hasInt32UpperBound());
         return upper_;
     }
 
@@ -497,7 +588,7 @@ class Range : public TempObject {
     }
 
     // Test whether a value in this range can possibly be a finite
-    // negative value.
+    // negative value. Note that "negative zero" is not considered negative.
     bool canBeFiniteNegative() const {
         return lower_ < 0;
     }
@@ -508,6 +599,12 @@ class Range : public TempObject {
         return upper_ >= 0;
     }
 
+    // Test whether a value in this range can have the sign bit set (not
+    // counting NaN, where the sign bit is meaningless).
+    bool canHaveSignBitSet() const {
+        return !hasInt32LowerBound() || canBeFiniteNegative() || canBeNegativeZero();
+    }
+
     // Set this range to have a lower bound not less than x.
     void refineLower(int32_t x) {
         assertInvariants();
@@ -524,26 +621,51 @@ class Range : public TempObject {
         optimize();
     }
 
+    // Set this range to exclude negative zero.
+    void refineToExcludeNegativeZero() {
+        assertInvariants();
+        canBeNegativeZero_ = ExcludesNegativeZero;
+        optimize();
+    }
+
     void setInt32(int32_t l, int32_t h) {
         hasInt32LowerBound_ = true;
         hasInt32UpperBound_ = true;
         lower_ = l;
         upper_ = h;
-        canHaveFractionalPart_ = false;
+        canHaveFractionalPart_ = ExcludesFractionalParts;
+        canBeNegativeZero_ = ExcludesNegativeZero;
         max_exponent_ = exponentImpliedByInt32Bounds();
         assertInvariants();
     }
 
+    // Set this range to include values >= l and <= h. Note that this
+    // function treats negative zero as equal to zero, as >= and <= do. If the
+    // range includes zero, it is assumed to include negative zero too.
     void setDouble(double l, double h);
 
+    // Set this range to the narrowest possible range containing d.
+    // This function treats negative zero as distinct from zero, since this
+    // makes the narrowest possible range containin zero a range which
+    // contains one value rather than two.
+    void setDoubleSingleton(double d);
+
     void setUnknown() {
-        set(NoInt32LowerBound, NoInt32UpperBound, true, IncludesInfinityAndNaN);
-        JS_ASSERT(isUnknown());
+        set(NoInt32LowerBound, NoInt32UpperBound,
+            IncludesFractionalParts,
+            IncludesNegativeZero,
+            IncludesInfinityAndNaN);
+        MOZ_ASSERT(isUnknown());
     }
 
-    void set(int64_t l, int64_t h, bool f, uint16_t e) {
+    void set(int64_t l, int64_t h,
+             FractionalPartFlag canHaveFractionalPart,
+             NegativeZeroFlag canBeNegativeZero,
+             uint16_t e)
+    {
         max_exponent_ = e;
-        canHaveFractionalPart_ = f;
+        canHaveFractionalPart_ = canHaveFractionalPart;
+        canBeNegativeZero_ = canBeNegativeZero;
         setLowerInit(l);
         setUpperInit(h);
         optimize();
