@@ -9,7 +9,7 @@
 #include "mozilla/MathAlgorithms.h"
 
 #include "jit/BitSet.h"
-#include "jit/IonSpewer.h"
+#include "jit/JitSpewer.h"
 #include "jit/LIR.h"
 
 using namespace js;
@@ -17,20 +17,20 @@ using namespace jit;
 
 using mozilla::FloorLog2;
 
-bool
-SafepointWriter::init(TempAllocator &alloc, uint32_t slotCount)
-{
-    frameSlots_ = BitSet::New(alloc, slotCount / sizeof(intptr_t));
-    if (!frameSlots_)
-        return false;
+SafepointWriter::SafepointWriter(uint32_t slotCount)
+  : frameSlots_(slotCount / sizeof(intptr_t))
+{ }
 
-    return true;
+bool
+SafepointWriter::init(TempAllocator &alloc)
+{
+    return frameSlots_.init(alloc);
 }
 
 uint32_t
 SafepointWriter::startEntry()
 {
-    IonSpew(IonSpew_Safepoints, "Encoding safepoint (position %d):", stream_.length());
+    JitSpew(JitSpew_Safepoints, "Encoding safepoint (position %d):", stream_.length());
     return uint32_t(stream_.length());
 }
 
@@ -65,7 +65,7 @@ WriteFloatRegisterMask(CompactBufferWriter &stream, uint64_t bits)
     } else if (sizeof(FloatRegisters::SetType) == 4) {
         stream.writeUnsigned(bits);
     } else {
-        JS_ASSERT(sizeof(FloatRegisters::SetType) == 8);
+        MOZ_ASSERT(sizeof(FloatRegisters::SetType) == 8);
         stream.writeUnsigned(bits & 0xffffffff);
         stream.writeUnsigned(bits >> 32);
     }
@@ -78,7 +78,7 @@ ReadFloatRegisterMask(CompactBufferReader &stream)
         return stream.readByte();
     if (sizeof(FloatRegisters::SetType) <= 4)
         return stream.readUnsigned();
-    JS_ASSERT(sizeof(FloatRegisters::SetType) == 8);
+    MOZ_ASSERT(sizeof(FloatRegisters::SetType) == 8);
     uint64_t ret = stream.readUnsigned();
     ret |= uint64_t(stream.readUnsigned()) << 32;
     return ret;
@@ -105,13 +105,13 @@ SafepointWriter::writeGcRegs(LSafepoint *safepoint)
     }
 
     // GC registers are a subset of the spilled registers.
-    JS_ASSERT((valueRegs.bits() & ~spilledGpr.bits()) == 0);
-    JS_ASSERT((gc.bits() & ~spilledGpr.bits()) == 0);
+    MOZ_ASSERT((valueRegs.bits() & ~spilledGpr.bits()) == 0);
+    MOZ_ASSERT((gc.bits() & ~spilledGpr.bits()) == 0);
 
     WriteFloatRegisterMask(stream_, spilledFloat.bits());
 
 #ifdef DEBUG
-    if (IonSpewEnabled(IonSpew_Safepoints)) {
+    if (JitSpewEnabled(JitSpew_Safepoints)) {
         for (GeneralRegisterForwardIterator iter(spilledGpr); iter.more(); iter++) {
             const char *type = gc.has(*iter)
                                ? "gc"
@@ -120,31 +120,31 @@ SafepointWriter::writeGcRegs(LSafepoint *safepoint)
                                  : valueRegs.has(*iter)
                                    ? "value"
                                    : "any";
-            IonSpew(IonSpew_Safepoints, "    %s reg: %s", type, (*iter).name());
+            JitSpew(JitSpew_Safepoints, "    %s reg: %s", type, (*iter).name());
         }
         for (FloatRegisterForwardIterator iter(spilledFloat); iter.more(); iter++)
-            IonSpew(IonSpew_Safepoints, "    float reg: %s", (*iter).name());
+            JitSpew(JitSpew_Safepoints, "    float reg: %s", (*iter).name());
     }
 #endif
 }
 
 static void
-MapSlotsToBitset(BitSet *set, CompactBufferWriter &stream, uint32_t nslots, uint32_t *slots)
+MapSlotsToBitset(BitSet &set, CompactBufferWriter &stream, uint32_t nslots, uint32_t *slots)
 {
-    set->clear();
+    set.clear();
 
     for (uint32_t i = 0; i < nslots; i++) {
         // Slots are represented at a distance from |fp|. We divide by the
         // pointer size, since we only care about pointer-sized/aligned slots
         // here. Since the stack grows down, this means slots start at index 1,
         // so we subtract 1 to pack the bitset.
-        JS_ASSERT(slots[i] % sizeof(intptr_t) == 0);
-        JS_ASSERT(slots[i] / sizeof(intptr_t) > 0);
-        set->insert(slots[i] / sizeof(intptr_t) - 1);
+        MOZ_ASSERT(slots[i] % sizeof(intptr_t) == 0);
+        MOZ_ASSERT(slots[i] / sizeof(intptr_t) > 0);
+        set.insert(slots[i] / sizeof(intptr_t) - 1);
     }
 
-    size_t count = set->rawLength();
-    const uint32_t *words = set->raw();
+    size_t count = set.rawLength();
+    const uint32_t *words = set.raw();
     for (size_t i = 0; i < count; i++)
         stream.writeUnsigned(words[i]);
 }
@@ -156,7 +156,7 @@ SafepointWriter::writeGcSlots(LSafepoint *safepoint)
 
 #ifdef DEBUG
     for (uint32_t i = 0; i < slots.length(); i++)
-        IonSpew(IonSpew_Safepoints, "    gc slot: %d", slots[i]);
+        JitSpew(JitSpew_Safepoints, "    gc slot: %d", slots[i]);
 #endif
 
     MapSlotsToBitset(frameSlots_,
@@ -174,7 +174,7 @@ SafepointWriter::writeSlotsOrElementsSlots(LSafepoint *safepoint)
 
     for (uint32_t i = 0; i < slots.length(); i++) {
 #ifdef DEBUG
-        IonSpew(IonSpew_Safepoints, "    slots/elements slot: %d", slots[i]);
+        JitSpew(JitSpew_Safepoints, "    slots/elements slot: %d", slots[i]);
 #endif
         stream_.writeUnsigned(slots[i]);
     }
@@ -187,7 +187,7 @@ SafepointWriter::writeValueSlots(LSafepoint *safepoint)
 
 #ifdef DEBUG
     for (uint32_t i = 0; i < slots.length(); i++)
-        IonSpew(IonSpew_Safepoints, "    gc value: %d", slots[i]);
+        JitSpew(JitSpew_Safepoints, "    gc value: %d", slots[i]);
 #endif
 
     MapSlotsToBitset(frameSlots_, stream_, slots.length(), slots.begin());
@@ -198,11 +198,11 @@ static void
 DumpNunboxPart(const LAllocation &a)
 {
     if (a.isStackSlot()) {
-        fprintf(IonSpewFile, "stack %d", a.toStackSlot()->slot());
+        fprintf(JitSpewFile, "stack %d", a.toStackSlot()->slot());
     } else if (a.isArgument()) {
-        fprintf(IonSpewFile, "arg %d", a.toArgument()->index());
+        fprintf(JitSpewFile, "arg %d", a.toArgument()->index());
     } else {
-        fprintf(IonSpewFile, "reg %s", a.toGeneralReg()->reg().name());
+        fprintf(JitSpewFile, "reg %s", a.toGeneralReg()->reg().name());
     }
 }
 #endif // DEBUG
@@ -251,7 +251,7 @@ AllocationToPartKind(const LAllocation &a)
         return Part_Reg;
     if (a.isStackSlot())
         return Part_Stack;
-    JS_ASSERT(a.isArgument());
+    MOZ_ASSERT(a.isArgument());
     return Part_Arg;
 }
 
@@ -281,17 +281,17 @@ SafepointWriter::writeNunboxParts(LSafepoint *safepoint)
     LSafepoint::NunboxList &entries = safepoint->nunboxParts();
 
 # ifdef DEBUG
-    if (IonSpewEnabled(IonSpew_Safepoints)) {
+    if (JitSpewEnabled(JitSpew_Safepoints)) {
         for (uint32_t i = 0; i < entries.length(); i++) {
             SafepointNunboxEntry &entry = entries[i];
             if (entry.type.isUse() || entry.payload.isUse())
                 continue;
-            IonSpewHeader(IonSpew_Safepoints);
-            fprintf(IonSpewFile, "    nunbox (type in ");
+            JitSpewHeader(JitSpew_Safepoints);
+            fprintf(JitSpewFile, "    nunbox (type in ");
             DumpNunboxPart(entry.type);
-            fprintf(IonSpewFile, ", payload in ");
+            fprintf(JitSpewFile, ", payload in ");
             DumpNunboxPart(entry.payload);
-            fprintf(IonSpewFile, ")\n");
+            fprintf(JitSpewFile, ")\n");
         }
     }
 # endif
@@ -299,18 +299,29 @@ SafepointWriter::writeNunboxParts(LSafepoint *safepoint)
     // Safepoints are permitted to have partially filled in entries for nunboxes,
     // provided that only the type is live and not the payload. Omit these from
     // the written safepoint.
-    uint32_t partials = safepoint->partialNunboxes();
 
-    stream_.writeUnsigned(entries.length() - partials);
+    size_t pos = stream_.length();
+    stream_.writeUnsigned(entries.length());
 
+    size_t count = 0;
     for (size_t i = 0; i < entries.length(); i++) {
         SafepointNunboxEntry &entry = entries[i];
 
-        if (entry.type.isUse() || entry.payload.isUse()) {
-            partials--;
+        if (entry.payload.isUse()) {
+            // No allocation associated with the payload.
             continue;
         }
 
+        if (entry.type.isUse()) {
+            // No allocation associated with the type. Look for another
+            // safepoint entry with an allocation for the type.
+            entry.type = safepoint->findTypeAllocation(entry.typeVreg);
+            if (entry.type.isUse())
+                continue;
+        }
+
+        count++;
+
         uint16_t header = 0;
 
         header |= (AllocationToPartKind(entry.type) << TYPE_KIND_SHIFT);
@@ -337,7 +348,8 @@ SafepointWriter::writeNunboxParts(LSafepoint *safepoint)
             stream_.writeUnsigned(payloadVal);
     }
 
-    JS_ASSERT(partials == 0);
+    // Update the stream with the actual number of safepoint entries written.
+    stream_.writeUnsignedAt(pos, count, entries.length());
 }
 #endif
 
@@ -346,7 +358,7 @@ SafepointWriter::encode(LSafepoint *safepoint)
 {
     uint32_t safepointOffset = startEntry();
 
-    JS_ASSERT(safepoint->osiCallPointOffset());
+    MOZ_ASSERT(safepoint->osiCallPointOffset());
 
     writeOsiCallPointOffset(safepoint->osiCallPointOffset());
     writeGcRegs(safepoint);
@@ -366,7 +378,7 @@ SafepointWriter::encode(LSafepoint *safepoint)
 void
 SafepointWriter::endEntry()
 {
-    IonSpew(IonSpew_Safepoints, "    -- entry ended at %d", uint32_t(stream_.length()));
+    JitSpew(JitSpew_Safepoints, "    -- entry ended at %d", uint32_t(stream_.length()));
 }
 
 SafepointReader::SafepointReader(IonScript *script, const SafepointIndex *si)
@@ -489,7 +501,7 @@ PartFromStream(CompactBufferReader &stream, NunboxPartKind kind, uint32_t info)
     if (kind == Part_Stack)
         return LStackSlot(info);
 
-    JS_ASSERT(kind == Part_Arg);
+    MOZ_ASSERT(kind == Part_Arg);
     return LArgument(info);
 }
 
