@@ -10,7 +10,7 @@
 #include "jit/BaselineFrameInfo.h"
 #include "jit/BaselineIC.h"
 #include "jit/BytecodeAnalysis.h"
-#include "jit/IonMacroAssembler.h"
+#include "jit/MacroAssembler.h"
 
 namespace js {
 namespace jit {
@@ -19,12 +19,12 @@ class BaselineCompilerShared
 {
   protected:
     JSContext *cx;
-    RootedScript script;
+    JSScript *script;
     jsbytecode *pc;
     MacroAssembler masm;
     bool ionCompileable_;
     bool ionOSRCompileable_;
-    bool debugMode_;
+    bool compileDebugInstrumentation_;
 
     TempAllocator &alloc_;
     BytecodeAnalysis analysis_;
@@ -47,7 +47,7 @@ class BaselineCompilerShared
         void fixupNativeOffset(MacroAssembler &masm) {
             CodeOffsetLabel offset(nativeOffset);
             offset.fixup(&masm);
-            JS_ASSERT(offset.offset() <= UINT32_MAX);
+            MOZ_ASSERT(offset.offset() <= UINT32_MAX);
             nativeOffset = (uint32_t) offset.offset();
         }
     };
@@ -68,17 +68,20 @@ class BaselineCompilerShared
     mozilla::DebugOnly<bool> inCall_;
 
     CodeOffsetLabel spsPushToggleOffset_;
+    CodeOffsetLabel traceLoggerEnterToggleOffset_;
+    CodeOffsetLabel traceLoggerExitToggleOffset_;
+    CodeOffsetLabel traceLoggerScriptTextIdOffset_;
 
-    BaselineCompilerShared(JSContext *cx, TempAllocator &alloc, HandleScript script);
+    BaselineCompilerShared(JSContext *cx, TempAllocator &alloc, JSScript *script);
 
-    ICEntry *allocateICEntry(ICStub *stub, bool isForOp) {
+    ICEntry *allocateICEntry(ICStub *stub, ICEntry::Kind kind) {
         if (!stub)
             return nullptr;
 
         // Create the entry and add it to the vector.
-        if (!icEntries_.append(ICEntry(script->pcToOffset(pc), isForOp)))
+        if (!icEntries_.append(ICEntry(script->pcToOffset(pc), kind)))
             return nullptr;
-        ICEntry &vecEntry = icEntries_[icEntries_.length() - 1];
+        ICEntry &vecEntry = icEntries_.back();
 
         // Set the first stub for the IC entry to the fallback stub
         vecEntry.setFirstStub(stub);
@@ -87,8 +90,15 @@ class BaselineCompilerShared
         return &vecEntry;
     }
 
+    // Append an ICEntry without a stub.
+    bool appendICEntry(ICEntry::Kind kind, uint32_t returnOffset) {
+        ICEntry entry(script->pcToOffset(pc), kind);
+        entry.setReturnOffset(CodeOffsetLabel(returnOffset));
+        return icEntries_.append(entry);
+    }
+
     bool addICLoadLabel(CodeOffsetLabel label) {
-        JS_ASSERT(!icEntries_.empty());
+        MOZ_ASSERT(!icEntries_.empty());
         ICLoadLabel loadLabel;
         loadLabel.label = label;
         loadLabel.icEntry = icEntries_.length() - 1;
@@ -96,11 +106,13 @@ class BaselineCompilerShared
     }
 
     JSFunction *function() const {
-        return script->function();
+        // Not delazifying here is ok as the function is guaranteed to have
+        // been delazified before compilation started.
+        return script->functionNonDelazifying();
     }
 
     PCMappingSlotInfo getStackTopSlotInfo() {
-        JS_ASSERT(frame.numUnsyncedSlots() <= 2);
+        MOZ_ASSERT(frame.numUnsyncedSlots() <= 2);
         switch (frame.numUnsyncedSlots()) {
           case 0:
             return PCMappingSlotInfo::MakeSlotInfo();
@@ -135,10 +147,21 @@ class BaselineCompilerShared
     };
     bool callVM(const VMFunction &fun, CallVMPhase phase=POST_INITIALIZE);
 
+    bool callVMNonOp(const VMFunction &fun, CallVMPhase phase=POST_INITIALIZE) {
+        if (!callVM(fun, phase))
+            return false;
+        icEntries_.back().setFakeKind(ICEntry::Kind_NonOpCallVM);
+        return true;
+    }
+
   public:
     BytecodeAnalysis &analysis() {
         return analysis_;
     }
+
+    void setCompileDebugInstrumentation() {
+        compileDebugInstrumentation_ = true;
+    }
 };
 
 } // namespace jit
