@@ -9,15 +9,14 @@
 
 #include "mozilla/Alignment.h"
 
-#include "jit/IonFrames.h"
-#include "jit/IonMacroAssembler.h"
+#include "jit/JitFrames.h"
 #include "jit/LIR.h"
+#include "jit/MacroAssembler.h"
 #include "jit/MIRGenerator.h"
 #include "jit/MIRGraph.h"
 #include "jit/Safepoints.h"
 #include "jit/Snapshots.h"
 #include "jit/VMFunctions.h"
-#include "vm/ForkJoin.h"
 
 namespace js {
 namespace jit {
@@ -48,7 +47,7 @@ struct ReciprocalMulConstants {
     int32_t shiftAmount;
 };
 
-class CodeGeneratorShared : public LInstructionVisitor
+class CodeGeneratorShared : public LElementVisitor
 {
     js::Vector<OutOfLineCode *, 0, SystemAllocPolicy> outOfLineCode_;
     OutOfLineCode *oolIns;
@@ -97,11 +96,32 @@ class CodeGeneratorShared : public LInstructionVisitor
     js::Vector<CodeOffsetLabel, 0, SystemAllocPolicy> patchableTLScripts_;
 #endif
 
+  public:
+    struct NativeToBytecode {
+        CodeOffsetLabel nativeOffset;
+        InlineScriptTree *tree;
+        jsbytecode *pc;
+    };
+
+  protected:
+    js::Vector<NativeToBytecode, 0, SystemAllocPolicy> nativeToBytecodeList_;
+    uint8_t *nativeToBytecodeMap_;
+    uint32_t nativeToBytecodeMapSize_;
+    uint32_t nativeToBytecodeTableOffset_;
+    uint32_t nativeToBytecodeNumRegions_;
+
+    JSScript **nativeToBytecodeScriptList_;
+    uint32_t nativeToBytecodeScriptListLength_;
+
     // When profiling is enabled, this is the instrumentation manager which
     // maintains state of what script is currently being generated (for inline
     // scripts) and when instrumentation needs to be emitted or skipped.
     IonInstrumentation sps_;
 
+    bool isNativeToBytecodeMapEnabled() {
+        return gen->isNativeToBytecodeMapEnabled();
+    }
+
   protected:
     // The offset of the first instruction of the OSR entry block from the
     // beginning of the code buffer.
@@ -112,7 +132,7 @@ class CodeGeneratorShared : public LInstructionVisitor
     }
 
     inline void setOsrEntryOffset(size_t offset) {
-        JS_ASSERT(osrEntryOffset_ == 0);
+        MOZ_ASSERT(osrEntryOffset_ == 0);
         osrEntryOffset_ = offset;
     }
     inline size_t getOsrEntryOffset() const {
@@ -124,7 +144,7 @@ class CodeGeneratorShared : public LInstructionVisitor
     size_t skipArgCheckEntryOffset_;
 
     inline void setSkipArgCheckEntryOffset(size_t offset) {
-        JS_ASSERT(skipArgCheckEntryOffset_ == 0);
+        MOZ_ASSERT(skipArgCheckEntryOffset_ == 0);
         skipArgCheckEntryOffset_ = offset;
     }
     inline size_t getSkipArgCheckEntryOffset() const {
@@ -137,6 +157,13 @@ class CodeGeneratorShared : public LInstructionVisitor
     void dropArguments(unsigned argc);
 
   protected:
+#ifdef CHECK_OSIPOINT_REGISTERS
+    // See js_JitOptions.checkOsiPointRegisters. We set this here to avoid
+    // races when enableOsiPointRegisterChecks is called while we're generating
+    // code off-thread.
+    bool checkOsiPointRegisters;
+#endif
+
     // The initial size of the frame in bytes. These are bytes beyond the
     // constant header present for every Ion frame, used for pre-determined
     // spills.
@@ -153,19 +180,19 @@ class CodeGeneratorShared : public LInstructionVisitor
     // For arguments to the current function.
     inline int32_t ArgToStackOffset(int32_t slot) const {
         return masm.framePushed() +
-               (gen->compilingAsmJS() ? AsmJSFrameSize : sizeof(IonJSFrameLayout)) +
+               (gen->compilingAsmJS() ? sizeof(AsmJSFrame) : sizeof(JitFrameLayout)) +
                slot;
     }
 
     // For the callee of the current function.
     inline int32_t CalleeStackOffset() const {
-        return masm.framePushed() + IonJSFrameLayout::offsetOfCalleeToken();
+        return masm.framePushed() + JitFrameLayout::offsetOfCalleeToken();
     }
 
     inline int32_t SlotToStackOffset(int32_t slot) const {
-        JS_ASSERT(slot > 0 && slot <= int32_t(graph.localSlotCount()));
+        MOZ_ASSERT(slot > 0 && slot <= int32_t(graph.localSlotCount()));
         int32_t offset = masm.framePushed() - frameInitialAdjustment_ - slot;
-        JS_ASSERT(offset >= 0);
+        MOZ_ASSERT(offset >= 0);
         return offset;
     }
     inline int32_t StackOffsetToSlot(int32_t offset) const {
@@ -181,7 +208,7 @@ class CodeGeneratorShared : public LInstructionVisitor
     // For argument construction for calls. Argslots are Value-sized.
     inline int32_t StackOffsetOfPassedArg(int32_t slot) const {
         // A slot of 0 is permitted only to calculate %esp offset for calls.
-        JS_ASSERT(slot >= 0 && slot <= int32_t(graph.argumentSlotCount()));
+        MOZ_ASSERT(slot >= 0 && slot <= int32_t(graph.argumentSlotCount()));
         int32_t offset = masm.framePushed() -
                        graph.paddedLocalSlotsSize() -
                        (slot * sizeof(Value));
@@ -192,8 +219,8 @@ class CodeGeneratorShared : public LInstructionVisitor
         // by sizeof(Value) is desirable since everything on the stack is a Value.
         // Note that paddedLocalSlotCount() aligns to at least a Value boundary
         // specifically to support this.
-        JS_ASSERT(offset >= 0);
-        JS_ASSERT(offset % sizeof(Value) == 0);
+        MOZ_ASSERT(offset >= 0);
+        MOZ_ASSERT(offset % sizeof(Value) == 0);
         return offset;
     }
 
@@ -223,7 +250,14 @@ class CodeGeneratorShared : public LInstructionVisitor
     void verifyOsiPointRegs(LSafepoint *safepoint);
 #endif
 
+    bool addNativeToBytecodeEntry(const BytecodeSite *site);
+    void dumpNativeToBytecodeEntries();
+    void dumpNativeToBytecodeEntry(uint32_t idx);
+
   public:
+    MIRGenerator &mirGen() const {
+        return *gen;
+    }
 
     // When appending to runtimeData_, the vector might realloc, leaving pointers
     // int the origianl vector stale and unusable. DataPtr acts like a pointer,
@@ -253,7 +287,7 @@ class CodeGeneratorShared : public LInstructionVisitor
   protected:
 
     size_t allocateData(size_t size) {
-        JS_ASSERT(size % sizeof(void *) == 0);
+        MOZ_ASSERT(size % sizeof(void *) == 0);
         size_t dataOffset = runtimeData_.length();
         masm.propagateOOM(runtimeData_.appendN(0, size));
         return dataOffset;
@@ -265,17 +299,16 @@ class CodeGeneratorShared : public LInstructionVisitor
         if (masm.oom())
             return SIZE_MAX;
         // Use the copy constructor on the allocated space.
-        JS_ASSERT(index == cacheList_.back());
+        MOZ_ASSERT(index == cacheList_.back());
         new (&runtimeData_[index]) T(cache);
         return index;
     }
 
   protected:
-    // Encodes an LSnapshot into the compressed snapshot buffer, returning
-    // false on failure.
-    bool encode(LRecoverInfo *recover);
-    bool encode(LSnapshot *snapshot);
-    bool encodeAllocation(LSnapshot *snapshot, MDefinition *def, uint32_t *startIndex);
+    // Encodes an LSnapshot into the compressed snapshot buffer.
+    void encode(LRecoverInfo *recover);
+    void encode(LSnapshot *snapshot);
+    void encodeAllocation(LSnapshot *snapshot, MDefinition *def, uint32_t *startIndex);
 
     // Attempts to assign a BailoutId to a snapshot, if one isn't already set.
     // If the bailout table is full, this returns false, which is not a fatal
@@ -286,16 +319,20 @@ class CodeGeneratorShared : public LInstructionVisitor
     // safepoint offsets.
     void encodeSafepoints();
 
+    // Fixup offsets of native-to-bytecode map.
+    bool createNativeToBytecodeScriptList(JSContext *cx);
+    bool generateCompactNativeToBytecodeMap(JSContext *cx, JitCode *code);
+    void verifyCompactNativeToBytecodeMap(JitCode *code);
+
     // Mark the safepoint on |ins| as corresponding to the current assembler location.
     // The location should be just after a call.
-    bool markSafepoint(LInstruction *ins);
-    bool markSafepointAt(uint32_t offset, LInstruction *ins);
+    void markSafepoint(LInstruction *ins);
+    void markSafepointAt(uint32_t offset, LInstruction *ins);
 
     // Mark the OSI point |ins| as corresponding to the current
     // assembler location inside the |osiIndices_|. Return the assembler
-    // location for the OSI point return location within
-    // |returnPointOffset|.
-    bool markOsiPoint(LOsiPoint *ins, uint32_t *returnPointOffset);
+    // location for the OSI point return location.
+    uint32_t markOsiPoint(LOsiPoint *ins);
 
     // Ensure that there is enough room between the last OSI point and the
     // current instruction, such that:
@@ -304,19 +341,21 @@ class CodeGeneratorShared : public LInstructionVisitor
     //      an invalidation marker.
     void ensureOsiSpace();
 
-    OutOfLineCode *oolTruncateDouble(FloatRegister src, Register dest);
-    bool emitTruncateDouble(FloatRegister src, Register dest);
-    bool emitTruncateFloat32(FloatRegister src, Register dest);
+    OutOfLineCode *oolTruncateDouble(FloatRegister src, Register dest, MInstruction *mir);
+    void emitTruncateDouble(FloatRegister src, Register dest, MInstruction *mir);
+    void emitTruncateFloat32(FloatRegister src, Register dest, MInstruction *mir);
 
-    void emitPreBarrier(Register base, const LAllocation *index, MIRType type);
-    void emitPreBarrier(Address address, MIRType type);
+    void emitAsmJSCall(LAsmJSCall *ins);
+
+    void emitPreBarrier(Register base, const LAllocation *index);
+    void emitPreBarrier(Address address);
 
     // We don't emit code for trivial blocks, so if we want to branch to the
     // given block, and it's trivial, return the ultimate block we should
     // actually branch directly to.
     MBasicBlock *skipTrivialBlocks(MBasicBlock *block) {
         while (block->lir()->isTrivial()) {
-            JS_ASSERT(block->lir()->rbegin()->numSuccessors() == 1);
+            MOZ_ASSERT(block->lir()->rbegin()->numSuccessors() == 1);
             block = block->lir()->rbegin()->getSuccessor(0);
         }
         return block;
@@ -413,14 +452,14 @@ class CodeGeneratorShared : public LInstructionVisitor
         masm.storeCallResultValue(t);
     }
 
-    bool callVM(const VMFunction &f, LInstruction *ins, const Register *dynStack = nullptr);
+    void callVM(const VMFunction &f, LInstruction *ins, const Register *dynStack = nullptr);
 
     template <class ArgSeq, class StoreOutputTo>
     inline OutOfLineCode *oolCallVM(const VMFunction &fun, LInstruction *ins, const ArgSeq &args,
                                     const StoreOutputTo &out);
 
-    bool callVM(const VMFunctionsModal &f, LInstruction *ins, const Register *dynStack = nullptr) {
-        return callVM(f[gen->info().executionMode()], ins, dynStack);
+    void callVM(const VMFunctionsModal &f, LInstruction *ins, const Register *dynStack = nullptr) {
+        callVM(f[gen->info().executionMode()], ins, dynStack);
     }
 
     template <class ArgSeq, class StoreOutputTo>
@@ -430,12 +469,13 @@ class CodeGeneratorShared : public LInstructionVisitor
         return oolCallVM(f[gen->info().executionMode()], ins, args, out);
     }
 
-    bool addCache(LInstruction *lir, size_t cacheIndex);
+    void addCache(LInstruction *lir, size_t cacheIndex);
     size_t addCacheLocations(const CacheLocationList &locs, size_t *numLocs);
     ReciprocalMulConstants computeDivisionConstants(int d);
 
   protected:
-    bool addOutOfLineCode(OutOfLineCode *code);
+    void addOutOfLineCode(OutOfLineCode *code, const MInstruction *mir);
+    void addOutOfLineCode(OutOfLineCode *code, const BytecodeSite *site);
     bool hasOutOfLineCode() { return !outOfLineCode_.empty(); }
     bool generateOutOfLineCode();
 
@@ -460,29 +500,29 @@ class CodeGeneratorShared : public LInstructionVisitor
 
   public:
     template <class ArgSeq, class StoreOutputTo>
-    bool visitOutOfLineCallVM(OutOfLineCallVM<ArgSeq, StoreOutputTo> *ool);
+    void visitOutOfLineCallVM(OutOfLineCallVM<ArgSeq, StoreOutputTo> *ool);
 
-    bool visitOutOfLineTruncateSlow(OutOfLineTruncateSlow *ool);
+    void visitOutOfLineTruncateSlow(OutOfLineTruncateSlow *ool);
 
     bool omitOverRecursedCheck() const;
 
 #ifdef JS_TRACE_LOGGING
   protected:
-    bool emitTracelogScript(bool isStart);
-    bool emitTracelogTree(bool isStart, uint32_t textId);
+    void emitTracelogScript(bool isStart);
+    void emitTracelogTree(bool isStart, uint32_t textId);
 
   public:
-    bool emitTracelogScriptStart() {
-        return emitTracelogScript(/* isStart =*/ true);
+    void emitTracelogScriptStart() {
+        emitTracelogScript(/* isStart =*/ true);
     }
-    bool emitTracelogScriptStop() {
-        return emitTracelogScript(/* isStart =*/ false);
+    void emitTracelogScriptStop() {
+        emitTracelogScript(/* isStart =*/ false);
     }
-    bool emitTracelogStartEvent(uint32_t textId) {
-        return emitTracelogTree(/* isStart =*/ true, textId);
+    void emitTracelogStartEvent(uint32_t textId) {
+        emitTracelogTree(/* isStart =*/ true, textId);
     }
-    bool emitTracelogStopEvent(uint32_t textId) {
-        return emitTracelogTree(/* isStart =*/ false, textId);
+    void emitTracelogStopEvent(uint32_t textId) {
+        emitTracelogTree(/* isStart =*/ false, textId);
     }
 #endif
 };
@@ -493,17 +533,15 @@ class OutOfLineCode : public TempObject
     Label entry_;
     Label rejoin_;
     uint32_t framePushed_;
-    jsbytecode *pc_;
-    JSScript *script_;
+    const BytecodeSite *site_;
 
   public:
     OutOfLineCode()
       : framePushed_(0),
-        pc_(nullptr),
-        script_(nullptr)
+        site_()
     { }
 
-    virtual bool generate(CodeGeneratorShared *codegen) = 0;
+    virtual void generate(CodeGeneratorShared *codegen) = 0;
 
     Label *entry() {
         return &entry_;
@@ -520,15 +558,17 @@ class OutOfLineCode : public TempObject
     uint32_t framePushed() const {
         return framePushed_;
     }
-    void setSource(JSScript *script, jsbytecode *pc) {
-        script_ = script;
-        pc_ = pc;
+    void setBytecodeSite(const BytecodeSite *site) {
+        site_ = site;
     }
-    jsbytecode *pc() {
-        return pc_;
+    const BytecodeSite *bytecodeSite() const {
+        return site_;
     }
-    JSScript *script() {
-        return script_;
+    jsbytecode *pc() const {
+        return site_->pc();
+    }
+    JSScript *script() const {
+        return site_->script();
     }
 };
 
@@ -537,12 +577,12 @@ template <typename T>
 class OutOfLineCodeBase : public OutOfLineCode
 {
   public:
-    virtual bool generate(CodeGeneratorShared *codegen) {
-        return accept(static_cast<T *>(codegen));
+    virtual void generate(CodeGeneratorShared *codegen) {
+        accept(static_cast<T *>(codegen));
     }
 
   public:
-    virtual bool accept(T *codegen) = 0;
+    virtual void accept(T *codegen) = 0;
 };
 
 // ArgSeq store arguments for OutOfLineCallVM.
@@ -709,8 +749,8 @@ class OutOfLineCallVM : public OutOfLineCodeBase<CodeGeneratorShared>
         out_(out)
     { }
 
-    bool accept(CodeGeneratorShared *codegen) {
-        return codegen->visitOutOfLineCallVM(this);
+    void accept(CodeGeneratorShared *codegen) {
+        codegen->visitOutOfLineCallVM(this);
     }
 
     LInstruction *lir() const { return lir_; }
@@ -724,26 +764,26 @@ inline OutOfLineCode *
 CodeGeneratorShared::oolCallVM(const VMFunction &fun, LInstruction *lir, const ArgSeq &args,
                                const StoreOutputTo &out)
 {
+    MOZ_ASSERT(lir->mirRaw());
+    MOZ_ASSERT(lir->mirRaw()->isInstruction());
+
     OutOfLineCode *ool = new(alloc()) OutOfLineCallVM<ArgSeq, StoreOutputTo>(lir, fun, args, out);
-    if (!addOutOfLineCode(ool))
-        return nullptr;
+    addOutOfLineCode(ool, lir->mirRaw()->toInstruction());
     return ool;
 }
 
 template <class ArgSeq, class StoreOutputTo>
-bool
+void
 CodeGeneratorShared::visitOutOfLineCallVM(OutOfLineCallVM<ArgSeq, StoreOutputTo> *ool)
 {
     LInstruction *lir = ool->lir();
 
     saveLive(lir);
     ool->args().generate(this);
-    if (!callVM(ool->function(), lir))
-        return false;
+    callVM(ool->function(), lir);
     ool->out().generate(this);
     restoreLiveIgnore(lir, ool->out().clobbered());
     masm.jump(ool->rejoin());
-    return true;
 }
 
 } // namespace jit
