@@ -15,13 +15,12 @@
 namespace js {
 namespace jit {
 
-bool
+void
 LIRGeneratorShared::emitAtUses(MInstruction *mir)
 {
-    JS_ASSERT(mir->canEmitAtUses());
+    MOZ_ASSERT(mir->canEmitAtUses());
     mir->setEmittedAtUses();
     mir->setVirtualRegister(0);
-    return true;
 }
 
 LUse
@@ -29,23 +28,20 @@ LIRGeneratorShared::use(MDefinition *mir, LUse policy)
 {
     // It is illegal to call use() on an instruction with two defs.
 #if BOX_PIECES > 1
-    JS_ASSERT(mir->type() != MIRType_Value);
+    MOZ_ASSERT(mir->type() != MIRType_Value);
 #endif
-    if (!ensureDefined(mir))
-        return policy;
+    ensureDefined(mir);
     policy.setVirtualRegister(mir->virtualRegister());
     return policy;
 }
 
-template <size_t X, size_t Y> bool
+template <size_t X, size_t Y> void
 LIRGeneratorShared::define(LInstructionHelper<1, X, Y> *lir, MDefinition *mir, const LDefinition &def)
 {
     // Call instructions should use defineReturn.
-    JS_ASSERT(!lir->isCall());
+    MOZ_ASSERT(!lir->isCall());
 
     uint32_t vreg = getVirtualRegister();
-    if (vreg >= MAX_VIRTUAL_REGISTERS)
-        return false;
 
     // Assign the definition and a virtual register. Then, propagate this
     // virtual register to the MIR, so we can map MIR to LIR during lowering.
@@ -53,17 +49,17 @@ LIRGeneratorShared::define(LInstructionHelper<1, X, Y> *lir, MDefinition *mir, c
     lir->getDef(0)->setVirtualRegister(vreg);
     lir->setMir(mir);
     mir->setVirtualRegister(vreg);
-    return add(lir);
+    add(lir);
 }
 
-template <size_t X, size_t Y> bool
+template <size_t X, size_t Y> void
 LIRGeneratorShared::define(LInstructionHelper<1, X, Y> *lir, MDefinition *mir, LDefinition::Policy policy)
 {
     LDefinition::Type type = LDefinition::TypeFrom(mir->type());
-    return define(lir, mir, LDefinition(type, policy));
+    define(lir, mir, LDefinition(type, policy));
 }
 
-template <size_t X, size_t Y> bool
+template <size_t X, size_t Y> void
 LIRGeneratorShared::defineFixed(LInstructionHelper<1, X, Y> *lir, MDefinition *mir, const LAllocation &output)
 {
     LDefinition::Type type = LDefinition::TypeFrom(mir->type());
@@ -73,66 +69,56 @@ LIRGeneratorShared::defineFixed(LInstructionHelper<1, X, Y> *lir, MDefinition *m
 
     // Add an LNop to avoid regalloc problems if the next op uses this value
     // with a fixed or at-start policy.
-    if (!define(lir, mir, def))
-        return false;
+    define(lir, mir, def);
 
-    if (gen->optimizationInfo().registerAllocator() == RegisterAllocator_LSRA) {
-        if (!add(new(alloc()) LNop))
-            return false;
-    }
-
-    return true;
+    if (gen->optimizationInfo().registerAllocator() == RegisterAllocator_LSRA)
+        add(new(alloc()) LNop);
 }
 
-template <size_t Ops, size_t Temps> bool
+template <size_t Ops, size_t Temps> void
 LIRGeneratorShared::defineReuseInput(LInstructionHelper<1, Ops, Temps> *lir, MDefinition *mir, uint32_t operand)
 {
     // The input should be used at the start of the instruction, to avoid moves.
-    JS_ASSERT(lir->getOperand(operand)->toUse()->usedAtStart());
+    MOZ_ASSERT(lir->getOperand(operand)->toUse()->usedAtStart());
 
     LDefinition::Type type = LDefinition::TypeFrom(mir->type());
 
     LDefinition def(type, LDefinition::MUST_REUSE_INPUT);
     def.setReusedInput(operand);
 
-    return define(lir, mir, def);
+    define(lir, mir, def);
 }
 
-template <size_t Ops, size_t Temps> bool
+template <size_t Ops, size_t Temps> void
 LIRGeneratorShared::defineBox(LInstructionHelper<BOX_PIECES, Ops, Temps> *lir, MDefinition *mir,
                               LDefinition::Policy policy)
 {
     // Call instructions should use defineReturn.
-    JS_ASSERT(!lir->isCall());
+    MOZ_ASSERT(!lir->isCall());
 
     uint32_t vreg = getVirtualRegister();
-    if (vreg >= MAX_VIRTUAL_REGISTERS)
-        return false;
 
 #if defined(JS_NUNBOX32)
     lir->setDef(0, LDefinition(vreg + VREG_TYPE_OFFSET, LDefinition::TYPE, policy));
     lir->setDef(1, LDefinition(vreg + VREG_DATA_OFFSET, LDefinition::PAYLOAD, policy));
-    if (getVirtualRegister() >= MAX_VIRTUAL_REGISTERS)
-        return false;
+    getVirtualRegister();
 #elif defined(JS_PUNBOX64)
     lir->setDef(0, LDefinition(vreg, LDefinition::BOX, policy));
 #endif
     lir->setMir(mir);
 
     mir->setVirtualRegister(vreg);
-    return add(lir);
+    add(lir);
 }
 
-bool
+void
 LIRGeneratorShared::defineReturn(LInstruction *lir, MDefinition *mir)
 {
     lir->setMir(mir);
 
-    JS_ASSERT(lir->isCall());
+    MOZ_ASSERT(lir->isCall());
 
     uint32_t vreg = getVirtualRegister();
-    if (vreg >= MAX_VIRTUAL_REGISTERS)
-        return false;
 
     switch (mir->type()) {
       case MIRType_Value:
@@ -141,9 +127,7 @@ LIRGeneratorShared::defineReturn(LInstruction *lir, MDefinition *mir)
                                             LGeneralReg(JSReturnReg_Type)));
         lir->setDef(PAYLOAD_INDEX, LDefinition(vreg + VREG_DATA_OFFSET, LDefinition::PAYLOAD,
                                                LGeneralReg(JSReturnReg_Data)));
-
-        if (getVirtualRegister() >= MAX_VIRTUAL_REGISTERS)
-            return false;
+        getVirtualRegister();
 #elif defined(JS_PUNBOX64)
         lir->setDef(0, LDefinition(vreg, LDefinition::BOX, LGeneralReg(JSReturnReg)));
 #endif
@@ -154,23 +138,24 @@ LIRGeneratorShared::defineReturn(LInstruction *lir, MDefinition *mir)
       case MIRType_Double:
         lir->setDef(0, LDefinition(vreg, LDefinition::DOUBLE, LFloatReg(ReturnDoubleReg)));
         break;
+      case MIRType_Int32x4:
+        lir->setDef(0, LDefinition(vreg, LDefinition::INT32X4, LFloatReg(ReturnSimdReg)));
+        break;
+      case MIRType_Float32x4:
+        lir->setDef(0, LDefinition(vreg, LDefinition::FLOAT32X4, LFloatReg(ReturnSimdReg)));
+        break;
       default:
         LDefinition::Type type = LDefinition::TypeFrom(mir->type());
-        JS_ASSERT(type != LDefinition::DOUBLE && type != LDefinition::FLOAT32);
+        MOZ_ASSERT(type != LDefinition::DOUBLE && type != LDefinition::FLOAT32);
         lir->setDef(0, LDefinition(vreg, type, LGeneralReg(ReturnReg)));
         break;
     }
 
     mir->setVirtualRegister(vreg);
-    if (!add(lir))
-        return false;
+    add(lir);
 
-    if (gen->optimizationInfo().registerAllocator() == RegisterAllocator_LSRA) {
-        if (!add(new(alloc()) LNop))
-            return false;
-    }
-
-    return true;
+    if (gen->optimizationInfo().registerAllocator() == RegisterAllocator_LSRA)
+        add(new(alloc()) LNop);
 }
 
 // In LIR, we treat booleans and integers as the same low-level type (INTEGER).
@@ -186,13 +171,16 @@ IsCompatibleLIRCoercion(MIRType to, MIRType from)
         (from == MIRType_Int32 || from == MIRType_Boolean)) {
         return true;
     }
+    // SIMD types can be coerced with from*Bits operators.
+    if (IsSimdType(to) && IsSimdType(from))
+        return true;
     return false;
 }
 
-bool
+void
 LIRGeneratorShared::redefine(MDefinition *def, MDefinition *as)
 {
-    JS_ASSERT(IsCompatibleLIRCoercion(def->type(), as->type()));
+    MOZ_ASSERT(IsCompatibleLIRCoercion(def->type(), as->type()));
 
     // Try to emit MIR marked as emitted-at-uses at, well, uses. For
     // snapshotting reasons we delay the MIRTypes match, or when we are
@@ -203,61 +191,32 @@ LIRGeneratorShared::redefine(MDefinition *def, MDefinition *as)
           (def->type() == MIRType_Int32 || def->type() == MIRType_Boolean) &&
           (as->type() == MIRType_Int32 || as->type() == MIRType_Boolean))))
     {
-        MDefinition *replacement;
+        MInstruction *replacement;
         if (def->type() != as->type()) {
             Value v = as->toConstant()->value();
             if (as->type() == MIRType_Int32)
                 replacement = MConstant::New(alloc(), BooleanValue(v.toInt32()));
             else
                 replacement = MConstant::New(alloc(), Int32Value(v.toBoolean()));
-            if (!emitAtUses(replacement->toInstruction()))
-                return false;
+            def->block()->insertBefore(def->toInstruction(), replacement);
+            emitAtUses(replacement->toInstruction());
         } else {
-            replacement = as;
+            replacement = as->toInstruction();
         }
         def->replaceAllUsesWith(replacement);
-        return true;
-    }
-
-    if (!ensureDefined(as))
-        return false;
-    def->setVirtualRegister(as->virtualRegister());
-    return true;
-}
-
-bool
-LIRGeneratorShared::defineAs(LInstruction *outLir, MDefinition *outMir, MDefinition *inMir)
-{
-    uint32_t vreg = inMir->virtualRegister();
-    LDefinition::Policy policy = LDefinition::PASSTHROUGH;
-
-    if (outMir->type() == MIRType_Value) {
-#ifdef JS_NUNBOX32
-        outLir->setDef(TYPE_INDEX,
-                       LDefinition(vreg + VREG_TYPE_OFFSET, LDefinition::TYPE, policy));
-        outLir->setDef(PAYLOAD_INDEX,
-                       LDefinition(vreg + VREG_DATA_OFFSET, LDefinition::PAYLOAD, policy));
-#elif JS_PUNBOX64
-        outLir->setDef(0, LDefinition(vreg, LDefinition::BOX, policy));
-#else
-# error "Unexpected boxing type"
-#endif
     } else {
-        outLir->setDef(0, LDefinition(vreg, LDefinition::TypeFrom(inMir->type()), policy));
+        ensureDefined(as);
+        def->setVirtualRegister(as->virtualRegister());
     }
-    outLir->setMir(outMir);
-    return redefine(outMir, inMir);
 }
 
-bool
+void
 LIRGeneratorShared::ensureDefined(MDefinition *mir)
 {
     if (mir->isEmittedAtUses()) {
-        if (!mir->toInstruction()->accept(this))
-            return false;
-        JS_ASSERT(mir->isLowered());
+        mir->toInstruction()->accept(this);
+        MOZ_ASSERT(mir->isLowered());
     }
-    return true;
 }
 
 LUse
@@ -414,12 +373,7 @@ LIRGeneratorShared::useFixed(MDefinition *mir, AnyRegister reg)
 LDefinition
 LIRGeneratorShared::temp(LDefinition::Type type, LDefinition::Policy policy)
 {
-    uint32_t vreg = getVirtualRegister();
-    if (vreg >= MAX_VIRTUAL_REGISTERS) {
-        gen->abort("max virtual registers");
-        return LDefinition();
-    }
-    return LDefinition(vreg, type, policy);
+    return LDefinition(getVirtualRegister(), type, policy);
 }
 
 LDefinition
@@ -445,7 +399,7 @@ LIRGeneratorShared::tempDouble()
 LDefinition
 LIRGeneratorShared::tempCopy(MDefinition *input, uint32_t reusedInput)
 {
-    JS_ASSERT(input->virtualRegister());
+    MOZ_ASSERT(input->virtualRegister());
     LDefinition t = temp(LDefinition::TypeFrom(input->type()), LDefinition::MUST_REUSE_INPUT);
     t.setReusedInput(reusedInput);
     return t;
@@ -457,17 +411,16 @@ LIRGeneratorShared::annotate(T *ins)
     ins->setId(lirGraph_.getInstructionId());
 }
 
-template <typename T> bool
+template <typename T> void
 LIRGeneratorShared::add(T *ins, MInstruction *mir)
 {
-    JS_ASSERT(!ins->isPhi());
+    MOZ_ASSERT(!ins->isPhi());
     current->add(ins);
     if (mir) {
-        JS_ASSERT(current == mir->block()->lir());
+        MOZ_ASSERT(current == mir->block()->lir());
         ins->setMir(mir);
     }
     annotate(ins);
-    return true;
 }
 
 #ifdef JS_NUNBOX32
@@ -493,7 +446,7 @@ VirtualRegisterOfPayload(MDefinition *mir)
 LUse
 LIRGeneratorShared::useType(MDefinition *mir, LUse::Policy policy)
 {
-    JS_ASSERT(mir->type() == MIRType_Value);
+    MOZ_ASSERT(mir->type() == MIRType_Value);
 
     return LUse(mir->virtualRegister() + VREG_TYPE_OFFSET, policy);
 }
@@ -501,7 +454,7 @@ LIRGeneratorShared::useType(MDefinition *mir, LUse::Policy policy)
 LUse
 LIRGeneratorShared::usePayload(MDefinition *mir, LUse::Policy policy)
 {
-    JS_ASSERT(mir->type() == MIRType_Value);
+    MOZ_ASSERT(mir->type() == MIRType_Value);
 
     return LUse(VirtualRegisterOfPayload(mir), policy);
 }
@@ -509,7 +462,7 @@ LIRGeneratorShared::usePayload(MDefinition *mir, LUse::Policy policy)
 LUse
 LIRGeneratorShared::usePayloadAtStart(MDefinition *mir, LUse::Policy policy)
 {
-    JS_ASSERT(mir->type() == MIRType_Value);
+    MOZ_ASSERT(mir->type() == MIRType_Value);
 
     return LUse(VirtualRegisterOfPayload(mir), policy, true);
 }
@@ -520,17 +473,32 @@ LIRGeneratorShared::usePayloadInRegisterAtStart(MDefinition *mir)
     return usePayloadAtStart(mir, LUse::REGISTER);
 }
 
-bool
+void
 LIRGeneratorShared::fillBoxUses(LInstruction *lir, size_t n, MDefinition *mir)
 {
-    if (!ensureDefined(mir))
-        return false;
+    ensureDefined(mir);
     lir->getOperand(n)->toUse()->setVirtualRegister(mir->virtualRegister() + VREG_TYPE_OFFSET);
     lir->getOperand(n + 1)->toUse()->setVirtualRegister(VirtualRegisterOfPayload(mir));
-    return true;
 }
 #endif
 
+LUse
+LIRGeneratorShared::useRegisterForTypedLoad(MDefinition *mir, MIRType type)
+{
+    MOZ_ASSERT(type != MIRType_Value && type != MIRType_None);
+    MOZ_ASSERT(mir->type() == MIRType_Object || mir->type() == MIRType_Slots);
+
+#ifdef JS_PUNBOX64
+    // On x64, masm.loadUnboxedValue emits slightly less efficient code when
+    // the input and output use the same register and we're not loading an
+    // int32/bool/double, so we just call useRegister in this case.
+    if (type != MIRType_Int32 && type != MIRType_Boolean && type != MIRType_Double)
+        return useRegister(mir);
+#endif
+
+    return useRegisterAtStart(mir);
+}
+
 } // namespace jit
 } // namespace js
 
