@@ -22,9 +22,9 @@ class LIRGeneratorX86 : public LIRGeneratorX86Shared
   protected:
     // Adds a box input to an instruction, setting operand |n| to the type and
     // |n+1| to the payload.
-    bool useBox(LInstruction *lir, size_t n, MDefinition *mir,
+    void useBox(LInstruction *lir, size_t n, MDefinition *mir,
                 LUse::Policy policy = LUse::REGISTER, bool useAtStart = false);
-    bool useBoxFixed(LInstruction *lir, size_t n, MDefinition *mir, Register reg1, Register reg2);
+    void useBoxFixed(LInstruction *lir, size_t n, MDefinition *mir, Register reg1, Register reg2);
 
     // It's a trap! On x86, the 1-byte store can only use one of
     // {al,bl,cl,dl,ah,bh,ch,dh}. That means if the register allocator
@@ -33,6 +33,7 @@ class LIRGeneratorX86 : public LIRGeneratorX86Shared
     // give us one of {al,bl,cl,dl}. For now, just useFixed(al).
     LAllocation useByteOpRegister(MDefinition *mir);
     LAllocation useByteOpRegisterOrNonDoubleConstant(MDefinition *mir);
+    LDefinition tempByteOpRegister();
 
     inline LDefinition tempToUnbox() {
         return LDefinition::BogusTemp();
@@ -40,22 +41,21 @@ class LIRGeneratorX86 : public LIRGeneratorX86Shared
 
     bool needTempForPostBarrier() { return true; }
 
-    LDefinition tempForDispatchCache(MIRType outputType = MIRType_None);
-
     void lowerUntypedPhiInput(MPhi *phi, uint32_t inputPosition, LBlock *block, size_t lirIndex);
-    bool defineUntypedPhi(MPhi *phi, size_t lirIndex);
+    void defineUntypedPhi(MPhi *phi, size_t lirIndex);
 
   public:
-    bool visitBox(MBox *box);
-    bool visitUnbox(MUnbox *unbox);
-    bool visitReturn(MReturn *ret);
-    bool visitAsmJSUnsignedToDouble(MAsmJSUnsignedToDouble *ins);
-    bool visitAsmJSUnsignedToFloat32(MAsmJSUnsignedToFloat32 *ins);
-    bool visitAsmJSLoadHeap(MAsmJSLoadHeap *ins);
-    bool visitAsmJSStoreHeap(MAsmJSStoreHeap *ins);
-    bool visitAsmJSLoadFuncPtr(MAsmJSLoadFuncPtr *ins);
-    bool visitStoreTypedArrayElementStatic(MStoreTypedArrayElementStatic *ins);
-    bool lowerPhi(MPhi *phi);
+    void visitBox(MBox *box);
+    void visitUnbox(MUnbox *unbox);
+    void visitReturn(MReturn *ret);
+    void visitAsmJSUnsignedToDouble(MAsmJSUnsignedToDouble *ins);
+    void visitAsmJSUnsignedToFloat32(MAsmJSUnsignedToFloat32 *ins);
+    void visitAsmJSLoadHeap(MAsmJSLoadHeap *ins);
+    void visitAsmJSStoreHeap(MAsmJSStoreHeap *ins);
+    void visitAsmJSLoadFuncPtr(MAsmJSLoadFuncPtr *ins);
+    void visitStoreTypedArrayElementStatic(MStoreTypedArrayElementStatic *ins);
+    void visitSubstr(MSubstr *ins);
+    void lowerPhi(MPhi *phi);
 
     static bool allowTypedElementHoleCheck() {
         return true;
@@ -64,14 +64,11 @@ class LIRGeneratorX86 : public LIRGeneratorX86Shared
     static bool allowStaticTypedArrayAccesses() {
         return true;
     }
-    static bool allowInlineForkJoinGetSlice() {
-        return true;
-    }
 };
 
 typedef LIRGeneratorX86 LIRGeneratorSpecific;
 
-} // namespace js
 } // namespace jit
+} // namespace js
 
 #endif /* jit_x86_Lowering_x86_h */
