@@ -24,23 +24,20 @@ PTest(JSContext* cx, unsigned argc, jsval *vp);
 static const JSClass ptestClass = {
     "PTest",
     JSCLASS_HAS_PRIVATE,
-
-    JS_PropertyStub,       // add
-    JS_DeletePropertyStub, // delete
-    test_prop_get,         // get
-    JS_StrictPropertyStub, // set
-    JS_EnumerateStub,
-    JS_ResolveStub,
-    JS_ConvertStub
+    nullptr, // addProperty
+    nullptr, // delProperty
+    test_prop_get,
+    nullptr // setProperty
 };
 
 static bool
 PTest(JSContext* cx, unsigned argc, jsval *vp)
 {
-    JSObject *obj = JS_NewObjectForConstructor(cx, &ptestClass, vp);
+    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);
+    JSObject *obj = JS_NewObjectForConstructor(cx, &ptestClass, args);
     if (!obj)
         return false;
-    JS_SET_RVAL(cx, vp, OBJECT_TO_JSVAL(obj));
+    args.rval().setObject(*obj);
     return true;
 }
 static bool test_fn(JSContext *cx, unsigned argc, jsval *vp)
@@ -56,14 +53,15 @@ static const JSFunctionSpec ptestFunctions[] = {
 
 BEGIN_TEST(testClassGetter_isCalled)
 {
-    CHECK(JS_InitClass(cx, global, nullptr, &ptestClass, PTest, 0,
+    CHECK(JS_InitClass(cx, global, js::NullPtr(), &ptestClass, PTest, 0,
                        nullptr, ptestFunctions, nullptr, nullptr));
 
     EXEC("function check() { var o = new PTest(); o.test_fn(); o.test_value1; o.test_value2; o.test_value1; }");
 
     for (int i = 1; i < 9; i++) {
         JS::RootedValue rval(cx);
-        CHECK(JS_CallFunctionName(cx, global, "check", 0, nullptr, rval.address()));
+        CHECK(JS_CallFunctionName(cx, global, "check", JS::HandleValueArray::empty(),
+                                  &rval));
         CHECK_SAME(INT_TO_JSVAL(called_test_fn), INT_TO_JSVAL(i));
         CHECK_SAME(INT_TO_JSVAL(called_test_prop_get), INT_TO_JSVAL(4 * i));
     }
