@@ -7,7 +7,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "js/OldDebugAPI.h"
+#include "jsfriendapi.h"
 #include "jsapi-tests/tests.h"
 
 BEGIN_TEST(test_cloneScript)
@@ -32,18 +32,19 @@ BEGIN_TEST(test_cloneScript)
     // compile for A
     {
         JSAutoCompartment a(cx, A);
-        JSFunction *fun;
+        JS::RootedFunction fun(cx);
         JS::CompileOptions options(cx);
         options.setFileAndLine(__FILE__, 1);
-        CHECK(fun = JS_CompileFunction(cx, A, "f", 0, nullptr, source,
-                                       strlen(source), options));
+        JS::AutoObjectVector emptyScopeChain(cx);
+        CHECK(JS::CompileFunction(cx, emptyScopeChain, options, "f", 0, nullptr,
+                                  source, strlen(source), &fun));
         CHECK(obj = JS_GetFunctionObject(fun));
     }
 
     // clone into B
     {
         JSAutoCompartment b(cx, B);
-        CHECK(JS_CloneFunctionObject(cx, obj, B));
+        CHECK(JS::CloneFunctionObject(cx, obj));
     }
 
     return true;
@@ -107,11 +108,12 @@ BEGIN_TEST(test_cloneScriptWithPrincipals)
     {
         JSAutoCompartment a(cx, A);
         JS::CompileOptions options(cx);
-        options.setFileAndLine(__FILE__, 1)
-               .setPrincipals(principalsA);
-        JS::RootedFunction fun(cx, JS_CompileFunction(cx, A, "f",
-                mozilla::ArrayLength(argnames), argnames, source,
-                strlen(source), options));
+        options.setFileAndLine(__FILE__, 1);
+        JS::RootedFunction fun(cx);
+        JS::AutoObjectVector emptyScopeChain(cx);
+        JS::CompileFunction(cx, emptyScopeChain, options, "f",
+                           mozilla::ArrayLength(argnames), argnames, source,
+                           strlen(source), &fun);
         CHECK(fun);
 
         JSScript *script;
@@ -125,9 +127,9 @@ BEGIN_TEST(test_cloneScriptWithPrincipals)
     {
         JSAutoCompartment b(cx, B);
         JS::RootedObject cloned(cx);
-        CHECK(cloned = JS_CloneFunctionObject(cx, obj, B));
+        CHECK(cloned = JS::CloneFunctionObject(cx, obj));
 
-        JSFunction *fun;
+        JS::RootedFunction fun(cx);
         JS::RootedValue clonedValue(cx, JS::ObjectValue(*cloned));
         CHECK(fun = JS_ValueToFunction(cx, clonedValue));
 
@@ -137,8 +139,8 @@ BEGIN_TEST(test_cloneScriptWithPrincipals)
         CHECK(JS_GetScriptPrincipals(script) == principalsB);
 
         JS::RootedValue v(cx);
-        JS::Value args[] = { JS::Int32Value(1) };
-        CHECK(JS_CallFunctionValue(cx, B, JS::ObjectValue(*cloned), 1, args, v.address()));
+        JS::RootedValue arg(cx, JS::Int32Value(1));
+        CHECK(JS_CallFunctionValue(cx, B, clonedValue, JS::HandleValueArray(arg), &v));
         CHECK(v.isObject());
 
         JSObject *funobj = &v.toObject();
