@@ -14,7 +14,7 @@ constructHook(JSContext *cx, unsigned argc, jsval *vp)
 
     // Check that arguments were passed properly from JS_New.
 
-    JS::RootedObject obj(cx, JS_NewObject(cx, js::Jsvalify(&JSObject::class_), nullptr, nullptr));
+    JS::RootedObject obj(cx, JS_NewObject(cx, js::Jsvalify(&js::PlainObject::class_), JS::NullPtr(), JS::NullPtr()));
     if (!obj) {
         JS_ReportError(cx, "test failed, could not construct object");
         return false;
@@ -38,7 +38,8 @@ constructHook(JSContext *cx, unsigned argc, jsval *vp)
 
     // Perform a side-effect to indicate that this hook was actually called.
     JS::RootedValue value(cx, args[0]);
-    if (!JS_SetElement(cx, &args.callee(), 0, &value))
+    JS::RootedObject callee(cx, &args.callee());
+    if (!JS_SetElement(cx, callee, 0, value))
         return false;
 
     args.rval().setObject(*obj);
@@ -54,39 +55,35 @@ constructHook(JSContext *cx, unsigned argc, jsval *vp)
 BEGIN_TEST(testNewObject_1)
 {
     static const size_t N = 1000;
-    jsval argv[N];
-
-    // Root the global argv test array. Only the first 2 entries really need to
-    // be rooted, since we're only putting integers in the rest.
-    CHECK(JS_AddNamedValueRoot(cx, &argv[0], "argv0"));
-    CHECK(JS_AddNamedValueRoot(cx, &argv[1], "argv1"));
+    JS::AutoValueVector argv(cx);
+    CHECK(argv.resize(N));
 
     JS::RootedValue v(cx);
-    EVAL("Array", v.address());
-    JS::RootedObject Array(cx, JSVAL_TO_OBJECT(v));
+    EVAL("Array", &v);
+    JS::RootedObject Array(cx, v.toObjectOrNull());
 
     // With no arguments.
-    JS::RootedObject obj(cx, JS_New(cx, Array, 0, nullptr));
+    JS::RootedObject obj(cx, JS_New(cx, Array, JS::HandleValueArray::empty()));
     CHECK(obj);
-    JS::RootedValue rt(cx, OBJECT_TO_JSVAL(obj));
+    JS::RootedValue rt(cx, JS::ObjectValue(*obj));
     CHECK(JS_IsArrayObject(cx, obj));
     uint32_t len;
     CHECK(JS_GetArrayLength(cx, obj, &len));
-    CHECK_EQUAL(len, 0);
+    CHECK_EQUAL(len, 0u);
 
     // With one argument.
-    argv[0] = INT_TO_JSVAL(4);
-    obj = JS_New(cx, Array, 1, argv);
+    argv[0].setInt32(4);
+    obj = JS_New(cx, Array, JS::HandleValueArray::subarray(argv, 0, 1));
     CHECK(obj);
     rt = OBJECT_TO_JSVAL(obj);
     CHECK(JS_IsArrayObject(cx, obj));
     CHECK(JS_GetArrayLength(cx, obj, &len));
-    CHECK_EQUAL(len, 4);
+    CHECK_EQUAL(len, 4u);
 
     // With N arguments.
     for (size_t i = 0; i < N; i++)
-        argv[i] = INT_TO_JSVAL(i);
-    obj = JS_New(cx, Array, N, argv);
+        argv[i].setInt32(i);
+    obj = JS_New(cx, Array, JS::HandleValueArray::subarray(argv, 0, N));
     CHECK(obj);
     rt = OBJECT_TO_JSVAL(obj);
     CHECK(JS_IsArrayObject(cx, obj));
@@ -99,21 +96,18 @@ BEGIN_TEST(testNewObject_1)
     static const JSClass cls = {
         "testNewObject_1",
         0,
-        JS_PropertyStub, JS_DeletePropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
-        JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, nullptr,
-        nullptr, nullptr, nullptr, constructHook
+        nullptr, nullptr, nullptr, nullptr,
+        nullptr, nullptr, nullptr, nullptr,
+        nullptr, nullptr, constructHook
     };
-    JS::RootedObject ctor(cx, JS_NewObject(cx, &cls, nullptr, nullptr));
+    JS::RootedObject ctor(cx, JS_NewObject(cx, &cls, JS::NullPtr(), JS::NullPtr()));
     CHECK(ctor);
     JS::RootedValue rt2(cx, OBJECT_TO_JSVAL(ctor));
-    obj = JS_New(cx, ctor, 3, argv);
+    obj = JS_New(cx, ctor, JS::HandleValueArray::subarray(argv, 0, 3));
     CHECK(obj);
     CHECK(JS_GetElement(cx, ctor, 0, &v));
     CHECK_SAME(v, JSVAL_ZERO);
 
-    JS_RemoveValueRoot(cx, &argv[0]);
-    JS_RemoveValueRoot(cx, &argv[1]);
-
     return true;
 }
 END_TEST(testNewObject_1)
