@@ -6,38 +6,32 @@
 
 static const JSClass ObjectEmulatingUndefinedClass = {
     "ObjectEmulatingUndefined",
-    JSCLASS_EMULATES_UNDEFINED,
-    JS_PropertyStub,
-    JS_DeletePropertyStub,
-    JS_PropertyStub,
-    JS_StrictPropertyStub,
-    JS_EnumerateStub,
-    JS_ResolveStub,
-    JS_ConvertStub
+    JSCLASS_EMULATES_UNDEFINED
 };
 
 static bool
 ObjectEmulatingUndefinedConstructor(JSContext *cx, unsigned argc, jsval *vp)
 {
-    JSObject *obj = JS_NewObjectForConstructor(cx, &ObjectEmulatingUndefinedClass, vp);
+    JS::CallArgs args = JS::CallArgsFromVp(argc, vp);
+    JSObject *obj = JS_NewObjectForConstructor(cx, &ObjectEmulatingUndefinedClass, args);
     if (!obj)
         return false;
-    JS_SET_RVAL(cx, vp, OBJECT_TO_JSVAL(obj));
+    args.rval().setObject(*obj);
     return true;
 }
 
 BEGIN_TEST(testObjectEmulatingUndefined_truthy)
 {
-    CHECK(JS_InitClass(cx, global, nullptr, &ObjectEmulatingUndefinedClass,
+    CHECK(JS_InitClass(cx, global, js::NullPtr(), &ObjectEmulatingUndefinedClass,
                        ObjectEmulatingUndefinedConstructor, 0,
                        nullptr, nullptr, nullptr, nullptr));
 
     JS::RootedValue result(cx);
 
-    EVAL("if (new ObjectEmulatingUndefined()) true; else false;", result.address());
+    EVAL("if (new ObjectEmulatingUndefined()) true; else false;", &result);
     CHECK_SAME(result, JSVAL_FALSE);
 
-    EVAL("if (!new ObjectEmulatingUndefined()) true; else false;", result.address());
+    EVAL("if (!new ObjectEmulatingUndefined()) true; else false;", &result);
     CHECK_SAME(result, JSVAL_TRUE);
 
     EVAL("var obj = new ObjectEmulatingUndefined(); \n"
@@ -45,7 +39,7 @@ BEGIN_TEST(testObjectEmulatingUndefined_truthy)
          "for (var i = 0; i < 50; i++) \n"
          "  res.push(Boolean(obj)); \n"
          "res.every(function(v) { return v === false; });",
-         result.address());
+         &result);
     CHECK_SAME(result, JSVAL_TRUE);
 
     return true;
@@ -54,22 +48,22 @@ END_TEST(testObjectEmulatingUndefined_truthy)
 
 BEGIN_TEST(testObjectEmulatingUndefined_equal)
 {
-    CHECK(JS_InitClass(cx, global, nullptr, &ObjectEmulatingUndefinedClass,
+    CHECK(JS_InitClass(cx, global, js::NullPtr(), &ObjectEmulatingUndefinedClass,
                        ObjectEmulatingUndefinedConstructor, 0,
                        nullptr, nullptr, nullptr, nullptr));
 
     JS::RootedValue result(cx);
 
-    EVAL("if (new ObjectEmulatingUndefined() == undefined) true; else false;", result.address());
+    EVAL("if (new ObjectEmulatingUndefined() == undefined) true; else false;", &result);
     CHECK_SAME(result, JSVAL_TRUE);
 
-    EVAL("if (new ObjectEmulatingUndefined() == null) true; else false;", result.address());
+    EVAL("if (new ObjectEmulatingUndefined() == null) true; else false;", &result);
     CHECK_SAME(result, JSVAL_TRUE);
 
-    EVAL("if (new ObjectEmulatingUndefined() != undefined) true; else false;", result.address());
+    EVAL("if (new ObjectEmulatingUndefined() != undefined) true; else false;", &result);
     CHECK_SAME(result, JSVAL_FALSE);
 
-    EVAL("if (new ObjectEmulatingUndefined() != null) true; else false;", result.address());
+    EVAL("if (new ObjectEmulatingUndefined() != null) true; else false;", &result);
     CHECK_SAME(result, JSVAL_FALSE);
 
     EVAL("var obj = new ObjectEmulatingUndefined(); \n"
@@ -77,7 +71,7 @@ BEGIN_TEST(testObjectEmulatingUndefined_equal)
          "for (var i = 0; i < 50; i++) \n"
          "  res.push(obj == undefined); \n"
          "res.every(function(v) { return v === true; });",
-         result.address());
+         &result);
     CHECK_SAME(result, JSVAL_TRUE);
 
     EVAL("var obj = new ObjectEmulatingUndefined(); \n"
@@ -85,7 +79,7 @@ BEGIN_TEST(testObjectEmulatingUndefined_equal)
          "for (var i = 0; i < 50; i++) \n"
          "  res.push(obj == null); \n"
          "res.every(function(v) { return v === true; });",
-         result.address());
+         &result);
     CHECK_SAME(result, JSVAL_TRUE);
 
     EVAL("var obj = new ObjectEmulatingUndefined(); \n"
@@ -93,7 +87,7 @@ BEGIN_TEST(testObjectEmulatingUndefined_equal)
          "for (var i = 0; i < 50; i++) \n"
          "  res.push(obj != undefined); \n"
          "res.every(function(v) { return v === false; });",
-         result.address());
+         &result);
     CHECK_SAME(result, JSVAL_TRUE);
 
     EVAL("var obj = new ObjectEmulatingUndefined(); \n"
@@ -101,7 +95,7 @@ BEGIN_TEST(testObjectEmulatingUndefined_equal)
          "for (var i = 0; i < 50; i++) \n"
          "  res.push(obj != null); \n"
          "res.every(function(v) { return v === false; });",
-         result.address());
+         &result);
     CHECK_SAME(result, JSVAL_TRUE);
 
     return true;
