@@ -22,8 +22,8 @@ my_convert(JSContext* context, JS::HandleObject obj, JSType type, JS::MutableHan
 static const JSClass myClass = {
     "MyClass",
     0,
-    JS_PropertyStub, JS_DeletePropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
-    JS_EnumerateStub, JS_ResolveStub, my_convert
+    nullptr, nullptr, nullptr, nullptr,
+    nullptr, nullptr, my_convert
 };
 
 static bool
@@ -33,7 +33,7 @@ createMyObject(JSContext* context, unsigned argc, jsval *vp)
 
     //JS_GC(context); //<- if we make GC here, all is ok
 
-    JSObject* myObject = JS_NewObject(context, &myClass, nullptr, nullptr);
+    JSObject* myObject = JS_NewObject(context, &myClass, JS::NullPtr(), JS::NullPtr());
     *vp = OBJECT_TO_JSVAL(myObject);
 
     JS_EndRequest(context);
@@ -55,7 +55,8 @@ BEGIN_TEST(testOps_bug559006)
 
     for (int i = 0; i < 9; i++) {
         JS::RootedValue rval(cx);
-        CHECK(JS_CallFunctionName(cx, global, "main", 0, nullptr, rval.address()));
+        CHECK(JS_CallFunctionName(cx, global, "main", JS::HandleValueArray::empty(),
+                                  &rval));
         CHECK_SAME(rval, INT_TO_JSVAL(123));
     }
     return true;
