@@ -9,17 +9,15 @@
 
 BEGIN_TEST(testSetProperty_NativeGetterStubSetter)
 {
-    JS::RootedObject obj(cx, JS_NewObject(cx, nullptr, nullptr, nullptr));
+    JS::RootedObject obj(cx, JS_NewObject(cx, nullptr, JS::NullPtr(), JS::NullPtr()));
     CHECK(obj);
-    JS::RootedValue vobj(cx, OBJECT_TO_JSVAL(obj));
 
-    CHECK(JS_DefineProperty(cx, global, "globalProp", vobj,
-                            JS_PropertyStub, JS_StrictPropertyStub,
-                            JSPROP_ENUMERATE));
+    CHECK(JS_DefineProperty(cx, global, "globalProp", obj, JSPROP_ENUMERATE,
+                            JS_STUBGETTER, JS_STUBSETTER));
 
-    CHECK(JS_DefineProperty(cx, obj, "prop", JSVAL_VOID,
-                            NativeGet, JS_StrictPropertyStub,
-                            JSPROP_SHARED));
+    CHECK(JS_DefineProperty(cx, obj, "prop", JS::UndefinedHandleValue,
+                            JSPROP_SHARED | JSPROP_PROPOP_ACCESSORS,
+                            JS_PROPERTYOP_GETTER(NativeGet), JS_STUBSETTER));
 
     EXEC("'use strict';                                     \n"
          "var error, passed = false;                        \n"
@@ -70,9 +68,9 @@ BEGIN_TEST(testSetProperty_InheritedGlobalSetter)
     // This is a JSAPI test because jsapi-test globals do not have a resolve
     // hook and therefore can use the property cache in some cases where the
     // shell can't.
-    JS_ASSERT(JS_GetClass(global)->resolve == &JS_ResolveStub);
+    MOZ_RELEASE_ASSERT(!JS_GetClass(global)->resolve);
 
-    CHECK(JS_DefineProperty(cx, global, "HOTLOOP", INT_TO_JSVAL(8), nullptr, nullptr, 0));
+    CHECK(JS_DefineProperty(cx, global, "HOTLOOP", 8, 0));
     EXEC("var n = 0;\n"
          "var global = this;\n"
          "function f() { n++; }\n"
@@ -84,4 +82,3 @@ BEGIN_TEST(testSetProperty_InheritedGlobalSetter)
     return true;
 }
 END_TEST(testSetProperty_InheritedGlobalSetter)
-
