@@ -13,7 +13,7 @@ static size_t uncaughtCount = 0;
 
 BEGIN_TEST(testUncaughtError)
 {
-    JSErrorReporter old = JS_SetErrorReporter(cx, UncaughtErrorReporter);
+    JSErrorReporter old = JS_SetErrorReporter(rt, UncaughtErrorReporter);
 
     CHECK(uncaughtCount == 0);
 
@@ -42,7 +42,7 @@ BEGIN_TEST(testUncaughtError)
 
     CHECK(uncaughtCount == 1);
 
-    JS_SetErrorReporter(cx, old);
+    JS_SetErrorReporter(rt, old);
 
     return true;
 }
