@@ -19,11 +19,4 @@ js_InitBooleanClass(JSContext *cx, js::HandleObject obj);
 extern JSString *
 js_BooleanToString(js::ExclusiveContext *cx, bool b);
 
-namespace js {
-
-inline bool
-BooleanGetPrimitiveValue(HandleObject obj, JSContext *cx);
-
-} /* namespace js */
-
 #endif /* jsbool_h */
