@@ -11,13 +11,14 @@
 #include "jscompartment.h"
 
 #include "jsiter.h"
-#include "jsworkers.h"
 
 #include "builtin/Object.h"
-#include "jit/IonFrames.h"
+#include "jit/JitFrames.h"
 #include "vm/ForkJoin.h"
+#include "vm/HelperThreads.h"
 #include "vm/Interpreter.h"
 #include "vm/ProxyObject.h"
+#include "vm/Symbol.h"
 
 namespace js {
 
@@ -29,7 +30,16 @@ class CompartmentChecker
   public:
     explicit CompartmentChecker(ExclusiveContext *cx)
       : compartment(cx->compartment())
-    {}
+    {
+#ifdef DEBUG
+        // In debug builds, make sure the embedder passed the cx it claimed it
+        // was going to use.
+        JSContext *activeContext = nullptr;
+        if (cx->isJSContext())
+            activeContext = cx->asJSContext()->runtime()->activeContext;
+        MOZ_ASSERT_IF(activeContext, cx == activeContext);
+#endif
+    }
 
     /*
      * Set a breakpoint here (break js::CompartmentChecker::fail) to debug
@@ -47,8 +57,8 @@ class CompartmentChecker
 
     /* Note: should only be used when neither c1 nor c2 may be the atoms compartment. */
     static void check(JSCompartment *c1, JSCompartment *c2) {
-        JS_ASSERT(!c1->runtimeFromAnyThread()->isAtomsCompartment(c1));
-        JS_ASSERT(!c2->runtimeFromAnyThread()->isAtomsCompartment(c2));
+        MOZ_ASSERT(!c1->runtimeFromAnyThread()->isAtomsCompartment(c1));
+        MOZ_ASSERT(!c2->runtimeFromAnyThread()->isAtomsCompartment(c2));
         if (c1 != c2)
             fail(c1, c2);
     }
@@ -72,6 +82,11 @@ class CompartmentChecker
             check(obj->compartment());
     }
 
+    template<typename T>
+    void check(const Rooted<T>& rooted) {
+        check(rooted.get());
+    }
+
     template<typename T>
     void check(Handle<T> handle) {
         check(handle.get());
@@ -99,32 +114,26 @@ class CompartmentChecker
             check(arr.array[i]);
     }
 
+    void check(const JS::HandleValueArray &arr) {
+        for (size_t i = 0; i < arr.length(); i++)
+            check(arr[i]);
+    }
+
     void check(const CallArgs &args) {
         for (Value *p = args.base(); p != args.end(); ++p)
             check(*p);
     }
 
-    void check(jsid id) {
-        if (JSID_IS_OBJECT(id))
-            check(JSID_TO_OBJECT(id));
-    }
-
-    void check(JSIdArray *ida) {
-        if (ida) {
-            for (int i = 0; i < ida->length; i++) {
-                if (JSID_IS_OBJECT(ida->vector[i]))
-                    check(ida->vector[i]);
-            }
-        }
-    }
+    void check(jsid id) {}
 
     void check(JSScript *script) {
         if (script)
             check(script->compartment());
     }
 
-    void check(StackFrame *fp);
+    void check(InterpreterFrame *fp);
     void check(AbstractFramePtr frame);
+    void check(SavedStacks *stacks);
 };
 #endif /* JS_CRASH_DIAGNOSTICS */
 
@@ -133,14 +142,12 @@ class CompartmentChecker
  * depends on other objects not having been swept yet.
  */
 #define START_ASSERT_SAME_COMPARTMENT()                                       \
-    if (!cx->isExclusiveContext())                                            \
-        return;                                                               \
     if (cx->isJSContext() && cx->asJSContext()->runtime()->isHeapBusy())      \
         return;                                                               \
-    CompartmentChecker c(cx->asExclusiveContext())
+    CompartmentChecker c(cx)
 
 template <class T1> inline void
-assertSameCompartment(ThreadSafeContext *cx, const T1 &t1)
+assertSameCompartment(ExclusiveContext *cx, const T1 &t1)
 {
 #ifdef JS_CRASH_DIAGNOSTICS
     START_ASSERT_SAME_COMPARTMENT();
@@ -149,16 +156,16 @@ assertSameCompartment(ThreadSafeContext *cx, const T1 &t1)
 }
 
 template <class T1> inline void
-assertSameCompartmentDebugOnly(ThreadSafeContext *cx, const T1 &t1)
+assertSameCompartmentDebugOnly(ExclusiveContext *cx, const T1 &t1)
 {
-#ifdef DEBUG
+#if defined(DEBUG) && defined(JS_CRASH_DIAGNOSTICS)
     START_ASSERT_SAME_COMPARTMENT();
     c.check(t1);
 #endif
 }
 
 template <class T1, class T2> inline void
-assertSameCompartment(ThreadSafeContext *cx, const T1 &t1, const T2 &t2)
+assertSameCompartment(ExclusiveContext *cx, const T1 &t1, const T2 &t2)
 {
 #ifdef JS_CRASH_DIAGNOSTICS
     START_ASSERT_SAME_COMPARTMENT();
@@ -168,7 +175,7 @@ assertSameCompartment(ThreadSafeContext *cx, const T1 &t1, const T2 &t2)
 }
 
 template <class T1, class T2, class T3> inline void
-assertSameCompartment(ThreadSafeContext *cx, const T1 &t1, const T2 &t2, const T3 &t3)
+assertSameCompartment(ExclusiveContext *cx, const T1 &t1, const T2 &t2, const T3 &t3)
 {
 #ifdef JS_CRASH_DIAGNOSTICS
     START_ASSERT_SAME_COMPARTMENT();
@@ -179,7 +186,7 @@ assertSameCompartment(ThreadSafeContext *cx, const T1 &t1, const T2 &t2, const T
 }
 
 template <class T1, class T2, class T3, class T4> inline void
-assertSameCompartment(ThreadSafeContext *cx,
+assertSameCompartment(ExclusiveContext *cx,
                       const T1 &t1, const T2 &t2, const T3 &t3, const T4 &t4)
 {
 #ifdef JS_CRASH_DIAGNOSTICS
@@ -192,7 +199,7 @@ assertSameCompartment(ThreadSafeContext *cx,
 }
 
 template <class T1, class T2, class T3, class T4, class T5> inline void
-assertSameCompartment(ThreadSafeContext *cx,
+assertSameCompartment(ExclusiveContext *cx,
                       const T1 &t1, const T2 &t2, const T3 &t3, const T4 &t4, const T5 &t5)
 {
 #ifdef JS_CRASH_DIAGNOSTICS
@@ -208,7 +215,7 @@ assertSameCompartment(ThreadSafeContext *cx,
 #undef START_ASSERT_SAME_COMPARTMENT
 
 STATIC_PRECONDITION_ASSUME(ubound(args.argv_) >= argc)
-JS_ALWAYS_INLINE bool
+MOZ_ALWAYS_INLINE bool
 CallJSNative(JSContext *cx, Native native, const CallArgs &args)
 {
     JS_CHECK_RECURSION(cx, return false);
@@ -220,13 +227,13 @@ CallJSNative(JSContext *cx, Native native, const CallArgs &args)
     bool ok = native(cx, args.length(), args.base());
     if (ok) {
         assertSameCompartment(cx, args.rval());
-        JS_ASSERT_IF(!alreadyThrowing, !cx->isExceptionPending());
+        MOZ_ASSERT_IF(!alreadyThrowing, !cx->isExceptionPending());
     }
     return ok;
 }
 
 STATIC_PRECONDITION_ASSUME(ubound(args.argv_) >= argc)
-JS_ALWAYS_INLINE bool
+MOZ_ALWAYS_INLINE bool
 CallNativeImpl(JSContext *cx, NativeImpl impl, const CallArgs &args)
 {
 #ifdef DEBUG
@@ -236,20 +243,20 @@ CallNativeImpl(JSContext *cx, NativeImpl impl, const CallArgs &args)
     bool ok = impl(cx, args);
     if (ok) {
         assertSameCompartment(cx, args.rval());
-        JS_ASSERT_IF(!alreadyThrowing, !cx->isExceptionPending());
+        MOZ_ASSERT_IF(!alreadyThrowing, !cx->isExceptionPending());
     }
     return ok;
 }
 
 STATIC_PRECONDITION(ubound(args.argv_) >= argc)
-JS_ALWAYS_INLINE bool
+MOZ_ALWAYS_INLINE bool
 CallJSNativeConstructor(JSContext *cx, Native native, const CallArgs &args)
 {
 #ifdef DEBUG
     RootedObject callee(cx, &args.callee());
 #endif
 
-    JS_ASSERT(args.thisv().isMagic());
+    MOZ_ASSERT(args.thisv().isMagic());
     if (!CallJSNative(cx, native, args))
         return false;
 
@@ -273,16 +280,16 @@ CallJSNativeConstructor(JSContext *cx, Native native, const CallArgs &args)
      *
      * - (new Object(Object)) returns the callee.
      */
-    JS_ASSERT_IF(native != ProxyObject::callableClass_.construct &&
-                 native != js::CallOrConstructBoundFunction &&
-                 native != js::IteratorConstructor &&
-                 (!callee->is<JSFunction>() || callee->as<JSFunction>().native() != obj_construct),
-                 !args.rval().isPrimitive() && callee != &args.rval().toObject());
+    MOZ_ASSERT_IF(native != js::proxy_Construct &&
+                  native != js::CallOrConstructBoundFunction &&
+                  native != js::IteratorConstructor &&
+                  (!callee->is<JSFunction>() || callee->as<JSFunction>().native() != obj_construct),
+                  args.rval().isObject() && callee != &args.rval().toObject());
 
     return true;
 }
 
-JS_ALWAYS_INLINE bool
+MOZ_ALWAYS_INLINE bool
 CallJSPropertyOp(JSContext *cx, PropertyOp op, HandleObject receiver, HandleId id, MutableHandleValue vp)
 {
     JS_CHECK_RECURSION(cx, return false);
@@ -294,7 +301,7 @@ CallJSPropertyOp(JSContext *cx, PropertyOp op, HandleObject receiver, HandleId i
     return ok;
 }
 
-JS_ALWAYS_INLINE bool
+MOZ_ALWAYS_INLINE bool
 CallJSPropertyOpSetter(JSContext *cx, StrictPropertyOp op, HandleObject obj, HandleId id,
                        bool strict, MutableHandleValue vp)
 {
@@ -311,12 +318,15 @@ CallJSDeletePropertyOp(JSContext *cx, JSDeletePropertyOp op, HandleObject receiv
     JS_CHECK_RECURSION(cx, return false);
 
     assertSameCompartment(cx, receiver, id);
-    return op(cx, receiver, id, succeeded);
+    if (op)
+        return op(cx, receiver, id, succeeded);
+    *succeeded = true;
+    return true;
 }
 
 inline bool
 CallSetter(JSContext *cx, HandleObject obj, HandleId id, StrictPropertyOp op, unsigned attrs,
-           unsigned shortid, bool strict, MutableHandleValue vp)
+           bool strict, MutableHandleValue vp)
 {
     if (attrs & JSPROP_SETTER) {
         RootedValue opv(cx, CastAsObjectJsval(op));
@@ -326,16 +336,14 @@ CallSetter(JSContext *cx, HandleObject obj, HandleId id, StrictPropertyOp op, un
     if (attrs & JSPROP_GETTER)
         return js_ReportGetterOnlyAssignment(cx, strict);
 
-    if (!(attrs & JSPROP_SHORTID))
-        return CallJSPropertyOpSetter(cx, op, obj, id, strict, vp);
+    if (!op)
+        return true;
 
-    RootedId nid(cx, INT_TO_JSID(shortid));
-
-    return CallJSPropertyOpSetter(cx, op, obj, nid, strict, vp);
+    return CallJSPropertyOpSetter(cx, op, obj, id, strict, vp);
 }
 
 inline uintptr_t
-GetNativeStackLimit(ThreadSafeContext *cx)
+GetNativeStackLimit(ExclusiveContext *cx)
 {
     StackKind kind;
     if (cx->isJSContext()) {
@@ -358,28 +366,22 @@ ExclusiveContext::typeLifoAlloc()
 }  /* namespace js */
 
 inline void
-JSContext::setPendingException(js::Value v) {
-    JS_ASSERT(!IsPoisonedValue(v));
+JSContext::setPendingException(js::Value v)
+{
+    MOZ_ASSERT(!IsPoisonedValue(v));
+    // overRecursed_ is set after the fact by js_ReportOverRecursed.
+    this->overRecursed_ = false;
     this->throwing = true;
-    this->exception = v;
-    js::assertSameCompartment(this, v);
+    this->unwrappedException_ = v;
+    // We don't use assertSameCompartment here to allow
+    // js::SetPendingExceptionCrossContext to work.
+    MOZ_ASSERT_IF(v.isObject(), v.toObject().compartment() == compartment());
 }
 
-inline void
-JSContext::setDefaultCompartmentObject(JSObject *obj)
+inline bool
+JSContext::runningWithTrustedPrincipals() const
 {
-    JS_ASSERT(!options().noDefaultCompartmentObject());
-    defaultCompartmentObject_ = obj;
-}
-
-inline void
-JSContext::setDefaultCompartmentObjectIfUnset(JSObject *obj)
-{
-    if (!options().noDefaultCompartmentObject() &&
-        !defaultCompartmentObject_)
-    {
-        setDefaultCompartmentObject(obj);
-    }
+    return !compartment() || compartment()->principals == runtime()->trustedPrincipals();
 }
 
 inline void
@@ -388,54 +390,52 @@ js::ExclusiveContext::enterCompartment(JSCompartment *c)
     enterCompartmentDepth_++;
     c->enter();
     setCompartment(c);
+}
 
-    if (JSContext *cx = maybeJSContext()) {
-        if (cx->throwing)
-            cx->wrapPendingException();
-    }
+inline void
+js::ExclusiveContext::enterNullCompartment()
+{
+    enterCompartmentDepth_++;
+    setCompartment(nullptr);
 }
 
 inline void
 js::ExclusiveContext::leaveCompartment(JSCompartment *oldCompartment)
 {
-    JS_ASSERT(hasEnteredCompartment());
+    MOZ_ASSERT(hasEnteredCompartment());
     enterCompartmentDepth_--;
 
     // Only call leave() after we've setCompartment()-ed away from the current
     // compartment.
     JSCompartment *startingCompartment = compartment_;
     setCompartment(oldCompartment);
-    startingCompartment->leave();
-
-    if (JSContext *cx = maybeJSContext()) {
-        if (cx->throwing && oldCompartment)
-            cx->wrapPendingException();
-    }
+    if (startingCompartment)
+        startingCompartment->leave();
 }
 
 inline void
 js::ExclusiveContext::setCompartment(JSCompartment *comp)
 {
     // ExclusiveContexts can only be in the atoms zone or in exclusive zones.
-    JS_ASSERT_IF(!isJSContext() && !runtime_->isAtomsCompartment(comp),
-                 comp->zone()->usedByExclusiveThread);
+    MOZ_ASSERT_IF(!isJSContext() && !runtime_->isAtomsCompartment(comp),
+                  comp->zone()->usedByExclusiveThread);
 
     // Normal JSContexts cannot enter exclusive zones.
-    JS_ASSERT_IF(isJSContext() && comp,
-                 !comp->zone()->usedByExclusiveThread);
+    MOZ_ASSERT_IF(isJSContext() && comp,
+                  !comp->zone()->usedByExclusiveThread);
 
     // Only one thread can be in the atoms compartment at a time.
-    JS_ASSERT_IF(runtime_->isAtomsCompartment(comp),
-                 runtime_->currentThreadHasExclusiveAccess());
+    MOZ_ASSERT_IF(runtime_->isAtomsCompartment(comp),
+                  runtime_->currentThreadHasExclusiveAccess());
 
     // Make sure that the atoms compartment has its own zone.
-    JS_ASSERT_IF(comp && !runtime_->isAtomsCompartment(comp),
-                 !runtime_->isAtomsZone(comp->zone()));
+    MOZ_ASSERT_IF(comp && !runtime_->isAtomsCompartment(comp),
+                  !runtime_->isAtomsZone(comp->zone()));
 
     // Both the current and the new compartment should be properly marked as
     // entered at this point.
-    JS_ASSERT_IF(compartment_, compartment_->hasBeenEntered());
-    JS_ASSERT_IF(comp, comp->hasBeenEntered());
+    MOZ_ASSERT_IF(compartment_, compartment_->hasBeenEntered());
+    MOZ_ASSERT_IF(comp, comp->hasBeenEntered());
 
     compartment_ = comp;
     zone_ = comp ? comp->zone() : nullptr;
@@ -456,9 +456,8 @@ JSContext::currentScript(jsbytecode **ppc,
     if (!act)
         return nullptr;
 
-    JS_ASSERT(act->cx() == this);
+    MOZ_ASSERT(act->cx() == this);
 
-#ifdef JS_ION
     if (act->isJit()) {
         JSScript *script = nullptr;
         js::jit::GetPcScript(const_cast<JSContext *>(this), &script, ppc);
@@ -466,12 +465,14 @@ JSContext::currentScript(jsbytecode **ppc,
             return nullptr;
         return script;
     }
-#endif
 
-    JS_ASSERT(act->isInterpreter());
+    if (act->isAsmJS())
+        return nullptr;
 
-    js::StackFrame *fp = act->asInterpreter()->current();
-    JS_ASSERT(!fp->runningInJit());
+    MOZ_ASSERT(act->isInterpreter());
+
+    js::InterpreterFrame *fp = act->asInterpreter()->current();
+    MOZ_ASSERT(!fp->runningInJit());
 
     JSScript *script = fp->script();
     if (!allowCrossCompartment && script->compartment() != compartment())
@@ -479,7 +480,7 @@ JSContext::currentScript(jsbytecode **ppc,
 
     if (ppc) {
         *ppc = act->asInterpreter()->regs().pc;
-        JS_ASSERT(script->containsPC(*ppc));
+        MOZ_ASSERT(script->containsPC(*ppc));
     }
     return script;
 }
@@ -493,9 +494,9 @@ JSNativeThreadSafeWrapper(JSContext *cx, unsigned argc, JS::Value *vp)
 
 template <JSThreadSafeNative threadSafeNative>
 inline bool
-JSParallelNativeThreadSafeWrapper(js::ForkJoinSlice *slice, unsigned argc, JS::Value *vp)
+JSParallelNativeThreadSafeWrapper(js::ForkJoinContext *cx, unsigned argc, JS::Value *vp)
 {
-    return threadSafeNative(slice, argc, vp);
+    return threadSafeNative(cx, argc, vp);
 }
 
 /* static */ inline JSContext *
