@@ -32,13 +32,14 @@
 #include "jsstr.h"
 #include "jstypes.h"
 #include "jsutil.h"
+#include "jswrapper.h"
 #include "prmjtime.h"
 
+#include "js/Conversions.h"
 #include "js/Date.h"
 #include "vm/DateTime.h"
 #include "vm/GlobalObject.h"
 #include "vm/Interpreter.h"
-#include "vm/NumericConversions.h"
 #include "vm/String.h"
 #include "vm/StringBuffer.h"
 
@@ -53,6 +54,7 @@ using mozilla::IsNaN;
 
 using JS::AutoCheckCannotGC;
 using JS::GenericNaN;
+using JS::ToInteger;
 
 /*
  * The JS 'Date' object is patterned after the Java 'Date' object.
@@ -121,7 +123,7 @@ TimeWithinDay(double t)
 static inline bool
 IsLeapYear(double year)
 {
-    JS_ASSERT(ToInteger(year) == year);
+    MOZ_ASSERT(ToInteger(year) == year);
     return fmod(year, 4) == 0 && (fmod(year, 100) != 0 || fmod(year, 400) == 0);
 }
 
@@ -154,7 +156,7 @@ YearFromTime(double t)
     if (!IsFinite(t))
         return GenericNaN();
 
-    JS_ASSERT(ToInteger(t) == t);
+    MOZ_ASSERT(ToInteger(t) == t);
 
     double y = floor(t / (msPerDay * 365.2425)) + 1970;
     double t2 = TimeFromYear(y);
@@ -183,7 +185,7 @@ DaysInFebruary(double year)
 static inline double
 DayWithinYear(double t, double year)
 {
-    JS_ASSERT_IF(IsFinite(t), YearFromTime(t) == year);
+    MOZ_ASSERT_IF(IsFinite(t), YearFromTime(t) == year);
     return Day(t) - DayFromYear(year);
 }
 
@@ -277,7 +279,7 @@ WeekDay(double t)
      * We can't assert TimeClip(t) == t because we call this function with
      * local times, which can be offset outside TimeClip's permitted range.
      */
-    JS_ASSERT(ToInteger(t) == t);
+    MOZ_ASSERT(ToInteger(t) == t);
     int result = (int(Day(t)) + 4) % 7;
     if (result < 0)
         result += 7;
@@ -296,13 +298,13 @@ DayFromMonth(int month, bool isLeapYear)
         {0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366}
     };
 
-    JS_ASSERT(0 <= month && month <= 12);
+    MOZ_ASSERT(0 <= month && month <= 12);
     return firstDayOfMonth[isLeapYear][month];
 }
 
 template<typename T>
 static inline int
-DayFromMonth(T month, bool isLeapYear) MOZ_DELETE;
+DayFromMonth(T month, bool isLeapYear) = delete;
 
 /* ES5 15.9.1.12 (out of order to accommodate DaylightSavingTA). */
 static double
@@ -517,8 +519,8 @@ MakeTime(double hour, double min, double sec, double ms)
 static bool
 date_convert(JSContext *cx, HandleObject obj, JSType hint, MutableHandleValue vp)
 {
-    JS_ASSERT(hint == JSTYPE_NUMBER || hint == JSTYPE_STRING || hint == JSTYPE_VOID);
-    JS_ASSERT(obj->is<DateObject>());
+    MOZ_ASSERT(hint == JSTYPE_NUMBER || hint == JSTYPE_STRING || hint == JSTYPE_VOID);
+    MOZ_ASSERT(obj->is<DateObject>());
 
     return DefaultValue(cx, obj, (hint == JSTYPE_VOID) ? JSTYPE_STRING : hint, vp);
 }
@@ -1022,7 +1024,7 @@ ParseDate(const CharT *s, size_t length, double *result, DateTimeInfo *dtInfo)
                              * AM/PM. Count 12:30 AM as 00:30, 12:30 PM as
                              * 12:30, instead of blindly adding 12 if PM.
                              */
-                            JS_ASSERT(action == -1 || action == -2);
+                            MOZ_ASSERT(action == -1 || action == -2);
                             if (hour > 12 || hour < 0)
                                 return false;
 
@@ -1194,8 +1196,8 @@ NowAsMillis()
     return (double) (PRMJ_Now() / PRMJ_USEC_PER_MSEC);
 }
 
-static bool
-date_now(JSContext *cx, unsigned argc, Value *vp)
+bool
+js::date_now(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
     args.rval().setDouble(NowAsMillis());
@@ -1203,14 +1205,19 @@ date_now(JSContext *cx, unsigned argc, Value *vp)
 }
 
 void
-DateObject::setUTCTime(double t, Value *vp)
+DateObject::setUTCTime(double t)
 {
     for (size_t ind = COMPONENTS_START_SLOT; ind < RESERVED_SLOTS; ind++)
         setReservedSlot(ind, UndefinedValue());
 
     setFixedSlot(UTC_TIME_SLOT, DoubleValue(t));
-    if (vp)
-        vp->setDouble(t);
+}
+
+void
+DateObject::setUTCTime(double t, MutableHandleValue vp)
+{
+    setUTCTime(t);
+    vp.setDouble(t);
 }
 
 void
@@ -1673,7 +1680,7 @@ date_setTime_impl(JSContext *cx, CallArgs args)
 {
     Rooted<DateObject*> dateObj(cx, &args.thisv().toObject().as<DateObject>());
     if (args.length() == 0) {
-        dateObj->setUTCTime(GenericNaN(), args.rval().address());
+        dateObj->setUTCTime(GenericNaN(), args.rval());
         return true;
     }
 
@@ -1681,7 +1688,7 @@ date_setTime_impl(JSContext *cx, CallArgs args)
     if (!ToNumber(cx, args[0], &result))
         return false;
 
-    dateObj->setUTCTime(TimeClip(result), args.rval().address());
+    dateObj->setUTCTime(TimeClip(result), args.rval());
     return true;
 }
 
@@ -1741,7 +1748,7 @@ date_setMilliseconds_impl(JSContext *cx, CallArgs args)
     double u = TimeClip(UTC(MakeDate(Day(t), time), &cx->runtime()->dateTimeInfo));
 
     /* Steps 4-5. */
-    dateObj->setUTCTime(u, args.rval().address());
+    dateObj->setUTCTime(u, args.rval());
     return true;
 }
 
@@ -1771,7 +1778,7 @@ date_setUTCMilliseconds_impl(JSContext *cx, CallArgs args)
     double v = TimeClip(MakeDate(Day(t), time));
 
     /* Steps 4-5. */
-    dateObj->setUTCTime(v, args.rval().address());
+    dateObj->setUTCTime(v, args.rval());
     return true;
 }
 
@@ -1808,7 +1815,7 @@ date_setSeconds_impl(JSContext *cx, CallArgs args)
     double u = TimeClip(UTC(date, &cx->runtime()->dateTimeInfo));
 
     /* Steps 6-7. */
-    dateObj->setUTCTime(u, args.rval().address());
+    dateObj->setUTCTime(u, args.rval());
     return true;
 }
 
@@ -1845,7 +1852,7 @@ date_setUTCSeconds_impl(JSContext *cx, CallArgs args)
     double v = TimeClip(date);
 
     /* Steps 6-7. */
-    dateObj->setUTCTime(v, args.rval().address());
+    dateObj->setUTCTime(v, args.rval());
     return true;
 }
 
@@ -1887,7 +1894,7 @@ date_setMinutes_impl(JSContext *cx, CallArgs args)
     double u = TimeClip(UTC(date, &cx->runtime()->dateTimeInfo));
 
     /* Steps 7-8. */
-    dateObj->setUTCTime(u, args.rval().address());
+    dateObj->setUTCTime(u, args.rval());
     return true;
 }
 
@@ -1929,7 +1936,7 @@ date_setUTCMinutes_impl(JSContext *cx, CallArgs args)
     double v = TimeClip(date);
 
     /* Steps 7-8. */
-    dateObj->setUTCTime(v, args.rval().address());
+    dateObj->setUTCTime(v, args.rval());
     return true;
 }
 
@@ -1976,7 +1983,7 @@ date_setHours_impl(JSContext *cx, CallArgs args)
     double u = TimeClip(UTC(date, &cx->runtime()->dateTimeInfo));
 
     /* Steps 7-8. */
-    dateObj->setUTCTime(u, args.rval().address());
+    dateObj->setUTCTime(u, args.rval());
     return true;
 }
 
@@ -2023,7 +2030,7 @@ date_setUTCHours_impl(JSContext *cx, CallArgs args)
     double v = TimeClip(newDate);
 
     /* Steps 8-9. */
-    dateObj->setUTCTime(v, args.rval().address());
+    dateObj->setUTCTime(v, args.rval());
     return true;
 }
 
@@ -2055,7 +2062,7 @@ date_setDate_impl(JSContext *cx, CallArgs args)
     double u = TimeClip(UTC(newDate, &cx->runtime()->dateTimeInfo));
 
     /* Steps 5-6. */
-    dateObj->setUTCTime(u, args.rval().address());
+    dateObj->setUTCTime(u, args.rval());
     return true;
 }
 
@@ -2087,7 +2094,7 @@ date_setUTCDate_impl(JSContext *cx, CallArgs args)
     double v = TimeClip(newDate);
 
     /* Steps 5-6. */
-    dateObj->setUTCTime(v, args.rval().address());
+    dateObj->setUTCTime(v, args.rval());
     return true;
 }
 
@@ -2144,7 +2151,7 @@ date_setMonth_impl(JSContext *cx, CallArgs args)
     double u = TimeClip(UTC(newDate, &cx->runtime()->dateTimeInfo));
 
     /* Steps 6-7. */
-    dateObj->setUTCTime(u, args.rval().address());
+    dateObj->setUTCTime(u, args.rval());
     return true;
 }
 
@@ -2181,7 +2188,7 @@ date_setUTCMonth_impl(JSContext *cx, CallArgs args)
     double v = TimeClip(newDate);
 
     /* Steps 6-7. */
-    dateObj->setUTCTime(v, args.rval().address());
+    dateObj->setUTCTime(v, args.rval());
     return true;
 }
 
@@ -2239,7 +2246,7 @@ date_setFullYear_impl(JSContext *cx, CallArgs args)
     double u = TimeClip(UTC(newDate, &cx->runtime()->dateTimeInfo));
 
     /* Steps 7-8. */
-    dateObj->setUTCTime(u, args.rval().address());
+    dateObj->setUTCTime(u, args.rval());
     return true;
 }
 
@@ -2281,7 +2288,7 @@ date_setUTCFullYear_impl(JSContext *cx, CallArgs args)
     double v = TimeClip(newDate);
 
     /* Steps 7-8. */
-    dateObj->setUTCTime(v, args.rval().address());
+    dateObj->setUTCTime(v, args.rval());
     return true;
 }
 
@@ -2308,7 +2315,7 @@ date_setYear_impl(JSContext *cx, CallArgs args)
 
     /* Step 3. */
     if (IsNaN(y)) {
-        dateObj->setUTCTime(GenericNaN(), args.rval().address());
+        dateObj->setUTCTime(GenericNaN(), args.rval());
         return true;
     }
 
@@ -2324,7 +2331,7 @@ date_setYear_impl(JSContext *cx, CallArgs args)
     double u = UTC(MakeDate(day, TimeWithinDay(t)), &cx->runtime()->dateTimeInfo);
 
     /* Steps 7-8. */
-    dateObj->setUTCTime(TimeClip(u), args.rval().address());
+    dateObj->setUTCTime(TimeClip(u), args.rval());
     return true;
 }
 
@@ -2352,7 +2359,7 @@ static const char * const months[] =
 static void
 print_gmt_string(char* buf, size_t size, double utctime)
 {
-    JS_ASSERT(TimeClip(utctime) == utctime);
+    MOZ_ASSERT(TimeClip(utctime) == utctime);
     JS_snprintf(buf, size, "%s, %.2d %s %.4d %.2d:%.2d:%.2d GMT",
                 days[int(WeekDay(utctime))],
                 int(DateFromTime(utctime)),
@@ -2366,7 +2373,7 @@ print_gmt_string(char* buf, size_t size, double utctime)
 static void
 print_iso_string(char* buf, size_t size, double utctime)
 {
-    JS_ASSERT(TimeClip(utctime) == utctime);
+    MOZ_ASSERT(TimeClip(utctime) == utctime);
     JS_snprintf(buf, size, "%.4d-%.2d-%.2dT%.2d:%.2d:%.2d.%.3dZ",
                 int(YearFromTime(utctime)),
                 int(MonthFromTime(utctime)) + 1,
@@ -2518,7 +2525,7 @@ date_format(JSContext *cx, double date, formatspec format, MutableHandleValue rv
     if (!IsFinite(date)) {
         JS_snprintf(buf, sizeof buf, js_NaN_date_str);
     } else {
-        JS_ASSERT(TimeClip(date) == date);
+        MOZ_ASSERT(TimeClip(date) == date);
 
         double local = LocalTime(date, &cx->runtime()->dateTimeInfo);
 
@@ -2555,7 +2562,7 @@ date_format(JSContext *cx, double date, formatspec format, MutableHandleValue rv
                 usetz = false;
             } else {
                 for (i = 0; i < tzlen; i++) {
-                    jschar c = tzbuf[i];
+                    char16_t c = tzbuf[i];
                     if (c > 127 ||
                         !(isalpha(c) || isdigit(c) ||
                           c == ' ' || c == '(' || c == ')')) {
@@ -2650,8 +2657,10 @@ ToLocaleFormatHelper(JSContext *cx, HandleObject obj, const char *format, Mutabl
             /* ...but not if starts with 4-digit year, like 2022/3/11. */
             !(isdigit(buf[0]) && isdigit(buf[1]) &&
               isdigit(buf[2]) && isdigit(buf[3]))) {
+            double localtime = obj->as<DateObject>().cachedLocalTime(&cx->runtime()->dateTimeInfo);
+            int year = IsNaN(localtime) ? 0 : (int) YearFromTime(localtime);
             JS_snprintf(buf + (result_len - 2), (sizeof buf) - (result_len - 2),
-                        "%d", js_DateGetYear(cx, obj));
+                        "%d", year);
         }
 
     }
@@ -2857,8 +2866,8 @@ date_valueOf_impl(JSContext *cx, CallArgs args)
     return true;
 }
 
-static bool
-date_valueOf(JSContext *cx, unsigned argc, Value *vp)
+bool
+js::date_valueOf(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
     return CallNonGenericMethod<IsDate, date_valueOf_impl>(cx, args);
@@ -3003,30 +3012,30 @@ FinishDateClassInit(JSContext *cx, HandleObject ctor, HandleObject proto)
     RootedValue toUTCStringFun(cx);
     RootedId toUTCStringId(cx, NameToId(cx->names().toUTCString));
     RootedId toGMTStringId(cx, NameToId(cx->names().toGMTString));
-    return baseops::GetProperty(cx, proto, toUTCStringId, &toUTCStringFun) &&
-           baseops::DefineGeneric(cx, proto, toGMTStringId, toUTCStringFun,
-                                  JS_PropertyStub, JS_StrictPropertyStub, 0);
+    return baseops::GetProperty(cx, proto.as<DateObject>(), toUTCStringId, &toUTCStringFun) &&
+           baseops::DefineGeneric(cx, proto.as<DateObject>(), toGMTStringId, toUTCStringFun,
+                                  nullptr, nullptr, 0);
 }
 
 const Class DateObject::class_ = {
     js_Date_str,
     JSCLASS_HAS_RESERVED_SLOTS(RESERVED_SLOTS) |
     JSCLASS_HAS_CACHED_PROTO(JSProto_Date),
-    JS_PropertyStub,         /* addProperty */
-    JS_DeletePropertyStub,   /* delProperty */
-    JS_PropertyStub,         /* getProperty */
-    JS_StrictPropertyStub,   /* setProperty */
-    JS_EnumerateStub,
-    JS_ResolveStub,
+    nullptr, /* addProperty */
+    nullptr, /* delProperty */
+    nullptr, /* getProperty */
+    nullptr, /* setProperty */
+    nullptr, /* enumerate */
+    nullptr, /* resolve */
     date_convert,
-    nullptr,                 /* finalize */
-    nullptr,                 /* call */
-    nullptr,                 /* hasInstance */
-    nullptr,                 /* construct */
-    nullptr,                 /* trace */
+    nullptr, /* finalize */
+    nullptr, /* call */
+    nullptr, /* hasInstance */
+    nullptr, /* construct */
+    nullptr, /* trace */
     {
         GenericCreateConstructor<js_Date, MAXARGS, JSFunction::FinalizeKind>,
-        GenericCreatePrototype<&DateObject::class_>,
+        GenericCreatePrototype,
         date_static_methods,
         date_methods,
         nullptr,
@@ -3048,87 +3057,43 @@ JS_FRIEND_API(JSObject *)
 js_NewDateObject(JSContext *cx, int year, int mon, int mday,
                  int hour, int min, int sec)
 {
-    JS_ASSERT(mon < 12);
+    MOZ_ASSERT(mon < 12);
     double msec_time = date_msecFromDate(year, mon, mday, hour, min, sec, 0);
     return js_NewDateObjectMsec(cx, UTC(msec_time, &cx->runtime()->dateTimeInfo));
 }
 
 JS_FRIEND_API(bool)
-js_DateIsValid(JSObject *obj)
+js::DateIsValid(JSContext *cx, JSObject *objArg)
 {
-    return obj->is<DateObject>() && !IsNaN(obj->as<DateObject>().UTCTime().toNumber());
-}
+    RootedObject obj(cx, objArg);
+    if (!ObjectClassIs(obj, ESClass_Date, cx))
+        return false;
 
-JS_FRIEND_API(int)
-js_DateGetYear(JSContext *cx, JSObject *obj)
-{
-    /* Preserve legacy API behavior of returning 0 for invalid dates. */
-    JS_ASSERT(obj);
-    double localtime = obj->as<DateObject>().cachedLocalTime(&cx->runtime()->dateTimeInfo);
-    if (IsNaN(localtime))
-        return 0;
+    RootedValue unboxed(cx);
+    if (!Unbox(cx, obj, &unboxed)) {
+        // This can't actually happen, so we don't force consumers to deal with
+        // a clunky out-param API. Do something sane-ish if it does happen.
+        cx->clearPendingException();
+        return false;
+    }
 
-    return (int) YearFromTime(localtime);
-}
-
-JS_FRIEND_API(int)
-js_DateGetMonth(JSContext *cx, JSObject *obj)
-{
-    JS_ASSERT(obj);
-    double localtime = obj->as<DateObject>().cachedLocalTime(&cx->runtime()->dateTimeInfo);
-    if (IsNaN(localtime))
-        return 0;
-
-    return (int) MonthFromTime(localtime);
-}
-
-JS_FRIEND_API(int)
-js_DateGetDate(JSContext *cx, JSObject *obj)
-{
-    JS_ASSERT(obj);
-    double localtime = obj->as<DateObject>().cachedLocalTime(&cx->runtime()->dateTimeInfo);
-    if (IsNaN(localtime))
-        return 0;
-
-    return (int) DateFromTime(localtime);
-}
-
-JS_FRIEND_API(int)
-js_DateGetHours(JSContext *cx, JSObject *obj)
-{
-    JS_ASSERT(obj);
-    double localtime = obj->as<DateObject>().cachedLocalTime(&cx->runtime()->dateTimeInfo);
-    if (IsNaN(localtime))
-        return 0;
-
-    return (int) HourFromTime(localtime);
-}
-
-JS_FRIEND_API(int)
-js_DateGetMinutes(JSContext *cx, JSObject *obj)
-{
-    JS_ASSERT(obj);
-    double localtime = obj->as<DateObject>().cachedLocalTime(&cx->runtime()->dateTimeInfo);
-    if (IsNaN(localtime))
-        return 0;
-
-    return (int) MinFromTime(localtime);
-}
-
-JS_FRIEND_API(int)
-js_DateGetSeconds(JSObject *obj)
-{
-    if (!obj->is<DateObject>())
-        return 0;
-
-    double utctime = obj->as<DateObject>().UTCTime().toNumber();
-    if (IsNaN(utctime))
-        return 0;
-    return (int) SecFromTime(utctime);
+    return !IsNaN(unboxed.toNumber());
 }
 
 JS_FRIEND_API(double)
-js_DateGetMsecSinceEpoch(JSObject *obj)
+js::DateGetMsecSinceEpoch(JSContext *cx, JSObject *objArg)
 {
-    return obj->is<DateObject>() ? obj->as<DateObject>().UTCTime().toNumber() : 0;
+    RootedObject obj(cx, objArg);
+    if (!ObjectClassIs(obj, ESClass_Date, cx))
+        return 0;
+
+    RootedValue unboxed(cx);
+    if (!Unbox(cx, obj, &unboxed)) {
+        // This can't actually happen, so we don't force consumers to deal with
+        // a clunky out-param API. Do something sane-ish if it does happen.
+        cx->clearPendingException();
+        return 0;
+    }
+
+    return unboxed.toNumber();
 }
