@@ -6,6 +6,8 @@
 
 #include "jspropertytree.h"
 
+#include "mozilla/DebugOnly.h"
+
 #include "jscntxt.h"
 #include "jsgc.h"
 #include "jstypes.h"
@@ -17,6 +19,9 @@
 #include "vm/Shape-inl.h"
 
 using namespace js;
+using namespace js::gc;
+
+using mozilla::DebugOnly;
 
 inline HashNumber
 ShapeHasher::hash(const Lookup &l)
@@ -30,15 +35,6 @@ ShapeHasher::match(const Key k, const Lookup &l)
     return k->matches(l);
 }
 
-Shape *
-PropertyTree::newShape(ExclusiveContext *cx)
-{
-    Shape *shape = js_NewGCShape(cx);
-    if (!shape)
-        js_ReportOutOfMemory(cx);
-    return shape;
-}
-
 static KidsHash *
 HashChildren(Shape *kid1, Shape *kid2)
 {
@@ -56,11 +52,11 @@ HashChildren(Shape *kid1, Shape *kid2)
 bool
 PropertyTree::insertChild(ExclusiveContext *cx, Shape *parent, Shape *child)
 {
-    JS_ASSERT(!parent->inDictionary());
-    JS_ASSERT(!child->parent);
-    JS_ASSERT(!child->inDictionary());
-    JS_ASSERT(child->compartment() == parent->compartment());
-    JS_ASSERT(cx->isInsideCurrentCompartment(this));
+    MOZ_ASSERT(!parent->inDictionary());
+    MOZ_ASSERT(!child->parent);
+    MOZ_ASSERT(!child->inDictionary());
+    MOZ_ASSERT(child->compartment() == parent->compartment());
+    MOZ_ASSERT(cx->isInsideCurrentCompartment(this));
 
     KidsPointer *kidp = &parent->kids;
 
@@ -72,8 +68,8 @@ PropertyTree::insertChild(ExclusiveContext *cx, Shape *parent, Shape *child)
 
     if (kidp->isShape()) {
         Shape *shape = kidp->toShape();
-        JS_ASSERT(shape != child);
-        JS_ASSERT(!shape->matches(child));
+        MOZ_ASSERT(shape != child);
+        MOZ_ASSERT(!shape->matches(child));
 
         KidsHash *hash = HashChildren(shape, child);
         if (!hash) {
@@ -97,29 +93,35 @@ PropertyTree::insertChild(ExclusiveContext *cx, Shape *parent, Shape *child)
 void
 Shape::removeChild(Shape *child)
 {
-    JS_ASSERT(!child->inDictionary());
-    JS_ASSERT(child->parent == this);
+    MOZ_ASSERT(!child->inDictionary());
+    MOZ_ASSERT(child->parent == this);
 
     KidsPointer *kidp = &kids;
 
     if (kidp->isShape()) {
-        JS_ASSERT(kidp->toShape() == child);
+        MOZ_ASSERT(kidp->toShape() == child);
         kidp->setNull();
         child->parent = nullptr;
         return;
     }
 
     KidsHash *hash = kidp->toHash();
-    JS_ASSERT(hash->count() >= 2);      /* otherwise kidp->isShape() should be true */
+    MOZ_ASSERT(hash->count() >= 2);      /* otherwise kidp->isShape() should be true */
+
+#ifdef DEBUG
+    size_t oldCount = hash->count();
+#endif
 
     hash->remove(StackShape(child));
     child->parent = nullptr;
 
+    MOZ_ASSERT(hash->count() == oldCount - 1);
+
     if (hash->count() == 1) {
         /* Convert from HASH form back to SHAPE form. */
         KidsHash::Range r = hash->all();
         Shape *otherChild = r.front();
-        JS_ASSERT((r.popFront(), r.empty()));    /* No more elements! */
+        MOZ_ASSERT((r.popFront(), r.empty()));    /* No more elements! */
         kidp->setShape(otherChild);
         js_delete(hash);
     }
@@ -129,7 +131,7 @@ Shape *
 PropertyTree::getChild(ExclusiveContext *cx, Shape *parentArg, StackShape &unrootedChild)
 {
     RootedShape parent(cx, parentArg);
-    JS_ASSERT(parent);
+    MOZ_ASSERT(parent);
 
     Shape *existingShape = nullptr;
 
@@ -145,25 +147,24 @@ PropertyTree::getChild(ExclusiveContext *cx, Shape *parentArg, StackShape &unroo
     if (kidp->isShape()) {
         Shape *kid = kidp->toShape();
         if (kid->matches(unrootedChild))
-        existingShape = kid;
+            existingShape = kid;
     } else if (kidp->isHash()) {
         if (KidsHash::Ptr p = kidp->toHash()->lookup(unrootedChild))
-        existingShape = *p;
+            existingShape = *p;
     } else {
         /* If kidp->isNull(), we always insert. */
     }
 
-#ifdef JSGC_INCREMENTAL
     if (existingShape) {
         JS::Zone *zone = existingShape->zone();
-        if (zone->needsBarrier()) {
+        if (zone->needsIncrementalBarrier()) {
             /*
              * We need a read barrier for the shape tree, since these are weak
              * pointers.
              */
             Shape *tmp = existingShape;
             MarkShapeUnbarriered(zone->barrierTracer(), &tmp, "read barrier");
-            JS_ASSERT(tmp == existingShape);
+            MOZ_ASSERT(tmp == existingShape);
         } else if (zone->isGCSweeping() && !existingShape->isMarked() &&
                    !existingShape->arenaHeader()->allocatedDuringIncremental)
         {
@@ -171,62 +172,27 @@ PropertyTree::getChild(ExclusiveContext *cx, Shape *parentArg, StackShape &unroo
              * The shape we've found is unreachable and due to be finalized, so
              * remove our weak reference to it and don't use it.
              */
-            JS_ASSERT(parent->isMarked());
+            MOZ_ASSERT(parent->isMarked());
             parent->removeChild(existingShape);
             existingShape = nullptr;
+        } else if (existingShape->isMarked(gc::GRAY)) {
+            UnmarkGrayShapeRecursively(existingShape);
         }
     }
-#endif
 
     if (existingShape)
         return existingShape;
 
-    RootedGeneric<StackShape*> child(cx, &unrootedChild);
-
-    Shape *shape = newShape(cx);
+    Shape *shape = Shape::new_(cx, unrootedChild, parent->numFixedSlots());
     if (!shape)
         return nullptr;
 
-    new (shape) Shape(*child, parent->numFixedSlots());
-
     if (!insertChild(cx, parent, shape))
         return nullptr;
 
     return shape;
 }
 
-Shape *
-PropertyTree::lookupChild(ThreadSafeContext *cx, Shape *parent, const StackShape &child)
-{
-    /* Keep this in sync with the logic of getChild above. */
-    Shape *shape = nullptr;
-
-    JS_ASSERT(parent);
-
-    KidsPointer *kidp = &parent->kids;
-    if (kidp->isShape()) {
-        Shape *kid = kidp->toShape();
-        if (kid->matches(child))
-            shape = kid;
-    } else if (kidp->isHash()) {
-        if (KidsHash::Ptr p = kidp->toHash()->readonlyThreadsafeLookup(child))
-            shape = *p;
-    } else {
-        return nullptr;
-    }
-
-#if defined(JSGC_INCREMENTAL) && defined(DEBUG)
-    if (shape) {
-        JS::Zone *zone = shape->arenaHeader()->zone;
-        JS_ASSERT(!zone->needsBarrier());
-        JS_ASSERT(!(zone->isGCSweeping() && !shape->isMarked() &&
-                    !shape->arenaHeader()->allocatedDuringIncremental));
-    }
-#endif
-
-    return shape;
-}
-
 void
 Shape::sweep()
 {
@@ -266,18 +232,136 @@ Shape::finalize(FreeOp *fop)
         fop->delete_(kids.toHash());
 }
 
+#ifdef JSGC_COMPACTING
+
+void
+Shape::fixupDictionaryShapeAfterMovingGC()
+{
+    if (!listp)
+        return;
+
+    // It's possible that this shape is unreachable and that listp points to the
+    // location of a dead object in the nursery, in which case we should never
+    // touch it again.
+    if (IsInsideNursery(reinterpret_cast<Cell *>(listp))) {
+        listp = nullptr;
+        return;
+    }
+
+    MOZ_ASSERT(!IsInsideNursery(reinterpret_cast<Cell *>(listp)));
+    AllocKind kind = TenuredCell::fromPointer(listp)->getAllocKind();
+    MOZ_ASSERT(kind == FINALIZE_SHAPE ||
+               kind == FINALIZE_ACCESSOR_SHAPE ||
+               kind <= FINALIZE_OBJECT_LAST);
+    if (kind == FINALIZE_SHAPE || kind == FINALIZE_ACCESSOR_SHAPE) {
+        // listp points to the parent field of the next shape.
+        Shape *next = reinterpret_cast<Shape *>(uintptr_t(listp) -
+                                                offsetof(Shape, parent));
+        listp = &gc::MaybeForwarded(next)->parent;
+    } else {
+        // listp points to the shape_ field of an object.
+        JSObject *last = reinterpret_cast<JSObject *>(uintptr_t(listp) -
+                                                      offsetof(JSObject, shape_));
+        listp = &gc::MaybeForwarded(last)->shape_;
+    }
+}
+
+void
+Shape::fixupShapeTreeAfterMovingGC()
+{
+    if (kids.isNull())
+        return;
+
+    if (kids.isShape()) {
+        if (gc::IsForwarded(kids.toShape()))
+            kids.setShape(gc::Forwarded(kids.toShape()));
+        return;
+    }
+
+    MOZ_ASSERT(kids.isHash());
+    KidsHash *kh = kids.toHash();
+    for (KidsHash::Enum e(*kh); !e.empty(); e.popFront()) {
+        Shape *key = e.front();
+        if (IsForwarded(key))
+            key = Forwarded(key);
+
+        BaseShape *base = key->base();
+        if (IsForwarded(base))
+            base = Forwarded(base);
+        UnownedBaseShape *unowned = base->unowned();
+        if (IsForwarded(unowned))
+            unowned = Forwarded(unowned);
+
+        PropertyOp getter = key->getter();
+        if (key->hasGetterObject())
+            getter = PropertyOp(MaybeForwarded(key->getterObject()));
+
+        StrictPropertyOp setter = key->setter();
+        if (key->hasSetterObject())
+            setter = StrictPropertyOp(MaybeForwarded(key->setterObject()));
+
+        StackShape lookup(unowned,
+                          const_cast<Shape *>(key)->propidRef(),
+                          key->slotInfo & Shape::SLOT_MASK,
+                          key->attrs,
+                          key->flags);
+        lookup.updateGetterSetter(getter, setter);
+        e.rekeyFront(lookup, key);
+    }
+}
+
+void
+Shape::fixupAfterMovingGC()
+{
+    if (inDictionary())
+        fixupDictionaryShapeAfterMovingGC();
+    else
+        fixupShapeTreeAfterMovingGC();
+}
+
+#endif // JSGC_COMPACTING
+
+void
+ShapeGetterSetterRef::mark(JSTracer *trc)
+{
+    // Update the current shape's entry in the parent KidsHash table if needed.
+    // This is necessary as the computed hash includes the getter/setter
+    // pointers.
+
+    JSObject *obj = *objp;
+    JSObject *prior = obj;
+    if (!prior)
+        return;
+
+    trc->setTracingLocation(&*prior);
+    gc::Mark(trc, &obj, "AccessorShape getter or setter");
+    if (obj == *objp)
+        return;
+
+    Shape *parent = shape->parent;
+    if (shape->inDictionary() || !parent->kids.isHash()) {
+        *objp = obj;
+        return;
+    }
+
+    KidsHash *kh = parent->kids.toHash();
+    kh->remove(StackShape(shape));
+    *objp = obj;
+    MOZ_ALWAYS_TRUE(kh->putNew(StackShape(shape), shape));
+}
+
 #ifdef DEBUG
 
 void
 KidsPointer::checkConsistency(Shape *aKid) const
 {
     if (isShape()) {
-        JS_ASSERT(toShape() == aKid);
+        MOZ_ASSERT(toShape() == aKid);
     } else {
-        JS_ASSERT(isHash());
+        MOZ_ASSERT(isHash());
         KidsHash *hash = toHash();
         KidsHash::Ptr ptr = hash->lookup(StackShape(aKid));
-        JS_ASSERT(*ptr == aKid);
+        MOZ_ASSERT(*ptr == aKid);
     }
 }
 
@@ -289,27 +373,23 @@ Shape::dump(JSContext *cx, FILE *fp) const
 
     jsid propid = this->propid();
 
-    JS_ASSERT(!JSID_IS_VOID(propid));
+    MOZ_ASSERT(!JSID_IS_VOID(propid));
 
     if (JSID_IS_INT(propid)) {
         fprintf(fp, "[%ld]", (long) JSID_TO_INT(propid));
+    } else if (JSID_IS_ATOM(propid)) {
+        if (JSLinearString *str = JSID_TO_ATOM(propid))
+            FileEscapedString(fp, str, '"');
+        else
+            fputs("<error>", fp);
     } else {
-        JSLinearString *str;
-        if (JSID_IS_ATOM(propid)) {
-            str = JSID_TO_ATOM(propid);
-            if (!str)
-                fputs("<error>", fp);
-            else
-                FileEscapedString(fp, str, '"');
-        } else {
-            JS_ASSERT(JSID_IS_SYMBOL(propid));
-            JSID_TO_SYMBOL(propid)->dump(fp);
-        }
+        MOZ_ASSERT(JSID_IS_SYMBOL(propid));
+        JSID_TO_SYMBOL(propid)->dump(fp);
     }
 
     fprintf(fp, " g/s %p/%p slot %d attrs %x ",
-            JS_FUNC_TO_DATA_PTR(void *, base()->rawGetter),
-            JS_FUNC_TO_DATA_PTR(void *, base()->rawSetter),
+            JS_FUNC_TO_DATA_PTR(void *, getter()),
+            JS_FUNC_TO_DATA_PTR(void *, setter()),
             hasSlot() ? slot() : -1, attrs);
 
     if (attrs) {
@@ -341,8 +421,8 @@ void
 Shape::dumpSubtree(JSContext *cx, int level, FILE *fp) const
 {
     if (!parent) {
-        JS_ASSERT(level == 0);
-        JS_ASSERT(JSID_IS_EMPTY(propid_));
+        MOZ_ASSERT(level == 0);
+        MOZ_ASSERT(JSID_IS_EMPTY(propid_));
         fprintf(fp, "class %s emptyShape\n", getObjectClass()->name);
     } else {
         fprintf(fp, "%*sid ", level, "");
@@ -353,14 +433,14 @@ Shape::dumpSubtree(JSContext *cx, int level, FILE *fp) const
         ++level;
         if (kids.isShape()) {
             Shape *kid = kids.toShape();
-            JS_ASSERT(kid->parent == this);
+            MOZ_ASSERT(kid->parent == this);
             kid->dumpSubtree(cx, level, fp);
         } else {
             const KidsHash &hash = *kids.toHash();
             for (KidsHash::Range range = hash.all(); !range.empty(); range.popFront()) {
                 Shape *kid = range.front();
 
-                JS_ASSERT(kid->parent == this);
+                MOZ_ASSERT(kid->parent == this);
                 kid->dumpSubtree(cx, level, fp);
             }
         }
