@@ -88,7 +88,7 @@ class JS_FRIEND_API(PerfMeasurement)
      * then the eventsMeasured bitmask will only include the subset of
      * |toMeasure| corresponding to the events that can be measured.
      */
-    PerfMeasurement(EventMask toMeasure);
+    explicit PerfMeasurement(EventMask toMeasure);
 
     /* Done with this set of measurements, tear down OS-level state. */
     ~PerfMeasurement();
@@ -118,7 +118,7 @@ class JS_FRIEND_API(PerfMeasurement)
  * global object).  The JS-visible API is identical to the C++ API.
  */
 extern JS_FRIEND_API(JSObject*)
-    RegisterPerfMeasurement(JSContext *cx, JSObject *global);
+    RegisterPerfMeasurement(JSContext *cx, JS::HandleObject global);
 
 /*
  * Given a Value which contains an instance of the aforementioned
