@@ -6,7 +6,8 @@ if (typeof Symbol === "function") {
         Symbol(),
         Symbol("one"),
         Symbol.for("two"),
-        Symbol.iterator
+        Symbol.iterator,
+        Object(Symbol())
     ];
 
     for (var sym of symbols) {
