@@ -121,6 +121,8 @@ def parse_args():
                          help="Print each test's output to the file given by --output-file.")
     output_og.add_option('-F', '--failed-only', action='store_true',
                          help="If a --show-* option is given, only print output for failed tests.")
+    output_og.add_option('--no-show-failed', action='store_true',
+                         help="Don't print output for failed tests (no-op with --show-output).")
     output_og.add_option('-O', '--output-file',
                          help='Write all output to the given file (default: stdout).')
     output_og.add_option('--failure-file',
@@ -200,8 +202,6 @@ def parse_args():
         except IOError as ex:
             raise SystemExit("Failed to open output file: " + str(ex))
 
-    options.show = options.show_cmd or options.show_output
-
     # Hide the progress bar if it will get in the way of other output.
     options.hide_progress = (options.tinderbox or
                              not ProgressBar.conservative_isatty() or
@@ -306,6 +306,9 @@ def main():
     if test_dir not in ('', '.'):
         os.chdir(test_dir)
 
+    # Force Pacific time zone to avoid failures in Date tests.
+    os.environ['TZ'] = 'PST8PDT'
+
     results = None
     try:
         results = ResultsSink(options, len(skip_list) + len(test_list))
