@@ -52,6 +52,30 @@ def _relpath(path, start=None):
         return os.curdir
     return os.path.join(*rel_list)
 
+# Mapping of Python chars to their javascript string representation.
+QUOTE_MAP = {
+    '\\': '\\\\',
+    '\b': '\\b',
+    '\f': '\\f',
+    '\n': '\\n',
+    '\r': '\\r',
+    '\t': '\\t',
+    '\v': '\\v'
+}
+
+# Quote the string S, javascript style.
+def js_quote(quote, s):
+    result = quote
+    for c in s:
+        if c == quote:
+            result += '\\' + quote
+        elif c in QUOTE_MAP:
+            result += QUOTE_MAP[c]
+        else:
+            result += c
+    result += quote
+    return result
+
 os.path.relpath = _relpath
 
 class Test:
@@ -89,6 +113,7 @@ class Test:
                                         # limits is not considered a failure.
         self.valgrind = False  # True means run under valgrind
         self.tz_pacific = False # True means force Pacific time for the test
+        self.test_also_noasmjs = False # True means run with and without asm.js enabled.
         self.expect_error = '' # Errors to expect and consider passing
         self.expect_status = 0 # Exit status to expect from shell
 
@@ -101,10 +126,28 @@ class Test:
         t.allow_overrecursed = self.allow_overrecursed
         t.valgrind = self.valgrind
         t.tz_pacific = self.tz_pacific
+        t.test_also_noasmjs = self.test_also_noasmjs
         t.expect_error = self.expect_error
         t.expect_status = self.expect_status
         return t
 
+    def copy_and_extend_jitflags(self, variant):
+        t = self.copy()
+        t.jitflags.extend(variant)
+        return t
+
+    def copy_variants(self, variants):
+        # If the tests are flagged with the |jit-test| test-also-noasmjs flags, then
+        # we duplicate the variants such that the test can be used both with the
+        # interpreter and asmjs.  This is a simple way to check for differential
+        # behaviour.
+        if self.test_also_noasmjs:
+            variants = variants + [['--no-asmjs']]
+
+        # For each list of jit flags, make a copy of the test.
+        return [ self.copy_and_extend_jitflags(v) for v in variants ]
+
+
     COOKIE = '|jit-test|'
     CacheDir = JS_CACHE_DIR
 
@@ -137,7 +180,7 @@ class Test:
                         except ValueError:
                             print("warning: couldn't parse thread-count %s" % value)
                     else:
-                        print('warning: unrecognized |jit-test| attribute %s' % part)
+                        print('%s: warning: unrecognized |jit-test| attribute %s' % (path, part))
                 else:
                     if name == 'slow':
                         test.slow = True
@@ -151,16 +194,16 @@ class Test:
                         test.valgrind = options.valgrind
                     elif name == 'tz-pacific':
                         test.tz_pacific = True
-                    elif name == 'debug':
-                        test.jitflags.append('--debugjit')
+                    elif name == 'test-also-noasmjs':
+                        test.test_also_noasmjs = True
                     elif name == 'ion-eager':
                         test.jitflags.append('--ion-eager')
-                    elif name == 'no-ion':
-                        test.jitflags.append('--no-ion')
                     elif name == 'dump-bytecode':
                         test.jitflags.append('--dump-bytecode')
+                    elif name.startswith('--'): # // |jit-test| --ion-gvn=off; --no-sse4
+                        test.jitflags.append(name)
                     else:
-                        print('warning: unrecognized |jit-test| attribute %s' % part)
+                        print('%s: warning: unrecognized |jit-test| attribute %s' % (path, part))
 
         if options.valgrind_all:
             test.valgrind = True
@@ -180,12 +223,14 @@ class Test:
         # whether we use double or single quotes. On windows and when using
         # a remote device, however, we have to be careful to use the quote
         # style that is the opposite of what the exec wrapper uses.
-        # This uses %r to get single quotes on windows and special cases
-        # the remote device.
-        fmt = 'const platform=%r; const libdir=%r; const scriptdir=%r'
         if remote_prefix:
-            fmt = 'const platform="%s"; const libdir="%s"; const scriptdir="%s"'
-        expr = fmt % (sys.platform, libdir, scriptdir_var)
+            quotechar = '"'
+        else:
+            quotechar = "'"
+        expr = ("const platform=%s; const libdir=%s; const scriptdir=%s"
+                % (js_quote(quotechar, sys.platform),
+                   js_quote(quotechar, libdir),
+                   js_quote(quotechar, scriptdir_var)))
 
         # We may have specified '-a' or '-d' twice: once via --jitflags, once
         # via the "|jit-test|" line.  Remove dups because they are toggles.
@@ -349,8 +394,11 @@ def run_test_remote(test, device, prefix, options):
     # the same buffer to both.
     return TestOutput(test, cmd, out, out, returncode, None, False)
 
-def check_output(out, err, rc, timed_out, test):
+def check_output(out, err, rc, timed_out, test, options):
     if timed_out:
+        if test.relpath_tests in options.ignore_timeouts:
+            return True
+
         # The shell sometimes hangs on shutdown on Windows 7 and Windows
         # Server 2008. See bug 970063 comment 7 for a description of the
         # problem. Until bug 956899 is fixed, ignore timeouts on these
@@ -621,14 +669,23 @@ def process_test_results(results, num_tests, options):
 
     try:
         for i, res in enumerate(results):
-            if options.show_output:
+            ok = check_output(res.out, res.err, res.rc, res.timed_out, res.test, options)
+
+            if ok:
+                show_output = options.show_output and not options.failed_only
+            else:
+                show_output = options.show_output or not options.no_show_failed
+
+            if show_output:
+                pb.beginline()
                 sys.stdout.write(res.out)
                 sys.stdout.write(res.err)
                 sys.stdout.write('Exit code: %s\n' % res.rc)
-            if res.test.valgrind:
+
+            if res.test.valgrind and not show_output:
+                pb.beginline()
                 sys.stdout.write(res.err)
 
-            ok = check_output(res.out, res.err, res.rc, res.timed_out, res.test)
             doing = 'after %s' % res.test.relpath_tests
             if not ok:
                 failures.append(res)
@@ -667,14 +724,22 @@ def run_tests(tests, prefix, options):
     return ok
 
 def get_remote_results(tests, device, prefix, options):
-    for i in xrange(0, options.repeat):
-        for test in tests:
-            yield run_test_remote(test, device, prefix, options)
+    from mozdevice import devicemanager
+
+    try:
+        for i in xrange(0, options.repeat):
+            for test in tests:
+                yield run_test_remote(test, device, prefix, options)
+    except devicemanager.DMError as e:
+        # After a devicemanager error, the device is typically in a
+        # state where all further tests will fail so there is no point in
+        # continuing here.
+        sys.stderr.write("Error running remote tests: %s" % e.message)
 
 def push_libs(options, device):
     # This saves considerable time in pushing unnecessary libraries
     # to the device but needs to be updated if the dependencies change.
-    required_libs = ['libnss3.so', 'libmozglue.so']
+    required_libs = ['libnss3.so', 'libmozglue.so', 'libnspr4.so', 'libplc4.so', 'libplds4.so']
 
     for file in os.listdir(options.local_lib):
         if file in required_libs:
