@@ -4,17 +4,12 @@
 /// "Use Terms").   Any redistribution of this code must retain the above
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch10/10.4/10.4.3/10.4.3-1-52gs.js
- * @description Strict - checking 'this' from a global scope (FunctionExpression with a strict directive prologue defined within an Anonymous FunctionExpression)
- * @noStrict
+ * @path ch10/10.4/10.4.3/10.4.3-1-20gs.js
+ * @description Strict - checking 'this' from a global scope (indirect eval includes strict directive prologue)
+ * @onlyStrict
  */
 
-if (! ((function () {
-    var f = function () {
-        "use strict";
-        return typeof this;
-    }
-    return (f()==="undefined") && (this===fnGlobalObject());
-})())) {
+var my_eval = eval;
+if (my_eval("\"use strict\";\nthis") !== fnGlobalObject() ) {
     throw "'this' had incorrect value!";
 }
\ No newline at end of file
