@@ -4,13 +4,16 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch10/10.4/10.4.3/10.4.3-1-75-s.js
- * @description Strict Mode - checking 'this' (strict function declaration called by Function.prototype.call(globalObject))
+ * @path ch10/10.4/10.4.3/10.4.3-1-56-s.js
+ * @description Strict Mode - checking 'this' (Literal setter defined within strict mode)
  * @onlyStrict
  */
     
 function testcase() {
-function f() { "use strict"; return this;};
-return f.call(fnGlobalObject()) === fnGlobalObject();
+"use strict";
+var x = 2;
+var o = { set foo(stuff) { x=this; } }
+o.foo = 3;
+return x===o;
 }
 runTestCase(testcase);
\ No newline at end of file
