@@ -4,21 +4,22 @@
 /// "Use Terms").   Any redistribution of this code must retain the above 
 /// copyright and this notice and otherwise comply with the Use Terms.
 /**
- * @path ch11/11.4/11.4.1/11.4.1-5-a-15-s.js
- * @description Strict Mode - SyntaxError is thrown when deleting a variable of type RegExp
+ * @path ch11/11.13/11.13.2/11.13.2-53-s.js
+ * @description Strict Mode - TypeError is thrown if The LeftHandSide of a Compound Assignment operator(&=) is a reference to a non-existent property of an object whose [[Extensible]] internal property if false
  * @onlyStrict
  */
 
 
 function testcase() {
         "use strict";
-        var regObj = new RegExp();
+        var obj = {};
+        Object.preventExtensions(obj);
 
         try {
-            eval("delete regObj;");
+            obj.len &= 10;
             return false;
         } catch (e) {
-            return e instanceof SyntaxError;
+            return e instanceof TypeError;
         }
     }
 runTestCase(testcase);
