@@ -33,13 +33,13 @@ JSString::sizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf)
     if (isRope())
         return 0;
 
-    JS_ASSERT(isLinear());
+    MOZ_ASSERT(isLinear());
 
     // JSDependentString: do nothing, we'll count the chars when we hit the base string.
     if (isDependent())
         return 0;
 
-    JS_ASSERT(isFlat());
+    MOZ_ASSERT(isFlat());
 
     // JSExtensibleString: count the full capacity, not just the used space.
     if (isExtensible()) {
@@ -80,7 +80,7 @@ JSString::dumpChars(const CharT *s, size_t n, FILE *fp)
 
     fputc('"', fp);
     for (size_t i = 0; i < n; i++) {
-        jschar c = s[i];
+        char16_t c = s[i];
         if (c == '\n')
             fprintf(fp, "\\n");
         else if (c == '\t')
@@ -99,7 +99,7 @@ template void
 JSString::dumpChars(const Latin1Char *s, size_t n, FILE *fp);
 
 template void
-JSString::dumpChars(const jschar *s, size_t n, FILE *fp);
+JSString::dumpChars(const char16_t *s, size_t n, FILE *fp);
 
 void
 JSString::dumpCharsNoNewline(FILE *fp)
@@ -126,8 +126,8 @@ JSString::dump()
                     (void *) chars);
             dumpChars(chars, length(), stderr);
         } else {
-            const jschar *chars = linear->twoByteChars(nogc);
-            fprintf(stderr, "JSString* (%p) = jschar * (%p) = ", (void *) this,
+            const char16_t *chars = linear->twoByteChars(nogc);
+            fprintf(stderr, "JSString* (%p) = char16_t * (%p) = ", (void *) this,
                     (void *) chars);
             dumpChars(chars, length(), stderr);
         }
@@ -150,30 +150,9 @@ JSString::equals(const char *s)
 }
 #endif /* DEBUG */
 
-void
-JSLinearString::debugUnsafeConvertToLatin1()
-{
-    // Temporary helper function to test changes for bug 998392.
-
-    MOZ_ASSERT(hasTwoByteChars());
-    MOZ_ASSERT(!hasBase());
-
-    size_t len = length();
-    const jschar *twoByteChars = rawTwoByteChars();
-    Latin1Char *latin1Chars = (Latin1Char *)twoByteChars;
-
-    for (size_t i = 0; i < len; i++) {
-        MOZ_ASSERT((twoByteChars[i] & 0xff00) == 0);
-        latin1Chars[i] = Latin1Char(twoByteChars[i]);
-    }
-
-    latin1Chars[len] = '\0';
-    d.u1.flags |= LATIN1_CHARS_BIT;
-}
-
 template <typename CharT>
 static MOZ_ALWAYS_INLINE bool
-AllocChars(ThreadSafeContext *maybecx, size_t length, CharT **chars, size_t *capacity)
+AllocChars(JSString *str, size_t length, CharT **chars, size_t *capacity)
 {
     /*
      * String length doesn't include the null char, so include it here before
@@ -194,38 +173,37 @@ AllocChars(ThreadSafeContext *maybecx, size_t length, CharT **chars, size_t *cap
     *capacity = numChars - 1;
 
     JS_STATIC_ASSERT(JSString::MAX_LENGTH * sizeof(CharT) < UINT32_MAX);
-    size_t bytes = numChars * sizeof(CharT);
-    *chars = (CharT *)(maybecx ? maybecx->malloc_(bytes) : js_malloc(bytes));
+    *chars = str->zone()->pod_malloc<CharT>(numChars);
     return *chars != nullptr;
 }
 
 bool
-JSRope::copyLatin1CharsZ(ThreadSafeContext *cx, ScopedJSFreePtr<Latin1Char> &out) const
+JSRope::copyLatin1CharsZ(ExclusiveContext *cx, ScopedJSFreePtr<Latin1Char> &out) const
 {
     return copyCharsInternal<Latin1Char>(cx, out, true);
 }
 
 bool
-JSRope::copyTwoByteCharsZ(ThreadSafeContext *cx, ScopedJSFreePtr<jschar> &out) const
+JSRope::copyTwoByteCharsZ(ExclusiveContext *cx, ScopedJSFreePtr<char16_t> &out) const
 {
-    return copyCharsInternal<jschar>(cx, out, true);
+    return copyCharsInternal<char16_t>(cx, out, true);
 }
 
 bool
-JSRope::copyLatin1Chars(ThreadSafeContext *cx, ScopedJSFreePtr<Latin1Char> &out) const
+JSRope::copyLatin1Chars(ExclusiveContext *cx, ScopedJSFreePtr<Latin1Char> &out) const
 {
     return copyCharsInternal<Latin1Char>(cx, out, false);
 }
 
 bool
-JSRope::copyTwoByteChars(ThreadSafeContext *cx, ScopedJSFreePtr<jschar> &out) const
+JSRope::copyTwoByteChars(ExclusiveContext *cx, ScopedJSFreePtr<char16_t> &out) const
 {
-    return copyCharsInternal<jschar>(cx, out, false);
+    return copyCharsInternal<char16_t>(cx, out, false);
 }
 
 template <typename CharT>
 bool
-JSRope::copyCharsInternal(ThreadSafeContext *cx, ScopedJSFreePtr<CharT> &out,
+JSRope::copyCharsInternal(ExclusiveContext *cx, ScopedJSFreePtr<CharT> &out,
                           bool nullTerminate) const
 {
     /*
@@ -259,7 +237,7 @@ JSRope::copyCharsInternal(ThreadSafeContext *cx, ScopedJSFreePtr<CharT> &out,
         }
     }
 
-    JS_ASSERT(pos == out + n);
+    MOZ_ASSERT(pos == out + n);
 
     if (nullTerminate)
         out[n] = 0;
@@ -271,7 +249,7 @@ namespace js {
 
 template <>
 void
-CopyChars(jschar *dest, const JSLinearString &str)
+CopyChars(char16_t *dest, const JSLinearString &str)
 {
     AutoCheckCannotGC nogc;
     if (str.hasTwoByteChars())
@@ -297,7 +275,7 @@ CopyChars(Latin1Char *dest, const JSLinearString &str)
          * range, so we can safely deflate here.
          */
         size_t len = str.length();
-        const jschar *chars = str.twoByteChars(nogc);
+        const char16_t *chars = str.twoByteChars(nogc);
         for (size_t i = 0; i < len; i++) {
             MOZ_ASSERT(chars[i] <= JSString::MAX_LATIN1_CHAR);
             dest[i] = chars[i];
@@ -366,19 +344,19 @@ JSRope::flattenInternal(ExclusiveContext *maybecx)
     if (leftMostRope->leftChild()->isExtensible()) {
         JSExtensibleString &left = leftMostRope->leftChild()->asExtensible();
         size_t capacity = left.capacity();
-        if (capacity >= wholeLength && left.hasTwoByteChars() == IsSame<CharT, jschar>::value) {
+        if (capacity >= wholeLength && left.hasTwoByteChars() == IsSame<CharT, char16_t>::value) {
             /*
              * Simulate a left-most traversal from the root to leftMost->leftChild()
              * via first_visit_node
              */
-            JS_ASSERT(str->isRope());
+            MOZ_ASSERT(str->isRope());
             while (str != leftMostRope) {
                 if (b == WithIncrementalBarrier) {
                     JSString::writeBarrierPre(str->d.s.u2.left);
                     JSString::writeBarrierPre(str->d.s.u3.right);
                 }
                 JSString *child = str->d.s.u2.left;
-                JS_ASSERT(child->isRope());
+                MOZ_ASSERT(child->isRope());
                 str->setNonInlineChars(left.nonInlineChars<CharT>(nogc));
                 child->d.u1.flattenData = uintptr_t(str) | Tag_VisitRightChild;
                 str = child;
@@ -400,7 +378,7 @@ JSRope::flattenInternal(ExclusiveContext *maybecx)
         }
     }
 
-    if (!AllocChars(maybecx, wholeLength, &wholeChars, &wholeCapacity))
+    if (!AllocChars(this, wholeLength, &wholeChars, &wholeCapacity))
         return nullptr;
 
     pos = wholeChars;
@@ -435,10 +413,10 @@ JSRope::flattenInternal(ExclusiveContext *maybecx)
     }
     finish_node: {
         if (str == this) {
-            JS_ASSERT(pos == wholeChars + wholeLength);
+            MOZ_ASSERT(pos == wholeChars + wholeLength);
             *pos = '\0';
             str->d.u1.length = wholeLength;
-            if (IsSame<CharT, jschar>::value)
+            if (IsSame<CharT, char16_t>::value)
                 str->d.u1.flags = EXTENSIBLE_FLAGS;
             else
                 str->d.u1.flags = EXTENSIBLE_FLAGS | LATIN1_CHARS_BIT;
@@ -449,7 +427,7 @@ JSRope::flattenInternal(ExclusiveContext *maybecx)
             return &this->asFlat();
         }
         uintptr_t flattenData = str->d.u1.flattenData;
-        if (IsSame<CharT, jschar>::value)
+        if (IsSame<CharT, char16_t>::value)
             str->d.u1.flags = DEPENDENT_FLAGS;
         else
             str->d.u1.flags = DEPENDENT_FLAGS | LATIN1_CHARS_BIT;
@@ -459,7 +437,7 @@ JSRope::flattenInternal(ExclusiveContext *maybecx)
         str = (JSString *)(flattenData & ~Tag_Mask);
         if ((flattenData & Tag_Mask) == Tag_VisitRightChild)
             goto visit_right_child;
-        JS_ASSERT((flattenData & Tag_Mask) == Tag_FinishNode);
+        MOZ_ASSERT((flattenData & Tag_Mask) == Tag_FinishNode);
         goto finish_node;
     }
 }
@@ -469,28 +447,26 @@ JSFlatString *
 JSRope::flattenInternal(ExclusiveContext *maybecx)
 {
     if (hasTwoByteChars())
-        return flattenInternal<b, jschar>(maybecx);
+        return flattenInternal<b, char16_t>(maybecx);
     return flattenInternal<b, Latin1Char>(maybecx);
 }
 
 JSFlatString *
 JSRope::flatten(ExclusiveContext *maybecx)
 {
-#ifdef JSGC_INCREMENTAL
-    if (zone()->needsBarrier())
+    if (zone()->needsIncrementalBarrier())
         return flattenInternal<WithIncrementalBarrier>(maybecx);
-#endif
     return flattenInternal<NoBarrier>(maybecx);
 }
 
 template <AllowGC allowGC>
 JSString *
-js::ConcatStrings(ThreadSafeContext *cx,
+js::ConcatStrings(ExclusiveContext *cx,
                   typename MaybeRooted<JSString*, allowGC>::HandleType left,
                   typename MaybeRooted<JSString*, allowGC>::HandleType right)
 {
-    JS_ASSERT_IF(!left->isAtom(), cx->isInsideCurrentZone(left));
-    JS_ASSERT_IF(!right->isAtom(), cx->isInsideCurrentZone(right));
+    MOZ_ASSERT_IF(!left->isAtom(), cx->isInsideCurrentZone(left));
+    MOZ_ASSERT_IF(!right->isAtom(), cx->isInsideCurrentZone(right));
 
     size_t leftLen = left->length();
     if (leftLen == 0)
@@ -509,32 +485,36 @@ js::ConcatStrings(ThreadSafeContext *cx,
                            ? JSFatInlineString::latin1LengthFits(wholeLength)
                            : JSFatInlineString::twoByteLengthFits(wholeLength);
     if (canUseFatInline && cx->isJSContext()) {
-        JSFatInlineString *str = NewGCFatInlineString<allowGC>(cx);
+        Latin1Char *latin1Buf;
+        char16_t *twoByteBuf;
+        JSInlineString *str = isLatin1
+            ? AllocateFatInlineString<allowGC>(cx, wholeLength, &latin1Buf)
+            : AllocateFatInlineString<allowGC>(cx, wholeLength, &twoByteBuf);
         if (!str)
             return nullptr;
 
         AutoCheckCannotGC nogc;
-        ScopedThreadSafeStringInspector leftInspector(left);
-        ScopedThreadSafeStringInspector rightInspector(right);
-        if (!leftInspector.ensureChars(cx, nogc) || !rightInspector.ensureChars(cx, nogc))
+        JSLinearString *leftLinear = left->ensureLinear(cx);
+        if (!leftLinear)
+            return nullptr;
+        JSLinearString *rightLinear = right->ensureLinear(cx);
+        if (!rightLinear)
             return nullptr;
 
         if (isLatin1) {
-            Latin1Char *buf = str->initLatin1(wholeLength);
-            PodCopy(buf, leftInspector.latin1Chars(), leftLen);
-            PodCopy(buf + leftLen, rightInspector.latin1Chars(), rightLen);
-            buf[wholeLength] = 0;
+            PodCopy(latin1Buf, leftLinear->latin1Chars(nogc), leftLen);
+            PodCopy(latin1Buf + leftLen, rightLinear->latin1Chars(nogc), rightLen);
+            latin1Buf[wholeLength] = 0;
         } else {
-            jschar *buf = str->initTwoByte(wholeLength);
-            if (leftInspector.hasTwoByteChars())
-                PodCopy(buf, leftInspector.twoByteChars(), leftLen);
+            if (leftLinear->hasTwoByteChars())
+                PodCopy(twoByteBuf, leftLinear->twoByteChars(nogc), leftLen);
             else
-                CopyAndInflateChars(buf, leftInspector.latin1Chars(), leftLen);
-            if (rightInspector.hasTwoByteChars())
-                PodCopy(buf + leftLen, rightInspector.twoByteChars(), rightLen);
+                CopyAndInflateChars(twoByteBuf, leftLinear->latin1Chars(nogc), leftLen);
+            if (rightLinear->hasTwoByteChars())
+                PodCopy(twoByteBuf + leftLen, rightLinear->twoByteChars(nogc), rightLen);
             else
-                CopyAndInflateChars(buf + leftLen, rightInspector.latin1Chars(), rightLen);
-            buf[wholeLength] = 0;
+                CopyAndInflateChars(twoByteBuf + leftLen, rightLinear->latin1Chars(nogc), rightLen);
+            twoByteBuf[wholeLength] = 0;
         }
 
         return str;
@@ -544,10 +524,10 @@ js::ConcatStrings(ThreadSafeContext *cx,
 }
 
 template JSString *
-js::ConcatStrings<CanGC>(ThreadSafeContext *cx, HandleString left, HandleString right);
+js::ConcatStrings<CanGC>(ExclusiveContext *cx, HandleString left, HandleString right);
 
 template JSString *
-js::ConcatStrings<NoGC>(ThreadSafeContext *cx, JSString *left, JSString *right);
+js::ConcatStrings<NoGC>(ExclusiveContext *cx, JSString *left, JSString *right);
 
 template <typename CharT>
 JSFlatString *
@@ -585,10 +565,10 @@ JSDependentString::undependInternal(ExclusiveContext *cx)
 JSFlatString *
 JSDependentString::undepend(ExclusiveContext *cx)
 {
-    JS_ASSERT(JSString::isDependent());
+    MOZ_ASSERT(JSString::isDependent());
     return hasLatin1Chars()
            ? undependInternal<Latin1Char>(cx)
-           : undependInternal<jschar>(cx);
+           : undependInternal<char16_t>(cx);
 }
 
 template <typename CharT>
@@ -643,56 +623,7 @@ template bool
 JSFlatString::isIndexSlow(const Latin1Char *s, size_t length, uint32_t *indexp);
 
 template bool
-JSFlatString::isIndexSlow(const jschar *s, size_t length, uint32_t *indexp);
-
-bool
-ScopedThreadSafeStringInspector::ensureChars(ThreadSafeContext *cx, const AutoCheckCannotGC &nogc)
-{
-    if (state_ != Uninitialized)
-        return true;
-
-    if (cx->isExclusiveContext()) {
-        JSLinearString *linear = str_->ensureLinear(cx->asExclusiveContext());
-        if (!linear)
-            return false;
-        if (linear->hasTwoByteChars()) {
-            state_ = TwoByte;
-            twoByteChars_ = linear->twoByteChars(nogc);
-        } else {
-            state_ = Latin1;
-            latin1Chars_ = linear->latin1Chars(nogc);
-        }
-    } else {
-        if (str_->isLinear()) {
-            if (str_->hasLatin1Chars()) {
-                state_ = Latin1;
-                latin1Chars_ = str_->asLinear().latin1Chars(nogc);
-            } else {
-                state_ = TwoByte;
-                twoByteChars_ = str_->asLinear().twoByteChars(nogc);
-            }
-        } else {
-            if (str_->hasLatin1Chars()) {
-                ScopedJSFreePtr<Latin1Char> chars;
-                if (!str_->asRope().copyLatin1Chars(cx, chars))
-                    return false;
-                state_ = Latin1;
-                latin1Chars_ = chars;
-                scopedChars_ = chars.forget();
-            } else {
-                ScopedJSFreePtr<jschar> chars;
-                if (!str_->asRope().copyTwoByteChars(cx, chars))
-                    return false;
-                state_ = TwoByte;
-                twoByteChars_ = chars;
-                scopedChars_ = chars.forget();
-            }
-        }
-    }
-
-    MOZ_ASSERT(state_ != Uninitialized);
-    return true;
-}
+JSFlatString::isIndexSlow(const char16_t *s, size_t length, uint32_t *indexp);
 
 /*
  * Set up some tools to make it easier to generate large tables. After constant
@@ -710,9 +641,9 @@ ScopedThreadSafeStringInspector::ensureChars(ThreadSafeContext *cx, const AutoCh
  * This is used when we generate our table of short strings, so the compiler is
  * happier if we use |c| as few times as possible.
  */
-#define FROM_SMALL_CHAR(c) jschar((c) + ((c) < 10 ? '0' :      \
-                                         (c) < 36 ? 'a' - 10 : \
-                                         'A' - 36))
+#define FROM_SMALL_CHAR(c) Latin1Char((c) + ((c) < 10 ? '0' :      \
+                                             (c) < 36 ? 'a' - 10 : \
+                                             'A' - 36))
 
 /*
  * Declare length-2 strings. We only store strings where both characters are
@@ -734,8 +665,11 @@ StaticStrings::init(JSContext *cx)
     AutoLockForExclusiveAccess lock(cx);
     AutoCompartment ac(cx, cx->runtime()->atomsCompartment());
 
+    static_assert(UNIT_STATIC_LIMIT - 1 <= JSString::MAX_LATIN1_CHAR,
+                  "Unit strings must fit in Latin1Char.");
+
     for (uint32_t i = 0; i < UNIT_STATIC_LIMIT; i++) {
-        jschar buffer[] = { jschar(i), '\0' };
+        Latin1Char buffer[] = { Latin1Char(i), '\0' };
         JSFlatString *s = NewStringCopyN<NoGC>(cx, buffer, 1);
         if (!s)
             return false;
@@ -743,7 +677,7 @@ StaticStrings::init(JSContext *cx)
     }
 
     for (uint32_t i = 0; i < NUM_SMALL_CHARS * NUM_SMALL_CHARS; i++) {
-        jschar buffer[] = { FROM_SMALL_CHAR(i >> 6), FROM_SMALL_CHAR(i & 0x3F), '\0' };
+        Latin1Char buffer[] = { FROM_SMALL_CHAR(i >> 6), FROM_SMALL_CHAR(i & 0x3F), '\0' };
         JSFlatString *s = NewStringCopyN<NoGC>(cx, buffer, 2);
         if (!s)
             return false;
@@ -758,10 +692,10 @@ StaticStrings::init(JSContext *cx)
                 TO_SMALL_CHAR((i % 10) + '0');
             intStaticTable[i] = length2StaticTable[index];
         } else {
-            jschar buffer[] = { jschar('0' + (i / 100)),
-                                jschar('0' + ((i / 10) % 10)),
-                                jschar('0' + (i % 10)),
-                                '\0' };
+            Latin1Char buffer[] = { Latin1Char('0' + (i / 100)),
+                                    Latin1Char('0' + ((i / 10) % 10)),
+                                    Latin1Char('0' + (i % 10)),
+                                    '\0' };
             JSFlatString *s = NewStringCopyN<NoGC>(cx, buffer, 3);
             if (!s)
                 return false;
@@ -793,8 +727,10 @@ template <typename CharT>
 StaticStrings::isStatic(const CharT *chars, size_t length)
 {
     switch (length) {
-      case 1:
-        return chars[0] < UNIT_STATIC_LIMIT;
+      case 1: {
+        char16_t c = chars[0];
+        return c < UNIT_STATIC_LIMIT;
+      }
       case 2:
         return fitsInSmallChar(chars[0]) && fitsInSmallChar(chars[1]);
       case 3:
@@ -829,7 +765,7 @@ AutoStableStringChars::~AutoStableStringChars()
         if (state_ == Latin1)
             js_free(const_cast<Latin1Char*>(latin1Chars_));
         else
-            js_free(const_cast<jschar*>(twoByteChars_));
+            js_free(const_cast<char16_t*>(twoByteChars_));
     }
 }
 
@@ -870,7 +806,7 @@ AutoStableStringChars::initTwoByte(JSContext *cx, JSString *s)
         return true;
     }
 
-    jschar *chars = cx->pod_malloc<jschar>(linearString->length() + 1);
+    char16_t *chars = cx->pod_malloc<char16_t>(linearString->length() + 1);
     if (!chars)
         return false;
 
@@ -885,8 +821,6 @@ AutoStableStringChars::initTwoByte(JSContext *cx, JSString *s)
     return true;
 }
 
-bool js::EnableLatin1Strings = true;
-
 #ifdef DEBUG
 void
 JSAtom::dump()
@@ -911,7 +845,7 @@ js::NewDependentString(JSContext *cx, JSString *baseArg, size_t start, size_t le
 
     if (base->hasTwoByteChars()) {
         AutoCheckCannotGC nogc;
-        const jschar *chars = base->twoByteChars(nogc) + start;
+        const char16_t *chars = base->twoByteChars(nogc) + start;
         if (JSLinearString *staticStr = cx->staticStrings().lookup(chars, length))
             return staticStr;
     } else {
@@ -924,31 +858,10 @@ js::NewDependentString(JSContext *cx, JSString *baseArg, size_t start, size_t le
     return JSDependentString::new_(cx, base, start, length);
 }
 
-template <typename CharT>
-static void
-CopyCharsMaybeInflate(jschar *dest, const CharT *src, size_t len);
-
-template <>
-void
-CopyCharsMaybeInflate(jschar *dest, const jschar *src, size_t len)
-{
-    PodCopy(dest, src, len);
-}
-
-template <>
-void
-CopyCharsMaybeInflate(jschar *dest, const Latin1Char *src, size_t len)
-{
-    CopyAndInflateChars(dest, src, len);
-}
-
 static bool
-CanStoreCharsAsLatin1(const jschar *s, size_t length)
+CanStoreCharsAsLatin1(const char16_t *s, size_t length)
 {
-    if (!EnableLatin1Strings)
-        return false;
-
-    for (const jschar *end = s + length; s < end; ++s) {
+    for (const char16_t *end = s + length; s < end; ++s) {
         if (*s > JSString::MAX_LATIN1_CHAR)
             return false;
     }
@@ -964,10 +877,8 @@ CanStoreCharsAsLatin1(const Latin1Char *s, size_t length)
 
 template <AllowGC allowGC>
 static MOZ_ALWAYS_INLINE JSInlineString *
-NewFatInlineStringDeflated(ThreadSafeContext *cx, mozilla::Range<const jschar> chars)
+NewFatInlineStringDeflated(ExclusiveContext *cx, mozilla::Range<const char16_t> chars)
 {
-    MOZ_ASSERT(EnableLatin1Strings);
-
     size_t len = chars.length();
     Latin1Char *storage;
     JSInlineString *str = AllocateFatInlineString<allowGC>(cx, len, &storage);
@@ -984,12 +895,10 @@ NewFatInlineStringDeflated(ThreadSafeContext *cx, mozilla::Range<const jschar> c
 
 template <AllowGC allowGC>
 static JSFlatString *
-NewStringDeflated(ThreadSafeContext *cx, const jschar *s, size_t n)
+NewStringDeflated(ExclusiveContext *cx, const char16_t *s, size_t n)
 {
-    MOZ_ASSERT(EnableLatin1Strings);
-
     if (JSFatInlineString::latin1LengthFits(n))
-        return NewFatInlineStringDeflated<allowGC>(cx, mozilla::Range<const jschar>(s, n));
+        return NewFatInlineStringDeflated<allowGC>(cx, mozilla::Range<const char16_t>(s, n));
 
     ScopedJSFreePtr<Latin1Char> news(cx->pod_malloc<Latin1Char>(n + 1));
     if (!news)
@@ -1011,17 +920,17 @@ NewStringDeflated(ThreadSafeContext *cx, const jschar *s, size_t n)
 
 template <AllowGC allowGC>
 static JSFlatString *
-NewStringDeflated(ThreadSafeContext *cx, const Latin1Char *s, size_t n)
+NewStringDeflated(ExclusiveContext *cx, const Latin1Char *s, size_t n)
 {
     MOZ_CRASH("Shouldn't be called for Latin1 chars");
 }
 
 template <AllowGC allowGC, typename CharT>
 JSFlatString *
-js::NewStringDontDeflate(ThreadSafeContext *cx, CharT *chars, size_t length)
+js::NewStringDontDeflate(ExclusiveContext *cx, CharT *chars, size_t length)
 {
     if (length == 1) {
-        jschar c = chars[0];
+        char16_t c = chars[0];
         if (StaticStrings::hasUnit(c)) {
             // Free |chars| because we're taking possession of it, but it's no
             // longer needed because we use the static string instead.
@@ -1030,28 +939,38 @@ js::NewStringDontDeflate(ThreadSafeContext *cx, CharT *chars, size_t length)
         }
     }
 
+    if (JSFatInlineString::lengthFits<CharT>(length)) {
+        JSInlineString *str =
+            NewFatInlineString<allowGC>(cx, mozilla::Range<const CharT>(chars, length));
+        if (!str)
+            return nullptr;
+
+        js_free(chars);
+        return str;
+    }
+
     return JSFlatString::new_<allowGC>(cx, chars, length);
 }
 
 template JSFlatString *
-js::NewStringDontDeflate<CanGC>(ThreadSafeContext *cx, jschar *chars, size_t length);
+js::NewStringDontDeflate<CanGC>(ExclusiveContext *cx, char16_t *chars, size_t length);
 
 template JSFlatString *
-js::NewStringDontDeflate<NoGC>(ThreadSafeContext *cx, jschar *chars, size_t length);
+js::NewStringDontDeflate<NoGC>(ExclusiveContext *cx, char16_t *chars, size_t length);
 
 template JSFlatString *
-js::NewStringDontDeflate<CanGC>(ThreadSafeContext *cx, Latin1Char *chars, size_t length);
+js::NewStringDontDeflate<CanGC>(ExclusiveContext *cx, Latin1Char *chars, size_t length);
 
 template JSFlatString *
-js::NewStringDontDeflate<NoGC>(ThreadSafeContext *cx, Latin1Char *chars, size_t length);
+js::NewStringDontDeflate<NoGC>(ExclusiveContext *cx, Latin1Char *chars, size_t length);
 
 template <AllowGC allowGC, typename CharT>
 JSFlatString *
-js::NewString(ThreadSafeContext *cx, CharT *chars, size_t length)
+js::NewString(ExclusiveContext *cx, CharT *chars, size_t length)
 {
-    if (IsSame<CharT, jschar>::value && CanStoreCharsAsLatin1(chars, length)) {
+    if (IsSame<CharT, char16_t>::value && CanStoreCharsAsLatin1(chars, length)) {
         if (length == 1) {
-            jschar c = chars[0];
+            char16_t c = chars[0];
             if (StaticStrings::hasUnit(c)) {
                 js_free(chars);
                 return cx->staticStrings().getUnit(c);
@@ -1071,50 +990,31 @@ js::NewString(ThreadSafeContext *cx, CharT *chars, size_t length)
 }
 
 template JSFlatString *
-js::NewString<CanGC>(ThreadSafeContext *cx, jschar *chars, size_t length);
+js::NewString<CanGC>(ExclusiveContext *cx, char16_t *chars, size_t length);
 
 template JSFlatString *
-js::NewString<NoGC>(ThreadSafeContext *cx, jschar *chars, size_t length);
+js::NewString<NoGC>(ExclusiveContext *cx, char16_t *chars, size_t length);
 
 template JSFlatString *
-js::NewString<CanGC>(ThreadSafeContext *cx, Latin1Char *chars, size_t length);
+js::NewString<CanGC>(ExclusiveContext *cx, Latin1Char *chars, size_t length);
 
 template JSFlatString *
-js::NewString<NoGC>(ThreadSafeContext *cx, Latin1Char *chars, size_t length);
+js::NewString<NoGC>(ExclusiveContext *cx, Latin1Char *chars, size_t length);
 
 namespace js {
 
 template <AllowGC allowGC, typename CharT>
 JSFlatString *
-NewStringCopyNDontDeflate(ThreadSafeContext *cx, const CharT *s, size_t n)
+NewStringCopyNDontDeflate(ExclusiveContext *cx, const CharT *s, size_t n)
 {
-    if (EnableLatin1Strings) {
-        if (JSFatInlineString::lengthFits<CharT>(n))
-            return NewFatInlineString<allowGC>(cx, mozilla::Range<const CharT>(s, n));
-
-        ScopedJSFreePtr<CharT> news(cx->pod_malloc<CharT>(n + 1));
-        if (!news)
-            return nullptr;
-
-        PodCopy(news.get(), s, n);
-        news[n] = 0;
-
-        JSFlatString *str = JSFlatString::new_<allowGC>(cx, news.get(), n);
-        if (!str)
-            return nullptr;
-
-        news.forget();
-        return str;
-    }
-
-    if (JSFatInlineString::twoByteLengthFits(n))
+    if (JSFatInlineString::lengthFits<CharT>(n))
         return NewFatInlineString<allowGC>(cx, mozilla::Range<const CharT>(s, n));
 
-    ScopedJSFreePtr<jschar> news(cx->pod_malloc<jschar>(n + 1));
+    ScopedJSFreePtr<CharT> news(cx->pod_malloc<CharT>(n + 1));
     if (!news)
         return nullptr;
 
-    CopyCharsMaybeInflate(news.get(), s, n);
+    PodCopy(news.get(), s, n);
     news[n] = 0;
 
     JSFlatString *str = JSFlatString::new_<allowGC>(cx, news.get(), n);
@@ -1126,37 +1026,37 @@ NewStringCopyNDontDeflate(ThreadSafeContext *cx, const CharT *s, size_t n)
 }
 
 template JSFlatString *
-NewStringCopyNDontDeflate<CanGC>(ThreadSafeContext *cx, const jschar *s, size_t n);
+NewStringCopyNDontDeflate<CanGC>(ExclusiveContext *cx, const char16_t *s, size_t n);
 
 template JSFlatString *
-NewStringCopyNDontDeflate<NoGC>(ThreadSafeContext *cx, const jschar *s, size_t n);
+NewStringCopyNDontDeflate<NoGC>(ExclusiveContext *cx, const char16_t *s, size_t n);
 
 template JSFlatString *
-NewStringCopyNDontDeflate<CanGC>(ThreadSafeContext *cx, const Latin1Char *s, size_t n);
+NewStringCopyNDontDeflate<CanGC>(ExclusiveContext *cx, const Latin1Char *s, size_t n);
 
 template JSFlatString *
-NewStringCopyNDontDeflate<NoGC>(ThreadSafeContext *cx, const Latin1Char *s, size_t n);
+NewStringCopyNDontDeflate<NoGC>(ExclusiveContext *cx, const Latin1Char *s, size_t n);
 
 template <AllowGC allowGC, typename CharT>
 JSFlatString *
-NewStringCopyN(ThreadSafeContext *cx, const CharT *s, size_t n)
+NewStringCopyN(ExclusiveContext *cx, const CharT *s, size_t n)
 {
-    if (IsSame<CharT, jschar>::value && CanStoreCharsAsLatin1(s, n))
+    if (IsSame<CharT, char16_t>::value && CanStoreCharsAsLatin1(s, n))
         return NewStringDeflated<allowGC>(cx, s, n);
 
     return NewStringCopyNDontDeflate<allowGC>(cx, s, n);
 }
 
 template JSFlatString *
-NewStringCopyN<CanGC>(ThreadSafeContext *cx, const jschar *s, size_t n);
+NewStringCopyN<CanGC>(ExclusiveContext *cx, const char16_t *s, size_t n);
 
 template JSFlatString *
-NewStringCopyN<NoGC>(ThreadSafeContext *cx, const jschar *s, size_t n);
+NewStringCopyN<NoGC>(ExclusiveContext *cx, const char16_t *s, size_t n);
 
 template JSFlatString *
-NewStringCopyN<CanGC>(ThreadSafeContext *cx, const Latin1Char *s, size_t n);
+NewStringCopyN<CanGC>(ExclusiveContext *cx, const Latin1Char *s, size_t n);
 
 template JSFlatString *
-NewStringCopyN<NoGC>(ThreadSafeContext *cx, const Latin1Char *s, size_t n);
+NewStringCopyN<NoGC>(ExclusiveContext *cx, const Latin1Char *s, size_t n);
 
 } /* namespace js */
