@@ -27,12 +27,15 @@ XDRBuffer::freeBuffer()
 bool
 XDRBuffer::grow(size_t n)
 {
-    JS_ASSERT(n > size_t(limit - cursor));
+    MOZ_ASSERT(n > size_t(limit - cursor));
 
-    const size_t MEM_BLOCK = 8192;
+    const size_t MIN_CAPACITY = 8192;
     size_t offset = cursor - base;
-    size_t newCapacity = JS_ROUNDUP(offset + n, MEM_BLOCK);
+    size_t newCapacity = mozilla::RoundUpPow2(offset + n);
+    if (newCapacity < MIN_CAPACITY)
+        newCapacity = MIN_CAPACITY;
     if (isUint32Overflow(newCapacity)) {
+        js::gc::AutoSuppressGC suppressGC(cx());
         JS_ReportErrorNumber(cx(), js_GetErrorMessage, nullptr, JSMSG_TOO_BIG_TO_ENCODE);
         return false;
     }
@@ -50,9 +53,25 @@ XDRBuffer::grow(size_t n)
 
 template<XDRMode mode>
 bool
-XDRState<mode>::codeChars(jschar *chars, size_t nchars)
+XDRState<mode>::codeChars(const Latin1Char *chars, size_t nchars)
 {
-    size_t nbytes = nchars * sizeof(jschar);
+    static_assert(sizeof(Latin1Char) == sizeof(uint8_t), "Latin1Char must fit in 1 byte");
+
+    MOZ_ASSERT(mode == XDR_ENCODE);
+
+    uint8_t *ptr = buf.write(nchars);
+    if (!ptr)
+        return false;
+
+    mozilla::PodCopy(ptr, chars, nchars);
+    return true;
+}
+
+template<XDRMode mode>
+bool
+XDRState<mode>::codeChars(char16_t *chars, size_t nchars)
+{
+    size_t nbytes = nchars * sizeof(char16_t);
     if (mode == XDR_ENCODE) {
         uint8_t *ptr = buf.write(nbytes);
         if (!ptr)
@@ -87,7 +106,7 @@ VersionCheck(XDRState<mode> *xdr)
 
 template<XDRMode mode>
 bool
-XDRState<mode>::codeFunction(MutableHandleObject objp)
+XDRState<mode>::codeFunction(MutableHandleFunction objp)
 {
     if (mode == XDR_DECODE)
         objp.set(nullptr);
@@ -102,37 +121,29 @@ template<XDRMode mode>
 bool
 XDRState<mode>::codeScript(MutableHandleScript scriptp)
 {
-    RootedScript script(cx());
-    if (mode == XDR_DECODE) {
-        script = nullptr;
+    if (mode == XDR_DECODE)
         scriptp.set(nullptr);
-    } else {
-        script = scriptp.get();
-    }
 
     if (!VersionCheck(this))
         return false;
 
-    if (!XDRScript(this, NullPtr(), NullPtr(), NullPtr(), &script))
+    if (!XDRScript(this, NullPtr(), NullPtr(), NullPtr(), scriptp))
         return false;
 
-    if (mode == XDR_DECODE) {
-        JS_ASSERT(!script->compileAndGo);
-        CallNewScriptHook(cx(), script, NullPtr());
-        Debugger::onNewScript(cx(), script, nullptr);
-        scriptp.set(script);
-    }
-
     return true;
 }
 
-XDRDecoder::XDRDecoder(JSContext *cx, const void *data, uint32_t length,
-                       JSPrincipals *principals, JSPrincipals *originPrincipals)
+template<XDRMode mode>
+bool
+XDRState<mode>::codeConstValue(MutableHandleValue vp)
+{
+    return XDRScriptConst(this, vp);
+}
+
+XDRDecoder::XDRDecoder(JSContext *cx, const void *data, uint32_t length)
   : XDRState<XDR_DECODE>(cx)
 {
     buf.setData(data, length);
-    this->principals_ = principals;
-    this->originPrincipals_ = NormalizeOriginPrincipals(principals, originPrincipals);
 }
 
 template class js::XDRState<XDR_ENCODE>;
