@@ -471,7 +471,7 @@ inline void XPCNativeSet::ASSERT_NotMarked()
 inline
 JSObject* XPCWrappedNativeTearOff::GetJSObjectPreserveColor() const
 {
-    return reinterpret_cast<JSObject *>(reinterpret_cast<uintptr_t>(mJSObject) & ~1);
+    return mJSObject.getPtr();
 }
 
 inline
@@ -491,6 +491,14 @@ void XPCWrappedNativeTearOff::SetJSObject(JSObject*  JSObj)
     mJSObject = JSObj;
 }
 
+inline
+void XPCWrappedNativeTearOff::JSObjectMoved(JSObject *obj, const JSObject *old)
+{
+    MOZ_ASSERT(!IsMarked());
+    MOZ_ASSERT(mJSObject == old);
+    mJSObject = obj;
+}
+
 inline
 XPCWrappedNativeTearOff::~XPCWrappedNativeTearOff()
 {
@@ -537,12 +545,9 @@ XPCWrappedNative::SweepTearOffs()
 inline bool
 xpc_ForcePropertyResolve(JSContext* cx, JS::HandleObject obj, jsid idArg)
 {
-    JS::RootedValue prop(cx);
     JS::RootedId id(cx, idArg);
-
-    if (!JS_LookupPropertyById(cx, obj, id, &prop))
-        return false;
-    return true;
+    bool dummy;
+    return JS_HasPropertyById(cx, obj, id, &dummy);
 }
 
 inline jsid
