@@ -9,9 +9,7 @@
 #include <unistd.h>
 #elif defined(XP_WIN)
 #include <windows.h>
-#elif defined(XP_OS2)
-#define INCL_DOSERRORS
-#include <os2.h>
+#include "nsILocalFileWin.h"
 #else
 // XXX add necessary include file for ftruncate (or equivalent)
 #endif
@@ -24,12 +22,14 @@
 #include "nsIClassInfoImpl.h"
 #include "mozilla/ipc/InputStreamUtils.h"
 #include "nsNetCID.h"
+#include "nsXULAppAPI.h"
 
 #define NS_NO_INPUT_BUFFERING 1 // see http://bugzilla.mozilla.org/show_bug.cgi?id=41067
 
 typedef mozilla::ipc::FileDescriptor::PlatformHandleType FileHandleType;
 
 using namespace mozilla::ipc;
+using mozilla::DebugOnly;
 
 ////////////////////////////////////////////////////////////////////////////////
 // nsFileStreamBase
@@ -46,9 +46,9 @@ nsFileStreamBase::~nsFileStreamBase()
     Close();
 }
 
-NS_IMPL_ISUPPORTS2(nsFileStreamBase,
-                   nsISeekableStream,
-                   nsIFileMetadata)
+NS_IMPL_ISUPPORTS(nsFileStreamBase,
+                  nsISeekableStream,
+                  nsIFileMetadata)
 
 NS_IMETHODIMP
 nsFileStreamBase::Seek(int32_t whence, int64_t offset)
@@ -92,7 +92,7 @@ nsFileStreamBase::SetEOF()
     if (mFD == nullptr)
         return NS_BASE_STREAM_CLOSED;
 
-#if defined(XP_UNIX) || defined(XP_OS2) || defined(XP_BEOS)
+#if defined(XP_UNIX) || defined(XP_BEOS)
     // Some system calls require an EOF offset.
     int64_t offset;
     rv = Tell(&offset);
@@ -109,11 +109,6 @@ nsFileStreamBase::SetEOF()
         NS_ERROR("SetEndOfFile failed");
         return NS_ERROR_FAILURE;
     }
-#elif defined(XP_OS2)
-    if (DosSetFileSize((HFILE) PR_FileDesc2NativeHandle(mFD), offset) != NO_ERROR) {
-        NS_ERROR("DosSetFileSize failed");
-        return NS_ERROR_FAILURE;
-    }
 #else
     // XXX not implemented
 #endif
@@ -338,9 +333,24 @@ nsFileStreamBase::DoOpen()
     NS_ASSERTION(mOpenParams.localFile, "Must have a file to open");
 
     PRFileDesc* fd;
-    nsresult rv = mOpenParams.localFile->OpenNSPRFileDesc(mOpenParams.ioFlags,
-                                                          mOpenParams.perm,
-                                                          &fd);
+    nsresult rv;
+
+#ifdef XP_WIN
+    if (mBehaviorFlags & nsIFileInputStream::SHARE_DELETE) {
+      nsCOMPtr<nsILocalFileWin> file = do_QueryInterface(mOpenParams.localFile);
+      MOZ_ASSERT(file);
+
+      rv = file->OpenNSPRFileDescShareDelete(mOpenParams.ioFlags,
+                                             mOpenParams.perm,
+                                             &fd);
+    } else
+#endif // XP_WIN
+    {
+      rv = mOpenParams.localFile->OpenNSPRFileDesc(mOpenParams.ioFlags,
+                                                   mOpenParams.perm,
+                                                   &fd);
+    }
+
     CleanUpOpen();
     if (NS_FAILED(rv))
         return rv;
@@ -376,11 +386,11 @@ NS_INTERFACE_MAP_BEGIN(nsFileInputStream)
     NS_IMPL_QUERY_CLASSINFO(nsFileInputStream)
 NS_INTERFACE_MAP_END_INHERITING(nsFileStreamBase)
 
-NS_IMPL_CI_INTERFACE_GETTER4(nsFileInputStream,
-                             nsIInputStream,
-                             nsIFileInputStream,
-                             nsISeekableStream,
-                             nsILineInputStream)
+NS_IMPL_CI_INTERFACE_GETTER(nsFileInputStream,
+                            nsIInputStream,
+                            nsIFileInputStream,
+                            nsISeekableStream,
+                            nsILineInputStream)
 
 nsresult
 nsFileInputStream::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
@@ -541,7 +551,8 @@ nsFileInputStream::Available(uint64_t *aResult)
 }
 
 void
-nsFileInputStream::Serialize(InputStreamParams& aParams)
+nsFileInputStream::Serialize(InputStreamParams& aParams,
+                             FileDescriptorArray& aFileDescriptors)
 {
     FileInputStreamParams params;
 
@@ -549,26 +560,23 @@ nsFileInputStream::Serialize(InputStreamParams& aParams)
         FileHandleType fd = FileHandleType(PR_FileDesc2NativeHandle(mFD));
         NS_ASSERTION(fd, "This should never be null!");
 
-        params.file() = FileDescriptor(fd);
-        NS_ASSERTION(params.file().IsValid(),
-                     "Sending an invalid file descriptor!");
+        DebugOnly<FileDescriptor*> dbgFD = aFileDescriptors.AppendElement(fd);
+        NS_ASSERTION(dbgFD->IsValid(), "Sending an invalid file descriptor!");
+
+        params.fileDescriptorIndex() = aFileDescriptors.Length() - 1;
+
+        Close();
     } else {
         NS_WARNING("This file has not been opened (or could not be opened). "
                    "Sending an invalid file descriptor to the other process!");
+
+        params.fileDescriptorIndex() = UINT32_MAX;
     }
 
     int32_t behaviorFlags = mBehaviorFlags;
 
-    // The other process shouldn't close when it reads the end because it will
-    // not be able to reopen the file later.
-    behaviorFlags &= ~nsIFileInputStream::CLOSE_ON_EOF;
-
-    // The other process will not be able to reopen the file so transferring
-    // this flag is meaningless.
-    behaviorFlags &= ~nsIFileInputStream::REOPEN_ON_REWIND;
-
-    // The other process is going to have an open file descriptor automatically
-    // so transferring this flag is meaningless.
+    // The receiving process (or thread) is going to have an open file
+    // descriptor automatically so transferring this flag is meaningless.
     behaviorFlags &= ~nsIFileInputStream::DEFER_OPEN;
 
     params.behaviorFlags() = behaviorFlags;
@@ -578,7 +586,8 @@ nsFileInputStream::Serialize(InputStreamParams& aParams)
 }
 
 bool
-nsFileInputStream::Deserialize(const InputStreamParams& aParams)
+nsFileInputStream::Deserialize(const InputStreamParams& aParams,
+                               const FileDescriptorArray& aFileDescriptors)
 {
     NS_ASSERTION(!mFD, "Already have a file descriptor?!");
     NS_ASSERTION(!mDeferredOpen, "Deferring open?!");
@@ -592,8 +601,15 @@ nsFileInputStream::Deserialize(const InputStreamParams& aParams)
 
     const FileInputStreamParams& params = aParams.get_FileInputStreamParams();
 
-    const FileDescriptor& fd = params.file();
-    NS_WARN_IF_FALSE(fd.IsValid(), "Received an invalid file descriptor!");
+    uint32_t fileDescriptorIndex = params.fileDescriptorIndex();
+
+    FileDescriptor fd;
+    if (fileDescriptorIndex < aFileDescriptors.Length()) {
+        fd = aFileDescriptors[fileDescriptorIndex];
+        NS_WARN_IF_FALSE(fd.IsValid(), "Received an invalid file descriptor!");
+    } else {
+        NS_WARNING("Received a bad file descriptor index!");
+    }
 
     if (fd.IsValid()) {
         PRFileDesc* fileDesc = PR_ImportFile(PROsfd(fd.PlatformHandle()));
@@ -605,6 +621,17 @@ nsFileInputStream::Deserialize(const InputStreamParams& aParams)
     }
 
     mBehaviorFlags = params.behaviorFlags();
+
+    if (XRE_GetProcessType() != GeckoProcessType_Default) {
+        // A child process shouldn't close when it reads the end because it will
+        // not be able to reopen the file later.
+        mBehaviorFlags &= ~nsIFileInputStream::CLOSE_ON_EOF;
+
+        // A child process will not be able to reopen the file so this flag is
+        // meaningless.
+        mBehaviorFlags &= ~nsIFileInputStream::REOPEN_ON_REWIND;
+    }
+
     mIOFlags = params.ioFlags();
 
     return true;
@@ -629,11 +656,11 @@ NS_INTERFACE_MAP_BEGIN(nsPartialFileInputStream)
     NS_IMPL_QUERY_CLASSINFO(nsPartialFileInputStream)
 NS_INTERFACE_MAP_END_INHERITING(nsFileStreamBase)
 
-NS_IMPL_CI_INTERFACE_GETTER4(nsPartialFileInputStream,
-                             nsIInputStream,
-                             nsIPartialFileInputStream,
-                             nsISeekableStream,
-                             nsILineInputStream)
+NS_IMPL_CI_INTERFACE_GETTER(nsPartialFileInputStream,
+                            nsIInputStream,
+                            nsIPartialFileInputStream,
+                            nsISeekableStream,
+                            nsILineInputStream)
 
 nsresult
 nsPartialFileInputStream::Create(nsISupports *aOuter, REFNSIID aIID,
@@ -734,16 +761,12 @@ nsPartialFileInputStream::Seek(int32_t aWhence, int64_t aOffset)
 }
 
 void
-nsPartialFileInputStream::Serialize(InputStreamParams& aParams)
+nsPartialFileInputStream::Serialize(InputStreamParams& aParams,
+                                    FileDescriptorArray& aFileDescriptors)
 {
     // Serialize the base class first.
     InputStreamParams fileParams;
-    nsFileInputStream::Serialize(fileParams);
-
-    if (fileParams.type() != InputStreamParams::TFileInputStreamParams) {
-        NS_ERROR("Base class serialize failed!");
-        return;
-    }
+    nsFileInputStream::Serialize(fileParams, aFileDescriptors);
 
     PartialFileInputStreamParams params;
 
@@ -755,7 +778,9 @@ nsPartialFileInputStream::Serialize(InputStreamParams& aParams)
 }
 
 bool
-nsPartialFileInputStream::Deserialize(const InputStreamParams& aParams)
+nsPartialFileInputStream::Deserialize(
+                                    const InputStreamParams& aParams,
+                                    const FileDescriptorArray& aFileDescriptors)
 {
     NS_ASSERTION(!mFD, "Already have a file descriptor?!");
     NS_ASSERTION(!mStart, "Already have a start?!");
@@ -763,7 +788,7 @@ nsPartialFileInputStream::Deserialize(const InputStreamParams& aParams)
     NS_ASSERTION(!mPosition, "Already have a position?!");
 
     if (aParams.type() != InputStreamParams::TPartialFileInputStreamParams) {
-        NS_ERROR("Received unknown parameters from the other process!");
+        NS_WARNING("Received unknown parameters from the other process!");
         return false;
     }
 
@@ -772,8 +797,8 @@ nsPartialFileInputStream::Deserialize(const InputStreamParams& aParams)
 
     // Deserialize the base class first.
     InputStreamParams fileParams(params.fileStreamParams());
-    if (!nsFileInputStream::Deserialize(fileParams)) {
-        NS_ERROR("Base class deserialize failed!");
+    if (!nsFileInputStream::Deserialize(fileParams, aFileDescriptors)) {
+        NS_WARNING("Base class deserialize failed!");
         return false;
     }
 
@@ -794,10 +819,10 @@ nsPartialFileInputStream::Deserialize(const InputStreamParams& aParams)
 ////////////////////////////////////////////////////////////////////////////////
 // nsFileOutputStream
 
-NS_IMPL_ISUPPORTS_INHERITED2(nsFileOutputStream,
-                             nsFileStreamBase,
-                             nsIOutputStream,
-                             nsIFileOutputStream)
+NS_IMPL_ISUPPORTS_INHERITED(nsFileOutputStream,
+                            nsFileStreamBase,
+                            nsIOutputStream,
+                            nsIFileOutputStream)
  
 nsresult
 nsFileOutputStream::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
@@ -834,11 +859,11 @@ nsFileOutputStream::Init(nsIFile* file, int32_t ioFlags, int32_t perm,
 ////////////////////////////////////////////////////////////////////////////////
 // nsAtomicFileOutputStream
 
-NS_IMPL_ISUPPORTS_INHERITED3(nsAtomicFileOutputStream,
-                             nsFileOutputStream,
-                             nsISafeOutputStream,
-                             nsIOutputStream,
-                             nsIFileOutputStream)
+NS_IMPL_ISUPPORTS_INHERITED(nsAtomicFileOutputStream,
+                            nsFileOutputStream,
+                            nsISafeOutputStream,
+                            nsIOutputStream,
+                            nsIFileOutputStream)
 
 NS_IMETHODIMP
 nsAtomicFileOutputStream::Init(nsIFile* file, int32_t ioFlags, int32_t perm,
@@ -932,7 +957,7 @@ nsAtomicFileOutputStream::Finish()
 #ifdef DEBUG
             bool equal;
             if (NS_FAILED(mTargetFile->Equals(mTempFile, &equal)) || !equal)
-                NS_ERROR("mTempFile not equal to mTargetFile");
+                NS_WARNING("mTempFile not equal to mTargetFile");
 #endif
         }
         else {
@@ -986,11 +1011,11 @@ nsSafeFileOutputStream::Finish()
 ////////////////////////////////////////////////////////////////////////////////
 // nsFileStream
 
-NS_IMPL_ISUPPORTS_INHERITED3(nsFileStream,
-                             nsFileStreamBase,
-                             nsIInputStream,
-                             nsIOutputStream,
-                             nsIFileStream)
+NS_IMPL_ISUPPORTS_INHERITED(nsFileStream,
+                            nsFileStreamBase,
+                            nsIInputStream,
+                            nsIOutputStream,
+                            nsIFileStream)
 
 NS_IMETHODIMP
 nsFileStream::Init(nsIFile* file, int32_t ioFlags, int32_t perm,
