@@ -17,6 +17,7 @@
 #include "nsIChannelEventSink.h"
 #include "nsCategoryCache.h"
 #include "nsISpeculativeConnect.h"
+#include "nsDataHashtable.h"
 #include "mozilla/Attributes.h"
 
 #define NS_N(x) (sizeof(x)/sizeof(*x))
@@ -37,6 +38,12 @@ class nsIProxyInfo;
 class nsPIDNSService;
 class nsPISocketTransportService;
 
+namespace mozilla {
+namespace net {
+    class NeckoChild;
+} // namespace net
+} // namespace mozilla
+
 class nsIOService MOZ_FINAL : public nsIIOService2
                             , public nsIObserver
                             , public nsINetUtil
@@ -56,8 +63,8 @@ public:
     // Returns an addrefed pointer.
     static nsIOService* GetInstance();
 
-    NS_HIDDEN_(nsresult) Init();
-    NS_HIDDEN_(nsresult) NewURI(const char* aSpec, nsIURI* aBaseURI,
+    nsresult Init();
+    nsresult NewURI(const char* aSpec, nsIURI* aBaseURI,
                                 nsIURI* *result,
                                 nsIProtocolHandler* *hdlrResult);
 
@@ -74,26 +81,29 @@ public:
       return mOffline && mSettingOffline && !mSetOfflineValue;
     }
 
+    // Should only be called from NeckoChild. Use SetAppOffline instead.
+    void SetAppOfflineInternal(uint32_t appId, int32_t status);
+
 private:
     // These shouldn't be called directly:
     // - construct using GetInstance
     // - destroy using Release
-    nsIOService() NS_HIDDEN;
-    ~nsIOService() NS_HIDDEN;
+    nsIOService();
+    ~nsIOService();
 
-    NS_HIDDEN_(nsresult) TrackNetworkLinkStatusForOffline();
+    nsresult OnNetworkLinkEvent(const char *data);
 
-    NS_HIDDEN_(nsresult) GetCachedProtocolHandler(const char *scheme,
+    nsresult GetCachedProtocolHandler(const char *scheme,
                                                   nsIProtocolHandler* *hdlrResult,
                                                   uint32_t start=0,
                                                   uint32_t end=0);
-    NS_HIDDEN_(nsresult) CacheProtocolHandler(const char *scheme,
+    nsresult CacheProtocolHandler(const char *scheme,
                                               nsIProtocolHandler* hdlr);
 
     // Prefs wrangling
-    NS_HIDDEN_(void) PrefsChanged(nsIPrefBranch *prefs, const char *pref = nullptr);
-    NS_HIDDEN_(void) GetPrefBranch(nsIPrefBranch **);
-    NS_HIDDEN_(void) ParsePortList(nsIPrefBranch *prefBranch, const char *pref, bool remove);
+    void PrefsChanged(nsIPrefBranch *prefs, const char *pref = nullptr);
+    void GetPrefBranch(nsIPrefBranch **);
+    void ParsePortList(nsIPrefBranch *prefBranch, const char *pref, bool remove);
 
     nsresult InitializeSocketTransportService();
     nsresult InitializeNetworkLinkService();
@@ -102,6 +112,16 @@ private:
     void LookupProxyInfo(nsIURI *aURI, nsIURI *aProxyURI, uint32_t aProxyFlags,
                          nsCString *aScheme, nsIProxyInfo **outPI);
 
+    // notify content processes of offline status
+    // 'status' must be a nsIAppOfflineInfo mode constant.
+    void NotifyAppOfflineStatus(uint32_t appId, int32_t status);
+    static PLDHashOperator EnumerateWifiAppsChangingState(const unsigned int &, int32_t, void*);
+
+    nsresult NewChannelFromURIWithProxyFlagsInternal(nsIURI* aURI,
+                                                     nsIURI* aProxyURI,
+                                                     uint32_t aProxyFlags,
+                                                     nsILoadInfo* aLoadInfo,
+                                                     nsIChannel** result);
 private:
     bool                                 mOffline;
     bool                                 mOfflineForProfileChange;
@@ -129,12 +149,50 @@ private:
     nsTArray<int32_t>                    mRestrictedPortList;
 
     bool                                 mAutoDialEnabled;
+    bool                                 mNetworkNotifyChanged;
+    int32_t                              mPreviousWifiState;
+    // Hashtable of (appId, nsIAppOffineInfo::mode) pairs
+    // that is used especially in IsAppOffline
+    nsDataHashtable<nsUint32HashKey, int32_t> mAppsOfflineStatus;
 public:
     // Used for all default buffer sizes that necko allocates.
     static uint32_t   gDefaultSegmentSize;
     static uint32_t   gDefaultSegmentCount;
 };
 
+/**
+ * This class is passed as the subject to a NotifyObservers call for the
+ * "network:app-offline-status-changed" topic.
+ * Observers will use the appId and mode to get the offline status of an app.
+ */
+class nsAppOfflineInfo : public nsIAppOfflineInfo
+{
+    NS_DECL_THREADSAFE_ISUPPORTS
+public:
+    nsAppOfflineInfo(uint32_t aAppId, int32_t aMode)
+        : mAppId(aAppId), mMode(aMode)
+    {
+    }
+
+    NS_IMETHODIMP GetMode(int32_t *aMode) MOZ_OVERRIDE
+    {
+        *aMode = mMode;
+        return NS_OK;
+    }
+
+    NS_IMETHODIMP GetAppId(uint32_t *aAppId) MOZ_OVERRIDE
+    {
+        *aAppId = mAppId;
+        return NS_OK;
+    }
+
+private:
+    virtual ~nsAppOfflineInfo() {}
+
+    uint32_t mAppId;
+    int32_t mMode;
+};
+
 /**
  * Reference to the IO service singleton. May be null.
  */
