@@ -50,7 +50,7 @@ static PRLogModuleInfo* gLoadGroupLog = nullptr;
 class RequestMapEntry : public PLDHashEntryHdr
 {
 public:
-    RequestMapEntry(nsIRequest *aRequest) :
+    explicit RequestMapEntry(nsIRequest *aRequest) :
         mKey(aRequest)
     {
     }
@@ -230,7 +230,7 @@ nsLoadGroup::Cancel(nsresult status)
 
     NS_ASSERTION(NS_FAILED(status), "shouldn't cancel with a success code");
     nsresult rv;
-    uint32_t count = mRequests.entryCount;
+    uint32_t count = mRequests.EntryCount();
 
     nsAutoTArray<nsIRequest*, 8> requests;
 
@@ -264,8 +264,7 @@ nsLoadGroup::Cancel(nsresult status)
 
         RequestMapEntry *entry =
             static_cast<RequestMapEntry *>
-                       (PL_DHashTableOperate(&mRequests, request,
-                                                PL_DHASH_LOOKUP));
+                       (PL_DHashTableLookup(&mRequests, request));
 
         if (PL_DHASH_ENTRY_IS_FREE(entry)) {
             // |request| was removed already
@@ -301,7 +300,7 @@ nsLoadGroup::Cancel(nsresult status)
     }
 
 #if defined(DEBUG)
-    NS_ASSERTION(mRequests.entryCount == 0, "Request list is not empty.");
+    NS_ASSERTION(mRequests.EntryCount() == 0, "Request list is not empty.");
     NS_ASSERTION(mForegroundCount == 0, "Foreground URLs are active.");
 #endif
 
@@ -316,7 +315,7 @@ NS_IMETHODIMP
 nsLoadGroup::Suspend()
 {
     nsresult rv, firstError;
-    uint32_t count = mRequests.entryCount;
+    uint32_t count = mRequests.EntryCount();
 
     nsAutoTArray<nsIRequest*, 8> requests;
 
@@ -368,7 +367,7 @@ NS_IMETHODIMP
 nsLoadGroup::Resume()
 {
     nsresult rv, firstError;
-    uint32_t count = mRequests.entryCount;
+    uint32_t count = mRequests.EntryCount();
 
     nsAutoTArray<nsIRequest*, 8> requests;
 
@@ -489,7 +488,7 @@ nsLoadGroup::AddRequest(nsIRequest *request, nsISupports* ctxt)
         nsAutoCString nameStr;
         request->GetName(nameStr);
         LOG(("LOADGROUP [%x]: Adding request %x %s (count=%d).\n",
-             this, request, nameStr.get(), mRequests.entryCount));
+             this, request, nameStr.get(), mRequests.EntryCount()));
     }
 #endif /* PR_LOGGING */
 
@@ -497,8 +496,7 @@ nsLoadGroup::AddRequest(nsIRequest *request, nsISupports* ctxt)
     {
       RequestMapEntry *entry =
           static_cast<RequestMapEntry *>
-                     (PL_DHashTableOperate(&mRequests, request,
-                                          PL_DHASH_LOOKUP));
+                     (PL_DHashTableLookup(&mRequests, request));
 
       NS_ASSERTION(PL_DHASH_ENTRY_IS_FREE(entry),
                    "Entry added to loadgroup twice, don't do that");
@@ -534,8 +532,7 @@ nsLoadGroup::AddRequest(nsIRequest *request, nsISupports* ctxt)
 
     RequestMapEntry *entry =
         static_cast<RequestMapEntry *>
-                   (PL_DHashTableOperate(&mRequests, request,
-                                        PL_DHASH_ADD));
+                   (PL_DHashTableAdd(&mRequests, request));
 
     if (!entry) {
         return NS_ERROR_OUT_OF_MEMORY;
@@ -572,7 +569,7 @@ nsLoadGroup::AddRequest(nsIRequest *request, nsISupports* ctxt)
                 // the damage...
                 //
 
-                PL_DHashTableOperate(&mRequests, request, PL_DHASH_REMOVE);
+                PL_DHashTableRemove(&mRequests, request);
 
                 rv = NS_OK;
 
@@ -602,7 +599,7 @@ nsLoadGroup::RemoveRequest(nsIRequest *request, nsISupports* ctxt,
         nsAutoCString nameStr;
         request->GetName(nameStr);
         LOG(("LOADGROUP [%x]: Removing request %x %s status %x (count=%d).\n",
-            this, request, nameStr.get(), aStatus, mRequests.entryCount-1));
+            this, request, nameStr.get(), aStatus, mRequests.EntryCount() - 1));
     }
 #endif
 
@@ -618,8 +615,7 @@ nsLoadGroup::RemoveRequest(nsIRequest *request, nsISupports* ctxt,
     //
     RequestMapEntry *entry =
         static_cast<RequestMapEntry *>
-                   (PL_DHashTableOperate(&mRequests, request,
-                                        PL_DHASH_LOOKUP));
+                   (PL_DHashTableLookup(&mRequests, request));
 
     if (PL_DHASH_ENTRY_IS_FREE(entry)) {
         LOG(("LOADGROUP [%x]: Unable to remove request %x. Not in group!\n",
@@ -664,7 +660,7 @@ nsLoadGroup::RemoveRequest(nsIRequest *request, nsISupports* ctxt,
         }
     }
 
-    if (mRequests.entryCount == 0) {
+    if (mRequests.EntryCount() == 0) {
         TelemetryReport();
     }
 
@@ -720,7 +716,7 @@ NS_IMETHODIMP
 nsLoadGroup::GetRequests(nsISimpleEnumerator * *aRequests)
 {
     nsCOMArray<nsIRequest> requests;
-    requests.SetCapacity(mRequests.entryCount);
+    requests.SetCapacity(mRequests.EntryCount());
 
     PL_DHashTableEnumerate(&mRequests, AppendRequestsToCOMArray, &requests);
 
@@ -1098,6 +1094,8 @@ nsresult nsLoadGroup::MergeLoadFlags(nsIRequest *aRequest, nsLoadFlags& outFlags
 
 class nsLoadGroupConnectionInfo MOZ_FINAL : public nsILoadGroupConnectionInfo
 {
+    ~nsLoadGroupConnectionInfo() {}
+
 public:
     NS_DECL_THREADSAFE_ISUPPORTS
     NS_DECL_NSILOADGROUPCONNECTIONINFO
@@ -1108,7 +1106,7 @@ private:
     nsAutoPtr<mozilla::net::SpdyPushCache> mSpdyCache;
 };
 
-NS_IMPL_ISUPPORTS1(nsLoadGroupConnectionInfo, nsILoadGroupConnectionInfo)
+NS_IMPL_ISUPPORTS(nsLoadGroupConnectionInfo, nsILoadGroupConnectionInfo)
 
 nsLoadGroupConnectionInfo::nsLoadGroupConnectionInfo()
     : mBlockingTransactionCount(0)
@@ -1168,12 +1166,8 @@ nsresult nsLoadGroup::Init()
         RequestHashInitEntry
     };
 
-    if (!PL_DHashTableInit(&mRequests, &hash_table_ops, nullptr,
-                           sizeof(RequestMapEntry), 16)) {
-        mRequests.ops = nullptr;
-
-        return NS_ERROR_OUT_OF_MEMORY;
-    }
+    PL_DHashTableInit(&mRequests, &hash_table_ops, nullptr,
+                      sizeof(RequestMapEntry));
 
     mConnectionInfo = new nsLoadGroupConnectionInfo();
 
