@@ -6,39 +6,46 @@
 #ifndef nsStreamLoader_h__
 #define nsStreamLoader_h__
 
+#include "nsIThreadRetargetableStreamListener.h"
 #include "nsIStreamLoader.h"
 #include "nsCOMPtr.h"
 #include "mozilla/Attributes.h"
+#include "mozilla/Vector.h"
 
 class nsIRequest;
 
 class nsStreamLoader MOZ_FINAL : public nsIStreamLoader
+                               , public nsIThreadRetargetableStreamListener
 {
 public:
-  NS_DECL_ISUPPORTS
+  NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSISTREAMLOADER
   NS_DECL_NSIREQUESTOBSERVER
   NS_DECL_NSISTREAMLISTENER
+  NS_DECL_NSITHREADRETARGETABLESTREAMLISTENER
 
   nsStreamLoader();
-  ~nsStreamLoader();
 
   static nsresult
   Create(nsISupports *aOuter, REFNSIID aIID, void **aResult);
 
 protected:
+  ~nsStreamLoader();
+
   static NS_METHOD WriteSegmentFun(nsIInputStream *, void *, const char *,
                                    uint32_t, uint32_t, uint32_t *);
 
+  // Utility method to free mData, if present, and update other state to
+  // reflect that no data has been allocated.
+  void ReleaseData();
+
   nsCOMPtr<nsIStreamLoaderObserver> mObserver;
   nsCOMPtr<nsISupports>             mContext;  // the observer's context
   nsCOMPtr<nsIRequest>              mRequest;
 
-  uint8_t  *mData;      // buffer to accumulate incoming data
-  uint32_t  mAllocated; // allocated size of data buffer (we preallocate if
-                        //   contentSize is available)
-  uint32_t  mLength;    // actual length of data in buffer
-                        //   (must be <= mAllocated)
+  // Buffer to accumulate incoming data. We preallocate if contentSize is
+  // available.
+  mozilla::Vector<uint8_t, 0> mData;
 };
 
 #endif // nsStreamLoader_h__
