@@ -23,13 +23,21 @@ public:
   NS_DECL_NSIDNSLISTENER
 
   DNSRequestParent();
-  virtual ~DNSRequestParent();
 
   void DoAsyncResolve(const nsACString  &hostname, uint32_t flags);
 
+  // Pass args here rather than storing them in the parent; they are only
+  // needed if the request is to be canceled.
+  bool RecvCancelDNSRequest(const nsCString& hostName,
+                            const uint32_t& flags,
+                            const nsresult& reason) MOZ_OVERRIDE;
+  bool Recv__delete__() MOZ_OVERRIDE;
+
 protected:
-  virtual void ActorDestroy(ActorDestroyReason why);
+  virtual void ActorDestroy(ActorDestroyReason why) MOZ_OVERRIDE;
 private:
+  virtual ~DNSRequestParent();
+
   uint32_t mFlags;
   bool mIPCClosed;  // true if IPDL channel has been closed (child crash)
 };
