@@ -7,6 +7,12 @@
 // HttpLog.h should generally be included first
 #include "HttpLog.h"
 
+// Log on level :5, instead of default :4.
+#undef LOG
+#define LOG(args) LOG5(args)
+#undef LOG_ENABLED
+#define LOG_ENABLED() LOG5_ENABLED()
+
 #include "mozilla/Telemetry.h"
 #include "nsAlgorithm.h"
 #include "nsHttp.h"
@@ -18,6 +24,7 @@
 #include "SpdyPush31.h"
 #include "SpdySession31.h"
 #include "SpdyStream31.h"
+#include "TunnelUtils.h"
 
 #include <algorithm>
 
@@ -32,36 +39,38 @@ namespace net {
 SpdyStream31::SpdyStream31(nsAHttpTransaction *httpTransaction,
                            SpdySession31 *spdySession,
                            int32_t priority)
-  : mStreamID(0),
-  mSession(spdySession),
-  mUpstreamState(GENERATING_SYN_STREAM),
-  mSynFrameComplete(0),
-  mSentFinOnData(0),
-  mTransaction(httpTransaction),
-  mSocketTransport(spdySession->SocketTransport()),
-  mSegmentReader(nullptr),
-  mSegmentWriter(nullptr),
-  mChunkSize(spdySession->SendingChunkSize()),
-  mRequestBlockedOnRead(0),
-  mRecvdFin(0),
-  mFullyOpen(0),
-  mSentWaitingFor(0),
-  mReceivedData(0),
-  mSetTCPSocketBuffer(0),
-  mTxInlineFrameSize(SpdySession31::kDefaultBufferSize),
-  mTxInlineFrameUsed(0),
-  mTxStreamFrameSize(0),
-  mZlib(spdySession->UpstreamZlib()),
-  mDecompressBufferSize(SpdySession31::kDefaultBufferSize),
-  mDecompressBufferUsed(0),
-  mDecompressedBytes(0),
-  mRequestBodyLenRemaining(0),
-  mPriority(priority),
-  mLocalUnacked(0),
-  mBlockedOnRwin(false),
-  mTotalSent(0),
-  mTotalRead(0),
-  mPushSource(nullptr)
+  : mStreamID(0)
+  , mSession(spdySession)
+  , mUpstreamState(GENERATING_SYN_STREAM)
+  , mSynFrameComplete(0)
+  , mSentFinOnData(0)
+  , mTransaction(httpTransaction)
+  , mSocketTransport(spdySession->SocketTransport())
+  , mSegmentReader(nullptr)
+  , mSegmentWriter(nullptr)
+  , mChunkSize(spdySession->SendingChunkSize())
+  , mRequestBlockedOnRead(0)
+  , mRecvdFin(0)
+  , mFullyOpen(0)
+  , mSentWaitingFor(0)
+  , mReceivedData(0)
+  , mSetTCPSocketBuffer(0)
+  , mTxInlineFrameSize(SpdySession31::kDefaultBufferSize)
+  , mTxInlineFrameUsed(0)
+  , mTxStreamFrameSize(0)
+  , mZlib(spdySession->UpstreamZlib())
+  , mDecompressBufferSize(SpdySession31::kDefaultBufferSize)
+  , mDecompressBufferUsed(0)
+  , mDecompressedBytes(0)
+  , mRequestBodyLenRemaining(0)
+  , mPriority(priority)
+  , mLocalUnacked(0)
+  , mBlockedOnRwin(false)
+  , mTotalSent(0)
+  , mTotalRead(0)
+  , mPushSource(nullptr)
+  , mIsTunnel(false)
+  , mPlainTextTunnel(false)
 {
   MOZ_ASSERT(PR_GetCurrentThread() == gSocketThread);
 
@@ -76,6 +85,7 @@ SpdyStream31::SpdyStream31(nsAHttpTransaction *httpTransaction,
 
 SpdyStream31::~SpdyStream31()
 {
+  ClearTransactionsBlockedOnTunnel();
   mStreamID = SpdySession31::kDeadStreamID;
 }
 
@@ -97,6 +107,13 @@ SpdyStream31::ReadSegments(nsAHttpSegmentReader *reader,
   nsresult rv = NS_ERROR_UNEXPECTED;
   mRequestBlockedOnRead = 0;
 
+  // avoid runt chunks if possible by anticipating
+  // full data frames
+  if (count > (mChunkSize + 8)) {
+    uint32_t numchunks = count / (mChunkSize + 8);
+    count = numchunks * (mChunkSize + 8);
+  }
+
   switch (mUpstreamState) {
   case GENERATING_SYN_STREAM:
   case GENERATING_REQUEST_BODY:
@@ -106,7 +123,8 @@ SpdyStream31::ReadSegments(nsAHttpSegmentReader *reader,
     mSegmentReader = reader;
     rv = mTransaction->ReadSegments(this, count, countRead);
     mSegmentReader = nullptr;
-
+    LOG3(("SpdyStream31::ReadSegments %p trans readsegments rv %x read=%d\n",
+          this, rv, *countRead));
     // Check to see if the transaction's request could be written out now.
     // If not, mark the stream for callback when writing can proceed.
     if (NS_SUCCEEDED(rv) &&
@@ -131,7 +149,8 @@ SpdyStream31::ReadSegments(nsAHttpSegmentReader *reader,
     if (!mBlockedOnRwin &&
         !mTxInlineFrameUsed && NS_SUCCEEDED(rv) && (!*countRead)) {
       LOG3(("SpdyStream31::ReadSegments %p 0x%X: Sending request data complete, "
-            "mUpstreamState=%x",this, mStreamID, mUpstreamState));
+            "mUpstreamState=%x finondata=%d",this, mStreamID,
+            mUpstreamState, mSentFinOnData));
       if (mSentFinOnData) {
         ChangeState(UPSTREAM_COMPLETE);
       }
@@ -181,9 +200,8 @@ SpdyStream31::ReadSegments(nsAHttpSegmentReader *reader,
 }
 
 // WriteSegments() is used to read data off the socket. Generally this is
-// just the SPDY frame header and from there the appropriate SPDYStream
-// is identified from the Stream-ID. The http transaction associated with
-// that read then pulls in the data directly.
+// just a call through to the associated nsHttpTransaction for this stream
+// for the remaining data bytes indicated by the current DATA frame.
 
 nsresult
 SpdyStream31::WriteSegments(nsAHttpSegmentWriter *writer,
@@ -224,13 +242,13 @@ SpdyStream31::CreatePushHashKey(const nsCString &scheme,
                                 nsCString &outKey)
 {
   outOrigin = scheme;
-  outOrigin.Append(NS_LITERAL_CSTRING("://"));
+  outOrigin.AppendLiteral("://");
   outOrigin.Append(hostHeader);
 
   outKey = outOrigin;
-  outKey.Append(NS_LITERAL_CSTRING("/[spdy3_1."));
+  outKey.AppendLiteral("/[spdy3_1.");
   outKey.AppendInt(serial);
-  outKey.Append(NS_LITERAL_CSTRING("]"));
+  outKey.Append(']');
   outKey.Append(pathInfo);
 }
 
@@ -272,17 +290,17 @@ SpdyStream31::ParseHttpRequestHeaders(const char *buf,
   *countUsed = avail - (oldLen - endHeader) + 4;
   mSynFrameComplete = 1;
 
-  nsCString hostHeader;
-  nsCString hashkey;
+  nsAutoCString hostHeader;
+  nsAutoCString hashkey;
   mTransaction->RequestHead()->GetHeader(nsHttp::Host, hostHeader);
 
-  CreatePushHashKey(NS_LITERAL_CSTRING("https"),
+  CreatePushHashKey(nsDependentCString(mTransaction->RequestHead()->IsHTTPS() ? "https" : "http"),
                     hostHeader, mSession->Serial(),
                     mTransaction->RequestHead()->RequestURI(),
                     mOrigin, hashkey);
 
   // check the push cache for GET
-  if (mTransaction->RequestHead()->Method() == nsHttp::Get) {
+  if (mTransaction->RequestHead()->IsGet()) {
     // from :scheme, :host, :path
     nsILoadGroupConnectionInfo *loadGroupCI = mTransaction->LoadGroupConnectionInfo();
     SpdyPushCache *cache = nullptr;
@@ -303,6 +321,12 @@ SpdyStream31::ParseHttpRequestHeaders(const char *buf,
       mPushSource = pushedStream;
       mSentFinOnData = 1;
 
+      // This stream has been activated (and thus counts against the concurrency
+      // limit intentionally), but will not be registered via
+      // RegisterStreamID (below) because of the push match. Therefore the
+      // concurrency sempahore needs to be balanced.
+      mSession->DecrementConcurrent(this);
+
       // There is probably pushed data buffered so trigger a read manually
       // as we can't rely on future network events to do it
       mSession->ConnectPushedStream(this);
@@ -382,7 +406,7 @@ SpdyStream31::ParseHttpRequestHeaders(const char *buf,
   // even though we are parsing the actual text stream because
   // it is legit to append headers.
   nsClassHashtable<nsCStringHashKey, nsCString>
-    hdrHash(1 + (mTransaction->RequestHead()->Headers().Count() * 2));
+    hdrHash(mTransaction->RequestHead()->Headers().Count());
 
   const char *beginBuffer = mFlatHttpRequestHeaders.BeginReading();
 
@@ -408,12 +432,12 @@ SpdyStream31::ParseHttpRequestHeaders(const char *buf,
     ToLowerCase(name);
 
     // exclusions.. mostly from 3.2.1
-    if (name.Equals("connection") ||
-        name.Equals("keep-alive") ||
-        name.Equals("host") ||
-        name.Equals("accept-encoding") ||
-        name.Equals("te") ||
-        name.Equals("transfer-encoding"))
+    if (name.EqualsLiteral("connection") ||
+        name.EqualsLiteral("keep-alive") ||
+        name.EqualsLiteral("host") ||
+        name.EqualsLiteral("accept-encoding") ||
+        name.EqualsLiteral("te") ||
+        name.EqualsLiteral("transfer-encoding"))
       continue;
 
     nsCString *val = hdrHash.Get(name);
@@ -432,7 +456,7 @@ SpdyStream31::ParseHttpRequestHeaders(const char *buf,
       val->Append(static_cast<char>(0));
     val->Append(v);
 
-    if (name.Equals("content-length")) {
+    if (name.EqualsLiteral("content-length")) {
       int64_t len;
       if (nsHttp::ParseInt64(val->get(), nullptr, &len))
         mRequestBodyLenRemaining = len;
@@ -446,24 +470,51 @@ SpdyStream31::ParseHttpRequestHeaders(const char *buf,
   // headers at this same level so it is not necessary to do so here.
 
   const char *methodHeader = mTransaction->RequestHead()->Method().get();
+  LOG3(("Stream method %p 0x%X %s\n", this, mStreamID, methodHeader));
 
   // The header block length
-  uint16_t count = hdrHash.Count() + 5; /* method, path, version, host, scheme */
+  uint16_t count = hdrHash.Count() + 4; /* :method, :path, :version, :host */
+  if (mTransaction->RequestHead()->IsConnect()) {
+    mRequestBodyLenRemaining = 0x0fffffffffffffffULL;
+  } else {
+    ++count; // :scheme used if not connect
+  }
   CompressToFrame(count);
 
   // :method, :path, :version comprise a HTTP/1 request line, so send those first
   // to make life easy for any gateways
   CompressToFrame(NS_LITERAL_CSTRING(":method"));
   CompressToFrame(methodHeader, strlen(methodHeader));
+
   CompressToFrame(NS_LITERAL_CSTRING(":path"));
-  CompressToFrame(mTransaction->RequestHead()->RequestURI());
+  if (!mTransaction->RequestHead()->IsConnect()) {
+    CompressToFrame(mTransaction->RequestHead()->Path());
+  } else {
+    MOZ_ASSERT(mTransaction->QuerySpdyConnectTransaction());
+    mIsTunnel = true;
+    // Connect places host:port in :path. Don't use default port.
+    nsHttpConnectionInfo *ci = mTransaction->ConnectionInfo();
+    if (!ci) {
+      return NS_ERROR_UNEXPECTED;
+    }
+    nsAutoCString route;
+    route = ci->GetHost();
+    route.Append(':');
+    route.AppendInt(ci->Port());
+    CompressToFrame(route);
+  }
+
   CompressToFrame(NS_LITERAL_CSTRING(":version"));
   CompressToFrame(versionHeader);
 
   CompressToFrame(NS_LITERAL_CSTRING(":host"));
   CompressToFrame(hostHeader);
-  CompressToFrame(NS_LITERAL_CSTRING(":scheme"));
-  CompressToFrame(NS_LITERAL_CSTRING("https"));
+
+  if (!mTransaction->RequestHead()->IsConnect()) {
+    // no :scheme with connect
+    CompressToFrame(NS_LITERAL_CSTRING(":scheme"));
+    CompressToFrame(nsDependentCString(mTransaction->RequestHead()->IsHTTPS() ? "https" : "http"));
+  }
 
   hdrHash.Enumerate(hdrHashEnumerate, this);
   CompressFlushFrame();
@@ -477,18 +528,18 @@ SpdyStream31::ParseHttpRequestHeaders(const char *buf,
   // Determine whether to put the fin bit on the syn stream frame or whether
   // to wait for a data packet to put it on.
 
-  if (mTransaction->RequestHead()->Method() == nsHttp::Get ||
-      mTransaction->RequestHead()->Method() == nsHttp::Connect ||
-      mTransaction->RequestHead()->Method() == nsHttp::Head) {
-    // for GET, CONNECT, and HEAD place the fin bit right on the
+  if (mTransaction->RequestHead()->IsGet() ||
+      mTransaction->RequestHead()->IsHead()) {
+    // for GET and HEAD place the fin bit right on the
     // syn stream packet
 
     mSentFinOnData = 1;
     mTxInlineFrame[4] = SpdySession31::kFlag_Data_FIN;
   }
-  else if (mTransaction->RequestHead()->Method() == nsHttp::Post ||
-           mTransaction->RequestHead()->Method() == nsHttp::Put ||
-           mTransaction->RequestHead()->Method() == nsHttp::Options) {
+  else if (mTransaction->RequestHead()->IsPost() ||
+           mTransaction->RequestHead()->IsPut() ||
+           mTransaction->RequestHead()->IsConnect() ||
+           mTransaction->RequestHead()->IsOptions()) {
     // place fin in a data frame even for 0 length messages, I've seen
     // the google gateway be unhappy with fin-on-syn for 0 length POST
   }
@@ -557,10 +608,8 @@ SpdyStream31::AdjustInitialWindow()
     return;
   toack = PR_htonl(toack);
 
-  SpdySession31::EnsureBuffer(mTxInlineFrame,
-                              mTxInlineFrameUsed + 16,
-                              mTxInlineFrameUsed,
-                              mTxInlineFrameSize);
+  EnsureBuffer(mTxInlineFrame, mTxInlineFrameUsed + 16,
+               mTxInlineFrameUsed, mTxInlineFrameSize);
 
   unsigned char *packet = mTxInlineFrame.get() + mTxInlineFrameUsed;
   mTxInlineFrameUsed += 16;
@@ -584,6 +633,9 @@ void
 SpdyStream31::UpdateTransportReadEvents(uint32_t count)
 {
   mTotalRead += count;
+  if (!mSocketTransport) {
+    return;
+  }
 
   mTransaction->OnTransportStatus(mSocketTransport,
                                   NS_NET_STATUS_RECEIVING_FROM,
@@ -688,10 +740,10 @@ SpdyStream31::TransmitFrame(const char *buf,
   // the socket write function. It will accept all of the inline and stream
   // data because of the above 'commitment' even if it has to buffer
 
-  rv = mSegmentReader->OnReadSegment(reinterpret_cast<char*>(mTxInlineFrame.get()),
-                                     mTxInlineFrameUsed,
-                                     &transmittedCount);
-  LOG3(("SpdyStream31::TransmitFrame for inline session=%p "
+  rv = mSession->BufferOutput(reinterpret_cast<char*>(mTxInlineFrame.get()),
+                              mTxInlineFrameUsed,
+                              &transmittedCount);
+  LOG3(("SpdyStream31::TransmitFrame for inline BufferOutput session=%p "
         "stream=%p result %x len=%d",
         mSession, this, rv, transmittedCount));
 
@@ -717,8 +769,15 @@ SpdyStream31::TransmitFrame(const char *buf,
       return NS_ERROR_UNEXPECTED;
     }
 
-    rv = mSegmentReader->OnReadSegment(buf, mTxStreamFrameSize,
-                                       &transmittedCount);
+    // If there is already data buffered, just add to that to form
+    // a single TLS Application Data Record - otherwise skip the memcpy
+    if (mSession->AmountOfOutputBuffered()) {
+      rv = mSession->BufferOutput(buf, mTxStreamFrameSize,
+                                  &transmittedCount);
+    } else {
+      rv = mSession->OnReadSegment(buf, mTxStreamFrameSize,
+                                   &transmittedCount);
+    }
 
     LOG3(("SpdyStream31::TransmitFrame for regular session=%p "
           "stream=%p result %x len=%d",
@@ -739,6 +798,8 @@ SpdyStream31::TransmitFrame(const char *buf,
     *countUsed += mTxStreamFrameSize;
   }
 
+  mSession->FlushOutputQueue();
+
   // calling this will trigger waiting_for if mRequestBodyLenRemaining is 0
   UpdateTransportSendEvents(mTxInlineFrameUsed + mTxStreamFrameSize);
 
@@ -981,26 +1042,16 @@ const unsigned char SpdyStream31::kDictionary[] = {
   0x2c, 0x65, 0x6e, 0x71, 0x3d, 0x30, 0x2e          // - e n q - 0 -
 };
 
-// use for zlib data types
-void *
-SpdyStream31::zlib_allocator(void *opaque, uInt items, uInt size)
-{
-  return moz_xmalloc(items * size);
-}
-
-// use for zlib data types
-void
-SpdyStream31::zlib_destructor(void *opaque, void *addr)
-{
-  moz_free(addr);
-}
-
 // This can be called N times.. 1 for syn_reply and 0->N for headers
 nsresult
 SpdyStream31::Uncompress(z_stream *context,
                          char *blockStart,
                          uint32_t blockLen)
 {
+  // ensure the minimum size
+  EnsureBuffer(mDecompressBuffer, SpdySession31::kDefaultBufferSize,
+               mDecompressBufferUsed, mDecompressBufferSize);
+
   mDecompressedBytes += blockLen;
 
   context->avail_in = blockLen;
@@ -1013,22 +1064,23 @@ SpdyStream31::Uncompress(z_stream *context,
       mDecompressBufferUsed;
     context->avail_out = mDecompressBufferSize - mDecompressBufferUsed;
     int zlib_rv = inflate(context, Z_NO_FLUSH);
+    LOG3(("SpdyStream31::Uncompress %p zlib_rv %d\n", this, zlib_rv));
 
     if (zlib_rv == Z_NEED_DICT) {
       if (triedDictionary) {
-        LOG3(("SpdySession31::Uncompress %p Dictionary Error\n", this));
+        LOG3(("SpdyStream31::Uncompress %p Dictionary Error\n", this));
         return NS_ERROR_ILLEGAL_VALUE;
       }
 
       triedDictionary = true;
       inflateSetDictionary(context, kDictionary, sizeof(kDictionary));
-    }
-
-    if (zlib_rv == Z_DATA_ERROR)
+    } else if (zlib_rv == Z_DATA_ERROR) {
+      LOG3(("SpdyStream31::Uncompress %p inflate returned data err\n", this));
       return NS_ERROR_ILLEGAL_VALUE;
-
-    if (zlib_rv == Z_MEM_ERROR)
+    } else  if (zlib_rv < Z_OK) { // probably Z_MEM_ERROR
+      LOG3(("SpdyStream31::Uncompress %p inflate returned %d\n", this, zlib_rv));
       return NS_ERROR_FAILURE;
+    }
 
     // zlib's inflate() decreases context->avail_out by the amount it places
     // in the output buffer
@@ -1042,10 +1094,8 @@ SpdyStream31::Uncompress(z_stream *context,
         !context->avail_out && context->avail_in) {
       LOG3(("SpdyStream31::Uncompress %p Large Headers - so far %d",
             this, mDecompressBufferSize));
-      SpdySession31::EnsureBuffer(mDecompressBuffer,
-                                  mDecompressBufferSize + 4096,
-                                  mDecompressBufferUsed,
-                                  mDecompressBufferSize);
+      EnsureBuffer(mDecompressBuffer, mDecompressBufferSize + 4096,
+                   mDecompressBufferUsed, mDecompressBufferSize);
     }
   }
   while (context->avail_in);
@@ -1135,9 +1185,9 @@ SpdyStream31::ConvertHeaders(nsACString &aHeadersOut)
   // create UI feedback.
 
   aHeadersOut.Append(version);
-  aHeadersOut.Append(NS_LITERAL_CSTRING(" "));
+  aHeadersOut.Append(' ');
   aHeadersOut.Append(status);
-  aHeadersOut.Append(NS_LITERAL_CSTRING("\r\n"));
+  aHeadersOut.AppendLiteral("\r\n");
 
   const unsigned char *nvpair = reinterpret_cast<unsigned char *>
     (mDecompressBuffer.get()) + 4;
@@ -1191,7 +1241,7 @@ SpdyStream31::ConvertHeaders(nsACString &aHeadersOut)
       // to look for chunked specifically because it is the only HTTP
       // allowed default encoding and we did not negotiate further encodings
       // via TE
-      if (nameString.Equals(NS_LITERAL_CSTRING("transfer-encoding"))) {
+      if (nameString.EqualsLiteral("transfer-encoding")) {
         LOG3(("SpdyStream31::ConvertHeaders session=%p stream=%p "
               "transfer-encoding found. Chunked is invalid and no TE sent.",
               mSession, this));
@@ -1208,15 +1258,15 @@ SpdyStream31::ConvertHeaders(nsACString &aHeadersOut)
 
       // spdy transport level headers shouldn't be gatewayed into http/1
       if (!nameString.IsEmpty() && nameString[0] != ':' &&
-          !nameString.Equals(NS_LITERAL_CSTRING("connection")) &&
-          !nameString.Equals(NS_LITERAL_CSTRING("keep-alive"))) {
+          !nameString.EqualsLiteral("connection") &&
+          !nameString.EqualsLiteral("keep-alive")) {
         nsDependentCSubstring valueString =
           Substring(reinterpret_cast<const char *>(nvpair) + 8 + nameLen,
                     reinterpret_cast<const char *>(nvpair) + 8 + nameLen +
                     valueLen);
 
         aHeadersOut.Append(nameString);
-        aHeadersOut.Append(NS_LITERAL_CSTRING(": "));
+        aHeadersOut.AppendLiteral(": ");
 
         // expand NULL bytes in the value string
         for (char *cPtr = valueString.BeginWriting();
@@ -1228,12 +1278,12 @@ SpdyStream31::ConvertHeaders(nsACString &aHeadersOut)
           }
 
           // NULLs are really "\r\nhdr: "
-          aHeadersOut.Append(NS_LITERAL_CSTRING("\r\n"));
+          aHeadersOut.AppendLiteral("\r\n");
           aHeadersOut.Append(nameString);
-          aHeadersOut.Append(NS_LITERAL_CSTRING(": "));
+          aHeadersOut.AppendLiteral(": ");
         }
 
-        aHeadersOut.Append(NS_LITERAL_CSTRING("\r\n"));
+        aHeadersOut.AppendLiteral("\r\n");
       }
       // move to the next name/value pair in this block
       nvpair += 8 + nameLen + valueLen;
@@ -1244,7 +1294,9 @@ SpdyStream31::ConvertHeaders(nsACString &aHeadersOut)
     nvpair += 4;
   } while (lastHeaderByte >= nvpair);
 
-  aHeadersOut.Append(NS_LITERAL_CSTRING("X-Firefox-Spdy: 3.1\r\n\r\n"));
+  // The decoding went ok. Now we can customize and clean up.
+
+  aHeadersOut.AppendLiteral("X-Firefox-Spdy: 3.1\r\n\r\n");
   LOG (("decoded response headers are:\n%s",
         aHeadersOut.BeginReading()));
 
@@ -1253,6 +1305,12 @@ SpdyStream31::ConvertHeaders(nsACString &aHeadersOut)
   mDecompressBufferSize = 0;
   mDecompressBufferUsed = 0;
 
+  if (mIsTunnel && !mPlainTextTunnel) {
+    aHeadersOut.Truncate();
+    LOG(("SpdyStream31::ConvertHeaders %p 0x%X headers removed for tunnel\n",
+         this, mStreamID));
+  }
+
   return NS_OK;
 }
 
@@ -1266,10 +1324,8 @@ SpdyStream31::ExecuteCompress(uint32_t flushMode)
   {
     uint32_t avail = mTxInlineFrameSize - mTxInlineFrameUsed;
     if (avail < 1) {
-      SpdySession31::EnsureBuffer(mTxInlineFrame,
-                                  mTxInlineFrameSize + 2000,
-                                  mTxInlineFrameUsed,
-                                  mTxInlineFrameSize);
+      EnsureBuffer(mTxInlineFrame, mTxInlineFrameSize + 2000,
+                   mTxInlineFrameUsed, mTxInlineFrameSize);
       avail = mTxInlineFrameSize - mTxInlineFrameUsed;
     }
 
@@ -1320,6 +1376,38 @@ SpdyStream31::CompressFlushFrame()
   ExecuteCompress(Z_SYNC_FLUSH);
 }
 
+bool
+SpdyStream31::GetFullyOpen()
+{
+  return mFullyOpen;
+}
+
+nsresult
+SpdyStream31::SetFullyOpen()
+{
+  MOZ_ASSERT(!mFullyOpen);
+  mFullyOpen = 1;
+  if (mIsTunnel) {
+    int32_t code = 0;
+    nsDependentCSubstring statusSubstring;
+    nsresult rv = FindHeader(NS_LITERAL_CSTRING(":status"), statusSubstring);
+    if (NS_SUCCEEDED(rv)) {
+      nsCString status(statusSubstring);
+      nsresult errcode;
+      code = status.ToInteger(&errcode);
+    }
+
+    LOG3(("SpdyStream31::SetFullyOpen %p Tunnel Response code %d", this, code));
+    if ((code / 100) != 2) {
+      MapStreamToPlainText();
+    }
+
+    MapStreamToHttpConnection();
+    ClearTransactionsBlockedOnTunnel();
+  }
+  return NS_OK;
+}
+
 void
 SpdyStream31::Close(nsresult reason)
 {
@@ -1413,11 +1501,15 @@ SpdyStream31::OnReadSegment(const char *buf,
     mRemoteWindow -= dataLength;
     mSession->DecrementRemoteSessionWindow(dataLength);
 
-    LOG3(("SpdyStream31 %p id %x request len remaining %d, "
-          "count avail %d, chunk used %d",
+    LOG3(("SpdyStream31 %p id %x request len remaining %u, "
+          "count avail %u, chunk used %u",
           this, mStreamID, mRequestBodyLenRemaining, count, dataLength));
-    if (dataLength > mRequestBodyLenRemaining)
+    if (!dataLength && mRequestBodyLenRemaining) {
+      return NS_BASE_STREAM_WOULD_BLOCK;
+    }
+    if (dataLength > mRequestBodyLenRemaining) {
       return NS_ERROR_UNEXPECTED;
+    }
     mRequestBodyLenRemaining -= dataLength;
     GenerateDataFrameHeader(dataLength, !mRequestBodyLenRemaining);
     ChangeState(SENDING_REQUEST_BODY);
@@ -1483,6 +1575,36 @@ SpdyStream31::OnWriteSegment(char *buf,
   return NS_OK;
 }
 
+/// connect tunnels
+
+void
+SpdyStream31::ClearTransactionsBlockedOnTunnel()
+{
+  MOZ_ASSERT(PR_GetCurrentThread() == gSocketThread);
+
+  if (!mIsTunnel) {
+    return;
+  }
+  gHttpHandler->ConnMgr()->ProcessPendingQ(mTransaction->ConnectionInfo());
+}
+
+void
+SpdyStream31::MapStreamToPlainText()
+{
+  nsRefPtr<SpdyConnectTransaction> qiTrans(mTransaction->QuerySpdyConnectTransaction());
+  MOZ_ASSERT(qiTrans);
+  mPlainTextTunnel = true;
+  qiTrans->ForcePlainText();
+}
+
+void
+SpdyStream31::MapStreamToHttpConnection()
+{
+  nsRefPtr<SpdyConnectTransaction> qiTrans(mTransaction->QuerySpdyConnectTransaction());
+  MOZ_ASSERT(qiTrans);
+  qiTrans->MapStreamToHttpConnection(mSocketTransport,
+                                     mTransaction->ConnectionInfo());
+}
+
 } // namespace mozilla::net
 } // namespace mozilla
-
