@@ -13,6 +13,7 @@
 #include "nsProxyRelease.h"
 #include "prinrval.h"
 #include "TunnelUtils.h"
+#include "mozilla/Mutex.h"
 
 #include "nsIAsyncInputStream.h"
 #include "nsIAsyncOutputStream.h"
@@ -35,13 +36,13 @@ class ASpdySession;
 // accessed from any other thread.
 //-----------------------------------------------------------------------------
 
-class nsHttpConnection : public nsAHttpSegmentReader
-                       , public nsAHttpSegmentWriter
-                       , public nsIInputStreamCallback
-                       , public nsIOutputStreamCallback
-                       , public nsITransportEventSink
-                       , public nsIInterfaceRequestor
-                       , public NudgeTunnelCallback
+class nsHttpConnection MOZ_FINAL : public nsAHttpSegmentReader
+                                 , public nsAHttpSegmentWriter
+                                 , public nsIInputStreamCallback
+                                 , public nsIOutputStreamCallback
+                                 , public nsITransportEventSink
+                                 , public nsIInterfaceRequestor
+                                 , public NudgeTunnelCallback
 {
     virtual ~nsHttpConnection();
 
@@ -200,6 +201,20 @@ public:
     void    SetupSecondaryTLS();
     void    SetInSpdyTunnel(bool arg);
 
+    // Check active connections for traffic (or not). SPDY connections send a
+    // ping, ordinary HTTP connections get some time to get traffic to be
+    // considered alive.
+    void CheckForTraffic(bool check);
+
+    // NoTraffic() returns true if there's been no traffic on the (non-spdy)
+    // connection since CheckForTraffic() was called.
+    bool NoTraffic() {
+        return mTrafficStamp &&
+            (mTrafficCount == (mTotalBytesWritten + mTotalBytesRead));
+    }
+    // override of nsAHttpConnection
+    virtual uint32_t Version();
+
 private:
     // Value (set in mTCPKeepaliveConfig) indicates which set of prefs to use.
     enum TCPKeepaliveConfig {
@@ -292,6 +307,10 @@ private:
     bool                            mInSpdyTunnel;
     bool                            mForcePlainText;
 
+    // A snapshot of current number of transfered bytes
+    int64_t                         mTrafficCount;
+    bool                            mTrafficStamp; // true then the above is set
+
     // The number of <= HTTP/1.1 transactions performed on this connection. This
     // excludes spdy transactions.
     uint32_t                        mHttp1xTransactionCount;
