@@ -8,24 +8,31 @@
 #include "nsWyciwygChannel.h"
 #include "nsILoadGroup.h"
 #include "nsNetUtil.h"
-#include "nsICacheService.h"
-#include "nsICacheSession.h"
+#include "LoadContextInfo.h"
+#include "nsICacheService.h" // only to initialize
+#include "nsICacheStorageService.h"
+#include "nsICacheStorage.h"
+#include "nsICacheEntry.h"
+#include "CacheObserver.h"
 #include "nsCharsetSource.h"
 #include "nsProxyRelease.h"
 #include "nsThreadUtils.h"
-#include "nsICacheEntryDescriptor.h"
 #include "nsIEventTarget.h"
 #include "nsIInputStream.h"
 #include "nsIInputStreamPump.h"
 #include "nsIOutputStream.h"
 #include "nsIProgressEventSink.h"
 #include "nsIURI.h"
-#include "nsWyciwygProtocolHandler.h"
+#include "mozilla/DebugOnly.h"
+#include "mozilla/unused.h"
+#include "nsProxyRelease.h"
+
+typedef mozilla::net::LoadContextInfo LoadContextInfo;
 
 // Must release mChannel on the main thread
 class nsWyciwygAsyncEvent : public nsRunnable {
 public:
-  nsWyciwygAsyncEvent(nsWyciwygChannel *aChannel) : mChannel(aChannel) {}
+  explicit nsWyciwygAsyncEvent(nsWyciwygChannel *aChannel) : mChannel(aChannel) {}
 
   ~nsWyciwygAsyncEvent()
   {
@@ -33,7 +40,7 @@ public:
     NS_WARN_IF_FALSE(thread, "Couldn't get the main thread!");
     if (thread) {
       nsIWyciwygChannel *chan = static_cast<nsIWyciwygChannel *>(mChannel);
-      mChannel.forget();
+      mozilla::unused << mChannel.forget();
       NS_ProxyRelease(thread, chan);
     }
   }
@@ -43,7 +50,7 @@ protected:
 
 class nsWyciwygSetCharsetandSourceEvent : public nsWyciwygAsyncEvent {
 public:
-  nsWyciwygSetCharsetandSourceEvent(nsWyciwygChannel *aChannel)
+  explicit nsWyciwygSetCharsetandSourceEvent(nsWyciwygChannel *aChannel)
     : nsWyciwygAsyncEvent(aChannel) {}
 
   NS_IMETHOD Run()
@@ -55,18 +62,16 @@ public:
 
 class nsWyciwygWriteEvent : public nsWyciwygAsyncEvent {
 public:
-  nsWyciwygWriteEvent(nsWyciwygChannel *aChannel, const nsAString &aData, 
-                      const nsACString &spec)
-    : nsWyciwygAsyncEvent(aChannel), mData(aData), mSpec(spec) {}
+  nsWyciwygWriteEvent(nsWyciwygChannel *aChannel, const nsAString &aData)
+    : nsWyciwygAsyncEvent(aChannel), mData(aData) {}
 
   NS_IMETHOD Run()
   {
-    mChannel->WriteToCacheEntryInternal(mData, mSpec);
+    mChannel->WriteToCacheEntryInternal(mData);
     return NS_OK;
   }
 private:
   nsString mData;
-  nsCString mSpec;
 };
 
 class nsWyciwygCloseEvent : public nsWyciwygAsyncEvent {
@@ -86,7 +91,8 @@ private:
 
 // nsWyciwygChannel methods 
 nsWyciwygChannel::nsWyciwygChannel()
-  : mStatus(NS_OK),
+  : mMode(NONE),
+    mStatus(NS_OK),
     mIsPending(false),
     mCharsetAndSourceSet(false),
     mNeedToWriteCharset(false),
@@ -100,16 +106,24 @@ nsWyciwygChannel::nsWyciwygChannel()
 
 nsWyciwygChannel::~nsWyciwygChannel() 
 {
+  if (mLoadInfo) {
+    nsCOMPtr<nsIThread> mainThread;
+    NS_GetMainThread(getter_AddRefs(mainThread));
+
+    nsILoadInfo *forgetableLoadInfo;
+    mLoadInfo.forget(&forgetableLoadInfo);
+    NS_ProxyRelease(mainThread, forgetableLoadInfo, false);
+  }
 }
 
-NS_IMPL_ISUPPORTS7(nsWyciwygChannel,
-                   nsIChannel,
-                   nsIRequest,
-                   nsIStreamListener,
-                   nsIRequestObserver,
-                   nsICacheListener,
-                   nsIWyciwygChannel,
-                   nsIPrivateBrowsingChannel)
+NS_IMPL_ISUPPORTS(nsWyciwygChannel,
+                  nsIChannel,
+                  nsIRequest,
+                  nsIStreamListener,
+                  nsIRequestObserver,
+                  nsICacheEntryOpenCallback,
+                  nsIWyciwygChannel,
+                  nsIPrivateBrowsingChannel)
 
 nsresult
 nsWyciwygChannel::Init(nsIURI* uri)
@@ -118,14 +132,24 @@ nsWyciwygChannel::Init(nsIURI* uri)
 
   nsresult rv;
 
+  if (!mozilla::net::CacheObserver::UseNewCache()) {
+    // Since nsWyciwygChannel can use the new cache API off the main thread
+    // and that API normally does this initiation, we need to take care
+    // of initiating the old cache service here manually.  Will be removed
+    // with bug 913828.
+    MOZ_ASSERT(NS_IsMainThread());
+    nsCOMPtr<nsICacheService> service =
+      do_GetService(NS_CACHESERVICE_CONTRACTID, &rv);
+  }
+
   mURI = uri;
   mOriginalURI = uri;
 
-  nsCOMPtr<nsICacheService> serv =
-    do_GetService(NS_CACHESERVICE_CONTRACTID, &rv);
+  nsCOMPtr<nsICacheStorageService> serv =
+    do_GetService("@mozilla.org/netwerk/cache-storage-service;1", &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = serv->GetCacheIOTarget(getter_AddRefs(mCacheIOTarget));
+  rv = serv->GetIoTarget(getter_AddRefs(mCacheIOTarget));
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -256,10 +280,7 @@ nsWyciwygChannel::GetURI(nsIURI* *aURI)
 NS_IMETHODIMP
 nsWyciwygChannel::GetOwner(nsISupports **aOwner)
 {
-  NS_PRECONDITION(mOwner, "Must have a principal!");
-  NS_ENSURE_STATE(mOwner);
-
-  NS_ADDREF(*aOwner = mOwner);
+  NS_IF_ADDREF(*aOwner = mOwner);
   return NS_OK;
 }
 
@@ -270,6 +291,20 @@ nsWyciwygChannel::SetOwner(nsISupports* aOwner)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsWyciwygChannel::GetLoadInfo(nsILoadInfo **aLoadInfo)
+{
+  NS_IF_ADDREF(*aLoadInfo = mLoadInfo);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsWyciwygChannel::SetLoadInfo(nsILoadInfo* aLoadInfo)
+{
+  mLoadInfo = aLoadInfo;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsWyciwygChannel::GetNotificationCallbacks(nsIInterfaceRequestor* *aCallbacks)
 {
@@ -321,7 +356,7 @@ nsWyciwygChannel::SetContentType(const nsACString &aContentType)
 NS_IMETHODIMP
 nsWyciwygChannel::GetContentCharset(nsACString &aContentCharset)
 {
-  aContentCharset.Assign("UTF-16");
+  aContentCharset.AssignLiteral("UTF-16");
   return NS_OK;
 }
 
@@ -386,28 +421,29 @@ NS_IMETHODIMP
 nsWyciwygChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctx)
 {
   LOG(("nsWyciwygChannel::AsyncOpen [this=%p]\n", this));
+  MOZ_ASSERT(mMode == NONE, "nsWyciwygChannel already open");
 
   NS_ENSURE_TRUE(!mIsPending, NS_ERROR_IN_PROGRESS);
+  NS_ENSURE_TRUE(mMode == NONE, NS_ERROR_IN_PROGRESS);
   NS_ENSURE_ARG_POINTER(listener);
 
-  nsAutoCString spec;
-  mURI->GetSpec(spec);
+  mMode = READING;
 
   // open a cache entry for this channel...
-  nsresult rv = OpenCacheEntry(spec, nsICache::ACCESS_READ);
-  if (rv == NS_ERROR_CACHE_KEY_NOT_FOUND) {
-    // Overwrite rv on purpose; if event dispatch fails we'll bail, and
-    // otherwise we'll wait until the event fires before calling back.
-    rv = NS_DispatchToCurrentThread(
-            NS_NewRunnableMethod(this, &nsWyciwygChannel::NotifyListener));
-  }
-
+  // mIsPending set to true since OnCacheEntryAvailable may be called
+  // synchronously and fails when mIsPending found false.
+  mIsPending = true;
+  nsresult rv = OpenCacheEntry(mURI, nsICacheStorage::OPEN_READONLY |
+                                     nsICacheStorage::CHECK_MULTITHREADED);
   if (NS_FAILED(rv)) {
     LOG(("nsWyciwygChannel::OpenCacheEntry failed [rv=%x]\n", rv));
+    mIsPending = false;
     return rv;
   }
 
-  mIsPending = true;
+  // There is no code path that would invoke the listener sooner than
+  // we get to this line in case OnCacheEntryAvailable is invoked
+  // synchronously.
   mListener = listener;
   mListenerContext = ctx;
 
@@ -421,29 +457,56 @@ nsWyciwygChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctx)
 // nsIWyciwygChannel
 //////////////////////////////////////////////////////////////////////////////
 
+nsresult
+nsWyciwygChannel::EnsureWriteCacheEntry()
+{
+  MOZ_ASSERT(mMode == WRITING, "nsWyciwygChannel not open for writing");
+
+  if (!mCacheEntry) {
+    // OPEN_TRUNCATE will give us the entry instantly
+    nsresult rv = OpenCacheEntry(mURI, nsICacheStorage::OPEN_TRUNCATE);
+    if (NS_FAILED(rv) || !mCacheEntry) {
+      LOG(("  could not synchronously open cache entry for write!"));
+      return NS_ERROR_FAILURE;
+    }
+  }
+
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsWyciwygChannel::WriteToCacheEntry(const nsAString &aData)
 {
-  // URIs not thread-safe, so get spec now in case we need it
-  nsAutoCString spec;
-  nsresult rv = mURI->GetAsciiSpec(spec);
-  if (NS_FAILED(rv)) 
-    return rv;
+  if (mMode == READING) {
+    LOG(("nsWyciwygChannel::WriteToCacheEntry already open for reading"));
+    MOZ_ASSERT(false);
+    return NS_ERROR_UNEXPECTED;
+  }
 
-  return mCacheIOTarget->Dispatch(new nsWyciwygWriteEvent(this, aData, spec),
+  mMode = WRITING;
+
+  if (mozilla::net::CacheObserver::UseNewCache()) {
+    nsresult rv = EnsureWriteCacheEntry();
+    if (NS_FAILED(rv)) return rv;
+  }
+
+  return mCacheIOTarget->Dispatch(new nsWyciwygWriteEvent(this, aData),
                                   NS_DISPATCH_NORMAL);
 }
 
 nsresult
-nsWyciwygChannel::WriteToCacheEntryInternal(const nsAString &aData, const nsACString& spec)
+nsWyciwygChannel::WriteToCacheEntryInternal(const nsAString &aData)
 {
+  LOG(("nsWyciwygChannel::WriteToCacheEntryInternal [this=%p]", this));
   NS_ASSERTION(IsOnCacheIOThread(), "wrong thread");
 
   nsresult rv;
 
-  if (!mCacheEntry) {
-    rv = OpenCacheEntry(spec, nsICache::ACCESS_WRITE);
-    if (NS_FAILED(rv)) return rv;
+  // With the new cache entry this will just pass as a no-op since we
+  // are opening the entry in WriteToCacheEntry.
+  rv = EnsureWriteCacheEntry();
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return rv;
   }
 
   if (mLoadFlags & INHIBIT_PERSISTENT_CACHING) {
@@ -459,7 +522,7 @@ nsWyciwygChannel::WriteToCacheEntryInternal(const nsAString &aData, const nsACSt
     WriteCharsetAndSourceToCache(mCharsetSource, mCharset);
     mNeedToWriteCharset = false;
   }
-  
+
   uint32_t out;
   if (!mCacheOutputStream) {
     // Get the outputstream from the cache entry.
@@ -468,13 +531,13 @@ nsWyciwygChannel::WriteToCacheEntryInternal(const nsAString &aData, const nsACSt
 
     // Write out a Byte Order Mark, so that we'll know if the data is
     // BE or LE when we go to read it.
-    PRUnichar bom = 0xFEFF;
+    char16_t bom = 0xFEFF;
     rv = mCacheOutputStream->Write((char *)&bom, sizeof(bom), &out);
     if (NS_FAILED(rv)) return rv;
   }
 
   return mCacheOutputStream->Write((const char *)PromiseFlatString(aData).get(),
-                                   aData.Length() * sizeof(PRUnichar), &out);
+                                   aData.Length() * sizeof(char16_t), &out);
 }
 
 
@@ -496,7 +559,7 @@ nsWyciwygChannel::CloseCacheEntryInternal(nsresult reason)
     mCacheInputStream = 0;
 
     if (NS_FAILED(reason))
-      mCacheEntry->Doom();
+      mCacheEntry->AsyncDoom(nullptr); // here we were calling Doom() ...
 
     mCacheEntry = 0;
   }
@@ -573,17 +636,29 @@ nsWyciwygChannel::GetCharsetAndSource(int32_t* aSource, nsACString& aCharset)
 }
 
 //////////////////////////////////////////////////////////////////////////////
-// nsICachelistener
+// nsICacheEntryOpenCallback
 //////////////////////////////////////////////////////////////////////////////
+
 NS_IMETHODIMP
-nsWyciwygChannel::OnCacheEntryAvailable(nsICacheEntryDescriptor * aCacheEntry, nsCacheAccessMode aMode, nsresult aStatus)
+nsWyciwygChannel::OnCacheEntryCheck(nsICacheEntry* entry, nsIApplicationCache* appCache,
+                                    uint32_t* aResult)
 {
-  LOG(("nsWyciwygChannel::OnCacheEntryAvailable [this=%p entry=%x "
-       "access=%x status=%x]\n", this, aCacheEntry, aMode, aStatus));
+  *aResult = ENTRY_WANTED;
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsWyciwygChannel::OnCacheEntryAvailable(nsICacheEntry *aCacheEntry,
+                                        bool aNew,
+                                        nsIApplicationCache* aAppCache,
+                                        nsresult aStatus)
+{
+  LOG(("nsWyciwygChannel::OnCacheEntryAvailable [this=%p entry=%p "
+       "new=%d status=%x]\n", this, aCacheEntry, aNew, aStatus));
 
   // if the channel's already fired onStopRequest, 
   // then we should ignore this event.
-  if (!mIsPending)
+  if (!mIsPending && !aNew)
     return NS_OK;
 
   // otherwise, we have to handle this event.
@@ -592,12 +667,12 @@ nsWyciwygChannel::OnCacheEntryAvailable(nsICacheEntryDescriptor * aCacheEntry, n
   else if (NS_SUCCEEDED(mStatus))
     mStatus = aStatus;
 
-  nsresult rv;
+  nsresult rv = NS_OK;
   if (NS_FAILED(mStatus)) {
     LOG(("channel was canceled [this=%p status=%x]\n", this, mStatus));
     rv = mStatus;
   }
-  else { // advance to the next state...
+  else if (!aNew) { // advance to the next state...
     rv = ReadFromCache();
   }
 
@@ -605,18 +680,17 @@ nsWyciwygChannel::OnCacheEntryAvailable(nsICacheEntryDescriptor * aCacheEntry, n
   if (NS_FAILED(rv)) {
     CloseCacheEntry(rv);
 
-    NotifyListener();
+    if (!aNew) {
+      // Since OnCacheEntryAvailable can be called directly from AsyncOpen
+      // we must dispatch.
+      NS_DispatchToCurrentThread(NS_NewRunnableMethod(
+        this, &nsWyciwygChannel::NotifyListener));
+    }
   }
 
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsWyciwygChannel::OnCacheEntryDoomed(nsresult status)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 //-----------------------------------------------------------------------------
 // nsWyciwygChannel::nsIStreamListener
 //-----------------------------------------------------------------------------
@@ -634,9 +708,10 @@ nsWyciwygChannel::OnDataAvailable(nsIRequest *request, nsISupports *ctx,
   rv = mListener->OnDataAvailable(this, mListenerContext, input, offset, count);
 
   // XXX handle 64-bit stuff for real
-  if (mProgressSink && NS_SUCCEEDED(rv) && !(mLoadFlags & LOAD_BACKGROUND))
+  if (mProgressSink && NS_SUCCEEDED(rv)) {
     mProgressSink->OnProgress(this, nullptr, offset + count,
                               uint64_t(mContentLength));
+  }
 
   return rv; // let the pump cancel on failure
 }
@@ -687,43 +762,30 @@ nsWyciwygChannel::OnStopRequest(nsIRequest *request, nsISupports *ctx, nsresult
 //////////////////////////////////////////////////////////////////////////////
 
 nsresult
-nsWyciwygChannel::OpenCacheEntry(const nsACString & aCacheKey,
-                                 nsCacheAccessMode aAccessMode)
+nsWyciwygChannel::OpenCacheEntry(nsIURI *aURI,
+                                 uint32_t aOpenFlags)
 {
-  nsresult rv = NS_ERROR_FAILURE;
-  // Get cache service
-  nsCOMPtr<nsICacheService> cacheService =
-    do_GetService(NS_CACHESERVICE_CONTRACTID, &rv);
+  nsresult rv;
+
+  nsCOMPtr<nsICacheStorageService> cacheService =
+    do_GetService("@mozilla.org/netwerk/cache-storage-service;1", &rv);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  // honor security settings
-  nsCacheStoragePolicy storagePolicy;
-  if (mPrivateBrowsing || mLoadFlags & INHIBIT_PERSISTENT_CACHING)
-    storagePolicy = nsICache::STORE_IN_MEMORY;
+  bool anonymous = mLoadFlags & LOAD_ANONYMOUS;
+  nsRefPtr<LoadContextInfo> loadInfo = mozilla::net::GetLoadContextInfo(
+    mPrivateBrowsing, mAppId, mInBrowser, anonymous);
+
+  nsCOMPtr<nsICacheStorage> cacheStorage;
+  if (mLoadFlags & INHIBIT_PERSISTENT_CACHING)
+    rv = cacheService->MemoryCacheStorage(loadInfo, getter_AddRefs(cacheStorage));
   else
-    storagePolicy = nsICache::STORE_ANYWHERE;
+    rv = cacheService->DiskCacheStorage(loadInfo, false, getter_AddRefs(cacheStorage));
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsICacheSession> cacheSession;
-  nsAutoCString sessionName;
-  nsWyciwygProtocolHandler::GetCacheSessionName(mAppId, mInBrowser,
-                                                mPrivateBrowsing,
-                                                sessionName);
+  rv = cacheStorage->AsyncOpenURI(aURI, EmptyCString(), aOpenFlags, this);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  // Open a stream based cache session.
-  rv = cacheService->CreateSession(sessionName.get(), storagePolicy, true,
-                                   getter_AddRefs(cacheSession));
-  if (!cacheSession) 
-    return NS_ERROR_FAILURE;
-
-  cacheSession->SetIsPrivate(mPrivateBrowsing);
-
-  if (aAccessMode == nsICache::ACCESS_WRITE)
-    rv = cacheSession->OpenCacheEntry(aCacheKey, aAccessMode, false,
-                                      getter_AddRefs(mCacheEntry));
-  else
-    rv = cacheSession->AsyncOpenCacheEntry(aCacheKey, aAccessMode, this, false);
-
-  return rv;
+  return NS_OK;
 }
 
 nsresult
@@ -740,7 +802,7 @@ nsWyciwygChannel::ReadFromCache()
   nsAutoCString tmpStr;
   rv = mCacheEntry->GetMetaDataElement("inhibit-persistent-caching",
                                        getter_Copies(tmpStr));
-  if (NS_SUCCEEDED(rv) && tmpStr == NS_LITERAL_CSTRING("1"))
+  if (NS_SUCCEEDED(rv) && tmpStr.EqualsLiteral("1"))
     mLoadFlags |= INHIBIT_PERSISTENT_CACHING;
 
   // Get a transport to the cached data...
