@@ -22,6 +22,8 @@ from mozbuild.frontend.reader import BuildReader
 
 from mozbuild.test.backend.common import BackendTester
 
+import mozpack.path as mozpath
+
 
 class TestRecursiveMakeTraversal(unittest.TestCase):
     def test_traversal(self):
@@ -30,21 +32,21 @@ class TestRecursiveMakeTraversal(unittest.TestCase):
         traversal.add('', dirs=['D'])
         traversal.add('A')
         traversal.add('B', dirs=['E', 'F'])
-        traversal.add('C', parallel=['G', 'H'])
-        traversal.add('D', parallel=['I'], dirs=['K'])
-        traversal.add('D', parallel=['J'], dirs=['L'])
+        traversal.add('C', dirs=['G', 'H'])
+        traversal.add('D', dirs=['I', 'K'])
+        traversal.add('D', dirs=['J', 'L'])
         traversal.add('E')
         traversal.add('F')
         traversal.add('G')
         traversal.add('H')
         traversal.add('I', dirs=['M', 'N'])
-        traversal.add('J', parallel=['O', 'P'])
-        traversal.add('K', parallel=['Q', 'R'])
+        traversal.add('J', dirs=['O', 'P'])
+        traversal.add('K', dirs=['Q', 'R'])
         traversal.add('L', dirs=['S'])
         traversal.add('M')
         traversal.add('N', dirs=['T'])
         traversal.add('O')
-        traversal.add('P', parallel=['U'])
+        traversal.add('P', dirs=['U'])
         traversal.add('Q')
         traversal.add('R', dirs=['V'])
         traversal.add('S', dirs=['W'])
@@ -54,8 +56,14 @@ class TestRecursiveMakeTraversal(unittest.TestCase):
         traversal.add('W', dirs=['X'])
         traversal.add('X')
 
-        start, deps = traversal.compute_dependencies()
+        parallels = set(('G', 'H', 'I', 'J', 'O', 'P', 'Q', 'R', 'U'))
+        def filter(current, subdirs):
+            return (current, [d for d in subdirs.dirs if d in parallels],
+                [d for d in subdirs.dirs if d not in parallels])
+
+        start, deps = traversal.compute_dependencies(filter)
         self.assertEqual(start, ('X',))
+        self.maxDiff = None
         self.assertEqual(deps, {
             'A': ('',),
             'B': ('A',),
@@ -83,21 +91,21 @@ class TestRecursiveMakeTraversal(unittest.TestCase):
             'X': ('W',),
         })
 
-        self.assertEqual(list(traversal.traverse('')),
+        self.assertEqual(list(traversal.traverse('', filter)),
                          ['', 'A', 'B', 'E', 'F', 'C', 'G', 'H', 'D', 'I',
                          'M', 'N', 'T', 'J', 'O', 'P', 'U', 'K', 'Q', 'R',
                          'V', 'L', 'S', 'W', 'X'])
 
-        self.assertEqual(list(traversal.traverse('C')),
+        self.assertEqual(list(traversal.traverse('C', filter)),
                          ['C', 'G', 'H'])
 
     def test_traversal_2(self):
         traversal = RecursiveMakeTraversal()
         traversal.add('', dirs=['A', 'B', 'C'])
         traversal.add('A')
-        traversal.add('B', static=['D'], dirs=['E', 'F'])
-        traversal.add('C', parallel=['G', 'H'], dirs=['I'])
-        # Don't register D
+        traversal.add('B', dirs=['D', 'E', 'F'])
+        traversal.add('C', dirs=['G', 'H', 'I'])
+        traversal.add('D')
         traversal.add('E')
         traversal.add('F')
         traversal.add('G')
@@ -114,16 +122,16 @@ class TestRecursiveMakeTraversal(unittest.TestCase):
             'E': ('D',),
             'F': ('E',),
             'G': ('C',),
-            'H': ('C',),
-            'I': ('G', 'H'),
+            'H': ('G',),
+            'I': ('H',),
         })
 
     def test_traversal_filter(self):
         traversal = RecursiveMakeTraversal()
         traversal.add('', dirs=['A', 'B', 'C'])
         traversal.add('A')
-        traversal.add('B', static=['D'], dirs=['E', 'F'])
-        traversal.add('C', parallel=['G', 'H'], dirs=['I'])
+        traversal.add('B', dirs=['D', 'E', 'F'])
+        traversal.add('C', dirs=['G', 'H', 'I'])
         traversal.add('D')
         traversal.add('E')
         traversal.add('F')
@@ -134,27 +142,28 @@ class TestRecursiveMakeTraversal(unittest.TestCase):
         def filter(current, subdirs):
             if current == 'B':
                 current = None
-            return current, subdirs.parallel, subdirs.dirs
+            return current, [], subdirs.dirs
 
         start, deps = traversal.compute_dependencies(filter)
         self.assertEqual(start, ('I',))
         self.assertEqual(deps, {
             'A': ('',),
             'C': ('F',),
-            'E': ('A',),
+            'D': ('A',),
+            'E': ('D',),
             'F': ('E',),
             'G': ('C',),
-            'H': ('C',),
-            'I': ('G', 'H'),
+            'H': ('G',),
+            'I': ('H',),
         })
 
 class TestRecursiveMakeBackend(BackendTester):
     def test_basic(self):
         """Ensure the RecursiveMakeBackend works without error."""
         env = self._consume('stub0', RecursiveMakeBackend)
-        self.assertTrue(os.path.exists(os.path.join(env.topobjdir,
+        self.assertTrue(os.path.exists(mozpath.join(env.topobjdir,
             'backend.RecursiveMakeBackend')))
-        self.assertTrue(os.path.exists(os.path.join(env.topobjdir,
+        self.assertTrue(os.path.exists(mozpath.join(env.topobjdir,
             'backend.RecursiveMakeBackend.pp')))
 
     def test_output_files(self):
@@ -164,8 +173,8 @@ class TestRecursiveMakeBackend(BackendTester):
         expected = ['', 'dir1', 'dir2']
 
         for d in expected:
-            out_makefile = os.path.join(env.topobjdir, d, 'Makefile')
-            out_backend = os.path.join(env.topobjdir, d, 'backend.mk')
+            out_makefile = mozpath.join(env.topobjdir, d, 'Makefile')
+            out_backend = mozpath.join(env.topobjdir, d, 'backend.mk')
 
             self.assertTrue(os.path.exists(out_makefile))
             self.assertTrue(os.path.exists(out_backend))
@@ -174,7 +183,7 @@ class TestRecursiveMakeBackend(BackendTester):
         """Ensure Makefile.in is converted properly."""
         env = self._consume('stub0', RecursiveMakeBackend)
 
-        p = os.path.join(env.topobjdir, 'Makefile')
+        p = mozpath.join(env.topobjdir, 'Makefile')
 
         lines = [l.strip() for l in open(p, 'rt').readlines()[1:] if not l.startswith('#')]
         self.assertEqual(lines, [
@@ -194,7 +203,7 @@ class TestRecursiveMakeBackend(BackendTester):
         """Ensure missing Makefile.in results in Makefile creation."""
         env = self._consume('stub0', RecursiveMakeBackend)
 
-        p = os.path.join(env.topobjdir, 'dir2', 'Makefile')
+        p = mozpath.join(env.topobjdir, 'dir2', 'Makefile')
         self.assertTrue(os.path.exists(p))
 
         lines = [l.strip() for l in open(p, 'rt').readlines()]
@@ -206,13 +215,11 @@ class TestRecursiveMakeBackend(BackendTester):
         """Ensure backend.mk file is written out properly."""
         env = self._consume('stub0', RecursiveMakeBackend)
 
-        p = os.path.join(env.topobjdir, 'backend.mk')
+        p = mozpath.join(env.topobjdir, 'backend.mk')
 
         lines = [l.strip() for l in open(p, 'rt').readlines()[2:]]
         self.assertEqual(lines, [
-            'MOZBUILD_DERIVED := 1',
-            'DIRS := dir1',
-            'PARALLEL_DIRS := dir2',
+            'DIRS := dir1 dir2',
             'TEST_DIRS := dir3',
         ])
 
@@ -221,8 +228,8 @@ class TestRecursiveMakeBackend(BackendTester):
 
         env = self._consume('stub0', RecursiveMakeBackend)
 
-        makefile_path = os.path.join(env.topobjdir, 'Makefile')
-        backend_path = os.path.join(env.topobjdir, 'backend.mk')
+        makefile_path = mozpath.join(env.topobjdir, 'Makefile')
+        backend_path = mozpath.join(env.topobjdir, 'backend.mk')
         makefile_mtime = os.path.getmtime(makefile_path)
         backend_mtime = os.path.getmtime(backend_path)
 
@@ -234,25 +241,11 @@ class TestRecursiveMakeBackend(BackendTester):
         self.assertEqual(os.path.getmtime(makefile_path), makefile_mtime)
         self.assertEqual(os.path.getmtime(backend_path), backend_mtime)
 
-    def test_external_make_dirs(self):
-        """Ensure we have make recursion into external make directories."""
-        env = self._consume('external_make_dirs', RecursiveMakeBackend)
-
-        backend_path = os.path.join(env.topobjdir, 'backend.mk')
-        lines = [l.strip() for l in open(backend_path, 'rt').readlines()[2:]]
-        self.assertEqual(lines, [
-            'MOZBUILD_DERIVED := 1',
-            'DIRS := dir',
-            'PARALLEL_DIRS := p_dir',
-            'DIRS += external',
-            'PARALLEL_DIRS += p_external',
-        ])
-
     def test_substitute_config_files(self):
         """Ensure substituted config files are produced."""
         env = self._consume('substitute_config_files', RecursiveMakeBackend)
 
-        p = os.path.join(env.topobjdir, 'foo')
+        p = mozpath.join(env.topobjdir, 'foo')
         self.assertTrue(os.path.exists(p))
         lines = [l.strip() for l in open(p, 'rt').readlines()]
         self.assertEqual(lines, [
@@ -263,7 +256,74 @@ class TestRecursiveMakeBackend(BackendTester):
         """Ensure variable passthru is written out correctly."""
         env = self._consume('variable_passthru', RecursiveMakeBackend)
 
-        backend_path = os.path.join(env.topobjdir, 'backend.mk')
+        backend_path = mozpath.join(env.topobjdir, 'backend.mk')
+        lines = [l.strip() for l in open(backend_path, 'rt').readlines()[2:]]
+
+        expected = {
+            'DISABLE_STL_WRAPPING': [
+                'DISABLE_STL_WRAPPING := 1',
+            ],
+            'EXTRA_COMPONENTS': [
+                'EXTRA_COMPONENTS += bar.js',
+                'EXTRA_COMPONENTS += foo.js',
+            ],
+            'EXTRA_PP_COMPONENTS': [
+                'EXTRA_PP_COMPONENTS += bar.pp.js',
+                'EXTRA_PP_COMPONENTS += foo.pp.js',
+            ],
+            'FAIL_ON_WARNINGS': [
+                'FAIL_ON_WARNINGS := 1',
+            ],
+            'MSVC_ENABLE_PGO': [
+                'MSVC_ENABLE_PGO := 1',
+            ],
+            'VISIBILITY_FLAGS': [
+                'VISIBILITY_FLAGS :=',
+            ],
+            'RCFILE': [
+                'RCFILE := foo.rc',
+            ],
+            'RESFILE': [
+                'RESFILE := bar.res',
+            ],
+            'RCINCLUDE': [
+                'RCINCLUDE := bar.rc',
+            ],
+            'DEFFILE': [
+                'DEFFILE := baz.def',
+            ],
+            'USE_STATIC_LIBS': [
+                'USE_STATIC_LIBS := 1',
+            ],
+            'MOZBUILD_CFLAGS': [
+                'MOZBUILD_CFLAGS += -fno-exceptions',
+                'MOZBUILD_CFLAGS += -w',
+            ],
+            'MOZBUILD_CXXFLAGS': [
+                'MOZBUILD_CXXFLAGS += -fcxx-exceptions',
+                'MOZBUILD_CXXFLAGS += -include foo.h',
+            ],
+            'MOZBUILD_LDFLAGS': [
+                'MOZBUILD_LDFLAGS += -framework Foo',
+                'MOZBUILD_LDFLAGS += -x',
+                'MOZBUILD_LDFLAGS += -DELAYLOAD:foo.dll',
+                'MOZBUILD_LDFLAGS += -DELAYLOAD:bar.dll',
+            ],
+            'WIN32_EXE_LDFLAGS': [
+                'WIN32_EXE_LDFLAGS += -subsystem:console',
+            ],
+        }
+
+        for var, val in expected.items():
+            # print("test_variable_passthru[%s]" % (var))
+            found = [str for str in lines if str.startswith(var)]
+            self.assertEqual(found, val)
+
+    def test_sources(self):
+        """Ensure SOURCES and HOST_SOURCES are handled properly."""
+        env = self._consume('sources', RecursiveMakeBackend)
+
+        backend_path = mozpath.join(env.topobjdir, 'backend.mk')
         lines = [l.strip() for l in open(backend_path, 'rt').readlines()[2:]]
 
         expected = {
@@ -275,32 +335,10 @@ class TestRecursiveMakeBackend(BackendTester):
                 'CMMSRCS += bar.mm',
                 'CMMSRCS += foo.mm',
             ],
-            'CPP_UNIT_TESTS': [
-                'CPP_UNIT_TESTS += foo.cpp',
-            ],
             'CSRCS': [
                 'CSRCS += bar.c',
                 'CSRCS += foo.c',
             ],
-            'EXTRA_COMPONENTS': [
-                'EXTRA_COMPONENTS += bar.js',
-                'EXTRA_COMPONENTS += foo.js',
-            ],
-            'EXTRA_PP_COMPONENTS': [
-                'EXTRA_PP_COMPONENTS += bar.pp.js',
-                'EXTRA_PP_COMPONENTS += foo.pp.js',
-            ],
-            'EXTRA_JS_MODULES': [
-                'EXTRA_JS_MODULES += bar.jsm',
-                'EXTRA_JS_MODULES += foo.jsm',
-            ],
-            'EXTRA_PP_JS_MODULES': [
-                'EXTRA_PP_JS_MODULES += bar.pp.jsm',
-                'EXTRA_PP_JS_MODULES += foo.pp.jsm',
-            ],
-            'FAIL_ON_WARNINGS': [
-                'FAIL_ON_WARNINGS := 1',
-            ],
             'HOST_CPPSRCS': [
                 'HOST_CPPSRCS += bar.cpp',
                 'HOST_CPPSRCS += foo.cpp',
@@ -309,35 +347,13 @@ class TestRecursiveMakeBackend(BackendTester):
                 'HOST_CSRCS += bar.c',
                 'HOST_CSRCS += foo.c',
             ],
-            'HOST_LIBRARY_NAME': [
-                'HOST_LIBRARY_NAME := host_bar',
-            ],
-            'LIBXUL_LIBRARY': [
-                'LIBXUL_LIBRARY := 1',
-            ],
-            'MSVC_ENABLE_PGO': [
-                'MSVC_ENABLE_PGO := 1',
-            ],
-            'OS_LIBS': [
-                'OS_LIBS += foo.so',
-                'OS_LIBS += -l123',
-                'OS_LIBS += bar.a',
-            ],
-            'SDK_LIBRARY': [
-                'SDK_LIBRARY += bar.sdk',
-                'SDK_LIBRARY += foo.sdk',
-            ],
             'SSRCS': [
                 'SSRCS += baz.S',
                 'SSRCS += foo.S',
             ],
-            'VISIBILITY_FLAGS': [
-                'VISIBILITY_FLAGS :=',
-            ],
         }
 
         for var, val in expected.items():
-            # print("test_variable_passthru[%s]" % (var))
             found = [str for str in lines if str.startswith(var)]
             self.assertEqual(found, val)
 
@@ -346,20 +362,36 @@ class TestRecursiveMakeBackend(BackendTester):
         env = self._consume('exports', RecursiveMakeBackend)
 
         # EXPORTS files should appear in the dist_include install manifest.
-        m = InstallManifest(path=os.path.join(env.topobjdir,
+        m = InstallManifest(path=mozpath.join(env.topobjdir,
             '_build_manifests', 'install', 'dist_include'))
         self.assertEqual(len(m), 7)
         self.assertIn('foo.h', m)
         self.assertIn('mozilla/mozilla1.h', m)
         self.assertIn('mozilla/dom/dom2.h', m)
 
+    def test_resources(self):
+        """Ensure RESOURCE_FILES is handled properly."""
+        env = self._consume('resources', RecursiveMakeBackend)
+
+        # RESOURCE_FILES should appear in the dist_bin install manifest.
+        m = InstallManifest(path=os.path.join(env.topobjdir,
+            '_build_manifests', 'install', 'dist_bin'))
+        self.assertEqual(len(m), 10)
+        self.assertIn('res/foo.res', m)
+        self.assertIn('res/fonts/font1.ttf', m)
+        self.assertIn('res/fonts/desktop/desktop2.ttf', m)
+
+        self.assertIn('res/bar.res', m)
+        self.assertIn('res/tests/test.manifest', m)
+        self.assertIn('res/tests/extra.manifest', m)
+
     def test_test_manifests_files_written(self):
         """Ensure test manifests get turned into files."""
         env = self._consume('test-manifests-written', RecursiveMakeBackend)
 
-        tests_dir = os.path.join(env.topobjdir, '_tests')
-        m_master = os.path.join(tests_dir, 'testing', 'mochitest', 'tests', 'mochitest.ini')
-        x_master = os.path.join(tests_dir, 'xpcshell', 'xpcshell.ini')
+        tests_dir = mozpath.join(env.topobjdir, '_tests')
+        m_master = mozpath.join(tests_dir, 'testing', 'mochitest', 'tests', 'mochitest.ini')
+        x_master = mozpath.join(tests_dir, 'xpcshell', 'xpcshell.ini')
         self.assertTrue(os.path.exists(m_master))
         self.assertTrue(os.path.exists(x_master))
 
@@ -371,7 +403,7 @@ class TestRecursiveMakeBackend(BackendTester):
             '[include:xpcshell.ini]',
         ])
 
-        all_tests_path = os.path.join(env.topobjdir, 'all-tests.json')
+        all_tests_path = mozpath.join(env.topobjdir, 'all-tests.json')
         self.assertTrue(os.path.exists(all_tests_path))
 
         with open(all_tests_path, 'rt') as fh:
@@ -382,39 +414,53 @@ class TestRecursiveMakeBackend(BackendTester):
 
             self.assertEqual(len(o['xpcshell.js']), 1)
 
+    def test_test_manifest_pattern_matches_recorded(self):
+        """Pattern matches in test manifests' support-files should be recorded."""
+        env = self._consume('test-manifests-written', RecursiveMakeBackend)
+        m = InstallManifest(path=mozpath.join(env.topobjdir,
+            '_build_manifests', 'install', 'tests'))
+
+        # This is not the most robust test in the world, but it gets the job
+        # done.
+        entries = [e for e in m._dests.keys() if '**' in e]
+        self.assertEqual(len(entries), 1)
+        self.assertIn('support/**', entries[0])
+
     def test_xpidl_generation(self):
         """Ensure xpidl files and directories are written out."""
         env = self._consume('xpidl', RecursiveMakeBackend)
 
         # Install manifests should contain entries.
-        install_dir = os.path.join(env.topobjdir, '_build_manifests',
+        install_dir = mozpath.join(env.topobjdir, '_build_manifests',
             'install')
-        self.assertTrue(os.path.isfile(os.path.join(install_dir, 'dist_idl')))
-        self.assertTrue(os.path.isfile(os.path.join(install_dir, 'xpidl')))
+        self.assertTrue(os.path.isfile(mozpath.join(install_dir, 'dist_idl')))
+        self.assertTrue(os.path.isfile(mozpath.join(install_dir, 'xpidl')))
 
-        m = InstallManifest(path=os.path.join(install_dir, 'dist_idl'))
+        m = InstallManifest(path=mozpath.join(install_dir, 'dist_idl'))
         self.assertEqual(len(m), 2)
         self.assertIn('bar.idl', m)
         self.assertIn('foo.idl', m)
 
-        m = InstallManifest(path=os.path.join(install_dir, 'xpidl'))
+        m = InstallManifest(path=mozpath.join(install_dir, 'xpidl'))
         self.assertIn('.deps/my_module.pp', m)
-        self.assertIn('xpt/my_module.xpt', m)
 
-        m = InstallManifest(path=os.path.join(install_dir, 'dist_include'))
+        m = InstallManifest(path=os.path.join(install_dir, 'dist_bin'))
+        self.assertIn('components/my_module.xpt', m)
+
+        m = InstallManifest(path=mozpath.join(install_dir, 'dist_include'))
         self.assertIn('foo.h', m)
 
-        p = os.path.join(env.topobjdir, 'config/makefiles/xpidl')
+        p = mozpath.join(env.topobjdir, 'config/makefiles/xpidl')
         self.assertTrue(os.path.isdir(p))
 
-        self.assertTrue(os.path.isfile(os.path.join(p, 'Makefile')))
+        self.assertTrue(os.path.isfile(mozpath.join(p, 'Makefile')))
 
     def test_old_install_manifest_deleted(self):
         # Simulate an install manifest from a previous backend version. Ensure
         # it is deleted.
         env = self._get_environment('stub0')
-        purge_dir = os.path.join(env.topobjdir, '_build_manifests', 'install')
-        manifest_path = os.path.join(purge_dir, 'old_manifest')
+        purge_dir = mozpath.join(env.topobjdir, '_build_manifests', 'install')
+        manifest_path = mozpath.join(purge_dir, 'old_manifest')
         os.makedirs(purge_dir)
         m = InstallManifest()
         m.write(path=manifest_path)
@@ -432,12 +478,12 @@ class TestRecursiveMakeBackend(BackendTester):
         m.add_symlink(__file__, 'self')
         backend.consume(objs)
 
-        man_dir = os.path.join(env.topobjdir, '_build_manifests', 'install')
+        man_dir = mozpath.join(env.topobjdir, '_build_manifests', 'install')
         self.assertTrue(os.path.isdir(man_dir))
 
         expected = ['testing']
         for e in expected:
-            full = os.path.join(man_dir, e)
+            full = mozpath.join(man_dir, e)
             self.assertTrue(os.path.exists(full))
 
             m2 = InstallManifest(path=full)
@@ -447,7 +493,7 @@ class TestRecursiveMakeBackend(BackendTester):
         """Test that IPDL_SOURCES are written to ipdlsrcs.mk correctly."""
         env = self._consume('ipdl_sources', RecursiveMakeBackend)
 
-        manifest_path = os.path.join(env.topobjdir,
+        manifest_path = mozpath.join(env.topobjdir,
             'ipc', 'ipdl', 'ipdlsrcs.mk')
         lines = [l.strip() for l in open(manifest_path, 'rt').readlines()]
 
@@ -469,20 +515,20 @@ class TestRecursiveMakeBackend(BackendTester):
         """Test that DEFINES are written to backend.mk correctly."""
         env = self._consume('defines', RecursiveMakeBackend)
 
-        backend_path = os.path.join(env.topobjdir, 'backend.mk')
+        backend_path = mozpath.join(env.topobjdir, 'backend.mk')
         lines = [l.strip() for l in open(backend_path, 'rt').readlines()[2:]]
 
         var = 'DEFINES'
         defines = [val for val in lines if val.startswith(var)]
 
-        expected = ['DEFINES += -DFOO -DBAZ=\'"ab\'\\\'\'cd"\' -DBAR=7 -DVALUE=\'xyz\'']
+        expected = ['DEFINES += -DFOO -DBAZ=\'"ab\'\\\'\'cd"\' -UQUX -DBAR=7 -DVALUE=\'xyz\'']
         self.assertEqual(defines, expected)
 
     def test_local_includes(self):
         """Test that LOCAL_INCLUDES are written to backend.mk correctly."""
         env = self._consume('local_includes', RecursiveMakeBackend)
 
-        backend_path = os.path.join(env.topobjdir, 'backend.mk')
+        backend_path = mozpath.join(env.topobjdir, 'backend.mk')
         lines = [l.strip() for l in open(backend_path, 'rt').readlines()[2:]]
 
         expected = [
@@ -497,7 +543,7 @@ class TestRecursiveMakeBackend(BackendTester):
         """Test that GENERATED_INCLUDES are written to backend.mk correctly."""
         env = self._consume('generated_includes', RecursiveMakeBackend)
 
-        backend_path = os.path.join(env.topobjdir, 'backend.mk')
+        backend_path = mozpath.join(env.topobjdir, 'backend.mk')
         lines = [l.strip() for l in open(backend_path, 'rt').readlines()[2:]]
 
         topobjdir = env.topobjdir.replace('\\', '/')
@@ -515,33 +561,178 @@ class TestRecursiveMakeBackend(BackendTester):
         env = self._consume('final_target', RecursiveMakeBackend)
 
         final_target_rule = "FINAL_TARGET = $(if $(XPI_NAME),$(DIST)/xpi-stage/$(XPI_NAME),$(DIST)/bin)$(DIST_SUBDIR:%=/%)"
-        print([x for x in os.walk(env.topobjdir)])
         expected = dict()
         expected[env.topobjdir] = []
-        expected[os.path.join(env.topobjdir, 'both')] = [
+        expected[mozpath.join(env.topobjdir, 'both')] = [
             'XPI_NAME = mycrazyxpi',
             'DIST_SUBDIR = asubdir',
             final_target_rule
         ]
-        expected[os.path.join(env.topobjdir, 'dist-subdir')] = [
+        expected[mozpath.join(env.topobjdir, 'dist-subdir')] = [
             'DIST_SUBDIR = asubdir',
             final_target_rule
         ]
-        expected[os.path.join(env.topobjdir, 'xpi-name')] = [
+        expected[mozpath.join(env.topobjdir, 'xpi-name')] = [
             'XPI_NAME = mycrazyxpi',
             final_target_rule
         ]
-        expected[os.path.join(env.topobjdir, 'final-target')] = [
+        expected[mozpath.join(env.topobjdir, 'final-target')] = [
             'FINAL_TARGET = $(DEPTH)/random-final-target'
         ]
         for key, expected_rules in expected.iteritems():
-            backend_path = os.path.join(key, 'backend.mk')
+            backend_path = mozpath.join(key, 'backend.mk')
             lines = [l.strip() for l in open(backend_path, 'rt').readlines()[2:]]
             found = [str for str in lines if
                 str.startswith('FINAL_TARGET') or str.startswith('XPI_NAME') or
                 str.startswith('DIST_SUBDIR')]
             self.assertEqual(found, expected_rules)
 
+    def test_config(self):
+        """Test that CONFIGURE_SUBST_FILES and CONFIGURE_DEFINE_FILES are
+        properly handled."""
+        env = self._consume('test_config', RecursiveMakeBackend)
+
+        self.assertEqual(
+            open(os.path.join(env.topobjdir, 'file'), 'r').readlines(), [
+                '#ifdef foo\n',
+                'bar baz\n',
+                '@bar@\n',
+            ])
+
+        self.assertEqual(
+            open(os.path.join(env.topobjdir, 'file.h'), 'r').readlines(), [
+                '/* Comment */\n',
+                '#define foo\n',
+                '#define foo baz qux\n',
+                '#define foo baz qux\n',
+                '#define bar\n',
+                '#define bar 42\n',
+                '/* #undef bar */\n',
+                '\n',
+                '# define baz 1\n',
+                '\n',
+                '#ifdef foo\n',
+                '#   define   foo baz qux\n',
+                '#  define foo    baz qux\n',
+                '  #     define   foo   baz qux   \n',
+                '#endif\n',
+            ])
+
+    def test_jar_manifests(self):
+        env = self._consume('jar-manifests', RecursiveMakeBackend)
+
+        with open(os.path.join(env.topobjdir, 'backend.mk'), 'rb') as fh:
+            lines = fh.readlines()
+
+        lines = [line.rstrip() for line in lines]
+
+        self.assertIn('JAR_MANIFEST := %s/jar.mn' % env.topsrcdir, lines)
+
+    def test_extra_js_modules(self):
+        env = self._consume('extra-js-modules', RecursiveMakeBackend)
+
+        with open(os.path.join(env.topobjdir, 'backend.mk'), 'rb') as fh:
+            lines = fh.readlines()
+
+        lines = [line.rstrip() for line in lines]
+        self.maxDiff = None
+        expected = [
+            'extra_js__FILES := module1.js module2.js',
+            'extra_js__DEST = $(FINAL_TARGET)/modules/',
+            'extra_js__TARGET := misc',
+            'INSTALL_TARGETS += extra_js_',
+            'extra_js_submodule_FILES := module3.js module4.js',
+            'extra_js_submodule_DEST = $(FINAL_TARGET)/modules/submodule',
+            'extra_js_submodule_TARGET := misc',
+            'INSTALL_TARGETS += extra_js_submodule',
+            'extra_pp_js_ := pp-module1.js',
+            'extra_pp_js__PATH = $(FINAL_TARGET)/modules/',
+            'extra_pp_js__TARGET := misc',
+            'PP_TARGETS += extra_pp_js_',
+            'extra_pp_js_ppsub := pp-module2.js',
+            'extra_pp_js_ppsub_PATH = $(FINAL_TARGET)/modules/ppsub',
+            'extra_pp_js_ppsub_TARGET := misc',
+            'PP_TARGETS += extra_pp_js_ppsub',
+        ]
+
+        found = [line for line in lines if line.startswith(('extra_',
+                                                            'INSTALL_TARGETS',
+                                                            'PP_TARGETS'))]
+        self.assertEqual(expected, found)
+
+    def test_test_manifests_duplicate_support_files(self):
+        """Ensure duplicate support-files in test manifests work."""
+        env = self._consume('test-manifests-duplicate-support-files',
+            RecursiveMakeBackend)
+
+        p = os.path.join(env.topobjdir, '_build_manifests', 'install', 'tests')
+        m = InstallManifest(p)
+        self.assertIn('testing/mochitest/tests/support-file.txt', m)
+
+    def test_android_eclipse(self):
+        env = self._consume('android_eclipse', RecursiveMakeBackend)
+
+        with open(mozpath.join(env.topobjdir, 'backend.mk'), 'rb') as fh:
+            lines = fh.readlines()
+
+        lines = [line.rstrip() for line in lines]
+
+        # Dependencies first.
+        self.assertIn('ANDROID_ECLIPSE_PROJECT_main1: target1 target2', lines)
+        self.assertIn('ANDROID_ECLIPSE_PROJECT_main4: target3 target4', lines)
+
+        command_template = '\t$(call py_action,process_install_manifest,' + \
+                           '--no-remove --no-remove-all-directory-symlinks ' + \
+                           '--no-remove-empty-directories %s %s.manifest)'
+        # Commands second.
+        for project_name in ['main1', 'main2', 'library1', 'library2']:
+            stem = '%s/android_eclipse/%s' % (env.topobjdir, project_name)
+            self.assertIn(command_template % (stem, stem), lines)
+
+        # Projects declared in subdirectories.
+        with open(mozpath.join(env.topobjdir, 'subdir', 'backend.mk'), 'rb') as fh:
+            lines = fh.readlines()
+
+        lines = [line.rstrip() for line in lines]
+
+        self.assertIn('ANDROID_ECLIPSE_PROJECT_submain: subtarget1 subtarget2', lines)
+
+        for project_name in ['submain', 'sublibrary']:
+            # Destination and install manifest are relative to topobjdir.
+            stem = '%s/android_eclipse/%s' % (env.topobjdir, project_name)
+            self.assertIn(command_template % (stem, stem), lines)
+
+    def test_install_manifests_package_tests(self):
+        """Ensure test suites honor package_tests=False."""
+        env = self._consume('test-manifests-package-tests', RecursiveMakeBackend)
+
+        tests_dir = mozpath.join(env.topobjdir, '_tests')
+
+        all_tests_path = mozpath.join(env.topobjdir, 'all-tests.json')
+        self.assertTrue(os.path.exists(all_tests_path))
+
+        with open(all_tests_path, 'rt') as fh:
+            o = json.load(fh)
+
+            self.assertIn('mochitest.js', o)
+            self.assertIn('not_packaged.java', o)
+
+        man_dir = mozpath.join(env.topobjdir, '_build_manifests', 'install')
+        self.assertTrue(os.path.isdir(man_dir))
+
+        full = mozpath.join(man_dir, 'tests')
+        self.assertTrue(os.path.exists(full))
+
+        m = InstallManifest(path=full)
+
+        # Only mochitest.js should be in the install manifest.
+        self.assertTrue('testing/mochitest/tests/mochitest.js' in m)
+
+        # The path is odd here because we do not normalize at test manifest
+        # processing time.  This is a fragile test because there's currently no
+        # way to iterate the manifest.
+        self.assertFalse('instrumentation/./not_packaged.java' in m)
+
 
 if __name__ == '__main__':
     main()
