@@ -2,9 +2,10 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-import time
-from marionette_test import MarionetteTestCase
 from by import By
+from errors import NoSuchElementException, ElementNotVisibleException
+from marionette_test import MarionetteTestCase
+from wait import Wait
 
 
 class TestClick(MarionetteTestCase):
@@ -15,15 +16,17 @@ class TestClick(MarionetteTestCase):
         link.click()
         self.assertEqual("Clicked", self.marionette.execute_script("return document.getElementById('mozLink').innerHTML;"))
 
-    def testClickingALinkMadeUpOfNumbersIsHandledCorrectly(self):
+    def test_clicking_a_link_made_up_of_numbers_is_handled_correctly(self):
         test_html = self.marionette.absolute_url("clicks.html")
         self.marionette.navigate(test_html)
         self.marionette.find_element(By.LINK_TEXT, "333333").click()
-        count = 0
-        while len(self.marionette.find_elements(By.ID, "username")) == 0:
-            count += 1
-            time.sleep(1)
-            if count == 30:
-                self.fail("Element id=username not found after 30 seconds")
-
+        Wait(self.marionette, timeout=30, ignored_exceptions=NoSuchElementException).until(
+            lambda m: m.find_element(By.ID, 'username'))
         self.assertEqual(self.marionette.title, "XHTML Test Page")
+
+    def test_clicking_an_element_that_is_not_displayed_raises(self):
+        test_html = self.marionette.absolute_url('hidden.html')
+        self.marionette.navigate(test_html)
+
+        with self.assertRaises(ElementNotVisibleException):
+            self.marionette.find_element(By.ID, 'child').click()
