@@ -1,26 +1,17 @@
 import os
 from setuptools import setup, find_packages
+import sys
 
-version = '0.7.1'
-
-# get documentation from the README
-try:
-    here = os.path.dirname(os.path.abspath(__file__))
-    description = file(os.path.join(here, 'README.md')).read()
-except (OSError, IOError):
-    description = ''
+version = '0.8.6'
 
 # dependencies
-deps = ['manifestdestiny', 'mozhttpd >= 0.5',
-        'mozprocess >= 0.9', 'mozrunner >= 5.15',
-        'mozdevice >= 0.22', 'moznetwork >= 0.21',
-        'mozcrash >= 0.5', 'mozprofile >= 0.7',
-        'moztest >= 0.1', 'py==1.4.14']
+with open('requirements.txt') as f:
+    deps = f.read().splitlines()
 
 setup(name='marionette_client',
       version=version,
       description="Marionette test automation client",
-      long_description=description,
+      long_description='See http://marionette-client.readthedocs.org/',
       classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
       keywords='mozilla',
       author='Jonathan Griffin',
