@@ -1,4 +1,4 @@
-/* -*- Mode: javascript; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -7,11 +7,12 @@
  * The ElementManager manages DOM references and interactions with elements.
  * According to the WebDriver spec (http://code.google.com/p/selenium/wiki/JsonWireProtocol), the
  * server sends the client an element reference, and maintains the map of reference to element.
- * The client uses this reference when querying/interacting with the element, and the 
+ * The client uses this reference when querying/interacting with the element, and the
  * server uses maps this reference to the actual element when it executes the command.
  */
 
 this.EXPORTED_SYMBOLS = [
+  "Accessibility",
   "ElementManager",
   "CLASS_NAME",
   "SELECTOR",
@@ -20,7 +21,9 @@ this.EXPORTED_SYMBOLS = [
   "LINK_TEXT",
   "PARTIAL_LINK_TEXT",
   "TAG",
-  "XPATH"
+  "XPATH",
+  "ANON",
+  "ANON_ATTRIBUTE"
 ];
 
 const DOCUMENT_POSITION_DISCONNECTED = 1;
@@ -36,6 +39,8 @@ this.LINK_TEXT = "link text";
 this.PARTIAL_LINK_TEXT = "partial link text";
 this.TAG = "tag name";
 this.XPATH = "xpath";
+this.ANON= "anon";
+this.ANON_ATTRIBUTE = "anon attribute";
 
 function ElementException(msg, num, stack) {
   this.message = msg;
@@ -43,10 +48,153 @@ function ElementException(msg, num, stack) {
   this.stack = stack;
 }
 
+this.Accessibility = function Accessibility() {
+  // A flag indicating whether the accessibility issue should be logged or cause
+  // an exception. Default: log to stdout.
+  this.strict = false;
+  // An interface for in-process accessibility clients
+  // Note: we access it lazily to not enable accessibility when it is not needed
+  Object.defineProperty(this, 'accessibleRetrieval', {
+    configurable: true,
+    get: function() {
+      delete this.accessibleRetrieval;
+      this.accessibleRetrieval = Components.classes[
+        '@mozilla.org/accessibleRetrieval;1'].getService(
+          Components.interfaces.nsIAccessibleRetrieval);
+      return this.accessibleRetrieval;
+    }
+  });
+};
+
+Accessibility.prototype = {
+  /**
+   * Accessible object roles that support some action
+   * @type Object
+   */
+  actionableRoles: new Set([
+    'pushbutton',
+    'checkbutton',
+    'combobox',
+    'key',
+    'link',
+    'menuitem',
+    'check menu item',
+    'radio menu item',
+    'option',
+    'radiobutton',
+    'slider',
+    'spinbutton',
+    'pagetab',
+    'entry',
+    'outlineitem'
+  ]),
+
+  /**
+   * Get an accessible object for a DOM element
+   * @param nsIDOMElement element
+   * @param Boolean mustHaveAccessible a flag indicating that the element must
+   * have an accessible object
+   * @return nsIAccessible object for the element
+   */
+  getAccessibleObject(element, mustHaveAccessible = false) {
+    let acc = this.accessibleRetrieval.getAccessibleFor(element);
+    if (!acc && mustHaveAccessible) {
+      this.handleErrorMessage('Element does not have an accessible object');
+    }
+    return acc;
+  },
+
+  /**
+   * Check if the accessible has a role that supports some action
+   * @param nsIAccessible object
+   * @return Boolean an indicator of role being actionable
+   */
+  isActionableRole(accessible) {
+    return this.actionableRoles.has(
+      this.accessibleRetrieval.getStringRole(accessible.role));
+  },
+
+  /**
+   * Determine if an accessible has at least one action that it supports
+   * @param nsIAccessible object
+   * @return Boolean an indicator of supporting at least one accessible action
+   */
+  hasActionCount(accessible) {
+    return accessible.actionCount > 0;
+  },
+
+  /**
+   * Determine if an accessible has a valid name
+   * @param nsIAccessible object
+   * @return Boolean an indicator that the element has a non empty valid name
+   */
+  hasValidName(accessible) {
+    return accessible.name && accessible.name.trim();
+  },
+
+  /**
+   * Check if an accessible has a set hidden attribute
+   * @param nsIAccessible object
+   * @return Boolean an indicator that the element has a hidden accessible
+   * attribute set to true
+   */
+  hasHiddenAttribute(accessible) {
+    let hidden;
+    try {
+      hidden = accessible.attributes.getStringProperty('hidden');
+    } finally {
+      // If the property is missing, exception will be thrown.
+      return hidden && hidden === 'true';
+    }
+  },
+
+  /**
+   * Verify if an accessible has a given state
+   * @param nsIAccessible object
+   * @param String stateName name of the state to match
+   * @return Boolean accessible has a state
+   */
+  matchState(accessible, stateName) {
+    let stateToMatch = Components.interfaces.nsIAccessibleStates[stateName];
+    let state = {};
+    accessible.getState(state, {});
+    return !!(state.value & stateToMatch);
+  },
+
+  /**
+   * Check if an accessible is hidden from the user of the accessibility API
+   * @param nsIAccessible object
+   * @return Boolean an indicator that the element is hidden from the user
+   */
+  isHidden(accessible) {
+    while (accessible) {
+      if (this.hasHiddenAttribute(accessible)) {
+        return true;
+      }
+      accessible = accessible.parent;
+    }
+    return false;
+  },
+
+  /**
+   * Send an error message or log the error message in the log
+   * @param String message
+   */
+  handleErrorMessage(message) {
+    if (!message) {
+      return;
+    }
+    if (this.strict) {
+      throw new ElementException(message, 56, null);
+    }
+    dump(Date.now() + " Marionette: " + message);
+  }
+};
+
 this.ElementManager = function ElementManager(notSupported) {
   this.seenItems = {};
   this.timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
-  this.elementStrategies = [CLASS_NAME, SELECTOR, ID, NAME, LINK_TEXT, PARTIAL_LINK_TEXT, TAG, XPATH];
+  this.elementStrategies = [CLASS_NAME, SELECTOR, ID, NAME, LINK_TEXT, PARTIAL_LINK_TEXT, TAG, XPATH, ANON, ANON_ATTRIBUTE];
   for (let i = 0; i < notSupported.length; i++) {
     this.elementStrategies.splice(this.elementStrategies.indexOf(notSupported[i]), 1);
   }
@@ -90,7 +238,7 @@ ElementManager.prototype = {
     this.seenItems[id] = Components.utils.getWeakReference(element);
     return id;
   },
-  
+
   /**
    * Retrieve element from its unique ID
    *
@@ -120,36 +268,47 @@ ElementManager.prototype = {
         !(XPCNativeWrapper(el).ownerDocument == wrappedWin.document) ||
         (XPCNativeWrapper(el).compareDocumentPosition(wrappedWin.document.documentElement) &
          DOCUMENT_POSITION_DISCONNECTED)) {
-      throw new ElementException("Stale element reference", 10, null);
+      throw new ElementException("The element reference is stale. Either the element " +
+                                 "is no longer attached to the DOM or the page has been refreshed.", 10, null);
     }
     return el;
   },
-  
+
   /**
-   * Convert values to primitives that can be transported over the Marionette
-   * JSON protocol.
-   * 
+   * Convert values to primitives that can be transported over the
+   * Marionette protocol.
+   *
+   * This function implements the marshaling algorithm defined in the
+   * WebDriver specification:
+   *
+   *     https://dvcs.w3.org/hg/webdriver/raw-file/tip/webdriver-spec.html#synchronous-javascript-execution
+   *
    * @param object val
-   *        object to be wrapped
+   *        object to be marshaled
    *
    * @return object
-   *        Returns a JSON primitive or Object
+   *         Returns a JSON primitive or Object
    */
   wrapValue: function EM_wrapValue(val) {
-    let result;
-    switch(typeof(val)) {
+    let result = null;
+
+    switch (typeof(val)) {
       case "undefined":
         result = null;
         break;
+
       case "string":
       case "number":
       case "boolean":
         result = val;
         break;
+
       case "object":
-        if (Object.prototype.toString.call(val) == '[object Array]') {
+        let type = Object.prototype.toString.call(val);
+        if (type == "[object Array]" ||
+            type == "[object NodeList]") {
           result = [];
-          for (let i in val) {
+          for (let i = 0; i < val.length; ++i) {
             result.push(this.wrapValue(val[i]));
           }
         }
@@ -167,9 +326,10 @@ ElementManager.prototype = {
         }
         break;
     }
+
     return result;
   },
-  
+
   /**
    * Convert any ELEMENT references in 'args' to the actual elements
    *
@@ -216,11 +376,11 @@ ElementManager.prototype = {
     }
     return converted;
   },
-  
+
   /*
    * Execute* helpers
    */
-  
+
   /**
    * Return an object with any namedArgs applied to it. Used
    * to let clients use given names when refering to arguments
@@ -245,7 +405,7 @@ ElementManager.prototype = {
     });
     return namedArgs;
   },
-  
+
   /**
    * Find an element or elements starting at the document root or
    * given node, using the given search strategy. Search
@@ -277,27 +437,26 @@ ElementManager.prototype = {
     let startNode = (values.element != undefined) ?
                     this.getKnownElement(values.element, win) : win.document;
     if (this.elementStrategies.indexOf(values.using) < 0) {
-      throw new ElementException("No such strategy.", 17, null);
+      throw new ElementException("No such strategy.", 32, null);
     }
     let found = all ? this.findElements(values.using, values.value, win.document, startNode) :
                       this.findElement(values.using, values.value, win.document, startNode);
-    if (found) {
-      let type = Object.prototype.toString.call(found);
-      if ((type == '[object Array]') || (type == '[object HTMLCollection]') || (type == '[object NodeList]')) {
-        let ids = []
-        for (let i = 0 ; i < found.length ; i++) {
-          ids.push(this.addToKnownElements(found[i]));
-        }
-        on_success(ids, command_id);
-      }
-      else {
-        let id = this.addToKnownElements(found);
-        on_success({'ELEMENT':id}, command_id);
-      }
-      return;
-    } else {
+    let type = Object.prototype.toString.call(found);
+    let isArrayLike = ((type == '[object Array]') || (type == '[object HTMLCollection]') || (type == '[object NodeList]'));
+    if (found == null || (isArrayLike && found.length <= 0)) {
       if (!searchTimeout || new Date().getTime() - startTime > searchTimeout) {
-        on_error("Unable to locate element: " + values.value, 7, null, command_id);
+        if (all) {
+          on_success([], command_id); // findElements should return empty list
+        } else {
+          // Format message depending on strategy if necessary
+          let message = "Unable to locate element: " + values.value;
+          if (values.using == ANON) {
+            message = "Unable to locate anonymous children";
+          } else if (values.using == ANON_ATTRIBUTE) {
+            message = "Unable to locate anonymous element: " + JSON.stringify(values.value);
+          }
+          on_error(message, 7, null, command_id);
+        }
       } else {
         values.time = startTime;
         this.timer.initWithCallback(this.find.bind(this, win, values,
@@ -307,12 +466,24 @@ ElementManager.prototype = {
                                     100,
                                     Components.interfaces.nsITimer.TYPE_ONE_SHOT);
       }
+    } else {
+      if (isArrayLike) {
+        let ids = []
+        for (let i = 0 ; i < found.length ; i++) {
+          ids.push(this.addToKnownElements(found[i]));
+        }
+        on_success(ids, command_id);
+      } else {
+        let id = this.addToKnownElements(found);
+        on_success({'ELEMENT':id}, command_id);
+      }
+      return;
     }
   },
 
   /**
    * Find a value by XPATH
-   * 
+   *
    * @param nsIDOMElement root
    *        Document root
    * @param string value
@@ -330,7 +501,7 @@ ElementManager.prototype = {
 
   /**
    * Find values by XPATH
-   * 
+   *
    * @param nsIDOMElement root
    *        Document root
    * @param string value
@@ -352,10 +523,10 @@ ElementManager.prototype = {
     }
     return elements;
   },
-  
+
   /**
    * Helper method to find. Finds one element using find's criteria
-   * 
+   *
    * @param string using
    *        String identifying which search method to use
    * @param string value
@@ -373,12 +544,12 @@ ElementManager.prototype = {
     switch (using) {
       case ID:
         element = startNode.getElementById ?
-                  startNode.getElementById(value) : 
+                  startNode.getElementById(value) :
                   this.findByXPath(rootNode, './/*[@id="' + value + '"]', startNode);
         break;
       case NAME:
         element = startNode.getElementsByName ?
-                  startNode.getElementsByName(value)[0] : 
+                  startNode.getElementsByName(value)[0] :
                   this.findByXPath(rootNode, './/*[@name="' + value + '"]', startNode);
         break;
       case CLASS_NAME:
@@ -407,6 +578,16 @@ ElementManager.prototype = {
       case SELECTOR:
         element = startNode.querySelector(value);
         break;
+      case ANON:
+        element = rootNode.getAnonymousNodes(startNode);
+        if (element != null) {
+          element = element[0];
+        }
+        break;
+      case ANON_ATTRIBUTE:
+        let attr = Object.keys(value)[0];
+        element = rootNode.getAnonymousElementByAttribute(startNode, attr, value[attr]);
+        break;
       default:
         throw new ElementException("No such strategy", 500, null);
     }
@@ -415,7 +596,7 @@ ElementManager.prototype = {
 
   /**
    * Helper method to find. Finds all element using find's criteria
-   * 
+   *
    * @param string using
    *        String identifying which search method to use
    * @param string value
@@ -438,7 +619,7 @@ ElementManager.prototype = {
         break;
       case NAME:
         elements = startNode.getElementsByName ?
-                   startNode.getElementsByName(value) : 
+                   startNode.getElementsByName(value) :
                    this.findByXPathAll(rootNode, './/*[@name="' + value + '"]', startNode);
         break;
       case CLASS_NAME:
@@ -464,6 +645,16 @@ ElementManager.prototype = {
       case SELECTOR:
         elements = Array.slice(startNode.querySelectorAll(value));
         break;
+      case ANON:
+        elements = rootNode.getAnonymousNodes(startNode) || [];
+        break;
+      case ANON_ATTRIBUTE:
+        let attr = Object.keys(value)[0];
+        let el = rootNode.getAnonymousElementByAttribute(startNode, attr, value[attr]);
+        if (el != null) {
+          elements = [el];
+        }
+        break;
       default:
         throw new ElementException("No such strategy", 500, null);
     }
