@@ -13,10 +13,16 @@ const Ci = Components.interfaces;
 const Cu = Components.utils;
 const Cr = Components.results;
 
-Cu.import("resource://testing-common/httpd.js");
+Cu.import("resource://testing-common/httpd.js", this);
 Cu.import("resource://gre/modules/Services.jsm");
-Cu.import("resource://gre/modules/LightweightThemeManager.jsm");
-Cu.import("resource://gre/modules/XPCOMUtils.jsm");
+Cu.import("resource://gre/modules/LightweightThemeManager.jsm", this);
+Cu.import("resource://gre/modules/XPCOMUtils.jsm", this);
+Cu.import("resource://gre/modules/TelemetryPing.jsm", this);
+Cu.import("resource://gre/modules/TelemetryFile.jsm", this);
+Cu.import("resource://gre/modules/Task.jsm", this);
+Cu.import("resource://gre/modules/Promise.jsm", this);
+Cu.import("resource://gre/modules/Preferences.jsm");
+Cu.import("resource://gre/modules/osfile.jsm", this);
 
 const IGNORE_HISTOGRAM = "test::ignore_me";
 const IGNORE_HISTOGRAM_TO_CLONE = "MEMORY_HEAP_ALLOCATED";
@@ -35,36 +41,42 @@ const PR_TRUNCATE = 0x20;
 const RW_OWNER = 0600;
 
 const NUMBER_OF_THREADS_TO_LAUNCH = 30;
-var gNumberOfThreadsLaunched = 0;
+let gNumberOfThreadsLaunched = 0;
+
+const PREF_BRANCH = "toolkit.telemetry.";
+const PREF_ENABLED = PREF_BRANCH + "enabled";
+const PREF_FHR_UPLOAD_ENABLED = "datareporting.healthreport.uploadEnabled";
+const PREF_FHR_SERVICE_ENABLED = "datareporting.healthreport.service.enabled";
+
+const HAS_DATAREPORTINGSERVICE = "@mozilla.org/datareporting/service;1" in Cc;
+const SESSION_RECORDER_EXPECTED = HAS_DATAREPORTINGSERVICE &&
+                                  Preferences.get(PREF_FHR_SERVICE_ENABLED, true);
 
 const Telemetry = Cc["@mozilla.org/base/telemetry;1"].getService(Ci.nsITelemetry);
-const TelemetryPing = Cc["@mozilla.org/base/telemetry-ping;1"].getService(Ci.nsITelemetryPing);
 
-var httpserver = new HttpServer();
-var serverStarted = false;
-var gFinished = false;
+let gHttpServer = new HttpServer();
+let gServerStarted = false;
+let gRequestIterator = null;
+let gDataReportingClientID = null;
 
-function telemetry_ping () {
+XPCOMUtils.defineLazyGetter(this, "gDatareportingService",
+  () => Cc["@mozilla.org/datareporting/service;1"]
+          .getService(Ci.nsISupports)
+          .wrappedJSObject);
+
+function sendPing () {
   TelemetryPing.gatherStartup();
-  TelemetryPing.enableLoadSaveNotifications();
-  TelemetryPing.cacheProfileDirectory();
-  if (serverStarted) {
-    TelemetryPing.testPing("http://localhost:" + httpserver.identity.primaryPort);
+  if (gServerStarted) {
+    return TelemetryPing.testPing("http://localhost:" + gHttpServer.identity.primaryPort);
   } else {
-    TelemetryPing.testPing("http://doesnotexist");
+    return TelemetryPing.testPing("http://doesnotexist");
   }
 }
 
-// Mostly useful so that you can dump payloads from decodeRequestPayload.
-function dummyHandler(request, response) {
-  let p = decodeRequestPayload(request);
-  return p;
-}
-
 function wrapWithExceptionHandler(f) {
-  function wrapper() {
+  function wrapper(...args) {
     try {
-      f.apply(null, arguments);
+      f(...args);
     } catch (ex if typeof(ex) == 'object') {
       dump("Caught exception: " + ex.message + "\n");
       dump(ex.stack);
@@ -74,37 +86,27 @@ function wrapWithExceptionHandler(f) {
   return wrapper;
 }
 
-function addWrappedObserver(f, topic) {
-  let wrappedObserver = wrapWithExceptionHandler(f);
-  Services.obs.addObserver(function g(aSubject, aTopic, aData) {
-    Services.obs.removeObserver(g, aTopic);
-    wrappedObserver(aSubject, aTopic, aData);
-  }, topic, false);
-}
-
 function registerPingHandler(handler) {
-  httpserver.registerPrefixHandler("/submit/telemetry/",
+  gHttpServer.registerPrefixHandler("/submit/telemetry/",
 				   wrapWithExceptionHandler(handler));
 }
 
-function nonexistentServerObserver(aSubject, aTopic, aData) {
-  httpserver.start(-1);
-  serverStarted = true;
-
-  // Provide a dummy function so it returns 200 instead of 404 to telemetry.
-  registerPingHandler(dummyHandler);
-  addWrappedObserver(telemetryObserver, "telemetry-test-xhr-complete");
-  telemetry_ping();
-}
-
 function setupTestData() {
-  Telemetry.newHistogram(IGNORE_HISTOGRAM, 1, 2, 3, Telemetry.HISTOGRAM_BOOLEAN);
+  Telemetry.newHistogram(IGNORE_HISTOGRAM, "never", Telemetry.HISTOGRAM_BOOLEAN);
   Telemetry.histogramFrom(IGNORE_CLONED_HISTOGRAM, IGNORE_HISTOGRAM_TO_CLONE);
   Services.startup.interrupted = true;
-  Telemetry.registerAddonHistogram(ADDON_NAME, ADDON_HISTOGRAM, 1, 5, 6,
-                                   Telemetry.HISTOGRAM_LINEAR);
-  h1 = Telemetry.getAddonHistogram(ADDON_NAME, ADDON_HISTOGRAM);
+  Telemetry.registerAddonHistogram(ADDON_NAME, ADDON_HISTOGRAM,
+                                   Telemetry.HISTOGRAM_LINEAR,
+                                   1, 5, 6);
+  let h1 = Telemetry.getAddonHistogram(ADDON_NAME, ADDON_HISTOGRAM);
   h1.add(1);
+  let h2 = Telemetry.getHistogramById("TELEMETRY_TEST_COUNT");
+  h2.add();
+
+  let k1 = Telemetry.getKeyedHistogramById("TELEMETRY_TEST_KEYED_COUNT");
+  k1.add("a");
+  k1.add("a");
+  k1.add("b");
 }
 
 function getSavedHistogramsFile(basename) {
@@ -123,16 +125,6 @@ function getSavedHistogramsFile(basename) {
   return histogramsFile;
 }
 
-function telemetryObserver(aSubject, aTopic, aData) {
-  registerPingHandler(checkHistogramsSync);
-  let histogramsFile = getSavedHistogramsFile("saved-histograms.dat");
-  setupTestData();
-
-  TelemetryPing.saveHistograms(histogramsFile, true);
-  TelemetryPing.testLoadHistograms(histogramsFile, true);
-  telemetry_ping();
-}
-
 function decodeRequestPayload(request) {
   let s = request.bodyInputStream;
   let payload = null;
@@ -189,18 +181,30 @@ function checkPayloadInfo(payload, reason) {
   do_check_true("appUpdateChannel" in payload.info);
   do_check_true("locale" in payload.info);
   do_check_true("revision" in payload.info);
-  do_check_true(payload.info.revision.startsWith("http"));
+  if (Services.appinfo.isOfficial) {
+    do_check_true(payload.info.revision.startsWith("http"));
+  }
+
+  if ("@mozilla.org/datareporting/service;1" in Cc &&
+      Services.prefs.getBoolPref(PREF_FHR_UPLOAD_ENABLED)) {
+    do_check_true("clientID" in payload);
+    do_check_neq(payload.clientID, null);
+    do_check_eq(payload.clientID, gDataReportingClientID);
+  }
 
   try {
     // If we've not got nsIGfxInfoDebug, then this will throw and stop us doing
     // this test.
-    var gfxInfo = Cc["@mozilla.org/gfx/info;1"].getService(Ci.nsIGfxInfoDebug);
-    var isWindows = ("@mozilla.org/windows-registry-key;1" in Components.classes);
-    var isOSX = ("nsILocalFileMac" in Components.interfaces);
+    let gfxInfo = Cc["@mozilla.org/gfx/info;1"].getService(Ci.nsIGfxInfoDebug);
+    let isWindows = ("@mozilla.org/windows-registry-key;1" in Components.classes);
+    let isOSX = ("nsILocalFileMac" in Components.interfaces);
 
     if (isWindows || isOSX) {
       do_check_true("adapterVendorID" in payload.info);
       do_check_true("adapterDeviceID" in payload.info);
+      if (isWindows) {
+        do_check_true("adapterSubsysID" in payload.info);
+      }
     }
   }
   catch (x) {
@@ -222,6 +226,9 @@ function checkPayload(request, reason, successfulPings) {
   do_check_true("maximalNumberOfConcurrentThreads" in payload.simpleMeasurements);
   do_check_true(payload.simpleMeasurements.maximalNumberOfConcurrentThreads >= gNumberOfThreadsLaunched);
 
+  let activeTicks = payload.simpleMeasurements.activeTicks;
+  do_check_true(SESSION_RECORDER_EXPECTED ? activeTicks >= 0 : activeTicks == -1);
+
   do_check_eq(payload.simpleMeasurements.failedProfileLockCount,
               FAILED_PROFILE_LOCK_ATTEMPTS);
   let profileDirectory = Services.dirsvc.get("ProfD", Ci.nsIFile);
@@ -230,7 +237,7 @@ function checkPayload(request, reason, successfulPings) {
   do_check_true(!failedProfileLocksFile.exists());
 
 
-  var isWindows = ("@mozilla.org/windows-registry-key;1" in Components.classes);
+  let isWindows = ("@mozilla.org/windows-registry-key;1" in Components.classes);
   if (isWindows) {
     do_check_true(payload.simpleMeasurements.startupSessionRestoreReadBytes > 0);
     do_check_true(payload.simpleMeasurements.startupSessionRestoreWriteBytes > 0);
@@ -239,11 +246,18 @@ function checkPayload(request, reason, successfulPings) {
   const TELEMETRY_PING = "TELEMETRY_PING";
   const TELEMETRY_SUCCESS = "TELEMETRY_SUCCESS";
   const TELEMETRY_TEST_FLAG = "TELEMETRY_TEST_FLAG";
+  const TELEMETRY_TEST_COUNT = "TELEMETRY_TEST_COUNT";
+  const TELEMETRY_TEST_KEYED_FLAG = "TELEMETRY_TEST_KEYED_FLAG";
+  const TELEMETRY_TEST_KEYED_COUNT = "TELEMETRY_TEST_KEYED_COUNT";
   const READ_SAVED_PING_SUCCESS = "READ_SAVED_PING_SUCCESS";
+
   do_check_true(TELEMETRY_PING in payload.histograms);
   do_check_true(READ_SAVED_PING_SUCCESS in payload.histograms);
-  let rh = Telemetry.registeredHistograms;
-  for (let name in rh) {
+  do_check_true(TELEMETRY_TEST_FLAG in payload.histograms);
+  do_check_true(TELEMETRY_TEST_COUNT in payload.histograms);
+
+  let rh = Telemetry.registeredHistograms([]);
+  for (let name of rh) {
     if (/SQLITE/.test(name) && name in payload.histograms) {
       do_check_true(("STARTUP_" + name) in payload.histograms); 
     }
@@ -264,6 +278,19 @@ function checkPayload(request, reason, successfulPings) {
   let flag = payload.histograms[TELEMETRY_TEST_FLAG];
   do_check_eq(uneval(flag), uneval(expected_flag));
 
+  // We should have a test count.
+  const expected_count = {
+    range: [1, 2],
+    bucket_count: 3,
+    histogram_type: 4,
+    values: {0:1, 1:0},
+    sum: 1,
+    sum_squares_lo: 1,
+    sum_squares_hi: 0,
+  };
+  let count = payload.histograms[TELEMETRY_TEST_COUNT];
+  do_check_eq(uneval(count), uneval(expected_count));
+
   // There should be one successful report from the previous telemetry ping.
   const expected_tc = {
     range: [1, 2],
@@ -300,72 +327,37 @@ function checkPayload(request, reason, successfulPings) {
 
   do_check_true(("mainThread" in payload.slowSQL) &&
                 ("otherThreads" in payload.slowSQL));
-}
 
-function checkPersistedHistogramsSync(request, response) {
-  // Even though we have had two successful pings when this handler is
-  // run, we only had one successful ping when the histograms were
-  // saved.
-  checkPayload(request, "saved-session", 1);
+  // Check keyed histogram payload.
 
-  addWrappedObserver(runAsyncTestObserver, "telemetry-test-xhr-complete");
-}
+  do_check_true("keyedHistograms" in payload);
+  let keyedHistograms = payload.keyedHistograms;
+  do_check_true(TELEMETRY_TEST_KEYED_FLAG in keyedHistograms);
+  do_check_true(TELEMETRY_TEST_KEYED_COUNT in keyedHistograms);
 
-function checkHistogramsSync(request, response) {
-  registerPingHandler(checkPersistedHistogramsSync);
-  checkPayload(request, "test-ping", 1);
-}
+  Assert.deepEqual({}, keyedHistograms[TELEMETRY_TEST_KEYED_FLAG]);
 
-function runAsyncTestObserver(aSubject, aTopic, aData) {
-  registerPingHandler(checkHistogramsAsync);
-  let histogramsFile = getSavedHistogramsFile("saved-histograms2.dat");
-
-  addWrappedObserver(function(aSubject, aTopic, aData) {
-    addWrappedObserver(function(aSubject, aTopic, aData) {
-      telemetry_ping();
-    }, "telemetry-test-load-complete");
-
-    TelemetryPing.testLoadHistograms(histogramsFile, false);
-  }, "telemetry-test-save-complete");
-  TelemetryPing.saveHistograms(histogramsFile, false);
-}
-
-function checkPersistedHistogramsAsync(request, response) {
-  // do not need the http server anymore
-  httpserver.stop(do_test_finished);
-  // Even though we have had four successful pings when this handler is
-  // run, we only had three successful pings when the histograms were
-  // saved.
-  checkPayload(request, "saved-session", 3);
-
-  runOldPingFileTest();
-
-  gFinished = true;
-}
-
-function checkHistogramsAsync(request, response) {
-  registerPingHandler(checkPersistedHistogramsAsync);
-  checkPayload(request, "test-ping", 3);
-}
-
-function runInvalidJSONTest() {
-  let histogramsFile = getSavedHistogramsFile("invalid-histograms.dat");
-  writeStringToFile(histogramsFile, "this.is.invalid.JSON");
-  do_check_true(histogramsFile.exists());
-  
-  TelemetryPing.testLoadHistograms(histogramsFile, true);
-  do_check_false(histogramsFile.exists());
-}
-
-function runOldPingFileTest() {
-  let histogramsFile = getSavedHistogramsFile("old-histograms.dat");
-  TelemetryPing.saveHistograms(histogramsFile, true);
-  do_check_true(histogramsFile.exists());
-
-  let mtime = histogramsFile.lastModifiedTime;
-  histogramsFile.lastModifiedTime = mtime - (14 * 24 * 60 * 60 * 1000 + 60000); // 14 days, 1m
-  TelemetryPing.testLoadHistograms(histogramsFile, true);
-  do_check_false(histogramsFile.exists());
+  const expected_keyed_count = {
+    "a": {
+      range: [1, 2],
+      bucket_count: 3,
+      histogram_type: 4,
+      values: {0:2, 1:0},
+      sum: 2,
+      sum_squares_lo: 2,
+      sum_squares_hi: 0,
+    },
+    "b": {
+      range: [1, 2],
+      bucket_count: 3,
+      histogram_type: 4,
+      values: {0:1, 1:0},
+      sum: 1,
+      sum_squares_lo: 1,
+      sum_squares_hi: 0,
+    },
+  };
+  Assert.deepEqual(expected_keyed_count, keyedHistograms[TELEMETRY_TEST_KEYED_COUNT]);
 }
 
 function dummyTheme(id) {
@@ -380,7 +372,7 @@ function dummyTheme(id) {
 }
 
 // A fake plugin host for testing flash version telemetry
-var PluginHost = {
+let PluginHost = {
   getPluginTags: function(countRef) {
     let plugins = [{name: "Shockwave Flash", version: FLASH_VERSION}];
     countRef.value = plugins.length;
@@ -396,7 +388,7 @@ var PluginHost = {
   }
 }
 
-var PluginHostFactory = {
+let PluginHostFactory = {
   createInstance: function (outer, iid) {
     if (outer != null)
       throw Components.results.NS_ERROR_NO_AGGREGATION;
@@ -408,7 +400,7 @@ const PLUGINHOST_CONTRACTID = "@mozilla.org/plugin/host;1";
 const PLUGINHOST_CID = Components.ID("{2329e6ea-1f15-4cbe-9ded-6e98e842de0e}");
 
 function registerFakePluginHost() {
-  var registrar = Components.manager.QueryInterface(Ci.nsIComponentRegistrar);
+  let registrar = Components.manager.QueryInterface(Ci.nsIComponentRegistrar);
   registrar.registerFactory(PLUGINHOST_CID, "Fake Plugin Host",
                             PLUGINHOST_CONTRACTID, PluginHostFactory);
 }
@@ -442,7 +434,7 @@ function write_fake_failedprofilelocks_file() {
 function run_test() {
   do_test_pending();
   try {
-    var gfxInfo = Cc["@mozilla.org/gfx/info;1"].getService(Ci.nsIGfxInfoDebug);
+    let gfxInfo = Cc["@mozilla.org/gfx/info;1"].getService(Ci.nsIGfxInfoDebug);
     gfxInfo.spoofVendorID("0xabcd");
     gfxInfo.spoofDeviceID("0x1234");
   } catch (x) {
@@ -453,6 +445,16 @@ function run_test() {
   do_get_profile();
   createAppInfo("xpcshell@tests.mozilla.org", "XPCShell", "1", "1.9.2");
 
+  Services.prefs.setBoolPref(PREF_ENABLED, true);
+  Services.prefs.setBoolPref(PREF_FHR_UPLOAD_ENABLED, true);
+
+  // Send the needed startup notifications to the datareporting service
+  // to ensure that it has been initialized.
+  if (HAS_DATAREPORTINGSERVICE) {
+    gDatareportingService.observe(null, "app-startup", null);
+    gDatareportingService.observe(null, "profile-after-change", null);
+  }
+
   // Make it look like we've previously failed to lock a profile a couple times.
   write_fake_failedprofilelocks_file();
 
@@ -485,8 +487,6 @@ function run_test() {
 }
 
 function actualTest() {
-  // ensure that test runs to completion
-  do_register_cleanup(function () do_check_true(gFinished));
   // try to make LightweightThemeManager do stuff
   let gInternalManager = Cc["@mozilla.org/addons/integration;1"]
                          .getService(Ci.nsIObserver)
@@ -498,11 +498,160 @@ function actualTest() {
   // fake plugin host for consistent flash version data
   registerFakePluginHost();
 
-  runInvalidJSONTest();
+  run_next_test();
+}
 
-  addWrappedObserver(nonexistentServerObserver, "telemetry-test-xhr-complete");
-  telemetry_ping();
-  // spin the event loop
-  do_test_pending();
-  do_test_finished();
+add_task(function* asyncSetup() {
+  yield TelemetryPing.setup();
+
+  if (HAS_DATAREPORTINGSERVICE) {
+    // force getSessionRecorder()==undefined to check the payload's activeTicks
+    gDatareportingService.simulateNoSessionRecorder();
+  }
+
+  // When no DRS or no DRS.getSessionRecorder(), activeTicks should be -1.
+  do_check_eq(TelemetryPing.getPayload().simpleMeasurements.activeTicks, -1);
+
+  if (HAS_DATAREPORTINGSERVICE) {
+    // Restore normal behavior for getSessionRecorder()
+    gDatareportingService.simulateRestoreSessionRecorder();
+
+    gDataReportingClientID = yield gDatareportingService.getClientID();
+
+    // We should have cached the client id now. Lets confirm that by
+    // checking the client id before the async ping setup is finished.
+    let promisePingSetup = TelemetryPing.reset();
+    do_check_eq(TelemetryPing.clientID, gDataReportingClientID);
+    yield promisePingSetup;
+  }
+});
+
+// Ensure that not overwriting an existing file fails silently
+add_task(function* test_overwritePing() {
+  let ping = {slug: "foo"}
+  yield TelemetryFile.savePing(ping, true);
+  yield TelemetryFile.savePing(ping, false);
+  yield TelemetryFile.cleanupPingFile(ping);
+});
+
+// Ensures that expired histograms are not part of the payload.
+add_task(function* test_expiredHistogram() {
+  let histogram_id = "FOOBAR";
+  let dummy = Telemetry.newHistogram(histogram_id, "30", Telemetry.HISTOGRAM_EXPONENTIAL, 1, 2, 3);
+
+  dummy.add(1);
+
+  do_check_eq(TelemetryPing.getPayload()["histograms"][histogram_id], undefined);
+  do_check_eq(TelemetryPing.getPayload()["histograms"]["TELEMETRY_TEST_EXPIRED"], undefined);
+});
+
+// Checks that an invalid histogram file is deleted if TelemetryFile fails to parse it.
+add_task(function* test_runInvalidJSON() {
+  let histogramsFile = getSavedHistogramsFile("invalid-histograms.dat");
+
+  writeStringToFile(histogramsFile, "this.is.invalid.JSON");
+  do_check_true(histogramsFile.exists());
+
+  yield TelemetryPing.testLoadHistograms(histogramsFile);
+  do_check_false(histogramsFile.exists());
+});
+
+// Sends a ping to a non existing server.
+add_task(function* test_noServerPing() {
+  yield sendPing();
+});
+
+// Checks that a sent ping is correctly received by a dummy http server.
+add_task(function* test_simplePing() {
+  gHttpServer.start(-1);
+  gServerStarted = true;
+  gRequestIterator = Iterator(new Request());
+
+  yield sendPing();
+  let request = yield gRequestIterator.next();
+  let payload = decodeRequestPayload(request);
+
+  checkPayloadInfo(payload, "test-ping");
+});
+
+// Saves the current session histograms, reloads them, perfoms a ping
+// and checks that the dummy http server received both the previously
+// saved histograms and the new ones.
+add_task(function* test_saveLoadPing() {
+  let histogramsFile = getSavedHistogramsFile("saved-histograms.dat");
+
+  setupTestData();
+  yield TelemetryPing.testSaveHistograms(histogramsFile);
+  yield TelemetryPing.testLoadHistograms(histogramsFile);
+  yield sendPing();
+
+  // Get requests received by dummy server.
+  let request1 = yield gRequestIterator.next();
+  let request2 = yield gRequestIterator.next();
+
+  // Check we have the correct two requests. Ordering is not guaranteed.
+  if (request1.path.contains("test-ping")) {
+    checkPayload(request1, "test-ping", 1);
+    checkPayload(request2, "saved-session", 1);
+  } else {
+    checkPayload(request1, "saved-session", 1);
+    checkPayload(request2, "test-ping", 1);
+  }
+});
+
+// Checks that an expired histogram file is deleted when loaded.
+add_task(function* test_runOldPingFile() {
+  let histogramsFile = getSavedHistogramsFile("old-histograms.dat");
+
+  yield TelemetryPing.testSaveHistograms(histogramsFile);
+  do_check_true(histogramsFile.exists());
+  let mtime = histogramsFile.lastModifiedTime;
+  histogramsFile.lastModifiedTime = mtime - (14 * 24 * 60 * 60 * 1000 + 60000); // 14 days, 1m
+
+  yield TelemetryPing.testLoadHistograms(histogramsFile);
+  do_check_false(histogramsFile.exists());
+});
+
+add_task(function* test_savedSessionClientID() {
+  // Assure that we store the ping properly when saving sessions on shutdown.
+  // We make the TelemetryPings shutdown to trigger a session save.
+  const dir = TelemetryFile.pingDirectoryPath;
+  yield OS.File.removeDir(dir, {ignoreAbsent: true});
+  yield OS.File.makeDir(dir);
+  yield TelemetryPing.shutdown();
+
+  yield TelemetryFile.loadSavedPings();
+  Assert.equal(TelemetryFile.pingsLoaded, 1);
+  let ping = TelemetryFile.popPendingPings().next();
+  Assert.equal(ping.value.payload.clientID, gDataReportingClientID);
+});
+
+add_task(function* stopServer(){
+  gHttpServer.stop(do_test_finished);
+});
+
+// An iterable sequence of http requests
+function Request() {
+  let defers = [];
+  let current = 0;
+
+  function RequestIterator() {}
+
+  // Returns a promise that resolves to the next http request
+  RequestIterator.prototype.next = function() {
+    let deferred = defers[current++];
+    return deferred.promise;
+  }
+
+  this.__iterator__ = function(){
+    return new RequestIterator();
+  }
+
+  registerPingHandler((request, response) => {
+    let deferred = defers[defers.length - 1];
+    defers.push(Promise.defer());
+    deferred.resolve(request);
+  });
+
+  defers.push(Promise.defer());
 }
