@@ -12,6 +12,7 @@
 #include "crashreporter.h"
 
 #include <windows.h>
+#include <versionhelpers.h>
 #include <commctrl.h>
 #include <richedit.h>
 #include <shellapi.h>
@@ -23,7 +24,6 @@
 #include "resource.h"
 #include "client/windows/sender/crash_report_sender.h"
 #include "common/windows/string_utils-inl.h"
-#include "mozilla/NullPtr.h"
 
 #define CRASH_REPORTER_VALUE L"Enabled"
 #define SUBMIT_REPORT_VALUE  L"SubmitCrashReport"
@@ -33,8 +33,15 @@
 #define EMAIL_VALUE          L"Email"
 #define MAX_EMAIL_LENGTH     1024
 
+#define SENDURL_ORIGINAL L"https://crash-reports.mozilla.com/submit"
+#define SENDURL_XPSP2 L"https://crash-reports-xpsp2.mozilla.com/submit"
+
 #define WM_UPLOADCOMPLETE WM_APP
 
+// Thanks, Windows.h :(
+#undef min
+#undef max
+
 using std::string;
 using std::wstring;
 using std::map;
@@ -48,8 +55,8 @@ using namespace CrashReporter;
 
 typedef struct {
   HWND hDlg;
-  wstring dumpFile;
   map<wstring,wstring> queryParameters;
+  map<wstring,wstring> files;
   wstring sendURL;
 
   wstring serverResponse;
@@ -395,7 +402,7 @@ static DWORD WINAPI SendThreadProc(LPVOID param)
     google_breakpad::CrashReportSender sender(L"");
     finishedOk = (sender.SendCrashReport(td->sendURL,
                                          td->queryParameters,
-                                         td->dumpFile,
+                                         td->files,
                                          &td->serverResponse)
                   == google_breakpad::RESULT_SUCCEEDED);
     if (finishedOk) {
@@ -1289,15 +1296,28 @@ void UIShowDefaultUI()
              MB_OK | MB_ICONSTOP);
 }
 
-bool UIShowCrashUI(const string& dumpFile,
+bool UIShowCrashUI(const StringTable& files,
                    const StringTable& queryParameters,
                    const string& sendURL,
                    const vector<string>& restartArgs)
 {
   gSendData.hDlg = nullptr;
-  gSendData.dumpFile = UTF8ToWide(dumpFile);
   gSendData.sendURL = UTF8ToWide(sendURL);
 
+  // Windows XP SP2 doesn't support the crash report server's crypto.
+  // This is a hack to use an alternate server.
+  if (!IsWindowsXPSP3OrGreater() &&
+      gSendData.sendURL.find(SENDURL_ORIGINAL) == 0) {
+    gSendData.sendURL.replace(0, ARRAYSIZE(SENDURL_ORIGINAL) - 1,
+                              SENDURL_XPSP2);
+  }
+
+  for (StringTable::const_iterator i = files.begin();
+       i != files.end();
+       i++) {
+    gSendData.files[UTF8ToWide(i->first)] = UTF8ToWide(i->second);
+  }
+
   for (StringTable::const_iterator i = queryParameters.begin();
        i != queryParameters.end();
        i++) {
@@ -1448,24 +1468,30 @@ ifstream* UIOpenRead(const string& filename)
   return file;
 }
 
-ofstream* UIOpenWrite(const string& filename, bool append) // append=false
+ofstream* UIOpenWrite(const string& filename,
+                      bool append, // append=false
+                      bool binary) // binary=false
 {
   // adapted from breakpad's src/common/windows/http_upload.cc
+  std::ios_base::openmode mode = ios::out;
+  if (append) {
+    mode = mode | ios::app;
+  }
+  if (binary) {
+    mode = mode | ios::binary;
+  }
 
-  // The "open" method on pre-MSVC8 ifstream implementations doesn't accept a
-  // wchar_t* filename, so use _wfopen directly in that case.  For VC8 and
-  // later, _wfopen has been deprecated in favor of _wfopen_s, which does
-  // not exist in earlier versions, so let the ifstream open the file itself.
+  // For VC8 and later, _wfopen has been deprecated in favor of _wfopen_s,
+  // which does not exist in earlier versions, so let the ifstream open the
+  // file itself.
 #if _MSC_VER >= 1400  // MSVC 2005/8
   ofstream* file = new ofstream();
-  file->open(UTF8ToWide(filename).c_str(), append ? ios::out | ios::app
-                                                  : ios::out);
+  file->open(UTF8ToWide(filename).c_str(), mode);
 #elif defined(_MSC_VER)
-  ofstream* file = new ofstream(_wfopen(UTF8ToWide(filename).c_str(),
-                                        append ? L"a" : L"w"));
+#error "Compiling with your version of MSVC is no longer supported."
 #else   // GCC
   ofstream* file = new ofstream(WideToMBCP(UTF8ToWide(filename), CP_ACP).c_str(),
-                                append ? ios::out | ios::app : ios::out);
+                                mode);
 #endif  // _MSC_VER >= 1400
 
   return file;
