@@ -11,14 +11,19 @@ let handlerCount = 0;
 
 let orig_w3c_touch_events = Services.prefs.getIntPref('dom.w3c_touch_events.enabled');
 
+let trackedWindows = new WeakMap();
+
 // =================== Touch ====================
 // Simulate touch events on desktop
 function TouchEventHandler (window) {
+  // Returns an already instanciated handler for this window
+  let cached = trackedWindows.get(window);
+  if (cached) {
+    return cached;
+  }
+
   let contextMenuTimeout = 0;
 
-  // This guard is used to not re-enter the events processing loop for
-  // self dispatched events
-  let ignoreEvents = false;
 
   let threshold = 25;
   try {
@@ -32,37 +37,67 @@ function TouchEventHandler (window) {
 
   let TouchEventHandler = {
     enabled: false,
-    events: ['mousedown', 'mousemove', 'mouseup', 'click'],
+    events: ['mousedown', 'mousemove', 'mouseup', 'touchstart', 'touchend'],
     start: function teh_start() {
-      let isReloadNeeded = Services.prefs.getIntPref('dom.w3c_touch_events.enabled') != 1;
-      handlerCount++;
-      Services.prefs.setIntPref('dom.w3c_touch_events.enabled', 1);
+      if (this.enabled)
+        return false;
       this.enabled = true;
+      let isReloadNeeded = Services.prefs.getIntPref('dom.w3c_touch_events.enabled') != 1;
+      Services.prefs.setIntPref('dom.w3c_touch_events.enabled', 1);
       this.events.forEach((function(evt) {
-        window.addEventListener(evt, this, true);
+        // Only listen trusted events to prevent messing with
+        // event dispatched manually within content documents
+        window.addEventListener(evt, this, true, false);
       }).bind(this));
       return isReloadNeeded;
     },
     stop: function teh_stop() {
-      handlerCount--;
-      if (handlerCount == 0)
-        Services.prefs.setIntPref('dom.w3c_touch_events.enabled', orig_w3c_touch_events);
+      if (!this.enabled)
+        return;
       this.enabled = false;
+      Services.prefs.setIntPref('dom.w3c_touch_events.enabled', orig_w3c_touch_events);
       this.events.forEach((function(evt) {
         window.removeEventListener(evt, this, true);
       }).bind(this));
     },
     handleEvent: function teh_handleEvent(evt) {
-      if (evt.button || ignoreEvents ||
-          evt.mozInputSource == Ci.nsIDOMMouseEvent.MOZ_SOURCE_UNKNOWN)
-        return;
-
       // The gaia system window use an hybrid system even on the device which is
       // a mix of mouse/touch events. So let's not cancel *all* mouse events
       // if it is the current target.
       let content = this.getContent(evt.target);
       let isSystemWindow = content.location.toString().indexOf("system.gaiamobile.org") != -1;
 
+      // App touchstart & touchend should also be dispatched on the system app
+      // to match on-device behavior.
+      if (evt.type.startsWith('touch') && !isSystemWindow) {
+        let sysFrame = content.realFrameElement;
+        let sysDocument = sysFrame.ownerDocument;
+        let sysWindow = sysDocument.defaultView;
+
+        let touchEvent = sysDocument.createEvent('touchevent');
+        let touch = evt.touches[0] || evt.changedTouches[0];
+        let point = sysDocument.createTouch(sysWindow, sysFrame, 0,
+                                            touch.pageX, touch.pageY,
+                                            touch.screenX, touch.screenY,
+                                            touch.clientX, touch.clientY,
+                                            1, 1, 0, 0);
+
+        let touches = sysDocument.createTouchList(point);
+        let targetTouches = touches;
+        let changedTouches = touches;
+        touchEvent.initTouchEvent(evt.type, true, true, sysWindow, 0,
+                                  false, false, false, false,
+                                  touches, targetTouches, changedTouches);
+        sysFrame.dispatchEvent(touchEvent);
+        return;
+      }
+
+      // Ignore all but real mouse event coming from physical mouse
+      // (especially ignore mouse event being dispatched from a touch event)
+      if (evt.button || evt.mozInputSource != Ci.nsIDOMMouseEvent.MOZ_SOURCE_MOUSE || evt.isSynthesized) {
+        return;
+      }
+
       let eventTarget = this.target;
       let type = '';
       switch (evt.type) {
@@ -105,6 +140,13 @@ function TouchEventHandler (window) {
 
           content.clearTimeout(contextMenuTimeout);
           type = 'touchend';
+
+          // Only register click listener after mouseup to ensure
+          // catching only real user click. (Especially ignore click
+          // being dispatched on form submit)
+          if (evt.detail == 1) {
+            window.addEventListener('click', this, true, false);
+          }
           break;
 
         case 'click':
@@ -113,14 +155,20 @@ function TouchEventHandler (window) {
           evt.preventDefault();
           evt.stopImmediatePropagation();
 
+          window.removeEventListener('click', this, true, false);
+
           if (this.cancelClick)
             return;
 
           ignoreEvents = true;
           content.setTimeout(function dispatchMouseEvents(self) {
-            self.fireMouseEvent('mousedown', evt);
-            self.fireMouseEvent('mousemove', evt);
-            self.fireMouseEvent('mouseup', evt);
+            try {
+              self.fireMouseEvent('mousedown', evt);
+              self.fireMouseEvent('mousemove', evt);
+              self.fireMouseEvent('mouseup', evt);
+            } catch(e) {
+              Cu.reportError('Exception in touch event helper: ' + e);
+            }
             ignoreEvents = false;
          }, 0, this);
 
@@ -141,7 +189,7 @@ function TouchEventHandler (window) {
       let content = this.getContent(evt.target);
       var utils = content.QueryInterface(Ci.nsIInterfaceRequestor)
                          .getInterface(Ci.nsIDOMWindowUtils);
-      utils.sendMouseEvent(type, evt.clientX, evt.clientY, 0, 1, 0, true);
+      utils.sendMouseEvent(type, evt.clientX, evt.clientY, 0, 1, 0, true, 0, Ci.nsIDOMMouseEvent.MOZ_SOURCE_TOUCH);
     },
     sendContextMenu: function teh_sendContextMenu(target, x, y, delay) {
       let doc = target.ownerDocument;
@@ -159,6 +207,30 @@ function TouchEventHandler (window) {
       return timeout;
     },
     sendTouchEvent: function teh_sendTouchEvent(evt, target, name) {
+      // When running OOP b2g desktop, we need to send the touch events
+      // using the mozbrowser api on the unwrapped frame.
+      if (target.localName == "iframe" && target.mozbrowser === true) {
+        if (name == "touchstart") {
+          this.touchstartTime = Date.now();
+        } else if (name == "touchend") {
+          // If we have a 'fast' tap, don't send a click as both will be turned
+          // into a click and that breaks eg. checkboxes.
+          if (Date.now() - this.touchstartTime < delay) {
+            this.cancelClick = true;
+          }
+        }
+        function clone(obj) {
+          return Cu.cloneInto(obj, target);
+        }
+        let unwraped = XPCNativeWrapper.unwrap(target);
+        unwraped.sendTouchEvent(name, clone([0]),       // event type, id
+                                clone([evt.clientX]),   // x
+                                clone([evt.clientY]),   // y
+                                clone([1]), clone([1]), // rx, ry
+                                clone([0]), clone([0]), // rotation, force
+                                1);                     // count
+        return;
+      }
       let document = target.ownerDocument;
       let content = this.getContent(target);
 
@@ -179,15 +251,10 @@ function TouchEventHandler (window) {
     },
     getContent: function teh_getContent(target) {
       let win = target.ownerDocument.defaultView;
-      let docShell = win.QueryInterface(Ci.nsIInterfaceRequestor)
-                    .getInterface(Ci.nsIWebNavigation)
-                    .QueryInterface(Ci.nsIDocShellTreeItem);
-      let topDocShell = docShell.sameTypeRootTreeItem;
-      topDocShell.QueryInterface(Ci.nsIDocShell);
-      let top = topDocShell.contentViewer.DOMDocument.defaultView;
-      return top;
+      return win;
     }
   };
+  trackedWindows.set(window, TouchEventHandler);
 
   return TouchEventHandler;
 }
