@@ -5,9 +5,6 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifdef MOZ_WIDGET_QT
-#include <QDesktopServices>
-#include <QUrl>
-#include <QString>
 #if (MOZ_ENABLE_CONTENTACTION)
 #include <contentaction/contentaction.h>
 #include "nsContentHandlerApp.h"
@@ -24,6 +21,9 @@
 #ifdef MOZ_ENABLE_DBUS
 #include "nsDBusHandlerApp.h"
 #endif
+#ifdef MOZ_WIDGET_QT
+#include "nsMIMEInfoQt.h"
+#endif
 
 nsresult
 nsMIMEInfoUnix::LoadUriInternal(nsIURI * aURI)
@@ -32,11 +32,7 @@ nsMIMEInfoUnix::LoadUriInternal(nsIURI * aURI)
 
 #ifdef MOZ_WIDGET_QT
   if (NS_FAILED(rv)) {
-    nsAutoCString spec;
-    aURI->GetAsciiSpec(spec);
-    if (QDesktopServices::openUrl(QUrl(spec.get()))) {
-      rv = NS_OK;
-    }
+    rv = nsMIMEInfoQt::LoadUriInternal(aURI);
   }
 #endif
 
@@ -53,16 +49,21 @@ nsMIMEInfoUnix::GetHasDefaultHandler(bool *_retval)
     return nsMIMEInfoImpl::GetHasDefaultHandler(_retval);
 
   *_retval = false;
-  nsRefPtr<nsMIMEInfoBase> mimeInfo = nsGNOMERegistry::GetFromType(mSchemeOrType);
-  if (!mimeInfo) {
-    nsAutoCString ext;
-    nsresult rv = GetPrimaryExtension(ext);
-    if (NS_SUCCEEDED(rv)) {
-      mimeInfo = nsGNOMERegistry::GetFromExtension(ext);
+
+  if (mClass ==  eProtocolInfo) {
+    *_retval = nsGNOMERegistry::HandlerExists(mSchemeOrType.get());
+  } else {
+    nsRefPtr<nsMIMEInfoBase> mimeInfo = nsGNOMERegistry::GetFromType(mSchemeOrType);
+    if (!mimeInfo) {
+      nsAutoCString ext;
+      nsresult rv = GetPrimaryExtension(ext);
+      if (NS_SUCCEEDED(rv)) {
+        mimeInfo = nsGNOMERegistry::GetFromExtension(ext);
+      }
     }
+    if (mimeInfo)
+      *_retval = true;
   }
-  if (mimeInfo)
-    *_retval = true;
 
   if (*_retval)
     return NS_OK;
@@ -165,7 +166,7 @@ nsMIMEInfoUnix::GetPossibleApplicationHandlers(nsIMutableArray ** aPossibleAppHa
 
     for (int i = 0; i < actions.size(); ++i) {
       nsContentHandlerApp* app =
-        new nsContentHandlerApp(nsString((PRUnichar*)actions[i].name().data()), 
+        new nsContentHandlerApp(nsString((char16_t*)actions[i].name().data()), 
                                 mSchemeOrType, actions[i]);
       mPossibleApplications->AppendElement(app, false);
     }
