@@ -10,7 +10,6 @@
 #include "nsIWebProgress.h"
 #include "nsCURILoader.h"
 #include "nsICachingChannel.h"
-#include "nsICacheVisitor.h"
 #include "nsIHttpChannel.h"
 #include "nsIURL.h"
 #include "nsISimpleEnumerator.h"
@@ -29,6 +28,7 @@
 #include "nsIDOMNode.h"
 #include "nsINode.h"
 #include "nsIDocument.h"
+#include "nsContentUtils.h"
 
 using namespace mozilla;
 
@@ -74,10 +74,11 @@ class nsPrefetchQueueEnumerator MOZ_FINAL : public nsISimpleEnumerator
 public:
     NS_DECL_ISUPPORTS
     NS_DECL_NSISIMPLEENUMERATOR
-    nsPrefetchQueueEnumerator(nsPrefetchService *aService);
-    ~nsPrefetchQueueEnumerator();
+    explicit nsPrefetchQueueEnumerator(nsPrefetchService *aService);
 
 private:
+    ~nsPrefetchQueueEnumerator();
+
     void Increment();
 
     nsRefPtr<nsPrefetchService> mService;
@@ -114,7 +115,7 @@ nsPrefetchQueueEnumerator::GetNext(nsISupports **aItem)
 {
     if (!mCurrent) return NS_ERROR_FAILURE;
 
-    NS_ADDREF(*aItem = static_cast<nsIDOMLoadStatus*>(mCurrent.get()));
+    NS_ADDREF(*aItem = static_cast<nsIStreamListener*>(mCurrent.get()));
 
     Increment();
 
@@ -156,7 +157,7 @@ nsPrefetchQueueEnumerator::Increment()
 // nsPrefetchQueueEnumerator::nsISupports
 //-----------------------------------------------------------------------------
 
-NS_IMPL_ISUPPORTS1(nsPrefetchQueueEnumerator, nsISimpleEnumerator)
+NS_IMPL_ISUPPORTS(nsPrefetchQueueEnumerator, nsISimpleEnumerator)
 
 //-----------------------------------------------------------------------------
 // nsPrefetchNode <public>
@@ -171,7 +172,6 @@ nsPrefetchNode::nsPrefetchNode(nsPrefetchService *aService,
     , mReferrerURI(aReferrerURI)
     , mService(aService)
     , mChannel(nullptr)
-    , mState(nsIDOMLoadStatus::UNINITIALIZED)
     , mBytesRead(0)
 {
     mSource = do_GetWeakReference(aSource);
@@ -189,9 +189,14 @@ nsPrefetchNode::OpenChannel()
     nsCOMPtr<nsILoadGroup> loadGroup = source->OwnerDoc()->GetDocumentLoadGroup();
     nsresult rv = NS_NewChannel(getter_AddRefs(mChannel),
                                 mURI,
-                                nullptr, loadGroup, this,
+                                nsContentUtils::GetSystemPrincipal(),
+                                nsILoadInfo::SEC_NORMAL,
+                                nsIContentPolicy::TYPE_OTHER,
+                                loadGroup, // aLoadGroup
+                                this,      // aCallbacks
                                 nsIRequest::LOAD_BACKGROUND |
                                 nsICachingChannel::LOAD_ONLY_IF_MODIFIED);
+
     NS_ENSURE_SUCCESS(rv, rv);
 
     // configure HTTP specific stuff
@@ -208,8 +213,6 @@ nsPrefetchNode::OpenChannel()
     rv = mChannel->AsyncOpen(this, nullptr);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    mState = nsIDOMLoadStatus::REQUESTED;
-
     return NS_OK;
 }
 
@@ -219,8 +222,6 @@ nsPrefetchNode::CancelChannel(nsresult error)
     mChannel->Cancel(error);
     mChannel = nullptr;
 
-    mState = nsIDOMLoadStatus::UNINITIALIZED;
-
     return NS_OK;
 }
 
@@ -228,13 +229,12 @@ nsPrefetchNode::CancelChannel(nsresult error)
 // nsPrefetchNode::nsISupports
 //-----------------------------------------------------------------------------
 
-NS_IMPL_ISUPPORTS6(nsPrefetchNode,
-                   nsIDOMLoadStatus,
-                   nsIRequestObserver,
-                   nsIStreamListener,
-                   nsIInterfaceRequestor,
-                   nsIChannelEventSink,
-                   nsIRedirectResultListener)
+NS_IMPL_ISUPPORTS(nsPrefetchNode,
+                  nsIRequestObserver,
+                  nsIStreamListener,
+                  nsIInterfaceRequestor,
+                  nsIChannelEventSink,
+                  nsIRedirectResultListener)
 
 //-----------------------------------------------------------------------------
 // nsPrefetchNode::nsIStreamListener
@@ -280,8 +280,6 @@ nsPrefetchNode::OnStartRequest(nsIRequest *aRequest,
         }
     }
 
-    mState = nsIDOMLoadStatus::RECEIVING;
-
     return NS_OK;
 }
 
@@ -307,8 +305,6 @@ nsPrefetchNode::OnStopRequest(nsIRequest *aRequest,
 {
     LOG(("done prefetching [status=%x]\n", aStatus));
 
-    mState = nsIDOMLoadStatus::LOADED;
-
     if (mBytesRead == 0 && aStatus == NS_OK) {
         // we didn't need to read (because LOAD_ONLY_IF_MODIFIED was
         // specified), but the object should report loadedSize as if it
@@ -487,7 +483,7 @@ nsPrefetchService::NotifyLoadRequested(nsPrefetchNode *node)
     if (!observerService)
       return;
 
-    observerService->NotifyObservers(static_cast<nsIDOMLoadStatus*>(node),
+    observerService->NotifyObservers(static_cast<nsIStreamListener*>(node),
                                      "prefetch-load-requested", nullptr);
 }
 
@@ -499,7 +495,7 @@ nsPrefetchService::NotifyLoadCompleted(nsPrefetchNode *node)
     if (!observerService)
       return;
 
-    observerService->NotifyObservers(static_cast<nsIDOMLoadStatus*>(node),
+    observerService->NotifyObservers(static_cast<nsIStreamListener*>(node),
                                      "prefetch-load-completed", nullptr);
 }
 
@@ -628,11 +624,11 @@ nsPrefetchService::StopPrefetching()
 // nsPrefetchService::nsISupports
 //-----------------------------------------------------------------------------
 
-NS_IMPL_ISUPPORTS4(nsPrefetchService,
-                   nsIPrefetchService,
-                   nsIWebProgressListener,
-                   nsIObserver,
-                   nsISupportsWeakReference)
+NS_IMPL_ISUPPORTS(nsPrefetchService,
+                  nsIPrefetchService,
+                  nsIWebProgressListener,
+                  nsIObserver,
+                  nsISupportsWeakReference)
 
 //-----------------------------------------------------------------------------
 // nsPrefetchService::nsIPrefetchService
@@ -765,84 +761,6 @@ nsPrefetchService::EnumerateQueue(nsISimpleEnumerator **aEnumerator)
     return NS_OK;
 }
 
-//-----------------------------------------------------------------------------
-// nsPrefetchNode::nsIDOMLoadStatus
-//-----------------------------------------------------------------------------
-NS_IMETHODIMP
-nsPrefetchNode::GetSource(nsIDOMNode **aSource)
-{
-    *aSource = nullptr;
-    nsCOMPtr<nsIDOMNode> source = do_QueryReferent(mSource);
-    if (source)
-        source.swap(*aSource);
-
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPrefetchNode::GetUri(nsAString &aURI)
-{
-    nsAutoCString spec;
-    nsresult rv = mURI->GetSpec(spec);
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    CopyUTF8toUTF16(spec, aURI);
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPrefetchNode::GetTotalSize(int32_t *aTotalSize)
-{
-    if (mChannel) {
-        int64_t size64;
-        nsresult rv = mChannel->GetContentLength(&size64);
-        NS_ENSURE_SUCCESS(rv, rv);
-        *aTotalSize = int32_t(size64); // XXX - loses precision
-        return NS_OK;
-    }
-
-    *aTotalSize = -1;
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPrefetchNode::GetLoadedSize(int32_t *aLoadedSize)
-{
-    *aLoadedSize = int32_t(mBytesRead); // XXX - loses precision
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPrefetchNode::GetReadyState(uint16_t *aReadyState)
-{
-    *aReadyState = mState;
-    return NS_OK;
-}
-
-NS_IMETHODIMP
-nsPrefetchNode::GetStatus(uint16_t *aStatus)
-{
-    if (!mChannel) {
-        *aStatus = 0;
-        return NS_OK;
-    }
-
-    nsresult rv;
-    nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(mChannel, &rv);
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    uint32_t httpStatus;
-    rv = httpChannel->GetResponseStatus(&httpStatus);
-    if (rv == NS_ERROR_NOT_AVAILABLE) {
-        *aStatus = 0;
-        return NS_OK;
-    }
-
-    NS_ENSURE_SUCCESS(rv, rv);
-    *aStatus = uint16_t(httpStatus);
-    return NS_OK;
-}
-
 //-----------------------------------------------------------------------------
 // nsPrefetchService::nsIWebProgressListener
 //-----------------------------------------------------------------------------
@@ -890,7 +808,7 @@ NS_IMETHODIMP
 nsPrefetchService::OnStatusChange(nsIWebProgress* aWebProgress,
                                   nsIRequest* aRequest,
                                   nsresult aStatus,
-                                  const PRUnichar* aMessage)
+                                  const char16_t* aMessage)
 {
     NS_NOTREACHED("notification excluded in AddProgressListener(...)");
     return NS_OK;
@@ -912,7 +830,7 @@ nsPrefetchService::OnSecurityChange(nsIWebProgress *aWebProgress,
 NS_IMETHODIMP
 nsPrefetchService::Observe(nsISupports     *aSubject,
                            const char      *aTopic,
-                           const PRUnichar *aData)
+                           const char16_t *aData)
 {
     LOG(("nsPrefetchService::Observe [topic=%s]\n", aTopic));
 
